! ========================================================================
! Copyright (C) by VMware, Inc. 1991-2011.  All Rights Reserved
! ========================================================================

expectvalue %String
run
(Object
	subclass: 'CDeclaration'
	instVarNames: #( header name storage
                    type count pointer fields
                    isStorage isConstant isFunction includesCode
                    isVaryingArgCount isTransparentUnion bitCount source
                    file line)
	classVars: #()
	classInstVars: #()
	poolDictionaries: { }
	inDictionary: Globals
	options: #( #logCreation )) definition
%

! set class category
run
CDeclaration category: 'FFI'.
true
%

! Remove existing behavior from CDeclaration
doit
CDeclaration removeAllMethods.
CDeclaration class removeAllMethods.
true
%
! ------------------- Class methods for CDeclaration
category: 'Documentation'
set compile_env: 0
classmethod: CDeclaration
comment
^'This class is internal to the FFI implementation and 
represents a declaration from a C header file.'
%
category: 'other'
set compile_env: 0
classmethod: CDeclaration
header: aHeader

	^self basicNew
		initialize: aHeader;
		yourself.
%
! ------------------- Instance methods for CDeclaration
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
accessorForOffset: anInteger

	1 < count size ifTrue: [self error: 'Multi-dimensional arrays not supported!'].
	0 == pointer ifTrue: [
		type == #'class' ifTrue: [^self structAccessorForOffset: anInteger].
		(type isKindOf: CDeclaration) ifTrue: [^self structAccessorForOffset: anInteger].
		count == nil ifTrue: [^self simpleAccessorForOffset: anInteger].
		type == #'uint8' ifTrue: [^self stringAccessorForOffset: anInteger].
		^self arrayAccessorForOffset: anInteger.
	].
	1 < pointer ifTrue: [self error: 'Multi-level pointers not supported!'].
	count == nil ifTrue: [^self singlePointerAccessorForOffset: anInteger].
	type == #'uint8' ifTrue: [^self stringPointerAccessorForOffset: anInteger].
	^self arrayPointersAccessorForOffset: anInteger.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
addSourceTo: aStream

	aStream 
		lf; tab; nextPut: $";
		nextPutAll: (file isNil ifTrue: ['test'] ifFalse: [file]);
		nextPutAll: ' line ';
		nextPutAll: line printString;
		yourself.
	source do: [:char | aStream nextPut: char. char == $" ifTrue: [aStream nextPut: char]].
	aStream nextPut: $"; lf.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
argumentType

	(self isSimple and: [self knownArgumentTypes includes: type]) ifTrue: [^type].
	(type == #'uint8' and: [isConstant]) ifTrue: [
		(count size == 1 and: [pointer == 0]) ifTrue: [^#'const char*'].		"const char foo[]"
		(count == nil and: [pointer == 1]) ifTrue: [^#'const char*'].			"const char * foo"
	].
	^#'ptr'.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
arrayAccessorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name; lf; lf; tab;
		nextPutAll: '| array offset |'; lf; tab;
		nextPutAll: 'array := Array new: ';
		nextPutAll: count first printString;
		nextPut: $.; lf; tab;
		nextPutAll: 'offset := ';
		nextPutAll: anInteger printString;
		nextPut: $.; lf; tab;
		nextPutAll: '1 to: array size do: [:i | '; lf; tab; tab;
		nextPutAll: 'array at: i put: (cByteArray ';
		nextPutAll: self getSelector;
		nextPutAll: ' offset).'; lf; tab; tab;
		nextPutAll: 'offset := offset + ';
		nextPutAll: self baseByteSize printString;
		nextPut: $.; lf; tab;
		nextPutAll: '].'; lf; tab;
		nextPutAll: '^array.'; lf;
		contents.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
arrayPointersAccessorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name; lf; lf; tab;
		nextPutAll: '| array offset |'; lf; tab;
		nextPutAll: 'array := Array new: ';
		nextPutAll: count first printString;
		nextPut: $.; lf; tab;
		nextPutAll: 'offset := ';
		nextPutAll: anInteger printString;
		nextPut: $.; lf; tab;
		nextPutAll: '1 to: array size do: [:i | '; lf; tab; tab;
		nextPutAll: 'array at: i put: (cByteArray pointerAt: offset resultClass: CByteArray).'; lf; tab; tab;
		nextPutAll: 'offset := offset + 8.'; lf; tab;
		nextPutAll: '].'; lf; tab;
		nextPutAll: '^array.'; lf;
		contents.
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
arrayPointersUpdatorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name;
		nextPutAll: ': anArrayOfCByteArrayInstances'; lf; lf; tab;
		nextPutAll: '| offset |'; lf; tab;
		nextPutAll: 'offset := ';
		nextPutAll: anInteger printString;
		nextPut: $.; lf; tab;
		nextPutAll: '1 to: anArrayOfCByteArrayInstances size do: [:i | '; lf; tab; tab;
		nextPutAll: 'cByteArray'; lf; tab; tab; tab;
		nextPutAll: 'pointerAt: offset'; lf; tab; tab; tab; 
		nextPutAll: 'put: (anArrayOfCByteArrayInstances at: i).'; lf; tab; tab;
		nextPutAll: 'offset := offset + 8.'; lf; tab;
		nextPutAll: '].'; lf;
		contents.
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
arrayUpdatorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name;
		nextPutAll: ': aSequenceableCollection'; lf; lf; tab;
		nextPutAll: '| offset |'; lf; tab;
		nextPutAll: 'offset := ';
		nextPutAll: anInteger printString;
		nextPut: $.; lf; tab;
		nextPutAll: '1 to: aSequenceableCollection size do: [:i | '; lf; tab; tab;
		nextPutAll: 'cByteArray '; lf; tab; tab; tab;
		nextPutAll: self getSelector; space;
		nextPutAll: 'offset'; lf; tab; tab; tab;
		nextPutAll: 'put: (aSequenceableCollection at: i).'; lf; tab; tab;
		nextPutAll: 'offset := offset + ';
		nextPutAll: self baseByteSize printString;
		nextPut: $.; lf; tab;
		nextPutAll: '].'; lf;
		contents.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
asClassVarName

	^'Func' , name.
%
category: 'other'
set compile_env: 0
method: CDeclaration
atEnd

	^header atEnd.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
baseAccessorForOffset: anInteger

	^'cByteArray ' , self getSelector , ' ' , anInteger printString.
%
category: 'size'
set compile_env: 0
method: CDeclaration
baseBitSize

self halt.
	0 < pointer ifTrue: [^64].
	bitCount notNil ifTrue: [^bitCount].
	(type == #'int8' or: [type == #'uint8']) ifTrue: [^8].
	(type == #'int16' or: [type == #'uint16']) ifTrue: [^16].
	(type == #'int32' or: [type == #'uint32']) ifTrue: [^32].
	(type == #'int64' or: [type == #'uint64']) ifTrue: [^64].
	(type == #'void' and: [0 < pointer]) ifTrue: [^64].
	type == #'struct' ifTrue: [
		^fields
			inject: 0
			into: [:sum :each | sum + each baseBitSize].
	].
	(type isKindOf: CDeclaration) ifTrue: [^type baseBitSize].
	self halt.
%
category: 'size'
set compile_env: 0
method: CDeclaration
baseByteSize

	0 < pointer ifTrue: [^8].
	bitCount notNil ifTrue: [^bitCount].
	(type == #'int8' or: [type == #'uint8']) ifTrue: [^1].
	(type == #'int16' or: [type == #'uint16']) ifTrue: [^2].
	(type == #'int32' or: [type == #'uint32']) ifTrue: [^4].
	(type == #'int64' or: [type == #'uint64']) ifTrue: [^8].
	(type == #'void' and: [0 < pointer]) ifTrue: [^8].
	type == #'class' ifTrue: [^self baseStructByteSize].
	type == #'struct' ifTrue: [^self baseStructByteSize].
	type == #'union' ifTrue: [^self baseUnionByteSize].
	type == #'enum' ifTrue: [^4].
	type == #'function' ifTrue: [^8].
	(type isKindOf: CDeclaration) ifTrue: [^type baseByteSize].
	self halt.
%
category: 'size'
set compile_env: 0
method: CDeclaration
baseStructByteSize

	| bits |
	bits := 0.
	fields do: [:each |  
		each bitCount isNil ifTrue: [
			bits := bits + (each byteSize * 8).
		] ifFalse: [
			each count notNil ifTrue: [self error: 'Array of bits!?'].
			bits := bits + each bitCount.
		].
	].
	^(bits / 8) ceiling.
%
category: 'size'
set compile_env: 0
method: CDeclaration
baseUnionByteSize

	^fields 
		inject: 0
		into: [:max :each | max max: each baseByteSize].
%
category: 'other'
set compile_env: 0
method: CDeclaration
beExternStorage

	storage := #'extern'.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
bitCount

	^bitCount.
%
category: 'size'
set compile_env: 0
method: CDeclaration
byteSize

	| x |
	x := self baseByteSize.
	count isNil ifTrue: [^x].
	count do: [:y | x := x * y].
	^x.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
cCalloutForLibrary: aCLibrary

	^CCallout
		library: aCLibrary 
		name: name 
		result: self resultType 
		args: (fields collect: [:each | each argumentType])
		varArgsAfter: (isVaryingArgCount ifTrue: [fields size] ifFalse: [-1]).
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
count

	^count.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
createFunctionInitializerCode

	| stream space |
	stream := WriteStream on: String new.
	stream
		nextPutAll: self initializerFunctionName;
		nextPutAll: ' cLibrary'; lf; lf; tab;
		nextPutAll: self asClassVarName;
		nextPutAll: ' := CCallout'; lf; tab; tab;
		nextPutAll: 'library: cLibrary'; lf; tab; tab;
		nextPutAll: 'name: ''';
		nextPutAll: name; 
		nextPut: $'; lf; tab; tab;
		nextPutAll: 'result: #''';
		nextPutAll: self resultType;
		nextPutAll: ''''; lf; tab; tab;
		nextPutAll: 'args: #(';
		yourself.
	space := ''.
	fields do: [:each | 
		stream 
			nextPutAll: space;
			nextPutAll: each argumentType printString;
			yourself.
		space := ' '.
	].
	^stream
		nextPutAll: ')'; lf; tab; tab;
		nextPutAll: 'varArgsAfter: ';
		nextPutAll: self varArgsAfter printString;
		nextPut: $.; lf;
		contents.
%
category: 'other'
set compile_env: 0
method: CDeclaration
errorExpectedIdentifier

	self error: 'Expected an identifier'.
%
category: 'other'
set compile_env: 0
method: CDeclaration
expectCloseParenthesisCount: anInteger

	anInteger timesRepeat: [
		| token |
		(token := self next) isCloseParenthesisToken ifFalse: [self error: 'Expected close parenthesis'].
	].
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
fields

	^type == #'struct'
		ifTrue: [fields]
		ifFalse: [nil].
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
file

	^file.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
getSelector

	(#(
		#'int8' #'int16' #'int32' #'int64'
		#'uint8' #'uint16' #'uint32' #'uint64'
	) includes: type) ifTrue: [^type , 'At:'].
	type == #'enum' ifTrue: [^'int32At:'].
	type == #'function' ifTrue: [^'uint64At:'].
	self halt.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
header

	^header.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
includesCode

	^includesCode.
%
category: 'initializing'
set compile_env: 0
method: CDeclaration
initialize: aHeader

	header := aHeader.
	[self peek isEmptyToken] whileTrue: [self next].
	pointer := 0.
	isStorage := false.
	isConstant := false.
	isFunction := false.
	includesCode := false.
	isTransparentUnion := false.
	self readStorageClassSpecifier.
%
category: 'initializing'
set compile_env: 0
method: CDeclaration
initializeAsStructOrUnion

	storage := nil.
	self peek isOpenCurlyBracketToken ifTrue: [
		self next.
		fields notNil ifTrue: [self error: 'Fields already defined!'].
		fields := Array new.
		[
			self peek isCloseCurlyBracketToken not.
		] whileTrue: [
			| field token |
			field := self class header: header.
			field storage == #'auto' ifFalse: [self error: 'Unexpected storage!'].
			[
				fields add: field.
				(token := self next) isCommaToken.
			] whileTrue: [
				field := field copy.
				field readDeclarator.
			].
			token isSemicolonToken ifFalse: [self error: 'Unexpected end-of-field!'].
		].
		self next.
	].
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
initializerFunctionName
	^'init_', name, '_inLibrary:'
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
invokeFunctionCode

	| args stream delimiter |
	args := fields collect: [:each | each name].
	1 to: args size do: [:i | 
		(args at: i) isNil ifTrue: [args at: i put: 'arg' , i printString].
	].
	stream := (WriteStream on: String new)
		nextPutAll: name;
		yourself.
	args do: [:each | 
		stream 
			nextPutAll: '_: ';
			nextPutAll: each;
			space;
			yourself.
	].
	self addSourceTo: stream.
	stream
		lf; tab; 
		nextPutAll: '"Interpreted as #';
		nextPutAll: self resultType;
		nextPutAll: ' from #(';
		yourself.
	fields do: [:each | stream space; nextPut: $#; nextPutAll: each argumentType].
	stream 
		nextPutAll: ' )';
		nextPut: $"; lf; lf; 
		tab; nextPut: $^;
		nextPutAll: self asClassVarName;
		nextPutAll: ' callWith: { ';
		yourself.
	delimiter := ''.
	args do: [:each | 
		stream 
			nextPutAll: delimiter;
			nextPutAll: each;
			yourself.
		delimiter := '. '.
	].
	isVaryingArgCount ifTrue: [stream nextPutAll: '"varArgs array should be appended to this array"'].
	^stream 
		nextPutAll: ' }'; lf;
		contents.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
isConstant

	^isConstant.
%
category: 'querying'
set compile_env: 0
method: CDeclaration
isEmptyVoid

	^name == nil and: [
		storage == #'auto' and: [
		type == #'void' and: [
		count == nil and: [
		pointer == 0 and: [
		fields == nil and: [
		isConstant == false and: [
		self isFunction == false and: [
		bitCount == nil]]]]]]]].
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
isFunction

	^isFunction.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
isSimple

	^count isNil and: [pointer = 0].
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
isStorage

	^isStorage.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
isTransparentUnion

	^isTransparentUnion.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
isVaryingArgCount

	^isVaryingArgCount.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
knownArgumentTypes

	^#( #int64 #uint64 #int32 #uint32 #int16 #uint16 #int8 #uint8 #bool #double #float #ptr #'char*' "#'&ptr'" #'const char*' ).
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
knownResultTypes

	^#( #int64 #uint64 #int32 #uint32 #int16 #uint16 #int8 #uint8 #bool #double #float #ptr #'char*' #void ).
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
line

	^line.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
members

	^type == #'union'
		ifTrue: [fields]
		ifFalse: [nil].
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
name

	^name.
%
category: 'other'
set compile_env: 0
method: CDeclaration
next

	| next |
	[
		(next := self _next) type == #'empty'.
	] whileTrue: [].
	^next.
%
category: 'other'
set compile_env: 0
method: CDeclaration
nextNonEmpty

	^self peek; next.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
parameters

	^self isFunction
		ifTrue: [fields]
		ifFalse: [nil].
%
category: 'other'
set compile_env: 0
method: CDeclaration
peek

	| next |
	[
		next := header peek.
		next isNil ifTrue: [^nil].
		next type == #'empty'.
	] whileTrue: [
		self _next.
	].
	^next.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
pointer

	^pointer.
%
category: 'other'
set compile_env: 0
method: CDeclaration
printFunctionParametersOn: aStream

	| comma |
	(name isNil or: [name last ~~ $(]) ifTrue: [
		aStream nextPut: $(.
	].
	(fields isNil or: [fields isEmpty]) ifTrue: [aStream nextPutAll: 'void)'. ^self].
	comma := ''.
	fields do: [:each | 
		aStream nextPutAll: comma.
		each printOn: aStream.
		comma := ', '.
	].
	aStream nextPut: $).
%
category: 'other'
set compile_env: 0
method: CDeclaration
printOn: aStream

	(storage notNil and: [storage ~~ #'auto']) ifTrue: [aStream nextPutAll: storage; space].
	type notNil ifTrue: [
		(type isKindOf: Symbol)
			ifTrue: [aStream nextPutAll: type]
			ifFalse: [type printOn: aStream]. 
		aStream space.
	].
	pointer timesRepeat: [aStream nextPut: $*].
	name notNil ifTrue: [aStream nextPutAll: name].
	count notNil ifTrue: [
		count do: [:each | 
			aStream nextPut: $[.
			0 < each ifTrue: [each printOn: aStream].
			aStream nextPut: $].
		].
	].
	isFunction ifTrue: [^self printFunctionParametersOn: aStream].
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
properties

	^type == #'class'
		ifTrue: [fields]
		ifFalse: [nil].
%
category: 'attributes'
set compile_env: 0
method: CDeclaration
readAttributes
"
	http://tigcc.ticalc.org/doc/gnuexts.html
"
	| token flag |
	self peek isOpenParenthesisToken ifFalse: [self error: 'Expected parenthesis'].
	self next.
	(token := self peek) key == #'identifier' ifFalse: [^self].
	token value = '__mode__(' 	ifTrue: [self next. ^self readModeAttributes].
	token value = '__nonnull__(' 	ifTrue: [self next. ^self readNonnullAttribute].
	token value = '__format__('	ifTrue: [self next. ^self readFormatAttribute].
	token value = '__sentinel__('	ifTrue: [self next. ^self readNonnullAttribute].
	flag := #(
		'__const__'
		'__deprecated__'
		'__malloc__'
		'__noreturn__'
		'__nothrow__'
		'__pure__'
		'__transparent_union__'
		'__warn_unused_result__'
	) includes: token value.
	flag ifTrue: [
		isTransparentUnion := token value = '__transparent_union__'.
		self next. 
		self expectCloseParenthesisCount: 2. 
		^self
	].
%
category: 'other'
set compile_env: 0
method: CDeclaration
readBitFields

	bitCount := CPreprocessor evaluateHeader: header.
	(bitCount isKindOf: SmallInteger) ifFalse: [self error: 'Expected a SmallInteger'].
%
category: 'type specifiers'
set compile_env: 0
method: CDeclaration
readCharSigned: aBooleanOrNil

	type := aBooleanOrNil == true
		ifTrue: [#'int8']
		ifFalse: [#'uint8'].
	^self readDeclarator.
%
category: 'other'
set compile_env: 0
method: CDeclaration
readClass

	| token |
	storage := #'typedef'.
	type := #'class'.
	(token := self next; next) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
	name := token value.
	(token := self next) isOpenCurlyBracketToken ifFalse: [self error: 'unexpected token'].
%
category: 'declarator'
set compile_env: 0
method: CDeclaration
readDeclarator

	| token |
	self readDeclaratorForConstAndStar.
	self atEnd ifTrue: [^self].
	(token := self peek) isOpenParenthesisToken ifTrue: [
		isStorage := true.
		(token := self next; next) isStarToken ifFalse: [self syntaxError].
		self readFunctionPointerDeclarator. 
	] ifFalse: [
		self readDeclaratorNameAndArraySizes.
	].
	self atEnd ifTrue: [^self].
	(name notNil and: [name last == $(]) ifTrue: [self readFunctionParameterTypeList].
	((token := self peek) notNil and: [token isIdentifierToken and: [token value = '__THROW']]) ifTrue: [self next].
	[
		(token := self peek) notNil and: [token isAttributeIdentifierToken].
	] whileTrue: [
		self next.
		self readAttributes.
	].
	self atEnd ifTrue: [^self].
	self peek isColonToken ifTrue: [
		self next. 
		self readBitFields.
	].
%
category: 'declarator'
set compile_env: 0
method: CDeclaration
readDeclaratorForConstAndStar

	pointer := 0.
	[
		| aToken |
		((aToken := self peek) notNil and: [aToken type == #'identifier' and: [
			(aToken value = 'const' or: [aToken value = '__const'])
			and: [self next. 0 = pointer]]]) ifTrue: [
				isConstant := true.
		].
		self atEnd not and: [self peek isStarToken].
	] whileTrue: [
		self next.
		pointer := pointer + 1.
	].
%
category: 'declarator'
set compile_env: 0
method: CDeclaration
readDeclaratorNameAndArraySizes

	| token |
	(token := self peek) isNil ifTrue: [^self].
	(token isIdentifierToken and: [token value = '__restrict' or: [token value = 'restrict']]) ifTrue: [
		self next.
	].
	(token := self peek) isNil ifTrue: [^self].
	token isIdentifierToken ifTrue: [
		name := self next value.
	].
	[
		self atEnd not and: [self peek isOpenSquareBracketToken].
	] whileTrue: [
		| value |
		self next.
		count isNil ifTrue: [count := OrderedCollection new].
		value := self peek isCloseSquareBracketToken 
			ifTrue: [0]
			ifFalse: [CPreprocessor evaluateHeader: header].
		(value isKindOf: SmallInteger) ifFalse: [self error: 'Expected a SmallInteger'].
		count add: value.
		(token := self next) isCloseSquareBracketToken ifFalse: [self halt].
	].
	count notNil ifTrue: [count := count asArray].
%
category: 'type specifiers'
set compile_env: 0
method: CDeclaration
readEnumeration

	| token tag integralConstant |
	token := self next.
	token key == #'identifier' ifTrue: [
		tag := token value.
		token := self next.
	].
	token isOpenCurlyBracketToken ifFalse: [self error: 'Invalid enum declaration!'].
	integralConstant := -1.
	[
		(token := self next) isCloseCurlyBracketToken.
	] whileFalse: [
		| identifier |
		token key == #identifier ifFalse: [self errorExpectedIdentifier].
		identifier := token value.
		integralConstant := self peek isEqualsToken ifTrue: [
			self next.
			integralConstant := CPreprocessor evaluateHeader: header.
			(integralConstant isKindOf: SmallInteger) ifFalse: [self error: 'Expected a SmallInteger'].
			integralConstant.
		] ifFalse: [
			integralConstant + 1.
		].
		(header enums includesKey: identifier) ifTrue: [self error: 'Duplicate enum definition!'].
		header enums at: identifier put: integralConstant.
		self peek isCommaToken ifTrue: [self next].
	].
	(token := self peek) key == #'identifier' ifTrue: [
		name := self next value.
	].
%
category: 'attributes'
set compile_env: 0
method: CDeclaration
readFormatAttribute
"
 ... __attribute__ ( ( __format__ ( __printf__ , 3 , 4 ) ) ) ;
"
	| token |
	(token := self next) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
	2 timesRepeat: [
		(token := self next) isCommaToken ifFalse: [self error: 'Expected comma'].
		(token := self next) key == #'int16' ifFalse: [self error: 'Expected number'].
	].
	self expectCloseParenthesisCount: 3.
%
category: 'declarator'
set compile_env: 0
method: CDeclaration
readFunctionParameterTypeList

	| continue token |
	(name notNil and: [name last = $(]) ifTrue: [name size: name size - 1].
	isFunction := true.
	isVaryingArgCount := false.
	self peek isStarToken ifTrue: [self next. ^self readFunctionPointerDeclarator].
	fields := OrderedCollection new.
	continue := ((token := self peek) isCloseParenthesisToken) not.
	[
		continue.
	] whileTrue: [
		| field |
		self peek isDotToken ifTrue: [
			self next.
			2 timesRepeat: [self next isDotToken ifFalse: [self error: 'Unexpected function declaration!']].
			isVaryingArgCount := true.
			continue := false.
		] ifFalse: [
			field := self class header: header.
			field storage == #'auto' ifFalse: [self error: 'Unexpected storage!'].
			fields add: field.
			self peek isEqualsToken ifTrue: [
				| value |
				self next.
				value := CPreprocessor evaluateHeader: header.
			].
			(continue := self peek isCommaToken) ifTrue: [self next].
		].
	].
	fields := fields asArray.
	(fields size = 1 and: [fields first isEmptyVoid]) ifTrue: [
		fields := #().
	].
	(token := self next) isCloseParenthesisToken ifFalse: [self error: 'Should be done!'].
	self peek isOpenCurlyBracketToken ifTrue: [self next. self skipToEndOfCodeBlock].
%
category: 'declarator'
set compile_env: 0
method: CDeclaration
readFunctionPointerDeclarator

	| token myName |
	(token := self next) key ~~ #'identifier' ifTrue: [
		token isCloseParenthesisToken ifFalse: [self error: 'Unexpected token!'].
		(token := self next) isOpenParenthesisToken ifFalse: [self error: 'Unexpected token!'].
		^self readFunctionParameterTypeList.
	].
	name := token value.
	name last =$( ifFalse: [
		(token := self next) isCloseParenthesisToken ifFalse: [self error: 'Unexpected token!'].
		(token := self next) isOpenParenthesisToken ifFalse: [self error: 'Unexpected token!'].
		^self readFunctionParameterTypeList.
	].
	"something like this: extern int (* zork( long foo ) )( short bar );"
	myName := name.
	name := nil.
	type := self copy.
	name := myName.
	isStorage := false.
	self readFunctionParameterTypeList.
	(token := self next) isCloseParenthesisToken ifFalse: [self error: 'Unexpected token!'].
	(token := self next) isOpenParenthesisToken ifFalse: [self error: 'Unexpected token!'].
	type readFunctionParameterTypeList.
%
category: 'type specifiers'
set compile_env: 0
method: CDeclaration
readIntSigned: aBooleanOrNil size: aStringOrNil

	type := self typeForIntSigned: aBooleanOrNil size: aStringOrNil.
	^self readDeclarator.
%
category: 'type specifiers'
set compile_env: 0
method: CDeclaration
readLocalDeclaration

	| other |
	other := header types at: self next value.
	other isFunction ifTrue: [
		type := #'function'.
	] ifFalse: [
		type := other type.
		count := other count.
		pointer := other pointer.
		fields := other _fields.
	].
	self readDeclarator.
%
category: 'attributes'
set compile_env: 0
method: CDeclaration
readModeAttributes
"
	http://tigcc.ticalc.org/doc/gnuexts.html#SEC91_mode
	http://www.linuxtopia.org/online_books/programming_tool_guides/linux_using_gnu_compiler_collection/vector-extensions.html
		QI = 8-bit integer, HI = 16-bit integer, SI = 32-bit integer, DI = 64-bit integer, SF = 32-bit float, DF = 64-bit float
		byte = 8-bit integer, word = 64-bit integer, pointer = 64-bit integer

	typedef int int8_t __attribute__ ( ( __mode__ ( __QI__ ) ) ) ;
"
	| token mode |
	(token := self next) key == #'identifier' ifFalse: [self error: 'Expected identifier'].
	mode := token value.
	type := 
		mode = '__QI__' ifTrue: [type first == $u ifTrue: [#'uint8'] ifFalse: [#'int8']] ifFalse: [
		mode = '__HI__' ifTrue: [type first == $u ifTrue: [#'uint16'] ifFalse: [#'int16']] ifFalse: [
		mode = '__SI__' ifTrue: [type first == $u ifTrue: [#'uint32'] ifFalse: [#'int32']] ifFalse: [
		mode = '__DI__' ifTrue: [type first == $u ifTrue: [#'uint64'] ifFalse: [#'int64']] ifFalse: [
		mode = '__byte__' ifTrue: [type first == $u ifTrue: [#'uint8'] ifFalse: [#'int8']] ifFalse: [
		mode = '__word__' ifTrue: [type first == $u ifTrue: [#'uint64'] ifFalse: [#'int64']] ifFalse: [
		mode = '__pointer__' ifTrue: [type first == $u ifTrue: [#'uint64'] ifFalse: [#'int64']] ifFalse: [
		self halt]]]]]]].
	self expectCloseParenthesisCount: 3.
%
category: 'attributes'
set compile_env: 0
method: CDeclaration
readNonnullAttribute

	| closeCount |
	closeCount := self peek isOpenParenthesisToken
		ifTrue: [self next. 3]
		ifFalse: [2].
	[
		self next isCloseParenthesisToken.
	] whileFalse: [].
	self expectCloseParenthesisCount: closeCount.
%
category: 'other'
set compile_env: 0
method: CDeclaration
readStorageClassSpecifier

	| token |
	self peek isTildeToken ifTrue: [self next].	"indicates a destructor function, which we can ignore"
	[self peek isExtensionIdentifierToken] whileTrue: [self next].	"safe to ignore"
	(token := self peek) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
	token value = 'class' ifTrue: [^self readClass].
	(token value = 'const' or: [token value = '__const']) ifTrue: [
		isConstant := true. 
		(token := self next; peek) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
	].
	(token value = 'restrict' or: [token value = '__restrict']) ifTrue: [
		(token := self next; peek) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
	].
	token value = 'typedef' ifTrue: [self next. storage := #'typedef'. ^self readTypeSpecifier].
	(#('auto' 'register' 'extern' 'static') includes: token value) ifTrue: [
		storage := self next value asSymbol.
	].
	(token := self peek) key == #'identifier' ifTrue: [
		(token value = 'const' or: [token value = '__const']) ifTrue: [
			isConstant := true. 
			(token := self next; peek) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
		].
	].
	token value = 'inline' ifTrue: [^self skipInline].
	storage isNil ifTrue: [storage := #'auto'].
	self readTypeSpecifier.
%
category: 'type specifiers'
set compile_env: 0
method: CDeclaration
readStructOrUnion

	self peek key == #'identifier' ifTrue: [
		| dictionary |
		name := self next value.
		dictionary := type == #'struct'
			ifTrue: [header structs]
			ifFalse: [header unions].
		type := dictionary
			at: name
			ifAbsentPut: [self copy].
	] ifFalse: [
		type := self copy.
	].
	type _fields isNil ifTrue: [type initializeAsStructOrUnion].
	name := nil.
	self peek isSemicolonToken ifTrue: [^self].
	self readDeclarator.
%
category: 'type specifiers'
set compile_env: 0
method: CDeclaration
readTypeSpecifier
 
	| token value signed size |
	token := self peek.
	(token type = #'string' and: [token value = 'C']) ifTrue: [
		(token := self next; next) isOpenCurlyBracketToken ifFalse: [self error: 'Unexpected token!'].
		name := 'C'.
		^self.
	].
	token key == #'identifier' ifFalse: [self errorExpectedIdentifier].
	token value = 'volatile' ifTrue: [
		(token := self next; peek) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
	].
	value := token value.
	value = 'enum'		ifTrue: [self next. type := #'enum'.	^self readEnumeration].
	value = 'float' 		ifTrue: [self next. type := #'float'. 	^self readDeclarator].
	value = 'double' 	ifTrue: [self next. type := #'double'. 	^self readDeclarator].

	"It seems that these should be defined through some other means"
	value = 'size_t' 				ifTrue: [self next. type := #'uint64'. 	^self readDeclarator].		"__SIZEOF_SIZE_T__ is 8 in Linux"
	value = 'wchar_t'				ifTrue: [self next. type := #'int32'. 		^self readDeclarator].		"__SIZEOF_WCHAR_T__ is 4 in Linux"
	value = '__gnuc_va_list'	ifTrue: [self next. type := #'ptr'. 		^self readDeclarator].	
	value = 'ptrdiff_t'				ifTrue: [self next. type := #'int64'. 		^self readDeclarator].	

	(#('signed' 'unsigned') includes: value) ifTrue: [
		signed := value = 'signed'.
		token := self next; peek.
		value := token value.
	].
	value = 'char' ifTrue: [self next. ^self readCharSigned: signed].
	(#('long' 'short') includes: value) ifTrue: [
		size := value.
		(token := self next; peek) key == #'identifier' ifFalse: [
			^self 
				readIntSigned: signed 
				size: size.
		].
		value := token value.
	].
	(size = 'long' and: [token value = 'long']) ifTrue: [
		(token := self next; peek) key == #'identifier' ifFalse: [self errorExpectedIdentifier].
		value := token value.
	].
	value = 'double' ifTrue: [self next. size = 'long' ifFalse: [self halt]. type := #'longDouble'. ^self readDeclarator].
	value = 'int' ifTrue: [self next. ^self readIntSigned: signed size: size].
	(size notNil or: [signed notNil]) ifTrue: [^self readIntSigned: signed size: size].
	value = 'void' 		ifTrue: [type := #'void'. 	self next. ^self readDeclarator].
	value = 'struct' 	ifTrue: [type := #'struct'. self next. ^self readStructOrUnion].
	value = 'union' 	ifTrue: [type := #'union'. 	self next. ^self readStructOrUnion].
	(header types includesKey: value) ifTrue: [^self readLocalDeclaration].
	^self 
		readIntSigned: signed 
		size: size.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
resultType

	isFunction ifFalse: [self error: 'Not a function!'].
	(self isSimple and: [self knownResultTypes includes: type]) ifTrue: [^type].
	type == #'uint8' ifTrue: [
		(count size == 1 and: [pointer == 0]) ifTrue: [^#'char*'].		"const char foo[]"
		(count == nil and: [pointer == 1]) ifTrue: [^#'char*'].			"const char * foo"
	].
	^#'ptr'.
%
category: 'other'
set compile_env: 0
method: CDeclaration
setClassProperties: anArray

	fields := anArray.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
simpleAccessorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name; lf; lf; tab;
		nextPutAll: '^cByteArray ';
		nextPutAll: self getSelector; space;
		nextPutAll: anInteger printString;
		nextPut: $.; lf;
		contents.
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
simpleUpdatorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name;
		nextPutAll: ': anObject'; lf; lf; tab;
		nextPutAll: 'cByteArray '; lf; tab; tab;
		nextPutAll: self getSelector; space;
		nextPutAll: anInteger printString; lf; tab; tab;
		nextPutAll: 'put: anObject.'; lf;
		contents.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
singlePointerAccessorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name; lf; lf; tab;
		nextPutAll: '^cByteArray'; lf; tab; tab;
		nextPutAll: 'pointerAt: ';
		nextPutAll: anInteger printString; lf; tab; tab;
		nextPutAll: 'resultClass: CByteArray.'; lf;
		contents.
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
singlePointerUpdatorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name; 
		nextPutAll: ': aCByteArray'; lf; lf; tab;
		nextPutAll: 'cByteArray'; lf; tab; tab;
		nextPutAll: 'pointerAt: ';
		nextPutAll: anInteger printString; lf; tab; tab;
		nextPutAll: 'put: aCByteArray.'; lf;
		contents.
%
category: 'other'
set compile_env: 0
method: CDeclaration
skipInline

	type := #'inline'.
	[self next isOpenCurlyBracketToken] whileFalse: [].
	self skipToEndOfCodeBlock.
%
category: 'other'
set compile_env: 0
method: CDeclaration
skipToEndOfCodeBlock

	| depth |
	includesCode := true.
	depth := 1.
	[
		0 < depth.
	] whileTrue: [
		| token |
		token := self next.
		token isOpenCurlyBracketToken ifTrue: [depth := depth + 1].
		token isCloseCurlyBracketToken ifTrue: [depth := depth - 1].
	].
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
source

	^source.
%
category: 'Updating'
set compile_env: 0
method: CDeclaration
source: aString

	source notNil ifTrue: [^self].
	source := aString.
	(type isKindOf: CDeclaration) ifTrue: [type source: aString].
	fields notNil ifTrue: [
		fields do: [:each | 
			each source isNil ifTrue: [
				each source: aString.
			].
		].
	].
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
sourceStringsForAccessors

	| list offset |
	list := OrderedCollection new.
	offset := 0.
	fields do: [:each | 
		list add: (each accessorForOffset: offset).
		offset := offset + each byteSize.
	].
	^list.
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
sourceStringsForUpdators

	| list offset |
	list := OrderedCollection new.
	offset := 0.
	fields do: [:each | 
		list add: (each updatorForOffset: offset).
		offset := offset + each byteSize.
	].
	^list.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
storage

	^storage.
%
category: 'other'
set compile_env: 0
method: CDeclaration
stream

	^header stream.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
stringAccessorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name; lf; lf; tab;
		nextPutAll: '^cByteArray stringFromCharStarAt: ';
		nextPutAll: anInteger printString; 
		nextPut: $.; lf; lf; tab;
		nextPutAll: '"^cByteArray '; lf; tab; tab;
		nextPutAll: 'byteArrayFrom: ';
		nextPutAll: anInteger printString; lf; tab; tab;
		nextPutAll: 'to: ';
		nextPutAll: (anInteger + count first - 1) printString;
		nextPutAll: '."'; lf;
		contents.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
stringPointerAccessorForOffset: anInteger

	1 == count ifTrue: [
		^(WriteStream on: String new)
			nextPutAll: name; lf; lf; tab;
			nextPutAll: '^cByteArray stringFromCharStarAt: ';
			nextPutAll: anInteger printString; 
			nextPut: $.; lf; tab;
			nextPutAll: '"^cByteArray'; lf; tab; tab;
			nextPutAll: 'pointerAt: ';
			nextPutAll: anInteger printString; lf; tab; tab;
			nextPutAll: 'resultClass: CByteArray."'; lf;
			contents.
	] ifFalse: [
		^(WriteStream on: String new)
			nextPutAll: name; lf; lf; tab;
			nextPutAll: '| array offset |'; lf; tab;
			nextPutAll: 'array := Array new: ';
			nextPutAll: count first printString;
			nextPut: $.; lf; tab;
			nextPutAll: 'offset := ';
			nextPutAll: anInteger printString;
			nextPut: $.; lf; tab;
			nextPutAll: '1 to: array size do: [:i | '; lf; tab; tab;
			nextPutAll: 'array at: i put: (cByteArray stringFromCharStarAt: offset).'; lf; tab; tab;
			nextPutAll: '"array at: i put: (cByteArray pointerAt: offset resultClass: CByteArray)."'; lf; tab; tab;
			nextPutAll: 'offset := offset + 8.'; lf; tab;
			nextPutAll: '].'; lf; tab;
			nextPutAll: '^array.'; lf;
			contents.
	].
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
stringPointerUpdatorForOffset: anInteger

	1 == count ifTrue: [
		^(WriteStream on: String new)
			nextPutAll: name;
			nextPutAll: ': aCByteArray'; lf; lf; tab;
			nextPutAll: '"cByteArray'; lf; tab; tab;
			nextPutAll: 'pointerAt: ';
			nextPutAll: anInteger printString; lf; tab; tab;
			nextPutAll: 'put: aCByteArray."'; lf;
			contents.
	] ifFalse: [
		^self arrayPointersUpdatorForOffset: anInteger.
	].
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
stringUpdatorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name;
		nextPutAll: ': aByteArray'; lf; lf; tab;
		nextPutAll: 'cByteArray'; lf; tab; tab; 
		nextPutAll: 'replaceFrom: ';
		nextPutAll: anInteger printString; lf; tab; tab;
		nextPutAll: 'to: ';
		nextPutAll: (anInteger - 1) printString;
		nextPutAll: ' + (';
		nextPutAll: count first printString;
		nextPutAll: ' min: aByteArray size)'; lf; tab; tab; 
		nextPutAll: 'with: aByteArray'; lf; tab; tab;
		nextPutAll: 'startingAt: 1.'; lf;
		contents.
%
category: 'CByteArray get'
set compile_env: 0
method: CDeclaration
structAccessorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name; lf; lf; tab;
		nextPutAll: '^cByteArray'; lf; tab; tab;
		nextPutAll: 'newFrom: ';
		nextPutAll: anInteger printString; lf; tab; tab;
		nextPutAll: 'numBytes: ';
		nextPutAll: self baseByteSize printString;
		nextPut: $.; lf;
		contents.
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
structUpdatorForOffset: anInteger

	^(WriteStream on: String new)
		nextPutAll: name;
		nextPutAll: ': aByteArray'; lf; lf; tab;
		nextPutAll: 'cByteArray'; lf; tab; tab; 
		nextPutAll: 'replaceFrom: ';
		nextPutAll: anInteger printString; lf; tab; tab;
		nextPutAll: 'to: ';
		nextPutAll: (anInteger - 1) printString;
		nextPutAll: ' + (';
		nextPutAll: self baseByteSize printString;
		nextPutAll: ' min: aByteArray size)'; lf; tab; tab; 
		nextPutAll: 'with: aByteArray'; lf; tab; tab;
		nextPutAll: 'startingAt: 1.'; lf;
		contents.
%
category: 'other'
set compile_env: 0
method: CDeclaration
syntaxError

	self error: 'Syntax error!'.
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
type

	^type.
%
category: 'type specifiers'
set compile_env: 0
method: CDeclaration
typeForIntSigned: aBooleanOrNil size: aStringOrNil
"
// From Linux, we have the following: 
#define __SIZEOF_SHORT__ 2
#define __SIZEOF_INT__ 4
#define __SIZEOF_LONG__ 8
#define __SIZEOF_LONG_LONG__ 8
"
	aStringOrNil = 'long' ifTrue: [
		^aBooleanOrNil == false 
			ifTrue: [#'uint64']
			ifFalse: [#'int64']. 
	].
	aStringOrNil = 'short' ifTrue: [
		^aBooleanOrNil == false 
			ifTrue: [#'uint16']
			ifFalse: [#'int16']. 
	].
	^aBooleanOrNil == false 
		ifTrue: [#'uint32']
		ifFalse: [#'int32'].
%
category: 'CByteArray put'
set compile_env: 0
method: CDeclaration
updatorForOffset: anInteger

	0 == pointer ifTrue: [
		type == #'class' ifTrue: [^self structUpdatorForOffset: anInteger].
		(type isKindOf: CDeclaration) ifTrue: [^self structUpdatorForOffset: anInteger].
		count == nil ifTrue: [^self simpleUpdatorForOffset: anInteger].
		1 < count size ifTrue: [self error: 'Multi-dimensional arrays not supported!'].
		type == #'uint8' ifTrue: [^self stringUpdatorForOffset: anInteger].
		^self arrayUpdatorForOffset: anInteger.
	].
	1 < pointer ifTrue: [self error: 'Multi-level pointers not supported!'].
	count == nil ifTrue: [^self singlePointerUpdatorForOffset: anInteger].
	type == #'uint8' ifTrue: [^self stringPointerUpdatorForOffset: anInteger].
	^self arrayPointersUpdatorForOffset: anInteger.
%
category: 'CCallout'
set compile_env: 0
method: CDeclaration
varArgsAfter
	^  isVaryingArgCount ifTrue: [fields size] ifFalse: [-1]
%
category: 'accessors'
set compile_env: 0
method: CDeclaration
_fields

	^fields.
%
category: 'other'
set compile_env: 0
method: CDeclaration
_next

	| next |
	next := header next.
	file isNil ifTrue: [
		file := next file.
		line := next line.
	].
	^next.
%
