!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: behavio.gs 26875 2011-09-23 16:25:58Z otisa $
!
! Superclass Hierarchy:
!   Behavior, Object.
!
!=========================================================================

removeallmethods Behavior
removeallclassmethods Behavior
set class Behavior

category: 'Documentation'
classmethod:
comment
^
'
 Behavior is an abstract superclass with the concrete subclasses
 ObsoleteMetaclass and Module .
 You may not create any other subclasses of Behavior.

 Behavior describes the protocol common to all kinds of Module
 and instances of ObsoleteMetaclass.  In other words, you can send the messages listed
 here to any Class or metaclass.  In the method descriptions below,
 "superclass" refers to the superclass of instances of the receiver,
 not to the superclass of the receiver itself.

 instVars:
  superclass - The Behavior''s immediate superclass in the 
		Smalltalk environment 0  class hierarchy 

  format - A SmallInteger that encodes the following information in its bits:
      (format bitAnd: 3) = 0 if instances of the Behavior are pointer objects.
      (format bitAnd: 3) = 1 if instances of the Behavior are byte objects.
      (format bitAnd: 3) = 2 if instances of the Behavior are non-sequenceable
                collections (NSCs, such as Bags and Sets).
      (format bitAnd: 3) = 3 if instances of the Behavior are special objects.

      and single bit fields per the following masks:
        16r4  instances of the Behavior have varying instVars addressable by
		a SmallInteger offset, (such as Array, String)
              previously documented as "have indexable instVars"
        16r8  instances of the Behavior are invariant.
        16r10  the Behavior has constraints upon what can be
      	        stored in its instance variables.
        16r20  the Behavior does not allow subclass creation.
	16r40
        16r80  the Behavior does not allow structural access from GCI C interface
	16r100  obsolete bit
        16r200  no stores to instvars via structural access from GCI 
        16r400  GCI traversal result via send of aClampSpecification.traversalCallBackSelector
        16r800  non-persistent bit, instances of the class may not be committed
        16r1000  dbTransient bit, instVar in instances of the class are always nil on disk.
        16r2000  self can be special , used by compiler and native code generator ,
		 set if instances of receiver or any subclass can be special objects.
        16r4000  Ruby virtual class bit
        16r8000  Ruby singleton class bit
       16r10000  Ruby module-include-self bit
       16r20000  Ruby module-functions bit
      C definitions for the bits are in $GEMSTONE/include/gcioc.ht 

  instVarsInfo - A non-negative SmallInteger encoding the number of instance variables in 
     instances of this class (including those inherited from superclasses) 
     and additional information.   The additional information is regenerated each 
     time the class is loaded into memory and is used only within the Virtual Machine.
     The mask to extract the number of instance variables is 16rFFF .
     Each instance of Behavior is limited to 2030 named instance variables.

  instVarNames - An invariant Array of Symbols giving the names of the Behavior''s 
     instance variables, including those inherited from superclasses.  Each instance
     variable name is limited to 64 Characters, and must begin with an
     alphabetic character or an underscore ("_").  For more information, see
     the GemStone Programming Guide.

  constraints - An invariant Array of Classes.  
    Each element in the Array is the class kind of a corresponding instance 
    variable defined in a class or inherited from a superclass.
    If constraints size is one larger than the size of instVarNames, the last element
    is the constraint on varying instVars .

  classVars - A SymbolDictionary used when compiling methods in this Behavior.  Each instance
     of a class has its own instance variables, which may differ in value.  Each
     class has its own class variables, which have the same value for all instances
     of the class.  For each SymbolAssociation in this dictionary, the key is a
     Symbol representing a class variable, and the corresponding value is the value
     of that class variable.  Each class variable name is limited to 64 Characters,
     and must begin with an alphabetic character or an underscore ("_").
     When resolving variable names, method compilation searches the classVars of 
     the class in which a method is being compiled, and also the classVars of superclasses. 

  methDicts -  A GsMethodDictionary that has all of the additional protocol 
	(not inherited from superclasses) for instances of this Behavior. 
    May also be an Array of pairs, ( GsMethodDictionary, RubyGlobalScope)
       to support persistent methods and ruby name spaces .
    If an Array, the array holds groups of 4 ,
        methodDict for envId is      methDicts at:(envId*4 + 1)
        rubyNameSpace for envId is   methDicts at:(envId*4 + 2)
        rubySuperclass for envId is  methDicts at:(envId*4 + 3)

        methodDict for env 1d is     methDicts at: 5  (or _rubyAt:4)
        rubyNameSpace for env 1 is   methDicts at: 6  (or _rubyAt:5)
        rubySuperclass for env 1 is  methDicts at: 7  (or _rubyAt:6)

  poolDictionaries -  An Array of SymbolDictionaries used when compiling methods in 
     this Behavior.  The dictionaries contain objects that can be shared by multiple classes 
     and multiple users.  When resolving variable names, method compilation searches 
     the poolDictionaries of the class in which a method is being compiled, but 
     does not search poolDictionaries of superclasses.

  categorys - A GsMethodDictionary that categorizes selectors in this Behavior.  
     For each element in this dictionary, the key is a method category Symbol, and the
     corresponding value is a SymbolSet of the selectors for that method category.
     May also be an Array of GsMethodDictionarys if class contains 
     methods with environmentId > 0.
     Support for categories for environmentId > 0 is mostly not implemented.

  primaryCopy - 
     In a Ruby Virtual class,  references the class of which this is a copy.
     Otherwise nil and reserved for future use by VMware, Inc. 
'
%

category: 'Instance Creation'
classmethod:
new

"Disallowed.  To create a new Class or metaclass, use
 Class | subclass:instVarNames:... instead."

self shouldNotImplement: #new .
^ nil
%

category: 'Instance Creation'
classmethod:
new: anInteger

"Disallowed.  To create a new Class or metaclass, use
 Class | subclass:instVarNames:... instead."

self shouldNotImplement: #new: .
^ nil
%

category: 'Testing Inheritance'
method:
isSubclassOf: aClassHistory

"Returns true if the receiver is identical to or is a subclass of any class
 in aClassHistory; otherwise, returns false.

 If the aClassHistory argument is actually a class rather than a class history,
 then this method uses the class history of the argument, instead of the class
 itself."

<primitive: 70>
self _primitiveFailed: #isSubclassOf: args: { aClassHistory } .
^ false
%

method:
_subclassOf: aClass
"Returns true if the receiver is identical to or is a subclass aClass.

 ClassHistories of the receiver and argument are ignored."

<primitive: 895>
self _primitiveFailed: #_subclassOf: args: { aClass } .
^ false
%


category: 'Testing Inheritance'
method:
validateSubclassOf: aClass

"Returns true if receiver is identical to aClass or is a subclass
 of aClass; otherwise, generates an error."

( self isSubclassOf: aClass) ifFalse:[
     ^ self _error: #rtErrNotASubclassOf args:{ aClass } ].
^ true
%

category: 'Testing Inheritance'
method:
_validateSubConstraintOf: aClass

"Returns true if receiver is identical to aClass or is a subclass of aClass,
 otherwise generates an error with message specific to class modification."

( self isSubclassOf: aClass) ifFalse:[
     ^ self _error: #rtErrConstrNotSubclassOf args:{ aClass } ].
^ true
%

category: 'Testing'
method:
isBehavior

"Returns true if the receiver is a kind of Behavior, and returns false
 otherwise."

  ^true
%

! fix bug 6601
category: 'Accessing the Class Format'
method:
isNonByteVarying

"Returns true if the instances of the receiver are not byte objects and have
 unnamed instance variables; returns false otherwise."

| bits |

bits := format bitAnd: 7.
^bits == 2 or: [bits == 4].
%

method:
_name
  ^ 'aBehavior'
%

category: 'Deprecated'
method:
_namedIvConstraintAtOffset: offset

"Returns the constraint, if any, on the named instance variable at the
 specified offset.  Returns Object if there is no such named instance variable,
 or if the instance variable at that offset is not constrained."

self deprecated: 'Constraints are no longer supported'.
(offset > self instSize ) ifTrue:[ ^ Object ] .
^ (constraints atOrNil: offset) ifNil:[ Object ].
%


category: 'Private Methods for Class Modification'
method:
_insertNamedInstVar: aSymbol atOffset: offset

"Receiver and all subclasses must have be modifiable.  aSymbol must be unique
 with respect to existing instance variables."

" add instance variable to self"
| mySubclasses constr ivn |

self _validatePrivilege ifTrue:[
  self _incrementInstVars: 1 .
  (ivn := instVarNames) == #() ifTrue:[ 
    ivn := { } . instVarNames := ivn .
  ].
  ivn insertObject: aSymbol at: offset .
  constr := constraints .
  constr size ~~ 0 ifTrue:[ constr  insertObject: Object  at: offset ].
  self _refreshClassCache: false .
  mySubclasses := self _subclasses .
  mySubclasses ~~ nil ifTrue:[
    mySubclasses do:[:x | x _insertNamedInstVar: aSymbol atOffset: offset ].
    ].
].
^ self
%

category: 'Private Methods for Class Modification'
method:
_incrementInstVars: delta

"Increment instVars by 1 to account for adding a named instance variable."
| mask n info_old |
mask := Class_numIvs_mask .
info_old := instVarsInfo .
delta _isSmallInteger ifFalse:[ delta _validateClass: SmallInteger ].
n := (info_old bitAnd: mask) + delta .
n < 0 ifTrue:[ self error:'numInstVars would go negative'].
n > mask ifTrue:[ self error:'numInstVars would exceed max'].

instVarsInfo := (info_old bitAnd:( mask bitInvert)) bitOr:( n bitAnd: mask)
%

category: 'Private Methods for Class Modification'
method:
_makeProtected

"Make the receiver a protected class by setting a bit in its format.  This
 protection disallows structural access through GemBuilder for C."

self _validatePrivilege ifTrue:[
  format := format bitOr: 128 
]
%

category: 'Deprecated'
method:
_newConstraint: aClass atOffset: offset 

"Execute the constraint change for Behavior | instvar:ConstraintTo:
 assuming all error and variance checks have been done."
| constrs |
self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifTrue:[ 
  (constrs := constraints) size == 0 ifTrue:[ | sz |
    aClass == Object ifTrue:[ ^ self "do nothing"].
    sz := self instSize .
    (constrs := Array new: sz ) replaceFrom: 1 to: sz withObject: Object.
    constraints := constrs .
  ].
  constrs at: offset put: aClass .
  (aClass == Object) ifFalse:[ self _setConstraintBit ].
  self _refreshClassCache: false .
]
%

category: 'Private Methods for Class Modification'
method:
_recompileMethodsAfterNewIvOffset: newOffset

| mySymList mySubclasses |
self _validatePrivilege ifTrue:[
  mySymList:= System myUserProfile symbolList .
  (newOffset < self instSize) ifTrue:[
      self recompileAllMethodsInContext: mySymList ] .
  mySubclasses := self _subclasses .
  mySubclasses ~~ nil ifTrue:[
    mySubclasses do:[:x | x _recompileMethodsAfterNewIvOffset: newOffset ] .
    ].
].
^ self
%

category: 'Private Methods for Class Modification'
method:
_refreshClassCache: clearLookupCachesBool

"Refreshes in-memory state of the class .
 If clearLookupCachesBool==true, also clears all method lookup caches.
 Must also be sent immediately after any change to the constraints ,  
 format or named instance variables of the receiver."

<primitive: 227>
self _primitiveFailed:#_refreshClassCache: args: { clearLookupCachesBool } .
self _uncontinuableError
%

category: 'Private Methods for Class Modification'
method:
_removeInstVarAtOffset: offset

"Remove named instance variable at specified offset from self and all
 subclasses assuming that all error checks have been done."

self _validatePrivilege ifTrue:[ | constr |
  self _incrementInstVars: -1 .
  instVarNames removeFrom: offset to: offset .
  constr := constraints.
  constr size ~~ 0 ifTrue:[ constr removeFrom: offset to: offset ] .

  self _refreshClassCache: false .
  self _subclasses do:[:x| x _removeInstVarAtOffset: offset ] .
].
%

category: 'Deprecated'
method:
_setConstraintBit

"Sets the constraint bit in the 'format' instance variable of the receiver."

self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifTrue:[
  format := format bitOr: 16#10 .
]
%

category: 'Private Methods for Class Modification'
method:
_disallowGciCreateStore

"Private.

 Sets bit in the format instance variable to cause GemBuilder for C instance
 creation and updates to go through message sends.  Semantics are private to
 GemBuilder for Smalltalk."

self _validatePrivilege ifTrue:[
  format := format bitOr: 16r200 "no update through structural access" .
  self _refreshClassCache: false
]
%

category: 'Private'
method:
_structuralUpdatesDisallowed

"Private.

 Returns true if GemBuilder for C (GCI) direct structural update of instances
 is disallowed, false otherwise.  A result of true means that the deferred
 update mechanism is used (see GciProcessDeferredUpdates in gci.hf) by
 GemBuilder for C store operations on instances of the receiver."

^ (format bitAnd: 16r200 "no update through structural access") ~~ 0
%

category: 'Private'
method:
_traversalByCallback

"Private.

 Returns true if GemBuilder for C (GCI) traversal results of instances 
 are obtained by message send of aClampSpecification.traversalCallBackSelector,
 false otherwise."

^ (format bitAnd: 16r400"travByCallback") ~~ 0 
%

category: 'Private'
method:
_makeTraversalByCallback

"Private.

 Make the receiver place its instances in a traversal buffer by
 invoking the clampSpecification's traversal callback method."

self _validatePrivilege ifTrue:[
  format := format bitOr: 16r400. "travByCallback"
  self _refreshClassCache: false .
]
%

category: 'Deprecated'
method:
_validateNewVaryingConstraint: aClass

"aClass is the proposed new constraint on unnamed instance variables.  If
 aClass is not a superclass of the already existing constraint, then check to
 ensure that the receiver is modifiable.  Recursively check all subclasses.
 If validation ok, return true, else signal an error. 
 If execution continued from the error, return false.  "

self deprecated: 'Constraints are no longer supported'.
(self varyingConstraint isSubclassOf: aClass) ifFalse:[
  self isModifiable ifFalse:[ 
    self validateIsModifiable  .
    ^ false
  ].
  self _subclasses do:[:x| 
    (x _validateNewVaryingConstraint: aClass) ifFalse:[ ^ false ].
  ].
] .
^ true
%

category: 'Deprecated'
method:
_validateNewInheritedConstraint: aClass atOffset: offset

"If the current constraint is not a subclass of the new constraint, checks to
 ensure that the receiver is modifiable.  Recursively checks subclasses.
 Return true if validation ok, othewise signal an error. If execution continued
 from the error, return false.  "

self deprecated: 'Constraints are no longer supported'.
( ((constraints atOrNil: offset) ifNil:[ Object])  isSubclassOf: aClass ) ifFalse:[
  self isModifiable ifFalse:[ 
    self validateIsModifiable . 
    ^ false 
  ].
  self _subclasses do:[:x|
     (x _validateNewInheritedConstraint: aClass atOffset: offset) ifFalse:[
       ^ false
     ].
  ] .
] .
^ true
%

category: 'Deprecated'
method:
_updateVaryingConstraint: aClass

"Update the receiver's constraint on unnamed variables to be consistent with
 inheritance rules, where aClass is the new constraint which has been placed on
 the superclass's unnamed part.  Recursively update subclasses.  Checks to
 ensure self and subclasses are modifiable must have already been made."

self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifTrue:[
  (self varyingConstraint isSubclassOf: aClass) ifFalse:[
      self _setVaryingConstraint: aClass .
      self _subclasses do:[:x| x _updateVaryingConstraint: aClass].
  ] .
]
%

category: 'Deprecated'
method:
_newInheritedConstraint: aClass atOffset: offset

"Change the constraint for offset to aClass if that offset is not
 already constrained to be a subclass of aClass.  Assume all
 variance and error checks have been done."

self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifTrue:[
  ( ((constraints atOrNil: offset) ifNil:[Object])  isSubclassOf: aClass ) ifFalse:[
    self _newConstraint: aClass atOffset: offset .
    self _subclasses do:[:aSubcls|
       aSubcls _newInheritedConstraint: aClass atOffset: offset ].
    ] .
]
%

category: 'Private Methods for Class Modification'
method:
_validateNewNamedInstVar: aSymbol

"Generate an error if the argument is the name of an already existing instance
 variable of the receiver or if the receiver is not modifiable and has not
 disallowed subclassing, otherwise return true. If execution is continued
 from the error, return false."

"reimplementation of self validateIsModifiable ."
self isModifiable ifFalse:[
  self isNsc ifTrue:[ 
    self _error: #rtErrClassNotModifiable .  
    ^ false .  
  ].
  self subclassesDisallowed ifFalse:[
    self _error: #rtErrClassNotModifiable .  
    ^ false .
  ] .
  self isIndexable ifTrue:[ 
     self _error: #rtErrClassNotModifiable.
    ^ false
  ].
] .
(instVarNames includesIdentical: aSymbol) ifTrue:[ 
  self _error: #rtErrAddDupInstvar args:{ aSymbol } .
  ^ false
].
self _subclasses ifNotNil:[ :mySubclasses |
  mySubclasses do:[:aSubCls| 
     (aSubCls _validateNewNamedInstVar: aSymbol) ifFalse:[ ^ false ].
  ].
].
^ true
%

category: 'Deprecated'
method:
_setVaryingConstraint: aClass

"Assign a new value to the constraint on unnamed variables of the receiver,
 assuming all checks have been made."

| constrs ofs |

self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifTrue:[
  constrs := constraints .
  ofs := self instSize + 1 .
  constrs size == 0 ifTrue:[ 
    aClass == Object ifTrue:[ ^ self "nothing to do"].
    (constrs := Array new: ofs) replaceFrom: 1 to: ofs withObject: Object .
    constraints := constrs .
  ].
  constrs at: ofs put: aClass .
  (aClass == Object) ifFalse:[ self _setConstraintBit ].
  self _refreshClassCache: false .
]
%

category: 'Error Handling'
method:
_isInstanceDisallowed

^ (InstancesDisallowed includesValue: self) or:[ self isMeta ]
%

category: 'Updating'
method:
instVarAt: anIndex put: aValue

anIndex == 7 ifTrue:[ 
  "Gs64 v3.0 structural update of the methDicts instVar disallowed"
   OffsetError  new _number: 2350; reason: #errNoStructuralUpdate ; 
	  details:'methDicts instVar'; object: self ; signal .
   self _uncontinuableError.
].
anIndex == 9 ifTrue:[
  "Gs64 v3.0 structural update of the categories instVar disallowed"
   OffsetError  new _number: 2350; reason: #errNoStructuralUpdate ; 
	details:'categorys instVar'; object: self ; signal .
   self _uncontinuableError.
].
^ super instVarAt: anIndex put: aValue
%

method:
persistentMethodDictForEnv: envId
"result will be nil if no methods exist for specified environmentId."
| mds |
(mds := methDicts) _isArray ifTrue:[ | idx |
  ^ mds atOrNil: (envId*4 + 1) 
].
envId == 0 ifTrue:[ ^ mds ].
^ nil
%

method:
persistentMethodDictForEnv: envId put: aValue
  "aValue should be a GsMethodDictionary, or nil ,
   caller responsible for _refreshClassCache "

<protected>
| ofs mds |
(mds := methDicts) _isArray ifFalse:[
  envId == 0 ifTrue:[ 
    methDicts := aValue .
    ^ self
  ].
  mds := { mds }.
  methDicts := mds .
].
ofs := envId*4 + 1 .
mds size < ofs ifTrue:[ mds size: ofs ].
mds at: ofs put: aValue
%

category: 'Enumerating'
method:
persistentMethodDictsDo: aBlock
| mds |
(mds := methDicts) ifNotNil: [ 
  mds _isArray ifTrue:[
    1 to: mds size by: 4 do:[:j |  | aDict |
      (aDict := mds at: j) ifNotNil:[ aBlock value: aDict ].
    ].
  ] ifFalse:[
    aBlock value: mds
  ]
]
%

method:
persistentNameSpacesDo: aBlock
| mds |
(mds := methDicts) ~~ nil ifTrue: [ 
  mds _isArray ifTrue:[
    2 to: mds size by: 4 do:[:j |  | aNs |
      (aNs := mds at: j) ifNotNil:[ aBlock value: aNs ].
    ].
  ]
]
%

! methods which use env:baseCategorysDo: may need reimplementation in behavio_2.gs 
!   to use GsPackagePolicy for the Session Methods functionality

method:
_baseCategorys: envId

  "Returns nil or the category dictionary for specified environment"
  | cats |
  (cats := categorys) _isArray ifTrue:[
    cats := cats atOrNil:  envId + 1 .
  ] ifFalse:[
    envId ~~ 0 ifTrue:[ cats := nil ].
  ].
  ^ cats
%

method
__categorys
  ^ categorys
%

method:
_baseCategorysForStore: envId
  | dict carr |
  (dict := self _baseCategorys: envId ) ifNil:[
     dict := GsMethodDictionary new  .
     dict _objectSecurityPolicy: self objectSecurityPolicy .
     (carr := categorys _isArray) ifTrue:[ | ofs |
       ofs := envId + 1 .
       (carr size < ofs) ifTrue:[ carr size: ofs ].
        carr at: ofs put: dict .
     ] ifFalse:[
       categorys := dict
     ].
  ]. 
  ^ dict
%

method
_unifiedCategorys: envId
  | cats | 
  cats := self _baseCategorys: envId .
  cats ifNil:[ cats := GsMethodDictionary new ] ifNotNil:[ cats copy ].
  ^ cats
%


method:
env: environmentId baseCategorysDo: aBlock
  "evaluates aBlock for each method category of receiver 
   in specified environment. Returns the receiver. 

   aBlock should be a two argument block 
   expecting the args  categoryNameSymbol ,   selectorsSet.
   aBlock will be invoked once for each category in the receiver.
   The iteration is directly over the receiver's categories."
     
  | cats |
  cats := self _baseCategorys: environmentId .
  cats ifNotNil:[ cats keysAndValuesDo: aBlock ].
%
method
env: envId unifiedCategoriesDo: aBlock
  "evaluates aBlock for each method category of receiver
   in specified environment. Returns the receiver.

   aBlock should be a two argument block
   expecting the args  categoryNameSymbol ,   selectorsSet.
   aBlock will be invoked once for each category in the receiver.
   The iteration is done over a copy of the receiver's categories,
   however the selectorSets are not copied ."

  (self _unifiedCategorys: envId ) keysAndValuesDo: aBlock
%

method:
env: environmentId categorysDo: aBlock
  "evaluates aBlock for each method category of receiver 
   in specified environment. Returns the receiver. 

   aBlock should be a two argument block 
   expecting the args  categoryNameSymbol ,   selectorsSet.
   If the package manager is active in the current session,
   aBlock may be invoked more than once for each category name.
   The iteration is done directly over the receiver's categories."
     
  self env: environmentId baseCategorysDo: aBlock
%

method:
categorysDo: aBlock
  "evaluates aBlock for each category in environment 0.

   aBlock should be a two argument block 
   expecting the args  categoryNameSymbol ,   selectorsSet,
   If the package manager is active in the current session,
   aBlock may be invoked more than once for each category name."

  self env: 0 categorysDo: aBlock
%

method:
env: envId methodsDo: aBlock
  "evaluates aBlock for each method in receiver's method dictionaries
   for specified environment. Returns the receiver.

   aBlock should be a two argument block
   expecting the args  selectorSymbol ,   aGsNMethod ."

  (self persistentMethodDictForEnv: envId ) keysAndValuesDo: aBlock 
%

method:
methodsDo: aBlock
  "evaluates aBlock for each method in receiver's method dictionaries
   for environment 0 . Returns the receiver.

   aBlock should be a two argument block
   expecting args  selectorSymbol ,   aGsNMethod ."

  self env: 0 methodsDo: aBlock
%

category: 'Accessing Categories'
method:
includesCategory: aString

"Returns true if aString is equivalent to the name of a category in the
 receiver, false otherwise."

^ self includesCategory: aString environmentId: 0
%

method:
includesCategory: aString environmentId: envId

"Returns true if aString is equivalent to the name of a category in the
 receiver, false otherwise."

| aSym |
aSym := Symbol _existingWithAll: aString.
aSym ifNil:[ ^ false ].
self env: envId categorysDo:[ :name :selectors |
   name == aSym ifTrue:[ ^ true ].
].
^ false
%

method:
_categoryNotFound: aString
  self _error: #classErrMethCatNotFound args: { aString } .
  ^ nil
%

category: 'Updating the Method Dictionary'
method:
_codeChangedForEnv: envId 
  "set bit in VM's cbCodeChanged word so it can propagate to
   other VMs upon commit.  Code which adds or removes entries
   in persistent method dictionaries must send this method. "
<primitive: 854>
envId _validateClass: SmallInteger .
self _primitiveFailed: #_codeChangedForEnv: args: {envId} "envId out of range"
%

method:
_basicRemoveSelector: aSymbol environmentId: envId
"returns  true if a method found to remove"
| md pmd removed tmeth pmeth |
self _validatePrivilege ifFalse:[ ^ false ].
md := self transientMethodDictForEnv: envId .
removed := false .
md ifNotNil:[
  tmeth := md removeKey: aSymbol otherwise: nil .
].
pmd := self persistentMethodDictForEnv: envId .
pmd ifNotNil:[
  pmeth := pmd removeKey: aSymbol otherwise: nil .
].
"refresh method lookup caches to account for removal of the selector
 and delete any breakpoints in the removed method(s) ."
tmeth ifNotNil:[ 
  removed := true 
  self _refreshLookupCache: aSymbol oldMethod: tmeth env: envId .
].
pmeth ifNotNil:[
  removed := true .
  self _codeChangedForEnv: envId .
  self _refreshLookupCache: aSymbol oldMethod: pmeth env: envId .
].
^ removed
%

category: 'Updating the Method Dictionary'
method:
removeSelector: aString 

  self removeSelector: aString environmentId: 0
%

method:
removeSelector: aString environmentId: envId

| sym |
sym := Symbol _existingWithAll: aString .
sym ifNil:[ self error:'the selector ', aString , '  does not exist' ].
self _basicRemoveSelector: sym environmentId: envId.
self env: envId baseCategorysDo: [ :categName :selectors |
  (selectors remove: sym ifAbsent:[ nil ]) ifNotNil:[
    ^ self "done"
  ].
].  
%

! deleted _compileMethod:dictionaries:category:environmentId:

category: 'Updating the Method Dictionary'
method:
compileMethod: sourceString
dictionaries: aSymbolList
category: categorySymbol
intoMethodDict: aMethodDict
intoCategories: aCategDict
intoPragmas: aPragmasArray
environmentId: environmentId

"Compiles sourceString as a method for the receiver in category categorySymbol,
 using the symbol list aSymbolList.  If the compilation succeeds, returns the
 GsNMethod produced by the compilation.  
 If the compilation fails, signals a CompileError .
 If the compilation has warnings, signals a CompileWarning, after the GsNMethod 
 has been generated and installed in any method dictionaries.

 If aMethodDict is not nil, and the compilation succeedd,
 the resulting method is added to aMethodDict instead of to 
 the receiver's method dictionary.  This is used to add methods 
 to per-session dictionaries.

 pragmasArray must be nil, or an instance of Array. If compilation
 is successful and pragmasArray is not nil, any pragmas found by the 
 compiler are appended (as selector, Array pairs) to pragmasArray .

 The caller is responsible for setting the current GsObjectSecurityPolicy to
 match the receiver's GsObjectSecurityPolicy if desired.

 environmentId must be a SmallInteger >= 0 and <= 255 .
 0 denotes the base Smalltalk image.  1 is reserved for use by Ruby .

 You must have code modification privilege to execute this method.
 "

^ self _checkCompileResult:( self _primitiveCompileMethod: sourceString
		symbolList: aSymbolList category: categorySymbol 
             oldLitVars: nil intoMethodDict: aMethodDict intoCategories: aCategDict 
	     intoPragmas: aPragmasArray environmentId: environmentId )
        source: sourceString
%
category: 'Private'
method:
_checkCompileResult: result source: sourceString

  ^self _checkCompileResult: result source: sourceString suppressCompileWarning: false
%
category: 'Private'
method:
_checkCompileResult: result source: sourceString suppressCompileWarning: suppressCompileWarning
  "process the result Array from _primitiveCompileMethod:... 
   Returns a GsNMethod or signals a CompileError or CompileWarning."

  result class == GsNMethod ifTrue:[ 
    ^ result 
  ].
  (result at: 2) ifNotNil:[ :errorArray|  
    "Fill in the error message text for each error in the result."
    [ | errDict |
      errDict := GemStoneError at: System myUserProfile nativeLanguage .
      1 to: errorArray size do:[:j | | thisErr |
	thisErr := errorArray at: j .
	(thisErr atOrNil: 3) ifNil:[  | msg |
	  thisErr size < 3 ifTrue:[ thisErr size: 3 ].
	  msg := errDict atOrNil:(thisErr at: 1) .
	  thisErr at: 3 put: ( msg ifNil:[ '(unknown error number)' ]).
	]. 
      ].
    ] onException: Error do:[:ex | "ignore"].
    ^ (CompileError new) args: { errorArray . sourceString . self } ; signal
  ].
  (result at: 3) ifNotNil:[ :warnStr | | meth |
    (meth := result at: 1) class == GsNMethod ifTrue:[ 
       suppressCompileWarning ifFalse: [ CompileWarning signal: warnStr method: meth ].
       ^ meth
    ].
  ].
  InternalError signal:'unrecognized result from _primitiveCompileMethod'
%

category: 'Updating the Method Dictionary'
method:
compileMethod: sourceString
dictionaries: aSymbolList
category: categorySymbol
intoMethodDict: aMethodDict
intoCategories: aCategDict

^ self compileMethod: sourceString 
       dictionaries: aSymbolList
       category: categorySymbol 
       intoMethodDict: aMethodDict
       intoCategories: aCategDict  
       intoPragmas: nil environmentId: 0
%

category: 'Private'
method:
_primitiveCompileMethod: sourceString
symbolList: aSymbolList
category: categorySymbol
oldLitVars: litVarArray
intoMethodDict: aMethodDict
intoCategories: aCategDict
intoPragmas: pragmasArray
environmentId: environmentId

  | src cvtAb |
  cvtAb := System _zeroArgPrim: 132 "fast System gemConfigurationAt:#GemConvertArrayBuilder" .
 cvtAb ifTrue:[ | cvtRes |
   sourceString _isOneByteString ifFalse:[
     sourceString _validateClasses: { String . DoubleByteString }.
   ].
   cvtRes := GsNMethod _convertArrayBuildersIfNeeded: sourceString .
   cvtRes _isArray ifTrue:[ 
     "error during ArrayBuilder conversion"
     ^ cvtRes "a compile error descriptor"
   ].
   src := cvtRes .
 ] ifFalse:[ 
   src := sourceString 
 ].
 ^ self _primCompileMethod: src  
     symbolList: aSymbolList
     category: categorySymbol
     oldLitVars: litVarArray
     intoMethodDict: aMethodDict
     intoCategories: aCategDict
     intoPragmas: pragmasArray
     environmentId: environmentId
%

category: 'Private'
method:
_primCompileMethod: sourceString
symbolList: aSymbolList
category: categorySymbol
oldLitVars: litVarArray
intoMethodDict: aMethodDict
intoCategories: aCategDict
intoPragmas: pragmasArray
environmentId: environmentId

"Compiles sourceString as a method for the receiver in category categorySymbol,
 using the symbol list aSymbolList.  If the compilation succeeds, the method
 dictionary of the receiver will have been updated.

 Returns the GsNMethod produced by the compilation if the compilation 
 succeeded with no warnings or errors, or an Array of of the form
    { (GsNMethod or nil if the compilation has errors) .
       (nil or an Array of error descriptors as described for 
        compileMethod:dictionaries:category: ) .
       (nil or a String describing warnings) 
     } .
      
 If litVarArray is not nil, it must be an Array of SymbolAssociations,
 and this Array will be searched prior to searching aSymbolList to
 resolve literal variables within the method.

 If the compilation succeeds,
   if aMethodDict == false, the method is added to no dictionaries,
      and aCategDict is ignored
   else if aMethodDict ~~ nil, the method is added to per-session
      method dictionaries,
   else aMethodDict == nil, the method is added to receiver's 
      persistent method dictionary.
  
 If aMethodDict is neither nil nor false, and aCategDict is not nil and
 the compilation succeeds, the resulting method is added aCategDict
 instead of the receiver's categories.

 pragmasArray must be nil, or an instance of Array. If compilation
 is successful and pragmasArray is not nil, any pragmas found by the 
 compiler are appended (as selector, Array pairs) to pragmasArray .

 If the compilation succeeds, the selector of the new method is
 removed from all method lookup caches for the receiver and all subclasses
 thereof,   independent of the value of aMethodDict argument.

 environmentId must be a SmallInteger >= 0 and <= 255 .
 0 denotes the base Smalltalk image.  1 is reserved for use by Ruby .

 You must have code modification privilege to execute this primitive.
"

<primitive: 228>
sourceString _validateClasses: { String . DoubleByteString }.
aSymbolList ~~ nil  ifTrue:[ aSymbolList _validateClass: SymbolList ].
categorySymbol _validateClass: Symbol.
litVarArray ~~ nil ifTrue:[ litVarArray _validateClass: Array].
(aMethodDict ~~ nil and:[ aMethodDict ~~ false]) ifTrue:[ aMethodDict _validateClass: GsMethodDictionary ].
aCategDict ~~ nil ifTrue:[ aCategDict _validateClass: GsMethodDictionary ].
pragmasArray ~~ nil ifTrue:[ pragmasArray _validateClass: Array].
environmentId _validateClass: SmallInteger .

^ self _primitiveFailed: 
  #_primCompileMethod:symbolList:category:oldLitVars:intoMethodDict:intoCategories:intoPragmas:environmentId: 
   args: { sourceString .  aSymbolList .  categorySymbol .  litVarArray .
	   aMethodDict .  aCategDict .  pragmasArray .  environmentId }
%

! reimplemented in sessionmethods.topaz
category: 'Updating the Method Dictionary'
method:
compileMethod: sourceString
dictionaries: aSymbolList
category: aCategoryString
environmentId: environmentId

"This compiles some source code for the receiver.  The first argument,
 sourceString, is the string of source code to be compiled.  The second
 argument is a SymbolList to be used in parsing, along with the list of all
 class variables and pool dictionaries for the receiver and all of its
 superclasses.  The third argument (a String) indicates the method's category.

 sourceString must be a kind of String or DoubleByteString.  Instances of
 JapaneseString are not supported as source strings.  String literals
 ('abc') are generated as instances of the class of sourceString, 
 unless sourceString is a Symbol, in which case 'abc' produces a String.
 If sourceString is a DoubleByteSymbol, 'abc' produces a DoubleByteString.

 anEnvironmentId must be a SmallInteger >= 0 and <= 16rFFFF.
 0 denotes the base Smalltalk image.  1 is reserved for use by Ruby .

 If there are no errors, this adds the resulting compiled method to the
 receiver's method dictionary and returns that method, 
 otherwise signals a CompileError .
 A CompileWarning may be signaled, after adding the new method
 to a receiver's method dictionary."

| symList categ meth |

self _validatePrivilege ifFalse:[ ^ nil ] .
aSymbolList class == SymbolList
  ifTrue:[ symList := aSymbolList ]
  ifFalse:[ aSymbolList _validateClass: Array .
            symList := SymbolList withAll: aSymbolList ] .
categ := aCategoryString asSymbol .

GsObjectSecurityPolicy setCurrent: self objectSecurityPolicy while:[
  meth := self compileMethod: sourceString
       dictionaries: symList
       category: categ  intoMethodDict: nil intoCategories: nil
       intoPragmas: nil environmentId: environmentId .

].
^ meth
%

category: 'Private'
method:
_gciCompileMethod: sourceString
dictionaries: aSymbolList
category: aCategoryString
environmentId: envId
  "used by GCI implementation , when session methods are enabled.
   Returns nil for successful compilation, a warning String,
   or signals a CompileError"

^ [
    self compileMethod: sourceString dictionaries: aSymbolList
          category: aCategoryString environmentId: envId .
    nil
  ] onException: CompileWarning do: [ :ex | "handle CompileWarning"
    ex warningString     
  ]   
%


category: 'Updating the Method Dictionary - Deprecated'
method:
compileMethod: sourceString
dictionaries: aSymbolList
category: aCategoryString

"Deprecated.
 Returns nil, or an Array of error descriptors"

^ [ self compileMethod: sourceString 
       dictionaries: aSymbolList 
       category: aCategoryString 
       environmentId: 0 .
    nil
  ] onException: CompileError do:[:ex | ex errorDetails ]
%

category: 'Updating the Method Dictionary'
method:
compileAccessingMethodsFor: anArrayOfSymbols

"This method is a simple way to create methods for reading and modifying
 instance variables in instances of the receiver.  Each element of
 anArrayOfSymbols must be an instance variable in the receiver.  For each
 instance variable 'x' in the Array, two methods are created: 'x' (read the
 variable) and 'x:newValue' (modify the variable).  The first method ('x') is
 placed in the category 'Accessing', while the second method ('x:newValue') is
 placed in the category 'Updating'.

 The method can also be used to create methods for accessing and modifying
 class and pool variables.  When creating class methods, the message must be
 sent to the class of the class.

 The new methods have environmentId == 0 .

 Returns the receiver.  Generates an error if any element of anArrayOfSymbols
 is not an instance variable, class variable, or pool variable of the
 receiver."

| allVarNames varName compileBlk |

self _validatePrivilege ifFalse:[ ^ nil ].
varName := 'newValue'.
allVarNames := self allInstVarNames.
[allVarNames includesValue: varName] whileTrue: [
  varName := 'z' , varName.
].
compileBlk := [ :src :cat :varName |
    [ self compileMethod: src
       dictionaries:  #() 
       category: cat
       environmentId: 0 
    ] onException: CompileError do:[:ex | 
      ^ self _error: #classErrNotAVar args: { varName } 
    ]
  ].
anArrayOfSymbols do: [ :var | | getSrc putSrc lf |
  lf := Character lf .
  (getSrc := String new) add: var; add: lf ; add: lf ;
      add: '   "Return the value of the instance variable '''; add: var ;
      add: '''."' ; add: lf ; add: '   ^' ; add: var ; add: lf .
  (putSrc := String new) add:var ; add: ': ' ; add: varName ; add: lf ; add: lf ;
    add: '   "Modify the value of the instance variable ''' ; add: var ;
    add: '''."' ; add: lf ; add: '   ' ; add: var ; add: ' := ' ; add: varName ; add: lf .
  compileBlk value: getSrc value: #Accessing value: var .
  compileBlk value: putSrc value: #Updating  value: var .
]
%

! _setMessageBreak: deleted

category: 'Debugging Support'
method:
_setMethodBreak: aSelector stepPoint: anInt

"Returns true to indicate success.  Otherwise it returns a string describing
 the error."

  | method selectorSym |

  (aSelector isByteKindOf: CharacterCollection)
  ifFalse:
    [ ^ 'Illegal selector' ].
  selectorSym := aSelector asSymbol .

  (#( ifFalse:  ifFalse:ifTrue:   and:   or:  _and: _or:  
      ifNil: ifNotNil:  ifNil:ifNotNil:  ifNotNil:ifNil:
     ifTrue:   ifTrue:ifFalse:   timesRepeat:   to:by:do:  repeat
     to:do:   untilFalse   untilTrue   whileFalse:   whileTrue:   
     _downTo:by:do: _downTo:do: 
     #==   #~~   isKindOf:   _leaveProtectedMode 
     _gsReturnNoResult _gsReturnNothingEnableEvents 
    _isInteger _isSmallInteger _isNumber _isFloat _isSymbol _isExceptionClass
    _isExecBlock _isBlock _isArray _isOneByteString _isString _isRubyHash _isRegexp
    _isRange
    )

    includesIdentical: aSelector)
  ifTrue:
     [ ^ 'You may not set a method break on an optimized selector' ].

  (self == SmallInteger 
      and: [ #(  #+  #-  #>=  #*  #=  ) includesIdentical: aSelector])
  ifTrue:
    [ ^ 'You may not set a method break on an optimized selector' ].

  (anInt _isSmallInteger)
  ifFalse:
    [ ^ 'Step point must be a SmallInteger' ].
  (self includesSelector: aSelector)
  ifFalse:
    [ ^ 'Selector does not exist in class' ].
  method := self compiledMethodAt: aSelector.


  ((self compiledMethodAt: aSelector ) setBreakAtStepPoint: anInt) == nil 
  ifTrue:
    [ ^ 'Step point does not exist in method' ].

  ^ true
%

! _messageBreakObject: deleted

! _methodBreakObject:stepPoint: deleted


category: 'Debugging Support'
method:
_sourceCodeAndOffsets: aSelector
  ^ self _sourceCodeAndOffsets: aSelector environmentId: 0
%
method:
_sourceCodeAndOffsets: aSelector environmentId: envId

"Returns an Array with two elements.  The first element is a String
 representing the source code for the argument, aSelector.  The second element
 is an InvariantArray (that holds SmallIntegers) is a list of offsets into
 sourceString, corresponding in order to the step points.  If aSelector (a
 String) is not a selector in the receiver's method dictionary, returns nil."

| method |
method := self compiledMethodAt: aSelector environmentId: envId otherwise: nil .
method ifNotNil:[ ^ { method _sourceString . method _sourceOffsets } ] .
^ nil
%

category: 'Accessing the Class Hierarchy'
method:
allSuperClasses

"Returns an Array of the superclasses of the receiver, beginning
 with the immediate superclass, and excluding the receiver."

| result cls |
result:= { } .
cls := self superClass .
[ true ] whileTrue:[
  cls == nil ifTrue:[ ^ result ].
  result add: cls .
  cls := cls superClass.
]
%

category: 'Accessing the Class Hierarchy'
method:
_allSuperList: includeRubyVirtual env: envId

"Returns an Array of the superclasses of the receiver, beginning
 with the most remote superclass, and excluding the receiver itself."

  | result currClass |
  result:= { } .
  currClass := self .
  includeRubyVirtual ifTrue:[
    [ true] whileTrue:[ 
      currClass:= currClass rubySuperclass: envId .
      currClass == nil ifTrue: [^ result].
      result insertObject: currClass at: 1.
    ].
  ] ifFalse:[
    [ true] whileTrue:[ 
      currClass:= currClass superClass.
      currClass == nil ifTrue: [^ result].
      result insertObject: currClass at: 1.
    ].
  ]
%

category: 'Jade browser support'
method
_allSuperList

 ^ self _allSuperList: true env: 0 
%

category: 'Accessing the Class Hierarchy'
method:
inheritsFrom: aClass

"Returns true if the argument aClass is on the receiver's
 superclass chain; returns false if it isn't."


(aClass isKindOf: Behavior)
ifFalse:
   [self _error: #rtErrBadArgKind args: { Class }].

(self isSubclassOf: aClass)
   ifTrue: [
     self == aClass
       ifTrue: [^false]
       ifFalse: [^true]
   ]
   ifFalse: [^false] .
   self _uncontinuableError " should never get here"
%

! see Behavior_ruby.gs for methods isRubyVirtual,
!  isRubyModuleIncludeSelf, isRubyModuleFunctions, isMetaModule
!  isRubySingletonClass

category: 'Accessing the Class Format'
method:
rubyPrimaryCopy
  "If receiver is a Ruby virtual class, return the class of which it
   is a copy, otherwise return the receiver."

  ^ self "alway self in pure Smalltalk "
%

category: 'Accessing the Class Hierarchy'
method:
superClass

"Returns the receiver's superclass."

 ^ superClass
%

category: 'Private'
method:
_superclass
  ^ superClass
%

category: 'Accessing the Class Hierarchy'
method:
superclass
 "for squeak compatibility"
 ^ self superClass
%

category: 'Accessing the Class Hierarchy'
method:
rubySuperclass: envId

 "Return receiver's superclass.
  Reimplemented in Module to handle virtual and singleton classes,
  and envId ~~ 0 ."

  ^ superClass
%

category: 'Accessing the Method Dictionary'
method:
canUnderstand: aSelector

"Returns true if the receiver can respond to the message indicated by
 aSelector, returns false otherwise.  The selector (a String) can be in the
 method dictionary of the receiver or any of the receiver's superclasses."

| aSymbol |
aSymbol := Symbol _existingWithAll: aSelector .
aSymbol == nil ifTrue:[ ^ false ].

((self whichClassIncludesSelector: aSymbol) ~~ nil)
ifTrue:
   [^ true].
^false
%

category: 'Accessing the Method Dictionary'
method:
compiledMethodAt: aSelector
  ^ self compiledMethodAt: aSelector environmentId: 0
%

method:
compiledMethodAt: aSelector environmentId: envId

"Returns the compiled method associated with the argument aSelector (a String).
 The argument must be a selector in the receiver's method dictionary; if it is
 not, this method generates an error."

| aMeth |
aMeth := self compiledMethodAt: aSelector environmentId: envId otherwise: nil.
aMeth == nil ifTrue:[ self _error: #rtErrKeyNotFound args: { aSelector } ].
^ aMeth
%

method:
compiledMethodAt: aSelector otherwise: notFoundVal
  ^ self compiledMethodAt: aSelector environmentId: 0 otherwise: notFoundVal 
%

method:
includesSelector: aString
  ^ self includesSelector: aString environmentId: 0
%


method:
selectors
^ self selectorsForEnvironment: 0
%

method:
allSelectors

^ self allSelectorsForEnvironment: 0
%

! should allSelectors return duplicates ??

method:
allSelectorsForEnvironment: envId
"Returns an Array of Symbols, consisting of all of the message
 selectors that instances of the receiver can understand, including
 those inherited from superclasses.  For keyword messages, the
 Symbol includes each of the keywords, concatenated together."

 | result currClass |
 result:= { } .
 currClass:= self.
 [ currClass == nil ] whileFalse: [
     result insert: (currClass selectorsForEnvironment: envId ) at: 1 .
     currClass:= currClass rubySuperclass: envId .
 ].
  ^ result 
%

method:
sourceCodeAt: aSelector

^ self sourceCodeAt: aSelector environmentId: 0
%

method:
sourceCodeAt: aSelector environmentId: envId

"Returns a String representing the source code for the argument, aSelector.  If
 aSelector (a String) is not a selector in the receiver's method dictionary,
 this generates an error."

^ (self compiledMethodAt: aSelector environmentId: envId) sourceString
%

method:
whichClassIncludesSelector: aString
  ^ self whichClassIncludesSelector: aString environmentId: 0
%

category: 'Virtual Machine Control'

! fixed 41412
method:
_refreshLookupCache: aSelector oldMethod: aMethod env: envId

"This message must be sent whenever a class's method dictionary
 changes, to keep lookup caches current.

 If not nil, aSelector specifieds a selector added or removed from receiver's
 method dictionary(s) for specified envId .

 If the argument aMethod is not nil, any breakpoints in the obsolete
 method aMethod are cleared.
 If aMethod is a SmallInteger, it is assumed to be a Ruby method-hidden
 value , and breakpoint clearing logic is skipped.

 envId must be a SmallInteger >= 0 and <= 255 .
"

<primitive: 374>
aSelector ifNotNil:[ aSelector _validateClass: Symbol ] .
aMethod ifNotNil:[
   (aMethod isKindOf: GsMethod) ifTrue: [  
     ^ self "ignore unconverted method from Gs64 2.x"
   ].
   aMethod _validateClass: GsNMethod 
].
envId _validateClass: SmallInteger .
self _primitiveFailed: #_refreshLookupCache:oldMethod:env: 
     args: { aSelector . aMethod . envId }.
self _uncontinuableError
%

method:
_clearLookupCaches: envId

"Invalidates method lookup caches for all classes.
 Invalidates all send-site caches for the specified environment.

 envId must be a SmallInteger >= 0 and <= 255 .
"
<primitive: 165>
envId _validateClass: SmallInteger .
self _primitiveFailed:#_clearLookupCaches: args: { envId }
%

! deleted _addCategory:

category: 'Browser Methods'
method:
_poolDictionaries

"Returns the object containing this instance's pool dictionaries."

^  poolDictionaries ifNil:[ #() ] 
%

! deleted _categories

category: 'Browser Methods'
method:
_categoriesReport

"Returns an Array containing key-value pairs from the receiver's categories
 for environment 0 .
 The key in each key-value pair is the name of a category; the value in each
 key-value pair is a sorted Array of selectors.

 Used by the Topaz 'list categories' command."

| result k sz sortedCats |
sortedCats := SortedCollection new:[ :a :b | a key <= b key ].
self env: 0 unifiedCategoriesDo:[ :categName :selectors |
  sortedCats add: ( Association newWithKey: categName value: selectors)
].
result := Array new: (sz := sortedCats size) * 2  .
k := 1 .
1 to: sz do:[:j | | anAssoc |
   anAssoc := sortedCats at: j . 
   result at: k put: anAssoc key .
   result at: k + 1 put: anAssoc _value .
   k := k + 2 .
].
^ result .
%

category: 'Browser Methods'
method:
_selectorsReport: envId
  ^ SortedCollection withAll:(self selectorsForEnvironment: envId) .
%

category: 'Browser Methods'
method:
_selectorPrefixesReport: envId
  "In base Smalltalk image, no ruby prefix logic"
  ^ self _selectorsReport: envId  
%

category: 'Accessing Variables'
method:
allClassVarNames

"Returns an Array of Symbols, consisting of the names of the 
 smalltalk class variables addressable by this class, 
 including those inherited from superclasses.  Contrast with classVarNames."

| result currClass |

result:= { } .
currClass:= self.
[ true ] whileTrue:[
  currClass == nil ifTrue:[ ^ result ].
  result insert: (currClass classVarNames) at: 1.
  currClass:= currClass rubySuperclass: 0
]
%

category: 'Accessing Variables'
method:
allInstVarNames

"Returns an Array of Symbols, consisting of the names of all the receiver's
 instance variables, including those inherited from superclasses.  The ordering
 of the names in the Array follows the ordering of the superclass hierarchy;
 that is, instance variable names inherited from Object are listed first, and
 those peculiar to the receiver are last."

^ Array withAll: instVarNames
%

category: 'Accessing Variables'
method:
allSharedPools

"Returns an Array of pool dictionaries used by this class and its superclasses.
 Contrast with sharedPools."

| result currClass |

result:= { } .
currClass:= self.
[ true ] whileTrue:[
  currClass == nil ifTrue:[ ^ result ].
  result insert: (currClass sharedPools) at: 1.
  currClass:= currClass superClass
].

^ result
%

category: 'Accessing Variables'
method:
classVarNames

"Returns an Array of Symbols naming the class variables defined by this class.
 Inherited class variables are not included; contrast with allClassVarNames."

| cvars |
(cvars := classVars) ifNil:[ ^ { } ].
^ cvars keys asArray
%

category: 'Accessing Variables'
method:
_classVars

"Returns the classVars instance variable."

^ classVars
%

category: 'Accessing Variables'
method:
classVarAt: aClassVar

"Returns the value of the class variable aClassVar."

classVars ifNil:[ ^ self _errorKeyNotFound: aClassVar ].
^ classVars at: aClassVar asSymbol.
%

category: 'Accessing Variables'
method:
instVarNames

"Returns an Array of Symbols naming the instance variables defined by the
 receiver, but not including those inherited from superclasses.  Contrast
 with allInstVarNames."

| inheritedInstVars "an Array of the inherited instance variables"
  size              "the size of inherited instance variables"
  myInstVars        "an Array of all of the instance variables"  
  supercls     |

(myInstVars := instVarNames) ifNil:[ ^ { } ].
(supercls := self superClass) ifNil: [ ^ myInstVars ].

inheritedInstVars := supercls _instVarNames.
(size := inheritedInstVars size) == myInstVars size ifTrue:[ ^{ } ].

"Assume that each inherited instance variable is added to the end of
 the Array result of allInstVarNames."

^ myInstVars copyFrom: size + 1 to: myInstVars size .
%

category: 'Accessing Variables'
method:
scopeHas: aVariableName
ifTrue: aBlock

"If aVariableName (a String) is specified as a variable in the receiver or one
 of its superclasses, this evaluates the zero-argument block aBlock and returns
 the result of evaluating aBlock.  Otherwise, returns false."

| allSharedPools |

(aVariableName isKindOf: String)
ifFalse:
   [ ^ aVariableName _error: #rtErrBadArgKind args: { String }].

( ((self allInstVarNames ) includesValue: aVariableName) or:
 [((self allClassVarNames) includesValue: aVariableName)])
ifTrue:
   [^ aBlock value]
ifFalse: "now check sharedPools"
[
   allSharedPools:= self allSharedPools.
   allSharedPools do: [:poolDict |
                         (poolDict includesKey: aVariableName)
                         ifTrue:
                            [ ^ aBlock value]
                      ]
].
^ false
%

category: 'Accessing Variables'
method:
sharedPools

"Returns an Array of pool dictionaries used by this class.  Superclasses
 are not included; contrast with allSharedPools."

| pd |
^ (pd := poolDictionaries) ifNil:[ { } ]
		      ifNotNil:[ Array withAll: pd ]
%

category: 'Deprecated'
method:
constraintOfInstVar: aString

"Returns the constraint on the instance variable named aString for instances
 of the receiver.  Generates an error if aString is not the name of an instance
 variable defined by the receiver."

| aSym offArray |
self deprecated: 'Constraints are no longer supported'.
aSym := Symbol _existingWithAll: aString .
aSym ~~ nil ifTrue:[
  offArray := self _ivOffsetAndConstraint: aSym .
  offArray ~~ nil ifTrue:[
    ^ offArray at: 2 .
    ].
  ].
^ self _errorInvalidOffset: aString .
%

category: 'Accessing the Class Format'
method:
format

"Returns the value of the format instance variable."

^ format
%

category: 'Accessing the Class Format'
method:
implementationFormat

"Returns the three least-significant bits of the receiver's format instance
 variable.  The values of those bits mean the following:

 0   OOP       non-indexable
 1   Byte      non-indexable
 2   NSC       non-indexable
 3   Special   non-indexable
 4   OOP       indexable
 5   Byte      indexable"

^ format bitAnd: 16r7
%

category: 'Accessing the Class Format'
method:
instSize

"Returns the number of named instance variables in the receiver, including all
 inherited instance variables."

^ instVarsInfo bitAnd: Class_numIvs_mask
%

category: 'Private
method:
_instVarsInfo
  ^ instVarsInfo
%

category: 'Accessing the Class Format'
method:
isBytes

"Returns true if instances of the receiver are byte objects.  Otherwise,
 returns false."

^ (format bitAnd: 16r3) == 1
%

category: 'Accessing the Class Format'
method:
isIndexable

"Returns true if instances of the receiver have indexed variables.
 Otherwise, returns false."

^ (format bitAnd: 16r4) ~~ 0 "that is, is indexable"
%

category: 'Accessing the Class Format'
method:
isNsc

"Returns true if instances of the receiver are non-sequenceable
 collections (UnorderedCollections).  Otherwise, returns false."

^ (format bitAnd: 16r3) == 2
%

category: 'Accessing the Class Format'
method:
isPointers

"Returns true if instances of the receiver are pointer objects.
 Otherwise, returns false."

^ (format bitAnd: 16r3) == 0
%
method:
isRubySingletonClass

  "Returns true if receiver is a Ruby singleton class."

  ^ (format bitAnd: 16r8000) ~~ 0
%

category: 'Accessing the Class Format'
method:
_isSpecial

"Returns true if instances of the receiver are special objects.
 Otherwise, returns false."

^ (format bitAnd: 16r3) == 3
%
category: 'Queries'
method:
isSpecial

"Returns whether instances of this class have their state encoded in their
 identities."

^ (format bitAnd: 16r3) == 3  "inline _isSpecial"
%

category: 'Accessing the Class Format'
method:
instancesInvariant

"Returns true if instances of the receiver may not change value after they have
 been committed to GemStone.  Otherwise, returns false."

^ (format bitAnd: 16r8) ~~ 0 
%

category: 'Accessing the Class Format'
method:
subclassesDisallowed

"Returns true if subclasses of the receiver have been disallowed by means of
 Behavior | disallowSubclasses.  Otherwise, returns false."

^ (format bitAnd: 16r20) ~~ 0
%

category: 'Accessing the Class Format'
method:
isProtected

"Returns true if instances of the receiver may not be accessed structurally
 through GemBuilder for C. "

^ (format bitAnd: 16r80) ~~ 0 
%

category: 'Accessing the Class Format'
method:
hasPublicInstVars

"Returns true if the receiver has publicly-visible instance variables."

^ self instSize >= self firstPublicInstVar
%

! comments added re: bug 12857
category: 'Accessing the Class Format'
method:
firstPublicInstVar

"Returns the index of the first publicly available instance variable storage
 location, whether or not a public instance variable has actually been
 defined."

"Currently, an instance variable is considered to be public if it is to be
 included when passivating the object through PassiveObject.  Being public has
 no relationship to whether or not accessing or updating methods are defined
 for the instance variable."

^ 1
%

category: 'Accessing the Class Format'
method:
_firstGciPublicInstVar

"Returns the index of the first instance variable defined in this class for
 which structural updates are allowed through GemBuilder for C, if any."

^ self firstPublicInstVar
%

! _selectorAtMethod deleted

category: 'Enumerating'
method:
allSuperClassesDo: aBlock

"Evaluates aBlock with each of the receiver's superclasses as
 the argument, beginning with the immediate superclass."

| currClass |

currClass:= self superClass .
[ true ] whileTrue:[
  currClass ifNil:[ ^ self ].   
  aBlock value: currClass .
  currClass:= currClass superClass
]
%

category: 'Modifying Classes'
method:
disallowSubclasses

"Disallows creation of subclasses of a class.  If the receiver is not
 modifiable, this method generates an error.  If the receiver is modifiable and
 already has subclasses, this method generates an error."

self _validatePrivilege ifTrue:[
  self isModifiable ifFalse:[ ^ self validateIsModifiable ].
  self _subclasses size ~~ 0 ifTrue:[ ^ self _error: #rtErrAlreadyHasSubclasses ].
  format := format bitOr: 32
]
%

category: 'Modifying Classes'
method:
allowSubclasses

"Allows creation of subclasses of a class."

format := (format bitOr:32) bitXor: 32 .  "clear bit 32"
%


! fix bug 11794 and 11457
category: 'Deprecated'
method:
addInstVar: aSymbol withConstraint: aClass

"Adds a new instance variable named aSymbol to the receiver and constrains the
 value of that variable to be of the kind aClass.  The instance variable is 
 added in the same way and under the same conditions as described for the
 addInstVar: method.

 The argument aClass must be a kind of Class; otherwise, this method generates
 an error.

 Note that this method can be used if the receiver disallows subclasses and has
 no unnamed instance variables, even if it is not modifiable.  Note also that
 the instVar:constrainTo: method cannot be used under those conditions."

| ivInfo offset |

self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifTrue:[
  aClass _validateIsClass ifTrue:[
    self addInstVar: aSymbol .
    "if addInstVar: succeeded then we should be able to add the constraint"
    self isModifiable
      ifTrue:[ self instVar: aSymbol constrainTo: aClass]
      ifFalse:[
	"do a subset of instVar:constrainTo: assuming subclasses disallowed"
	ivInfo := self _ivOffsetAndConstraint: aSymbol .
	(ivInfo == nil) ifTrue:[ ^ self _error: #classErrNotAVar args:{ aSymbol } ] .
	offset := ivInfo at: 1 .
	aClass validateSubclassOf: (self superClass _namedIvConstraintAtOffset: offset) .
	self _newConstraint: aClass atOffset: offset .
      ].
  ].
].
%

category: 'Deprecated'
method:
instVar: aString constrainTo: aClass

"Changes the receiver's constraint on the instance variable named
 aString to aClass.

 The argument aString must be the name of an instance variable defined in the
 receiver or inherited from a superclass.  aClass must be a kind of Class.
 The receiver, and any subclasses for which a constraint change will result,
 must be modifiable; otherwise, an error will be generated.

 If the superclass of the receiver has a constraint on the same instance
 variable, then aClass must be identical to, or a subclass of, that inherited
 constraint.

 For each of the receiver's subclasses, if the constraint on the specified
 instance variable is aClass or is a subclass of aClass, then that constraint
 will be unchanged.  Otherwise, the subclass's constraint will be changed to
 aClass."

| aSymbol ivInfo offset mySubclasses |
self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifFalse:[ ^ nil ].
aClass _validateIsClass ifFalse:[ ^ nil ].
aSymbol := aString asSymbol .
ivInfo := self _ivOffsetAndConstraint: aSymbol .
(ivInfo == nil) ifTrue:[ ^ self _error: #classErrNotAVar args:{ aSymbol } ] .
offset := ivInfo at: 1 .
(aClass validateSubclassOf: (self superClass _namedIvConstraintAtOffset: offset)) ifFalse:[
  ^ nil
].
self isModifiable ifFalse:[ ^ self validateIsModifiable ].
mySubclasses := self _subclasses .
mySubclasses do:[:x|
  (x _validateNewInheritedConstraint:aClass atOffset:offset) ifFalse:[ ^ nil ]
] .
self _newConstraint: aClass atOffset: offset .
mySubclasses do:[:x| x _newInheritedConstraint: aClass atOffset: offset ] .
%

category: 'Modifying Classes'
method:
isModifiable

"Returns true if the receiver may be modified (that is, if the receiver, its
 Array of constraints, and its Array of instance variable names are all
 variant, and the receiver has a 'subclasses' class variable).  Returns false
 otherwise."

 | constr ivn |
 self isInvariant ifTrue:[ ^ false ].

 ((constr := constraints) size ~~ 0 and:[ constr isInvariant]) ifTrue:[ ^ false ].

 ((ivn := instVarNames) size ~~ 0 and:[ ivn isInvariant]) ifTrue:[ ^ false ].

 ^ self _subclasses ~~ nil 
%

category: 'Modifying Classes'
method:
validateIsModifiable

"Returns the receiver if the receiver, its Array of constraints, and its
 Array of instance variables are modifiable.  Generates an error if the
 receiver cannot be modified (that is, if the receiver, its Array of
 constraints, or its Array of instance variable names is not variant)."

self isModifiable ifFalse:[ ^ self _error: #rtErrClassNotModifiable ]
%

category: 'Modifying Classes'
method:
validateSubclassesAreModifiable

"Generates an error if the receiver or any of its subclasses cannot be
 modified."

self validateIsModifiable .
self _subclasses do:[:x| x validateSubclassesAreModifiable ].
^ self
%

category: 'Modifying Classes'
method:
_removeClassVar: aSymbol

"Removes the class variable with name aSymbol from the receiver's
 class variables dictionary.  Generates an error if aSymbol is not
 the name of a class variable of the receiver."

  self _validatePrivilege ifTrue:[
    classVars ifNil:[ ^ self _errorKeyNotFound: aSymbol ].
    classVars removeKey: aSymbol
            ifAbsent: [classVars _errorKeyNotFound: aSymbol].
  ]
%

category: 'Modifying Classes'
method:
_addClassVar: aSymbol value: aVal 

"Adds the class variable with name aSymbol to the receiver's
 class variables dictionary."
  | dict |
  self _validatePrivilege ifTrue:[
    (dict := classVars) ifNil:[
      dict := self _createClassVarsDict .
    ].
    dict at: aSymbol put: aVal .
  ]
%

category: 'Modifying Classes'
method:
_createClassVarsDict
  | dict |
  (dict := classVars) ifNil:[
    (dict := SymbolDictionary new ) _objectSecurityPolicy: self objectSecurityPolicy .
    self _setClassVars: dict
  ].
  ^ dict
%


category: 'Modifying Classes'
method:
_addInvariantClassVar: aSymbol value: aVal 

"Adds the class variable with name aSymbol to the receiver's
 class variables dictionary, and make the resulting SymbolAssocation
 invariant."
| assoc dict |
self _validatePrivilege ifTrue:[
  (dict := classVars) ifNil:[
    dict := self _createClassVarsDict .
  ].
  assoc := dict associationAt: aSymbol otherwise: nil .
  assoc ifNil:[
    assoc := SymbolAssociation new key: aSymbol value: aVal .
    assoc _objectSecurityPolicy: self objectSecurityPolicy .
    dict addAssociation: assoc .
    assoc immediateInvariant 
  ] ifNotNil:[
    assoc _value = aVal ifFalse:[
      assoc _value: aVal .
      assoc immediateInvariant 
    ]
  ].
]
%

category: 'Modifying Classes'
method:
_removeClassVar: aSymbol ifAbsent: exceptionBlock

"Removes the class variable with name aSymbol from the receiver's
 class variables dictionary.  Executes exceptionBlock if aSymbol is not
 the name of a class variable of the receiver."
  | dict |
  self _validatePrivilege ifTrue:[
    (dict := classVars) ifNil:[ exceptionBlock value ]
       ifNotNil:[ dict removeKey: aSymbol ifAbsent: exceptionBlock ].
  ]
%

category: 'Modifying Classes'
method:
_subclasses

"Returns the class variable that is the list of subclasses, or
 nil if the receiver does not keep track of subclasses."

^ nil
%

category: 'Modifying Classes'
method:
immediateInvariant

"If the receiver is not invariant, make it invariant and
 conditionally also make the receiver's instVarNames invariant.
 Also clears the subclasses instVar .  

 After this method executes,  addInstVar: and removeInstVar: will
 no longer be allowed for the receiver."

self isInvariant ifFalse:[
  self _validatePrivilege ifFalse:[ ^ nil ].
  " Gs64 v3.0 and later, 
    We no longer recompile methods here, the assumption is that removeInstVar:   
    has recompiled methods as needed or has failed and invoked
      System _disallowCommitClassModFailure  .
  "   
  self subclassesDisallowed ifFalse:[
      "only make the instVarNames and Constraints collections invariant if we
       are not permitting possible dynamic addition of named instance variables.
      "
    constraints _isArray ifTrue:[ constraints immediateInvariant].
    instVarNames immediateInvariant.
    ] .
  "clear the subclasses class instance variable to prevent concurrency problems"
  self _subclasses ~~ nil ifTrue:[ 
    self _subclasses: nil .
    ] .
  super immediateInvariant .  "make self invariant"
  "_codeChangedForEnv: not needed, since no instances can exist yet"
  self _refreshClassCache: false .
].
%


category: 'Modifying Classes'
method:
removeInstVar: aString

"Removes the instance variable named aString from the receiver and from all of
 the receiver's subclasses.  The receiver and all of its subclasses must be
 modifiable.

 All instance methods for the receiver and its subclasses are recompiled using
 the symbol list of the current user.  If an error occurs during recompilation
 of methods, the instance variable will have been removed from the receiver and
 from all of its subclasses, but some methods in some subclasses will not have
 been recompiled, and subsquent commit will be disallowed.

 You may not use this method to remove an inherited instance variable."

| ivInfo offset aSymbol success |
self _validatePrivilege ifFalse:[ ^ nil ].

"validate that the instance variable exists"
aSymbol := aString asSymbol .
ivInfo := self _ivOffsetAndConstraint: aSymbol .
(ivInfo == nil) ifTrue:[ ^ self _error: #classErrNotAVar args:{ aSymbol } ] .

((self superClass _ivOffsetAndConstraint: aSymbol) == nil)
     ifFalse:[ ^ self _error: #classErrRemoveInherIv args:{ aSymbol } ].

offset := ivInfo at: 1 .
"validate that self and all subclasses are modifiable"
self validateSubclassesAreModifiable .
success := false .
[ self _removeInstVarAtOffset: offset .  "remove from self and all subclasses"
  self recompileAllSubclassMethodsInContext: System myUserProfile symbolList .
  success := true 
] ensure:[
  success ifFalse:[ System _disallowCommitClassModFailure ].
]
%

category: 'Deprecated'
method:
varyingConstraint: aClass

"Changes the constraint on the unnamed variables of the receiver.

 The argument aClass must be a kind of Class.  The receiver, and any subclasses
 for which a constraint change will result, must be modifiable.  Otherwise, an
 error will be generated.

 If the superclass of the receiver has a constraint on its unnamed part, then
 aClass must be identical to, or a subclass of, that inherited constraint.

 For each of the receiver's subclasses, if the constraint on that subclass's
 unnamed part is either aClass or a subclass of aClass, that constraint will
 be unchanged.  Otherwise, the subclass's constraint will be changed to
 aClass."

self deprecated: 'Constraints are no longer supported'.
self _validatePrivilege ifFalse:[ ^ nil ].
(self isBytes)
    ifTrue:[ ^ self _error: #rtErrVaryingConstrBytes ].
(self isNsc) ifFalse:[
  (self isIndexable) ifFalse:[ ^ self _error: #rtErrVaryingConstrNonidx] ] .
self isModifiable ifFalse:[ ^ self validateIsModifiable ].
aClass _validateIsClass ifFalse:[ ^ nil ].
(aClass validateSubclassOf:( self superClass varyingConstraint ) ) ifFalse:[ ^ nil ].
self _subclasses do:[:x| 
  (x _validateNewVaryingConstraint: aClass) ifFalse:[ ^ nil ].
].
self _setVaryingConstraint: aClass .
self _subclasses do:[:x| x _updateVaryingConstraint: aClass].
^ self
%

category: 'Modifying Classes'
method:
addInstVar: aString

"Adds a new instance variable named aString to the receiver.  The argument
 aString must be a valid GemStone Smalltalk identifier and must be distinct from
 the names of all other instance variables previously defined for the receiver,
 its superclasses, and its subclasses.

 The new instance variable becomes the last named instance variable in the
 receiver, and is inserted at the appropriate position in each of the
 receiver's subclasses, to preserve the rules for inheritance of instance
 variables.  If, for any of the receiver's subclasses, the new instance
 variable is not the last named instance variable, then all instance methods
 for that subclass are recompiled using the symbol list of the current user.
 If an error occurs during recompilation of methods, the new instance
 variable will have been added to the receiver and all of its subclasses, but
 some methods in some subclasses will not have been recompiled.

 To successfully invoke this method, the receiver must meet one of these two
 conditions:

 * The receiver and all of its subclasses must be modifiable.
 * The receiver must disallow subclasses and must have no unnamed instance
   variables."

| newOffset theSymbol |
self _validatePrivilege ifTrue:[
  "3.2, no error check for NSCs"
  (self isBytes) ifTrue:[ ^ self _error: #rtErrInstvarAddToBytes ] .
  theSymbol:= aString asSymbol .
  theSymbol validateIsIdentifier .
  (self _validateNewNamedInstVar: theSymbol ) ifFalse:[ ^ nil ].

  newOffset := self instSize + 1 .  "the offset of the new instance variable"
  self _insertNamedInstVar: theSymbol atOffset: newOffset .
  self _recompileMethodsAfterNewIvOffset: newOffset .
]
%

category: 'Deprecated'
method:
allConstraints

"If the receiver defines a non-sequenceable collection (UnorderedCollection)
 class, this returns a single Class, the element kind of the receiver.

 Otherwise, this returns an Array of Classes.  Each element in that Array is
 the class kind of a corresponding instance variable.  The ordering of the
 elements in the Array is the same as the ordering of instance variables in the
 receiver."

| constr |
self deprecated: 'Constraints are no longer supported'.
constr := Array withAll: constraints .
constr size == 0 ifTrue:[
  1 to: self instSize do:[:j | constr add: Object ].
].
^ constr
%

! fix bug 11627
category: 'Deprecated'
method:
constraintOn: aString

"Returns the class kind constraint for the instance variable represented by
 aString.  If the instance variable named aString is not constrained, returns
 Object.  If no instance variable named aString exists for objects whose
 Behavior is defined by the receiver, returns nil."

| aSymbol |
self deprecated: 'Constraints are no longer supported'.
aSymbol := Symbol _existingWithAll: aString .
aSymbol == nil ifTrue:[ ^ nil ].
^ self _constraintOn: aSymbol
%

category: 'Private'
method:
_constraintOn: aSymbol

"Returns the class kind constraint for the instance variable represented by
 aSymbol.  If aSymbol does not represent an instance variable of objects whose
 behavior is defined by the receiver, returns nil.
 If the instance variable aSymbol is not constrained, returns Object ."

| ivNams constrs |

ivNams := instVarNames .
constrs := constraints .
1 to: self instSize do: [ :i |
  aSymbol == (ivNams  at: i) ifTrue:[ 
    ^ (( constrs atOrNil: i) ifNil:[ Object]) 
  ].
].
^ nil
%

category: 'Private'
method:
_ivOffsetAndConstraint: aSymbol

"Searches the instVarNames instance variable of the receiver for an instance
 variable named aSymbol, and returns an Array containing the offset and the
 constraint for that instance variable.  Returns nil if no instance variable
 exists with the name aSymbol."

| idx |
idx := instVarNames indexOfIdentical: aSymbol .
idx == 0 ifTrue:[ ^ nil ].
^ { idx . ((constraints atOrNil: idx) ifNil:[ Object ])  }
%

category: 'Private'
method:
_ivNameAndConstraintAt: anInteger

"Returns an Array containing the name and constraint for the
 instance variable of the receiver at the specified offset.
 Returns nil if there is no named instance variable at that offset."

( anInteger < 1 or:[ anInteger > instVarNames size]) ifTrue:[ ^ nil ].
^ { instVarNames at:anInteger . ((constraints atOrNil: anInteger) ifNil:[ Object]) }
%

category: 'Accessing Constraints'
method:
_stringToPath: aString

"Returns an Array of SmallIntegers representing offsets to the named instance
 variables named in the argument.  Called by GciStrToPath.  See specification
 of GciStrToPath for further details.  Receiver is the class at the root of the
 path."

| pathTerms stringPosition argSize nextDotPos nextName result
  termResult pathSize currClass |

aString _validateByteClass: CharacterCollection .

" following code should ideally be String|_asArrayOfPathTerms "
argSize := aString size .
pathTerms := { }  .
stringPosition := 1 .
[ (stringPosition <= argSize) ] whileTrue: [
  nextDotPos := aString findString: '.' startingAt: stringPosition .
  (nextDotPos == 0) ifTrue:[ nextDotPos := argSize + 1 ] .
  nextName := aString class new.
 "aString copyFrom: stringPosition to: (nextDotPos - 1) into: nextName startingAt: 1 . "
  nextName replaceFrom: 1 to: nextDotPos - stringPosition 
	   with: aString startingAt: stringPosition .
  nextName := Symbol _existingWithAll: nextName .
  nextName == nil ifTrue:[ nextName := #'' "no instVar has this name" ].
  pathTerms add: nextName.
  stringPosition := nextDotPos + 1 .
  ].
pathSize := pathTerms size .  " end of _asArrayOfPathTerms "

result := { } .
currClass := self .
1 to: pathSize do:[:j|
  currClass == Object ifTrue:[
    ^ self _error: #rtErrStrToPathConstraint args:{ j }
    ].
  termResult := currClass _ivOffsetAndConstraint: (pathTerms at:j) .
  termResult == nil ifTrue:[
    ^ self _error: #rtErrStrToPathIvname
           args: { j . (pathTerms at:j) . currClass }.
    ] .
  result add:(termResult at: 1).
  currClass := termResult at: 2.
  ] .
^ result
%

category: 'Accessing Constraints'
method:
_pathToString: anArray

"Returns a String representing symbolic form of the path given in the argument.
 Called by GciPathToStr.  See specification of GciPathToStr further details.
 Receiver is the class at the root of the path.  The argument must be an Array
 of positive SmallIntegers."

| argSize result termResult currClass |

anArray _validateClass: Array .

argSize := anArray size .
result := String new .

currClass := self .
1 to: argSize do:[:j|
  currClass == Object ifTrue:[
    ^ self _error: #rtErrStrToPathConstraint args:{ j }
    ].
  termResult := currClass _ivNameAndConstraintAt: (anArray at:j) .
  termResult == nil ifTrue:[
    ^ self _error: #rtErrPathToStrIvname args:{ j . anArray at: j . currClass . 
                            currClass allInstVarNames size }.
    ].
  result addAll:(termResult at: 1); addAll: '.' .
  currClass := termResult at: 2.
  ] .
result size: (result size - 1).  "chop off trailing dot character"
^ result
%

category: 'Deprecated'
method:
_constraints

"Returns the constraints themselves, not a copy."

self deprecated: 'Constraints are no longer supported'.
^ constraints
%

category: 'Deprecated'
method:
_namedIvConstraintAt: anInteger

"Returns the constraint at the specified offset, or Object."

self deprecated: 'Constraints are no longer supported'.
(anInteger > self instSize ) ifTrue:[ ^ Object].
^ (constraints atOrNil: anInteger) ifNil:[ Object ]
%

category: 'Deprecated'
method:
varyingConstraint

"Returns the constraint on the unnamed part of the receiver (a kind of Class).
 If the receiver has no constraint on its unnamed part, or if it has no unnamed
 part, this method returns Object."

self deprecated: 'Constraints are no longer supported'.
^ (constraints atOrNil: self instSize + 1) ifNil:[ Object ].
%

category: 'Deprecated'
method:
elementConstraint

"Returns the kind of objects that instances of the receiver can hold."

self deprecated: 'Constraints are no longer supported'.
^ self varyingConstraint
%

category: 'Instance Creation'
method:
_basicNew

"This is a primitive.  Returns an instance of the receiver, with no indexed
 variables.  Do not override this method; contrast with Behavior | new."

<primitive: 50>

(self _isInstanceDisallowed)
ifTrue:[ self _error: #objErrCantCreateInstance args:  #()  .
         self _uncontinuableError
       ].
self _primitiveFailed: #_basicNew .
self _uncontinuableError
%

category: 'Instance Creation'
method:
new

"Returns an instance of the receiver with no indexed variables."

<primitive: 51>

self _primitiveFailed: #new .
self _uncontinuableError
%

category: 'Instance Creation'
method:
new: anInteger

"Returns an instance of the receiver with the specified number of indexed
 variables.  Generates an error if the receiver is not indexable or if
 anInteger is not a positive SmallInteger.

 For new byte objects, all indexed variables are set to zero;
 for new pointer objects, all indexed variables are set to nil."

<primitive: 53>

(self isIndexable) ifFalse:[ self _errorNotIndexable. ^ self new ].
(anInteger _isSmallInteger) ifFalse:[ 
  anInteger _isInteger ifTrue:[
    anInteger _error: #errArgTooLarge args:{ SmallInteger maximumValue }. 
    ^ self new 
  ].
  anInteger _validateClass: SmallInteger. 
  ^ self new 
].
(anInteger < 0) ifTrue:[ anInteger _error: #rtErrArgNotPositive. ^ self new].
self _primitiveFailed: #new: args: { anInteger }.
self _uncontinuableError
%

category: 'Instance Creation'
method:
migrateNew

"Create a new instance to use in migration.  By default, use #new.
Override in subclasses that can't use #new with #_basicNew. "

^ self new
%

category: 'Instance Creation'
method:
_basicNew: anInteger

"This is a primitive.  Returns with an instance of the receiver, with the given
 number of fields.  Generates an error if the Behavior is not indexable or if
 anInteger is bad.  Do not override this method; contrast with Behavior | new:."

<primitive: 52>
| isz |
(self _isInstanceDisallowed) ifTrue: [
   self _error: #objErrCantCreateInstance args:  #()  .
   self _primitiveFailed: #_basicNew: args: { anInteger }.
   self _uncontinuableError
].
(self isIndexable) ifFalse:[self _errorNotIndexable .  ^ self _basicNew ].
anInteger _isSmallInteger ifFalse:[ 
  anInteger _validateClass: SmallInteger . ^ self _basicNew
].
anInteger < 0 ifTrue:[
  anInteger _error: #rtErrArgNotPositive. ^ self _basicNew
].
(anInteger + (isz := self instSize)) _isSmallInteger ifFalse: [
  anInteger _error: #errArgTooLarge args:[ SmallInteger maximumValue - isz] .
  ^ self _basicNew 
].
self _primitiveFailed: #_basicNew: args: { anInteger }.
self _uncontinuableError
%

category: 'Formatting'
method:
asString

"Returns a String that indicates the class of the receiver."
| n |
n := self _name .  "avoid use of squeak Object>>name"
n ifNil:[ ^ '(unNamedClass)' copy ].
^ String withAll: n
%

!edited to fix 41513
category: 'Clustering'
method:
clusterBehavior

"This method clusters, in depth-first order, the parts of the receiver required
 for executing GemStone Smalltalk code (the receiver and its method dictionary).
 Returns true if the receiver has already been clustered during the current
 transaction; returns false otherwise.

 It is recommended that when several classes are being clustered in a
 transaction, send clusterBehavior to all classes to be clustered, then send
 clusterDescription."

self cluster ifTrue:[ ^ true ].
self isMeta ifFalse:[ | arr |
  (arr := instVarNames) ~~ #() ifTrue:[ arr cluster ] .
  (arr := constraints) ~~ #()  ifTrue:[ arr cluster ] .
].
methDicts _isArray ifTrue:[ methDicts cluster ].
self persistentMethodDictsDo:[:aDict| 
  aDict cluster .
  "note GsNmethod>>clusterDepthFirst clusters execution related parts
  to current bucket and other(debug) info info to bucket 5"
  aDict values do:[ :meth | meth clusterDepthFirst ].
].
self persistentNameSpacesDo:[:aNs| aNs cluster ].
classVars ifNotNil:[
  classVars cluster ifFalse:[
    classVars doAssociations:[:assoc| assoc cluster ].
  ].
].
^ false
%

!  _rebuildMethodDict deleted

! fix bug 14726
category: 'Clustering'
method:
clusterDescription

"This method clusters, in depth-first order, those instance variables in the
 receiver that are not expected to be accessed heavily during method execution,
 specifically the  poolDictionaries, and categories dictionaries.
 
 (The receiver itself is not clustered.)  Returns true if the description
 already been clustered during the current transaction; returns
 false otherwise.

 It is recommended that when several classes are being clustered in a
 transaction, send clusterBehavior to all classes to be clustered, then send
 clusterDescription."

| result |
result := false .
self isMeta ifFalse:[
  poolDictionaries ifNotNil:[ poolDictionaries cluster] .
].
result ifFalse:[
  categorys _isArray ifTrue:[ 
    result := categorys cluster .
    categorys do:[ :aDict | aDict clusterDepthFirst ].
  ] ifFalse:[
    categorys ifNotNil:[ result := categorys clusterDepthFirst ].
  ]
].
^ result
%

category: 'Clustering'
method:
clusterBehaviorExceptMethods: aCollectionOfMethodNames

"This method allows you to cluster the receiver more efficiently by omitting
 infrequently-used methods from the clustering.  The methods that you designate
 as aCollectionOfMethodNames will not be clustered with the receiver.  Thus,
 the methods that are frequently used will be packed more densely.  Returns
 true if the receiver has already been clustered during the current
 transaction; returns false otherwise.

 This method works by first clustering all methods named into the max cluster
 bucket, preventing them from showing up in the default cluster bucket.  It
 then uses the standard method to cluster behavior."

| savedBucket otherBucket systm |
systm := System .
savedBucket := systm clusterBucket.
otherBucket := 6 .
systm clusterBucket: otherBucket.
self persistentMethodDictsDo:[ :aDict |
   aDict keysAndValuesDo:[ :aSelector :aMethod |
     (aCollectionOfMethodNames includesValue: aSelector )
       ifTrue: [ aMethod clusterDepthFirst ].
   ].
].
systm clusterBucket: savedBucket.
^ self clusterBehavior
%

category: 'Copying'
method:
copy

"Returns the receiver. Copies of classes and metaclasses are not made."

^ self.
%

category: 'Private'
method:
_deepCopyWith: copiedObjDict

"Private. Used internally to implement deepCopy."

^ self.
%

category: 'Accessing Categories'
method:
categoryNames

"Returns an Array of Symbols.  The elements of the Array are the
 receiver's environment 0 category names (excluding names inherited from superclasses)."

 | set |
 set := IdentitySet new .
 self env: 0 categorysDo:[ :categName :selectors | set add: categName ].
 ^ Array withAll: set
%

category: 'Accessing Categories'
method:
sortedCategoryNames

"Returns an Array of Symbols.  The elements of the collection are the
 receiver's category names (excluding names inherited from superclasses)."

 | set |
 set := IdentitySet new .
 self env: 0 categorysDo:[ :categName :selectors | set add: categName ].
 ^ set sortAscending:'' .
%

category: 'Indexing Support'
method:
_ivOffsetOf: aSymbol

"Searches the instVarNames instance variable of the receiver for an instance
 variable named aSymbol, and returns the offset for that instance variable.
 Returns nil if no instance variable exists with the name aSymbol."

| idx |
idx := instVarNames indexOfIdentical: aSymbol .
^ idx == 0 ifTrue:[ nil ] ifFalse:[ idx ].
%

category: 'Browsing'
method:
_categoryOfSelector: selector

"Returns the category of the given selector, or 'unknown' if it isn't found."

 | result |
 (result := self categoryOfSelector: selector) == nil ifTrue:[ 
    ^ 'unknown'
    ].
 ^ result
%

! fix bug 8527 
category: 'Browsing'
method:
_copyMethodsAndVariablesFrom: sourceClass except: except dictionaries: dicts

"Copies all instance and class methods, pool dictionaries, and references
 to class variables from the given class to ourselves.  Returns an Array of
 methods in the source class which failed to compile.  Some of them
 might be class methods.  The Array will be empty if none failed.

 Method environmentIds are copied from the source methods.

 The except: argument is a list of categories and/or methods that should
 not be copied.  Each exception is two elements in the Array: a Character
 followed by a String or Symbol.
    $C a category of class methods follows
    $c a category of instance methods follows
    $S the selector of a class method follows
    $s the selector of an instance method follows
    $V a list of class variable names follows
    $P a list of pool dictionaries follows"

| failed srccls targcls sel doit pds i otherCvs excludeCvs |

self _validatePrivilege ifFalse:[ ^ nil ].
self == sourceClass ifTrue:[ 
  "because iteration is directly over the source's categories dictionaries"
  ^ self error: 'source of copy must not be self' 
].
failed := { } .
pds :=  #() .
excludeCvs :=  #() .

except ifNotNil: [
  i := except indexOf: $P.
  i ~~ 0 ifTrue: [ pds := except at: i+1 ].
  i := except indexOf: $V.
  i ~~ 0 ifTrue: [ excludeCvs := except at: i+1 ].
  ].

"Copy pool dictionaries"
sourceClass _poolDictionaries do: [ :dict | | poolDicts |
  poolDicts := poolDictionaries .
  (poolDicts ~~ nil and:[ poolDicts includesIdentical: dict]) ifFalse: [
    (pds includesIdentical: dict) ifFalse: [
      poolDicts ifNil:[ poolDicts := { } . poolDictionaries := poolDicts ]
          ifNotNil:[ poolDicts isInvariant ifTrue:[
                       poolDicts := Array withAll: poolDicts . poolDictionaries := poolDicts
                     ]].
      poolDicts add: dict
    ].
  ].
].

"Copy class variables"
otherCvs := sourceClass _classVars .
otherCvs ifNotNil:[ | destCvs |
   destCvs := classVars . 
   otherCvs associationsDo: [ :assn | | other |
    destCvs ifNotNil:[ other := destCvs associationAt: assn key otherwise: nil ].
    (other == nil or: [other value == nil and: [assn value ~~ nil]]) ifTrue: [
      (excludeCvs includesValue: assn key) ifFalse:[
        destCvs ifNil:[ destCvs := self _createClassVarsDict ].
        destCvs add: assn
      ].
    ].
  ].
].

"Copy class and instance methods"
1 to: 2 do: [ :i | | envId |
  i == 1 ifTrue:[ srccls := sourceClass.  targcls := self ] 
        ifFalse:[ srccls := sourceClass class.  targcls := self class ].
  envId := 0 .
  srccls categorysDo:[ :cat :sels |
      1 to: sels size do: [ :s | | oldMeth |
	sel := sels at: s.
	doit := true.
	1 to: except size by: 2 do: [ :i | | ch exCat |
          ch := except at: i .
          exCat := except at: i + 1 .
	  (((( ch == $C and: [targcls isMeta and: [cat == exCat ]]) or:
	    [ ch == $S and: [targcls isMeta and: [sel == exCat ]]]) or:
	    [ ch == $c and: [targcls isMeta not and: [cat ==  exCat  ]]]) or:
	    [ ch == $s and: [targcls isMeta not and: [sel ==  exCat ]]]) ifTrue: [
	      doit := false 
	  ].
	].
	doit ifTrue: [ | methEnvId |
          oldMeth := srccls compiledMethodAt: sel environmentId: envId .
	  methEnvId := oldMeth environmentId .
          methEnvId == envId ifFalse:[ self error:'environmentId mismatch']. 
	  ( targcls 
	     _compileMethodTrappingErrors: oldMeth sourceString 
	     dictionaries: dicts category: cat environmentId: envId ) ifNotNil:[
	    failed add: oldMeth 
          ].
        ].
      ].
  ].
].

^failed.
%

! fix bug 8086
category: 'Updating the Method Dictionary'
method:
_compileMethodTrappingErrors: sourceString
dictionaries: aSymbolList
category: aCategoryString
environmentId: envId

"If there are no errors, add the resulting compiled method to the receiver's
 method dictionary and returns nil.
 
 If errors occur, returns an Array of pairs.  The first element of each pair is
 the GemStone error number, and the second element is the offset into the
 sourceString where the error occurred.

 This method differs from compileMethod:dictionaries:category:environmentId: 
 in that it traps all errors (not just compiler errors).  
 Non-compiler errors are reported with a source offset of 0."

  self _validatePrivilege ifFalse:[ ^ nil ].
  ^ [ self compileMethod: sourceString dictionaries: aSymbolList
           category: aCategoryString environmentId: envId  .
      nil
    ] onException: { CompileError , Error } do:
      { [:ex | { ex gsArguments at: 1 }  "a CompileError" ] .
        [:ex | { { ex number . 0 } }  "any other Error" ]
      }
     
%

category: 'Accessing Variables'
method:
_instVarNames

"Returns the receiver's instance variables list.  Contrast with the public
 method #allInstVarNames."

^ instVarNames
%

category: 'Accessing the Method Dictionary'
method:
_methodWithSource: aString

"Returns the environment 0 method from the receivers' method dictionary 
 whose source string is equal to aString , or nil if not found."

self env: 0 methodsDo:[ :aSelector :aMethod |
   aMethod sourceString = aString ifTrue:[ ^ aMethod ].
].
^ nil
%

! _recategorize: deleted

category: 'Instance Creation'
method:
basicNew

"Creates a new, uninitialized instance of the receiver."

^self _basicNew
%

category: 'Instance Creation'
method:
basicNew: anInteger

"Creates a new, uninitialized instance of the receiver with the given
 number of indexed instance variables."

^self _basicNew: anInteger
%

category: 'Category'
method:
category

"Returns the classCategory instance variable of the receiver.  If the receiver's
 category is nil, returns its superclass's category."

| categ supercls |
categ := self _classCategory .
supercls := self superClass .
categ == nil ifTrue: [
  supercls == Object ifTrue:[ ^'User Classes' ].
  supercls == nil ifTrue: [ ^'Kernel' ].
  ^ supercls category
].
^ categ
%

category: 'Private'
method:
_classCategory

"Private."

"The classCategory instance variable is defined in Class, so return nil here."

^ nil
%

category: 'Browsing'
method:
copyMethodsFrom: sourceClass dictionaries: dicts

"Copies all instance and class methods from the sourceClass.  Returns an Array
 of methods in the source class which failed to compile in this class.  Some of
 them might be class methods.  The Array is empty if no methods failed to
 compile."

| failed srccls targcls |
self == sourceClass ifTrue:[
  "because iteration is directly over the source's categories dictionaries"
  ^ self error: 'source of copy must not be self' 
].
self _validatePrivilege ifFalse:[ ^ nil ].
failed := { } .

"Copy class and instance methods"
1 to: 2 do: [ :i | | envId |
  i == 1 ifTrue:[ srccls := sourceClass.  targcls := self ] 
        ifFalse:[ srccls := sourceClass class.  targcls := self class ].
  envId := 0 .
  srccls categorysDo:[ :cat :sels |
     sels do: [ :sel | | oldMeth |
        [
          targcls compileMethod: (oldMeth := srccls sourceCodeAt: sel environmentId: envId)
                    dictionaries: dicts category: cat environmentId: envId .
        ] onException: CompileError do:[:ex |
          failed add: oldMeth 
        ].
     ].
  ].
].

^failed.
%

category: 'Fileout'
method:
fileOutCategories

"Returns a string with all the receiver's methods in Topaz Filein format."

| str |

str := String new.
self fileOutCategoriesOn: (WriteStream on: str).
^str
%

category: 'Fileout'
method:
nameForFileout

"Returns the name to be used for this class for fileout."

| pair nm |
pair := System myUserProfile dictionaryAndSymbolOf: self.
nm := pair == nil ifTrue: [ self thisClass name ] ifFalse: [ pair at: 2 ].
^nm
%

category: 'Fileout'
method:
fileOutCategoriesOn: stream

"Writes the receiver's categories and methods onto the given stream
 in Topaz filein format."

| lf cls cats nm |

cls := self isMeta ifTrue: [self thisClass] ifFalse: [self].
lf := Character lf.
stream nextPut: lf.
nm := self nameForFileout.

self fileOutMethodRemovalOn: stream name: nm.

self fileOutPreMethodsOn: stream.

stream nextPutAll: '! ------------------- Class methods for '; nextPutAll: nm; nextPut: lf.
cats := SortedCollection new.
cats addAll: (cls class categoryNames).
cats do: [:category |
  cls class fileOutCategory: category on: stream
].

stream nextPutAll: '! ------------------- Instance methods for '; nextPutAll: nm; nextPut: lf.
cats := SortedCollection new.
cats addAll: (cls categoryNames).
cats do: [:category |
  cls fileOutCategory: category on: stream
].

self fileOutPostMethodsOn: stream.

^stream
%

category: 'Fileout'
method:
fileOutCategory: catName

"Returns a string containing the methods of the given category in
 Topaz Filein format."

| str |

str := String new.
self fileOutCategory: catName on: (WriteStream on: str).
^str
%

category: 'Fileout'
method:
fileOutCategory: catName on: stream

"Files out the given category on the given stream."

| hdr sels |

hdr := 'category: ''' , catName , $' , Character lf.
sels := self sortedSelectorsIn: catName.
sels do: [:selector |
  stream nextPutAll: hdr.
  self fileOutMethod: selector on: stream
]
%

category: 'Fileout'
method:
fileOutClass

"Returns a string with the receiver's class definition and all the
 receiver's methods in Topaz Filein format."

| str |

str := String new.
self fileOutClassOn: (WriteStream on: str).
^str
%

category: 'Fileout'
method:
fileOutClassByCategoryOn: stream

"Writes the receiver's definition and methods onto the given stream in
 filein format."

self fileOutPreClassOn: stream.
stream nextPutAll: 'doit'; nextPut: Character lf;
      nextPutAll: self definition; nextPut: Character lf;
      nextPut: $%; nextPut: Character lf.
self fileOutCategoriesOn: stream.
%

category: 'Fileout'
method:
fileOutClassOn: stream

"Writes the receiver's definition and methods onto the given stream in
 filein format."

self fileOutPreClassOn: stream.
stream nextPutAll: 'doit'; nextPut: Character lf;
      nextPutAll: self definition; nextPut: Character lf;
      nextPut: $%; nextPut: Character lf.
self fileOutMethodsOn: stream.
%

category: 'Accessing the Method Dictionary'
method:
categoryOfSelector: aSelector
 ^ self categoryOfSelector: aSelector environmentId: 0
%
method:
categoryOfSelector: aSelector environmentId: envId

"Returns a Symbol which is the name of the category for the specified
 selector, or nil if the selector was not found in any category."

| aSymbol |
aSymbol := Symbol _existingWithAll: aSelector .
aSymbol ifNil:[ ^ nil ].
self env: envId categorysDo:[ :categName :selectors |
  (selectors includesIdentical: aSymbol) ifTrue:[ ^ categName ]
].
^ nil
%

category: 'Fileout'
method:
fileOutMethod: selector
  ^ self fileOutMethod: selector environmentId: 0
%
method:
fileOutMethod: selector environmentId: envId

"Returns a string with the given method's category and source in Topaz
 Filein format."

| str cat strm |
cat := self categoryOfSelector: selector environmentId: envId.
cat ifNil:[ 
  ^ self _error: #assocErrNoElementsDetected 
].
str := String new.
strm := WriteStream on: str .
strm nextPutAll: 'category: '''; nextPutAll: cat; nextPut: $'; nextPut: Character lf.
self fileOutMethod: selector on: strm  .
^ str
%

method:
fileOutMethod: selector on: stream
 ^ self fileOutMethod: selector environmentId: 0 on: stream 
%

method:
fileOutMethod: selector environmentId: envId on: stream

"Writes the given method's source to the given stream in Topaz Filein format."

| lf src |

lf := Character lf.
stream nextPutAll:'set compile_env: '; nextPutAll: envId asString ; nextPut: lf ;
  nextPutAll: (self isMeta ifTrue: [ 'classmethod: ' ]
              ifFalse: [ 'method: ' ]) ;
  nextPutAll: self nameForFileout; nextPut: lf .
src := self sourceCodeAt: selector environmentId: envId .
stream nextPutAll: src.
src last == lf ifFalse: [
  stream nextPut: lf
].
stream nextPutAll: '%
'
%

category: 'Fileout'
method:
fileOutMethodRemovalOn: stream name: nm

"Writes code to remove all the receiver's methods onto the given stream
 in filein format."

stream nextPutAll: '! Remove existing behavior from ';
       nextPutAll: nm;
       nextPutAll: '
doit
';
       nextPutAll: nm;
       nextPutAll: ' removeAllMethods.
';
       nextPutAll: nm;
       nextPutAll: ' class removeAllMethods.
';
       nextPutAll: '%
'.
%

category: 'Fileout'
method:
fileOutMethods

"Returns a string with all the receiver's methods in Topaz Filein format."

| str lf cls |

cls := self isMeta ifTrue: [self thisClass] ifFalse: [self].
lf := Character lf.
str := String new.
self fileOutMethodsOn: (WriteStream on: str).
^str
%

category: 'Fileout'
method:
fileOutMethodsOn: stream

"File out this class's methods, but sort the selectors alphabetically."

| lf cls sels sel clamps cat nm |

stream == nil ifTrue: [^self].
cls := self thisClass.
nm := self nameForFileout.
lf := Character lf.
stream nextPut: lf.

self fileOutMethodRemovalOn: stream name: nm.

self fileOutPreMethodsOn: stream.

clamps := false.
stream nextPutAll: '! ------------------- Class methods for '; nextPutAll: nm; nextPut: lf.
sels := SortedCollection withAll: (cls class selectors).
1 to: sels size do: [:i |
  sel := sels at: i.
  stream nextPutAll: 'category: '''; nextPutAll: (cls class _categoryOfSelector: sel);
      nextPut: $'; nextPut: lf.
  cls class fileOutMethod: sel on: stream.
  (clamps not and: [sel == #clampedInstVars]) ifTrue: [
    clamps := true
  ].
].

stream nextPutAll: '! ------------------- Instance methods for '; nextPutAll: nm; nextPut: lf.
sels := SortedCollection withAll: (cls selectors).
1 to: sels size do: [:i |
  sel := sels at: i.
  stream nextPutAll: 'category: '''; nextPutAll: (cls _categoryOfSelector: sel);
      nextPut: $'; nextPut: lf.
  cls fileOutMethod: sel on: stream
].

"Special check for #clampedInstVars."
clamps ifTrue:
  [
  stream nextPutAll: 'doit
';
          nextPutAll: nm;
          nextPutAll: ' compileClampedInstVars
';
    nextPut: $%;
    nextPut: lf.
  ].

"File out the category name"
cat := self _classCategory .
cat ~~ nil ifTrue:
  [
  stream nextPutAll: 'doit
';
          nextPutAll: nm;
          nextPutAll: ' category: ';
          nextPutAll: cat quoted;
          nextPut: lf; nextPut: $%; nextPut: lf.
  ].

self fileOutPostMethodsOn: stream.

^stream
%

category: 'Fileout'
method:
fileOutPostMethodsOn: stream

"This method gives classes an opportunity to file out information not
 normally emitted by the predefined fileout methods.  Classes may override
 this method (as a class method, of course) to add extra code to the
 output stream.  Emitted code should be in Topaz filein format.  It will
 be placed after method creation."

%

category: 'Fileout'
method:
fileOutPreClassOn: stream

"This method gives classes an opportunity to file out information not
 normally emitted by the predefined fileout methods.  Classes may override
 this method (as a class method, of course) to add extra code to the
 output stream.  Emitted code should be in Topaz filein format.  It will
 be placed before any other fileout information for the class."

%

category: 'Fileout'
method:
fileOutPreMethodsOn: stream

"This method gives classes an opportunity to file out information not
 normally emitted by the predefined fileout methods.  Classes may override
 this method (as a class method, of course) to add extra code to the
 output stream.  Emitted code should be in Topaz filein format.  It will
 be placed after existing method removal and before method creation."

%

category: 'Accessing the Class Format'
method:
isBytesOrSpecial

"Returns whether instances of the receiver are byte objects."

^ (format bitAnd: 1) == 1
%

method:
instancesNonPersistent

 "Returns true if instances may not be committed.
  See also makeInstancesNonPersistent ."

^ (format bitAnd: 16r800) ~~ 0  
%

method:
instancesDbTransient

  "Return true if the class format has the DbTransient bit set.
   See also  makeInstancesDbTransient ."

^ (format bitAnd: 16r1000) ~~ 0
%

method:
_instancesNpDbtransient

  ^ (format bitAnd: 16r1800)  "used by _become:fullChecks:  methods"
%

category: 'Accessing the Class Format'
method:
selfCanBeSpecial

  "Return true if the class format has the SelfCanBeSpecial bit set"

^ (format bitAnd: 16r2000) ~~ 0
%

category: 'Accessing the Class Format'
method:
isVariable

"Returns true if instances of the receiver have an unnamed part."

self isIndexable ifTrue: [^true].
self isNsc ifTrue: [^true].
^false
%

category: 'Accessing Variables'
method:
offsetOfInstVar: aSymbol

"Returns the integer offset at which the instance variable named aString is
 stored in instances of the receiver.  Returns zero if the instance variable
 is not found."

^ instVarNames indexOfIdentical: aSymbol.
%

category: 'Analysis'
method:
referencedStrings

"Returns a Set containing all Strings and InvariantStrings referenced by
 the environment 0 methods in this Class and its metaclass."

| set blk |

set := IdentitySet new.
blk := [ :selector :method | method _referencedStringsInto: set ].
self env: 0 methodsDo: blk .
self class env: 0 methodsDo: blk .
^set.
%

category: 'Browsing'
method:
removeSelector: aString ifAbsent: aBlock
  ^ self removeSelector: aString environmentId: 0 ifAbsent: aBlock
%

method:
removeSelector: aString environmentId: envId ifAbsent: aBlock
"Removes the method whose selector is aString from the receiver's
 method dictionary.  If the selector is not in the method
 dictionary, returns the result of evaluating the
 zero-argument block aBlock.  Otherwise, returns the receiver."

| aKey meth |

self _validatePrivilege ifFalse:[ ^ nil ].
aKey := aString asSymbol .
meth := self compiledMethodAt: aKey environmentId: envId otherwise: nil .
meth ifNil:[ ^ aBlock value ].
self removeSelector: aKey environmentId: envId
%

category: 'Accessing Categories'
method:
selectorsIn: categoryName
  ^ self selectorsIn: categoryName environmentId: 0
%
method:
selectorsIn: categoryName environmentId: envId

"Returns an Array of all selectors in the specified category.  If categoryName
 is not in the receiver's method dictionary, generates an error."

| set count catSym |
catSym := Symbol _existingWithAll: categoryName .
catSym ifNil:[ ^ self _categoryNotFound:  categoryName ].
count := 0 .
set := IdentitySet new .
self env: envId categorysDo:[ :categName :selectors |
  categName == catSym ifTrue:[ set addAll: selectors . count := count + 1 ].
].
count == 0 ifTrue:[ ^ self _categoryNotFound:  categoryName ].
^ Array withAll: set 
%

category: 'Accessing Categories'
method:
sortedSelectorsIn: categoryName 
  ^ self sortedSelectorsIn: categoryName environmentId: 0
%
method:
sortedSelectorsIn: categoryName environmentId: envId

"Returns an Array of all selectors in the specified category, sorted
 in ascending order."

 ^ (self selectorsIn: categoryName environmentId: envId) sortAscending:''
%

category: 'Indexing Support'
method:
btreeLeafNodeClass

"Returns the class of BtreeLeafNode to create for an equality index whose last
 object along the path is an instance of the receiver."

| result |
(RangeEqualityIndex isBasicClass: self)
    ifTrue: [ result := self indexManager btreeBasicLeafNodeClass]
    ifFalse: [ result := self indexManager btreeLeafNodeClass ].
^ result
%

category: 'Indexing Support'
method:
rcBtreeLeafNodeClass

"Returns the class of BtreeLeafNode to create for an equality index whose last
 object along the path is an instance of the receiver."

| result |
(RangeEqualityIndex isBasicClass: self)
    ifTrue: [ result := self indexManager rcBtreeBasicLeafNodeClass]
    ifFalse: [ result := self indexManager rcBtreeLeafNodeClass ].
^ result
%

category: 'Indexing Support'
method:
sortNodeClass

"Returns the class of SortNode to create for sorting on instances of the
 receiver."

| result |
(RangeEqualityIndex isBasicClass: self)
    ifTrue: [ result := self indexManager sortNodeClass ]
    ifFalse: [ result := SortNode ].
^ result
%

category: 'Stripping Sources'
method:
_hideSourceCode

"For each environment 0 method defined for this class, hide the source code.  All that
 remains of the source is the method signature and the initial comment if one
 exists."

self _validatePrivilege ifFalse:[ ^ nil ].
self env: 0 methodsDo:[ :selector :method | method _removeAllSourceButFirstComment ]
%

! deleted _addMethod:inCategory:
! deleted _refreshCodeCache:newMethod:

category: 'Private'
method:
_noStrippedMethodSelectors

"Returns an Array of two empty Arrays, signifying that no methods are to have
 their source stripped."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  result := #( #() #() ).
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Private'
method:
_allStrippedMethodSelectors

"Returns an Array of two Arrays, signifying that all methods of the receiver
 are to have their source stripped."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  result := { self selectors . self class selectors }.
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Private'
method:
_stripAllMethodSources

"Returns true if all method sources should be stripped for the receiver, and
 false otherwise."

"Returns true for the list of classes given in the code, false for others."

| classNameList |
" classNameList is an Array of Symbols because some of these classes 
are not known at filein "
classNameList := #( ).

1 to: classNameList size do: [ :i |
  ((Globals at: (classNameList at: i)) == self)
    ifTrue: [ ^ true ]
].
^ false
%

! fix bug 11618
category: 'Reloading Decompiled Methods'
method:
_resolveClassVar: aSymbol 

"Searches the receiver's class variables dictionary, to attempt to resolve
 aSymbol.  Returns the SymbolAssociation for the variable with name aSymbol, or
 nil if aSymbol could not be found."

| assoc |

classVars size ~~ 0 ifTrue:[
  assoc := classVars associationAt: aSymbol otherwise: nil. 
  ].
^ assoc
%

category: 'Reloading Decompiled Methods'
method:
_resolveClassOrPoolVar: aSymbol

"Searches the receiver's class variables dictionary and pool dictionaries to
 attempt to resolve aSymbol.  Returns the SymbolAssociation for the variable
 with name aSymbol, or nil if aSymbol could not be found."

| assoc |
assoc := self _resolveClassVar: aSymbol .
assoc ~~ nil ifTrue:[ ^ assoc ].

poolDictionaries size ~~ 0 ifTrue:[
  1 to: poolDictionaries size do:[:j | | aDict |
    aDict := poolDictionaries at: j .
    assoc := aDict associationAt: aSymbol otherwise: nil .
    assoc ~~ nil ifTrue:[ ^ assoc ].
    ]. 
  ].
^ nil
%

! fix bug 11618
category: 'Reloading Decompiled Methods'
method:
_resolveLiteralVar: aSymbol

"Attempts to resolve a literal variable with name aSymbol.  To attempt to
 resolve aSymbol, searches the receiver's class variable dictionary and its pool
 dictionaries, then the superclasses' class variables, then the current default
 SymbolList.

 Returns the SymbolAssociation for the variable or nil if aSymbol could not be
 found."

"Implementation here must agree with comgen.c:searchClassOrPool()"

| assoc aClass |

aClass := self .
"search receiver's class variables and pool variables"
assoc := aClass _resolveClassOrPoolVar: aSymbol . 
assoc ~~ nil ifTrue:[ ^ assoc ].

"search for a class variable inherited from superclasses"
[ (aClass := aClass superClass) ~~ nil 
 ]  
whileTrue: [
  assoc := aClass _resolveClassVar: aSymbol . 
  assoc ~~ nil ifTrue:[ ^ assoc ].
] .
"search the symbol list"
^ GsSession currentSession resolveSymbol: aSymbol
%

! deleted convRecompileWith:

category: 'Private'
method:
_isKernel

"Private.  Returns true if the given class is a GemStone kernel class."

<primitive: 480>
self _primitiveFailed: #_isKernel
%

category: 'Private'
method:
_validatePrivilege

^ System myUserProfile _validateCodeModificationPrivilege
%

category: 'Repository Conversion'
method:
recompileAllMethods

"Recompile all environment 0 methods for execution in a Gs64 v3.0 or later system."

| meths |
meths := { } .
self env: 0 methodsDo:[ :selector :aMethod | meths add: aMethod ].
1 to: meths size do:[:j | (meths at: j) recompile ].
%

method:
needsRecompileFor30

 "Returns true if the receiver needs to have methods recompiled."

  methDicts _isArray ifTrue:[ ^ false ].
  ^ methDicts valueConstraint ~~ GsNMethod
%


category: 'Pragmas'
method:
pragmasForMethod: selector
        "Need to install SessionMethod support to access Pragmas"

        ^#()
%

!-------------------------------------------------------
!  following methods are replaced in behavio_2.gs 

method:
compiledMethodAt: aSelector environmentId: envId otherwise: notFoundVal
  | sym pmd md meth |
  sym := Symbol _existingWithAll: aSelector.
  sym ifNil:[ ^ notFoundVal ].
  md := self transientMethodDictForEnv: envId .
  md ifNotNil:[
    meth := md at: aSelector otherwise: nil .
    meth ifNotNil:[ ^ meth ].
  ].
  pmd := self persistentMethodDictForEnv: envId .
  meth := notFoundVal .
  pmd ifNotNil:[
    meth := pmd at: aSelector otherwise: notFoundVal .
  ].
  ^ meth
%

method:
includesSelector: aString environmentId: envId

"Returns true if the receiver defines a method for responding to aString."
 | sym | 
  sym := Symbol _existingWithAll: aString .
^ (self compiledMethodAt: sym environmentId: envId otherwise: nil) ~~ nil 
%

method:
whichClassIncludesSelector: aString environmentId: envId

"behavio_2.gs installs real implementation"

  | currClass aSymbol |
  aSymbol := Symbol _existingWithAll: aString .
  aSymbol == nil ifTrue:[ ^ nil ].

  currClass := self.
  [ currClass == nil ] whileFalse:[ | meth |
    (currClass includesSelector: aSymbol) ifTrue:[ ^ currClass ].
    currClass := currClass superClass
  ].
  ^ nil
%

category: 'Updating the Method Dictionary'
method:
removeAllMethods

"Removes all methods from receiver's environment 0 .  
 This should not be done without considerable forethought! ."

 "behavio_2.gs contains alternate implementation."
| dict cats envId |
self _validatePrivilege ifFalse:[ ^ nil ].
envId := 0 .
cats := self _baseCategorys: envId .
cats ifNotNil:[ cats removeAll ].
dict := self persistentMethodDictForEnv: envId .
dict ifNotNil:[
  dict removeAll .
  dict valueConstraint: GsNMethod . "to handle v2.3 to v3.0 image upgrade"
].
self _clearLookupCaches: envId
%

category: 'Updating Categories'
method:
removeCategory: categoryName 
  ^ self removeCategory: categoryName environmentId: 0
%

method:
removeCategory: categoryName environmentId: envId

"Removes the specified category and all its methods from the receiver's
 method dictionary.  If categoryName is not in the receiver's categorys
 generates an error.  Any breakpoints in removed methods are cleared."

"behavio_2.gs installs real implementation"

  | cats |
  self _validatePrivilege ifFalse:[ ^ nil ].
  cats := self _baseCategorys: envId .
  cats ifNotNil:[ | catName selectors |
    selectors := cats at: (catName := categoryName asSymbol) .
    selectors do:[ :aSelector |
       self _basicRemoveSelector: aSelector environmentId: envId
    ]. 
    cats removeKey: catName  
  ]  ifNil:[
    self _error: #rtErrKeyNotFound args: { categoryName }
  ]
%

category: 'Modifying Classes'
method:
recompileAllSubclassMethodsInContext: aSymbolList

"behavio_2.gs installs real implementation"

^ self
%

method:
recompileAllMethodsInContext: aSymbolList

"behavio_2.gs installs real implementation"

^ self
%

category: 'Private'
method: 
_instVarsEqual: anArrayOfInstvarNames

  "Return true if the argument matches the instVarNames
   defined by the receiver (excluding inherited instVars), false otherwise."

  ^ (Array withAll: self instVarNames) = (anArrayOfInstvarNames collect:[:n | n asSymbol ])
%

