!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: behavio_2.gs 26868 2011-09-22 20:05:59Z otisa $
!

!------------- session methods support for Behavior ------------------
set class Behavior

category: 'Accessing the Method Dictionary'
method:
_fullMethodDictEnv0

"Returns a dictionary of combined base and override methods."
 | aDict |
 aDict :=  self persistentMethodDictForEnv: 0 .
 aDict :=  aDict ifNil:[ GsMethodDictionary new ] ifNotNil:[ aDict copy ].
 GsPackagePolicy current copyMethodDictFor: self into: aDict .
^ aDict 
%

category: 'Enumerating'

! replaces categorysForEnv:
method:
_unifiedCategorys: envId
  | cats | 
  cats := self _baseCategorys: envId .
  cats ifNil:[ cats := GsMethodDictionary new ] ifNotNil:[ cats copy ].
  envId == 0 ifTrue:[ 
    cats := cats ifNil:[ GsMethodDictionary new ] ifNotNil:[ cats copy ].
    GsPackagePolicy current copyCategoryDictFor: self into: cats .
  ].
  ^ cats
%

method:
env: envId categorysDo: aBlock
  "evaluates aBlock for each method category of receiver
   in specified environment. Returns the receiver.

   aBlock should be a two argument block
   expecting the args  categoryNameSymbol ,   selectorsSet.
   If the package manager is active in the current session,
   aBlock may be invoked more than once for each category name.
   The iteration is done directly over the receiver's categories."

  envId == 0 ifTrue:[ GsPackagePolicy current categorysDo: aBlock for: self ].
  self env: envId baseCategorysDo: aBlock
%

category: 'Updating Categories'
method:
renameOrMergeCategory: oldName to: newName 
  ^ self renameOrMergeCategory: oldName to: newName environmentId: 0
%

category: 'Updating Categories'
method:
renameOrMergeCategory: oldName to: newName environmentId: envId

"Changes the name of the specified category to newName (a String), and
 returns the receiver.  If oldName is not an existing category in the receiver
 generates an error.  If newName is already in the receiver's
 category list, moves all the methods from the old category to the new
 category, and removes the old category."

| oldsym newsym oldCatSet |
self _validatePrivilege ifFalse:[ ^ nil ].
(self includesCategory: oldName environmentId: envId) ifFalse:[
  ^ self _categoryNotFound: oldName 
].
(self includesCategory: newName environmentId: envId) ifFalse:[
  ^ self renameCategory: oldName to: newName environmentId: envId
].
oldsym := oldName asSymbol.
newsym := newName asSymbol.

(envId ~~ 0 or:[ self canWriteMethods])  ifTrue:[ | catDict |
  catDict := self _baseCategorysForStore: envId . 
  oldCatSet := catDict at: oldsym .
  (catDict at: newsym ) addAll: oldCatSet .
  catDict removeKey: oldsym .
] ifFalse:[ |  pkgCats policy |
  pkgCats := GsMethodDictionary new  .
  (policy := GsPackagePolicy current) copyCategoryDictFor: self into: pkgCats .
  oldCatSet := (pkgCats at: oldsym ) copy .
  oldCatSet do:[ :aSelector |
     policy moveSelector: aSelector toCategory: newsym for: self .
  ].
  policy removeCategory: oldsym for: self.
].
self _needsAnnouncement ifTrue:[
  oldCatSet do: [:sel | 
     self _announceMethodMoved: (self compiledMethodAt: sel) oldCategory: oldsym
  ].
].
%

category: 'Updating Categories'
method:
renameCategory: categoryName to: newCategoryName
  ^ self renameCategory: categoryName to: newCategoryName environmentId: 0
%

method:
renameCategory: categoryName to: newCategoryName environmentId: envId

"Changes the name of the specified category to newCategoryName (a
 String), and returns the receiver.  If categoryName is not in the
 receiver's method dictionary, or if newCategoryName is already in the
 receiver's method dictionary, generates an error."

| catSym newCatSym oldCatSet |
self _validatePrivilege ifFalse:[ ^ nil ].
(self includesCategory: newCategoryName environmentId: envId ) ifTrue:[
  ^ self _error: #classErrMethCatExists args: { newCategoryName . self }
].
(self includesCategory: categoryName environmentId: envId ) ifFalse:[
   ^ self _error: #classErrMethCatNotFound args: { categoryName }
].
newCatSym := newCategoryName asSymbol .
catSym := categoryName asSymbol.

(envId ~~ 0 or:[ self canWriteMethods]) ifTrue:[ | catDict|
   catDict :=  self _baseCategorysForStore: envId .
   catDict at: newCatSym put: (oldCatSet := catDict at: catSym ).
   catDict removeKey: catSym .
] ifFalse: [   | pkgCats oldCatSet catSet |
   pkgCats := GsMethodDictionary new  .
   GsPackagePolicy current copyCategoryDictFor: self into: pkgCats .
   oldCatSet := pkgCats at: catSym otherwise: { } .
   catSet := self addCategory: newCatSym environmentId: envId .
   catSet addAll: oldCatSet.
   GsPackagePolicy current removeCategory: catSym for: self.
].
self _needsAnnouncement ifTrue:[
  oldCatSet do: [:sel | 
     self _announceMethodMoved: (self compiledMethodAt: sel) oldCategory: catSym
  ].
].
%

category: 'Updating Categories'
method:
removeCategory: categoryName environmentId: envId

"Removes the specified category and all its methods from the receiver's
 method dictionary.  If categoryName is not in the receiver's categories,
 generates an error.
 Any breakpoints in removed methods are cleared."

| theSelectors catSym allCats |
self _validatePrivilege ifFalse:[ ^ nil ].
(self includesCategory: categoryName environmentId: envId ) ifFalse:[
  ^ self _categoryNotFound: categoryName .
].
catSym := categoryName asSymbol .
allCats := self _unifiedCategorys: envId .
theSelectors := (allCats at: catSym ) copy.
1 to: theSelectors size do: [ :j |
  self _basicRemoveSelector: (theSelectors at: j) environmentId: envId
].
envId == 0 ifTrue:[ 
  GsPackagePolicy current removeCategory: catSym for: self.
  self canBeWritten ifTrue:[
    (self _baseCategorys: envId) removeKey: catSym otherwise: nil .
  ].
] ifFalse: [
  (self _baseCategorys: envId) removeKey: catSym
]. 
self _announceReorganized .
%

category: 'Updating Categories'
method:
addCategory: aString
  ^ self addCategory: aString environmentId: 0
%

method:
addCategory: aString environmentId: envId

"Adds aString as a method category for the receiver.  If aString is already a
 method category, generates an error.
 Returns the newly added category (a SymbolSet)."

| aSymbol canWrite res |
self _validatePrivilege ifFalse:[ ^ nil ].
aSymbol := aString asSymbol .
(self includesCategory: aSymbol environmentId: envId ) ifTrue:[ 
  ^ self _error: #classErrMethCatExists args: { aString . self }
].
canWrite := self canWriteMethods .
(canWrite or:[ envId ~~ 0]) ifTrue:[ | aSet catDict |
  catDict :=  self _baseCategorysForStore: envId .
  catDict at: aSymbol put: ( aSet := SymbolSet new ) .
  res := aSet
] ifFalse: [ 
  res := GsPackagePolicy current addCategory: aSymbol for: self 
].
self _announceReorganized .
^ res
%

category: 'Updating Categories'
method:
moveMethod: aSelector toCategory: categoryName
  ^ self moveMethod: aSelector toCategory: categoryName environmentId:0
%
category: 'Updating Categories'
method: Behavior
moveMethod: aSelector toCategory: categoryName environmentId: envId

"Moves the method aSelector (a String) from its current category to the
 specified category (also a String).  If either aSelector or categoryName is
 not in the receiver's method dictionary, or if aSelector is already in
 categoryName, generates an error..
 This method does not account for selectors inherited from superclasses."

| selSym catSym oldCat |

self _validatePrivilege ifFalse:[ ^ nil ].
(self includesCategory: categoryName environmentId: envId ) ifFalse:[
  ^ self _categoryNotFound: categoryName .
].
catSym := categoryName asSymbol .
oldCat := self categoryOfSelector: aSelector environmentId: envId .
oldCat ifNil:[ ^ self _error: #classErrSelectorNotFound args: { aSelector } ].
selSym := aSelector asSymbol .

(envId ~~ 0 or:[ self canWriteMethods])  ifTrue:[ | catDict |
  catDict := self _baseCategorysForStore: envId .
  (catDict at: oldCat ) remove: selSym .
  (catDict at: catSym) add: selSym .
] ifFalse:[
  GsPackagePolicy current moveSelector: selSym toCategory: catSym for: self
].
self _needsAnnouncement ifTrue:[
  self _announceMethodMoved: (self compiledMethodAt: selSym)  oldCategory: oldCat
].
%

method:
_announceReorganized
  ^ self
%
method:
_needsAnnouncement
 "Result controls sends of  _announceMethodMoved:oldCategory: .
  This implementation replaced when Seaside or other GUI tools installed. "

  ^ false
%

category: 'Browser Methods'
method:
_selectorWithSource: aString

"Returns the selector for the environment 0 method 
 whose source string is identical to aString."

self _fullMethodDictEnv0 keysAndValuesDo:[:aSelector :aMethod |
  (aMethod _sourceString == aString) ifTrue: [^ aSelector ]
].
^nil
%

method:
compiledMethodAt: aSelector environmentId: envId otherwise: notFoundVal usePackages: usePkg

"Returns the compiled method associated with the argument aSelector (a String).
 The argument must be a selector in the receiver's method dictionary; if it is
 not, this method returns notFoundVal.
 If usePkg is true, looks in GsPackagePolicy, then in persistent method dicts;
 else looks in session method dicts then in persistent method dicts"

  <primitive: 2001>  "enter protected mode"
  | prot |
  prot := System _protectedMode .
  ^ [ | aSym meth md cls fmt |
      (aSym := Symbol _existingWithAll: aSelector) ifNotNil:[
        cls := self . 
        fmt := format .
        [ cls ~~ nil ] whileTrue:[
          (envId == 0 and:[ usePkg]) ifTrue:[
            meth := GsPackagePolicy current compiledMethodAt: aSym for: cls.
          ] ifFalse:[
            (md := cls transientMethodDictForEnv: envId) ifNotNil:[
              meth := md at: aSym otherwise: notFoundVal .
            ] 
          ].
          meth ifNil:[
            (md := cls persistentMethodDictForEnv: envId ) ifNotNil:[
              meth := md at: aSym otherwise: notFoundVal .
            ].
          ].
          meth ifNotNil:[ cls := nil ] 
                ifNil:[ (fmt bitAnd: 16r14000) ~~ 0 
                           ifTrue:[ cls := primaryCopy "RUBY_VIRTUAL|MODULE_inclSelf" ]
                           ifFalse:[ cls := nil ]].
          fmt := 0 .
        ].
      ].
      meth ifNil:[ notFoundVal]
    ] ensure:[
      prot _leaveProtectedMode
    ].
%

method:
compiledMethodAt: aSelector environmentId: envId otherwise: notFoundVal
  ^ self compiledMethodAt: aSelector environmentId: envId otherwise: notFoundVal 
         usePackages: true
%

method:
compiledMethodAt: aSelector environmentId: envId usePackages: usePkg
| aMeth |
aMeth := self compiledMethodAt: aSelector environmentId: envId otherwise: nil
		usePackages: usePkg .
aMeth == nil ifTrue:[ self _error: #rtErrKeyNotFound args: { aSelector } ].
^ aMeth
%

method:
includesSelector: aString environmentId: envId

^ (self compiledMethodAt: aString environmentId: envId otherwise: nil
	usePackages: true) ~~ nil
%


category: 'Accessing the Method Dictionary'
method:
selectorsForEnvironment: envId usePackages: usePkg

"Returns an Array of Symbols, consisting of all of the message selectors
 defined by the receiver.  (Selectors inherited from superclasses are not
 included.)  For keyword messages, the Symbol includes each of the keywords,
 concatenated together.  
 If usePkg is true, result consists of persistent methods
 plus GsPackagePolicy methods ; if false  persistent methods plus
 session methods."

 <primitive: 2001>  "enter protected mode"
 | set prot |
 prot := System _protectedMode .
 [ | aDict |
   set := IdentitySet new.
   (aDict := self persistentMethodDictForEnv: envId ) ifNotNil:[
     set addAll: aDict keys.
   ].
   (envId == 0 and:[ usePkg]) ifTrue:[
     GsPackagePolicy current selectorsFor: self into: set.
   ] ifFalse:[
     (aDict := self transientMethodDictForEnv: envId) ifNotNil:[
       set addAll: aDict keys.
     ].
   ].
 ] ensure:[
   prot _leaveProtectedMode
 ].
 ^ set asArray .
%

method:
selectorsForEnvironment: envId
  ^ self selectorsForEnvironment: envId usePackages: true
%

method:
whichClassIncludesSelector: aString environmentId: envId usePackages: usePkg

"If the selector aString is in the receiver's method dictionary,
 returns the receiver.  Otherwise, returns the most immediate superclass
 of the receiver where aString is found as a message selector.  Returns
 nil if the selector is not in the method dictionary of the receiver or
 any of its superclasses.
 If usePkg is true, result consists of persistent methods
 plus GsPackagePolicy methods ; if false  persistent methods plus
 session methods."

  | currClass aSymbol |
  aSymbol := Symbol _existingWithAll: aString .
  aSymbol ifNil:[ ^ nil ].

  currClass := self.
  envId == 1 ifTrue:[
    [ currClass == nil ] whileFalse:[ | mDict |
      (currClass compiledMethodAt: aSymbol environmentId: envId otherwise: nil 
                  usePackages: usePkg) ifNotNil:[
         ^currClass
      ].
      "include virtual classes for ruby modules. "
      currClass := currClass rubySuperclass: envId 
    ].
  ] ifFalse:[
    [ currClass == nil ] whileFalse:[ | mDict |
      (currClass compiledMethodAt: aSymbol environmentId: envId otherwise: nil 
              usePackages: usePkg) ifNotNil:[ ^ currClass ].
      currClass := currClass superClass
    ].
  ].
  ^ nil
%

method:
whichClassIncludesSelector: aString environmentId: envId
  ^ self whichClassIncludesSelector: aString environmentId: envId usePackages: true
%

!-------------------------------------------------------------------
category: 'Class Timestamps'
method:
commentStamp

  "Return the comment timeStamp.
   Both a Class and it's metaclass share a single comment timeStamp"
  | eDict |
  (eDict := self extraDict) ifNil:[ ^ nil ].
  ^ eDict at: #GSClassCommentStamp otherwise: nil
%

method:
commentStamp: aStamp
  "Set the receiver's comment timeStamp to aStamp.
   Both a Class and it's metaclass share a single comment stamp"

  | eDict |
  (eDict := self extraDict) ifNotNil:[
    eDict at: #GSClassCommentStamp put: aStamp
  ].
%

method: 
theNonMetaClass
  "Sent to a class or metaclass, always return the class.
   Used by Monticello"

  ^self
%


!-------------------------------------------------------------------
category: 'Method Timestamps'
method:
authorInitials
    ^GsPackagePolicy current authorInitials
%

method:
changeStamp 
  "Answer a string to be pasted into source code to mark who changed it and when."
  ^ self authorInitials , ' ' , (Date today asStringUsingFormat: #(2 1 3 $/ 1 1 $: false )), ' ',
    (Time now asString copyFrom: 1 to: 5)
%

!-------------------------------------------------------------------
category: 'Method Timestamps'
method:
methodStampDictName
  ^ #GSMethodStampDict
%

method:
setStamp: aStamp forMethod: selector
  "Set the timestamp for the given method. 
   This code shared by Classes and metaclasses, 
   parameterized by methodStampDictName."

  | methodStampDict eDict selSym |
  selSym := Symbol _existingWithAll: selector  .
  selSym ifNil:[ self error:'invalid selector'].
  (GsPackagePolicy current 
        setStamp: aStamp forBehavior: self forMethod: selSym ) ifNotNil:[
    ^self 
  ].

  (eDict := self extraDict) ifNotNil:[  | dictName |
    dictName := self methodStampDictName .
    methodStampDict := eDict at: dictName otherwise: nil .
    methodStampDict ifNil: [ 
      GsObjectSecurityPolicy setCurrent: self objectSecurityPolicy while:[
        methodStampDict := IdentityKeyValueDictionary new.
      ].
      eDict at: dictName put: methodStampDict.
    ].
    aStamp ifNil: [ methodStampDict removeKey: selSym otherwise: nil ]
        ifNotNil: [ methodStampDict at: selSym put: aStamp ]
  ].
%

method:
stampForMethod: selector
  "Get the timestamp for the given method"

  | stamp methodStampDict eDict selSym |
  selSym := Symbol _existingWithAll: selector  .
  selSym ifNil:[ ^ '' ].
  stamp := GsPackagePolicy current stampForMethod: selSym in: self.
  stamp ifNotNil: [ ^stamp ].

  (eDict := self extraDict) ifNil:[ ^'' ]. 
  methodStampDict := eDict at: self methodStampDictName ifAbsent: [ ^'' ].
  ^ methodStampDict at: selSym ifAbsent: [ '' ]
%
!-------------------------------------------------------------------
category: 'Pragmas'

method:
pragmaDictName

  ^ #GSMethodPragmaDict
%
method:
createPragmaFrom: aPragmaArray for: aGsMethod
    "Placeholder for creating an instance of Pragma"
    ^nil
%

method:
pragmasForMethod: selector
  "Get the Pragmas for the given method
   This code shared by Classes and metaclasses, 
   parameterized by pragmaDictName."

  | pragmas methodPragmaDict eDict selSym |
  selSym := Symbol _existingWithAll: selector  .
  selSym ifNil:[ ^ #() ] .
  pragmas := GsPackagePolicy current pragmasForMethod: selSym in: self.
  pragmas ifNotNil: [ ^pragmas ].

  (eDict := self extraDict) ifNil:[ ^#() ].
  methodPragmaDict := eDict at: self pragmaDictName ifAbsent: [ ^#() ].
  ^methodPragmaDict at: selSym ifAbsent: [ #() ]
%

category: 'Pragmas'
method:
setPragmas: pragmaArray forMethod: selector
  "Set the pragmas for the given method .
   This code shared by Classes and metaclasses, 
   parameterized by pragmaDictName."

  | methodPragmaDict eDict selSym |
  selSym := Symbol _existingWithAll: selector  .
  selSym ifNil:[ self error:'invalid selector'].
  (GsPackagePolicy current
      setPragmas: pragmaArray forBehavior: self forMethod: selSym ) ifNotNil:[
     ^self 
  ].

  (eDict := self extraDict) ifNotNil:[ | dictName |
     dictName := self pragmaDictName .
     methodPragmaDict := eDict at: dictName otherwise: nil .
     methodPragmaDict ifNil: [ 
       GsObjectSecurityPolicy setCurrent: self objectSecurityPolicy while:[
         methodPragmaDict := IdentityKeyValueDictionary new.
       ].
       eDict at: dictName put: methodPragmaDict.
     ].
     pragmaArray ifNil:[ methodPragmaDict removeKey: selSym otherwise: nil  ]
         ifNotNil: [ methodPragmaDict at: selSym put: pragmaArray ]
  ].
%

!-------------------------------------------------------------------
category: 'Updating the Method Dictionary'
method:
removeSelector: aString environmentId: envId

"Removes the method whose selector is aString from the receiver's method
 dictionary.  If the selector is not in the method dictionary, generates an
 error."

| selector removedFromBaseDict |

self _validatePrivilege ifFalse:[ ^ nil ].
selector := aString asSymbol .

removedFromBaseDict := self _basicRemoveSelector: selector environmentId: envId.
removedFromBaseDict ifTrue:[ | cDict |
    cDict := self _baseCategorys: envId .
    cDict keysAndValuesDo: [:aKey :setOfSelectors |
        (setOfSelectors remove: selector otherwise: nil ) ifNotNil:[ 
           ^ self "done"
        ]
     ].
    "if we get here , we could not find the selector in any category "
    ^ self _error: #classErrSelectorNotFound args: { aString } .
]
%

method:
__basicRemoveSelector: aSymbol environmentId: envId
"returns  true if a method found to remove"
| md pmd removed tmeth pmeth |
self _validatePrivilege ifFalse:[ ^ nil ].
md := self transientMethodDictForEnv: envId .
removed := false .
md ifNotNil:[
  tmeth := md removeKey: aSymbol otherwise: nil .
].
pmd := self persistentMethodDictForEnv: envId .
pmd ifNotNil:[
  pmeth := pmd removeKey: aSymbol otherwise: nil .
].
"Now refresh method lookup caches to account for removal of the selector
 and delete any breakpoints in the removed method(s) ."
tmeth ifNotNil:[
  self _refreshLookupCache: aSymbol oldMethod: tmeth env: envId .
  removed := true
].
pmeth ifNotNil:[
  removed := true .
  self _codeChangedForEnv: envId .
  self _refreshLookupCache: aSymbol oldMethod: pmeth env: envId
].
^ removed
%

method:
_basicRemoveSelector: aSymbol environmentId: envId

"Private."  
| oldMeth removedFromSelf |
self _validatePrivilege ifFalse:[ ^ nil ].
removedFromSelf := false.
envId == 0 ifTrue:[
  oldMeth := GsPackagePolicy current removeMethodAt: aSymbol for: self .
  oldMeth ifNil: [
    oldMeth := (self persistentMethodDictForEnv: 0) removeKey: aSymbol . "runtime error here if key not found"
    self setStamp: nil forMethod: aSymbol.
    self setPragmas: nil forMethod: aSymbol.
    removedFromSelf := true.
  ].
  self _refreshLookupCache: aSymbol oldMethod: oldMeth env: envId
] ifFalse:[
  removedFromSelf := self __basicRemoveSelector: aSymbol environmentId: envId .
].
^removedFromSelf
%

method:
removeAllMethods

"Removes all methods from the receiver.  This should not be done without
 considerable forethought!"
| baseCats baseMeths envId |
self _validatePrivilege ifFalse:[ ^ nil ].
envId := 0 .
baseCats := self _baseCategorys: envId .
baseCats ifNotNil: [ baseCats removeAll ].
baseMeths := self persistentMethodDictForEnv: envId .
baseMeths ifNotNil: [ 
  baseMeths keysDo: [:sel | 
    self setStamp: nil forMethod: sel.
    self setPragmas: nil forMethod: sel 
  ].
  baseMeths removeAll .
  baseMeths valueConstraint: GsNMethod . "to handle v2.3 to v3.0 image upgrade"
].
GsPackagePolicy current removeAllMethodsFor: self.
self _clearLookupCaches: envId .
%

!-------------------------------------------------------------------
category: 'Modifying Classes'
method:
recompileAllMethodsInContext: aSymbolList

"Recompiles all methods for the receiver, using the specified symbol list.

 The environmentId of each method is preserved by the recompilation.
 
 This method is designed to allow a user interface to issue GciContinue after
 fixing the source code for a method in error.  GciContinue will reattempt the
 compilation of the method which contained an error, then proceed to the next
 method."
 
| envId |
self _validatePrivilege ifFalse:[ ^ nil ].
envId := 0 .
self env: envId unifiedCategoriesDo:[ :theCategory :selectorList |
    selectorList do: [ :aSelector| | aMeth err |
      [ "This Block supports GciContinue"
        "Make sure the method is there before trying to recompile it.
         An exception handler may have removed the method!"
        err := nil .
        aMeth := self compiledMethodAt: aSelector environmentId: envId otherwise: nil .
        aMeth ifNotNil:[ | theSource mEnvId |
          theSource := aMeth sourceString .
          (mEnvId := aMeth environmentId) == envId ifFalse:[
             self error:'mismatched environmentId'
          ].
          [ | stamp |
            envId == 0 ifTrue:[ stamp := self stampForMethod: aSelector].
            self compileMethod: theSource dictionaries: aSymbolList
                          category: theCategory environmentId: envId .
            envId == 0 ifTrue:[ self setStamp: stamp forMethod: aSelector].
          ] onException: CompileError do:[:ex |
            err := ex .
            ex outer .
          ]
        ].
        err == nil 
      ] untilTrue .
    ] .
].
^ self
%

method:
_removeAllSubclassCode

"Dereference the code objects of all environment 0 GsMethods of the receiver,
 to force recompilation of those methods."

| theSubclasses aDict envId |
self _validatePrivilege ifFalse:[ ^ nil ].
envId := 0 .
theSubclasses := self _subclasses .
theSubclasses ifNotNil:[
  theSubclasses do:[:x| x _removeAllSubclassCode ] .
  ].
GsPackagePolicy current removeAllSubclassCodeFor: self. "envId 0 only"
aDict := self persistentMethodDictForEnv: envId .
aDict ifNotNil:[
  aDict keysAndValuesDo:[ :aKey :aMethod |
    aDict at: aKey put: ((aDict at: aKey) _copyToForceRecompilation).
  ].
  self _codeChangedForEnv: envId  .
].
self _refreshClassCache: true .
^ self
%

method:
recompileAllSubclassMethodsInContext: aSymbolList

"Recompiles all methods for the receiver and its subclasses, using
 the specified symbol list.  If the receiver is not modifiable,
 then methods in subclasses will not be recompiled, since only
 modifiable classes should have the Subclasses class variable present."

| theSubclasses |
self _validatePrivilege ifFalse:[ ^ nil ].
self _removeAllSubclassCode .
self recompileAllMethodsInContext: aSymbolList .
theSubclasses := self _subclasses .
theSubclasses ifNotNil:[
  theSubclasses do:[:x | x recompileAllSubclassMethodsInContext: aSymbolList ]
  ].
^ self
%
!-------------------------------------------------------------------
category: 'Testing
method:
canWriteMethods

  ^ self canWriteMethodsEnv: 0
%

method:
canWriteMethodsEnv: envId

  | mds dict |
  (mds := methDicts) _isArray ifTrue:[
    dict := mds atOrNil: (envId*4 + 1) .  
    dict ifNotNil:[ ^ dict canBeWritten ].
    ^ mds canBeWritten
  ] ifFalse:[
    mds ifNotNil:[ 
      envId == 0 ifTrue:[ ^ mds canBeWritten ] 
    ].
  ].
  ^ self canBeWritten
%


! removeRubySelector: moved to .mcz
! big deletion

category: 'Accessing the Method Dictionary'
method:
methodDictForEnv: envId usePackages: usePkg
 | dict md |
 (md := self persistentMethodDictForEnv: envId ) ifNotNil:[
   dict := md copy 
 ].
 dict ifNil:[ dict := GsMethodDictionary new ].
 (envId == 0 and:[ usePkg]) ifTrue:[
   GsPackagePolicy current copyMethodDictFor: self into: dict 
 ] ifFalse:[
   (md := self transientMethodDictForEnv: envId ) ifNotNil:[
     dict addAll: md 
   ].
 ].
 ^ dict
%

method:
methodDictForEnv: envId
  ^ self methodDictForEnv: envId usePackages: true
%

