!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: bomlastconv.gs 26918 2011-09-27 23:54:19Z lalmarod $
!
! Description - 
!   This file contains the subset of bomlast.gs used for repository
!   conversion. (bomlast.gs is not used for conversion).
!=========================================================================

! Fix up methods that would have contained forward reference to PrintStream.
!  PrintStream requires execution to create class; execution not
!  possible until filein2 stage, but we need Object>>printString in 
!  filein1 in order to make topaz execution work ... 
category: 'Formatting'
method: Object
printString

"Returns a String whose contents are a displayable representation of the
 receiver."

"This method uses the printOn: method to create the String."

| ws str |

str := String new.

ws := PrintStream printingOn: str.  

self printOn: ws.
^ ws _collection "contents might have been converted to an EUCString."
%


! compile methods that contain forward references to Float if they are compiled
! earlier in the filein when building a fresh GemStone repository.

category: 'Converting'
method: Integer
asDecimalFloat

"Returns a DecimalFloat representing the receiver."

<primitive: 137>

| factor sum oldRoundingMode radix |

oldRoundingMode := Float roundingMode.  "nearest even is only right way"
Float roundingMode: 'nearestEven'.

sum := 0.0F0 .
radix := 4294967295.0F0 .
factor := self sign asFloat.
1 to: self size do: [:i |
  sum := (self _digitAt: i) * factor + sum.
  factor := factor * radix 
].
DecimalFloat roundingMode: oldRoundingMode.
^sum
%

category: 'Truncation and Rounding'
method: Number
ceiling

"Returns the integer that is closest to the receiver, on the same side
 of the receiver as positive infinity.  In particular, returns the
 receiver if the receiver is an integer."

(self <= 0.0)
   ifTrue: [^self truncated]
   ifFalse: [^self negated floor negated truncated]
%

category: 'Arithmetic'
method: Number
degreesToRadians

"Assuming the receiver represents an angle in degrees, returns the angle
 in radians."

^ self / 57.29577951308232
%

category: 'Arithmetic'
method: Number
radiansToDegrees

"Assuming the receiver represents an angle in radians, returns the angle
 in degrees."

^ self * 57.29577951308232
%

method: GsMethodDictionary
rebuildTable: newSize

"Rebuilds the method dictionary by populating a larger method dictionary
 first and doing a (primitive) become:"

"This is a reimplementation of the method in object.gs to handle
instances that have a dependency list."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  | newGsMethodDict depList |

  tableSize = newSize ifTrue:[ 
    ^ self "no change in table size" 
  ].
  collisionLimit == 536870911 ifTrue:[
    ^ self 		"avoid recursive rebuild"
  ].

  newGsMethodDict := self class new: (newSize * 2).
  newGsMethodDict valueConstraint: valueConstraint.
  newGsMethodDict keyConstraint: keyConstraint.
  
  self keysAndValuesDo: [ :aKey :aValue |
    newGsMethodDict at: aKey put: aValue.
    ].
  
  (depList := DependencyList for: self) ~~ nil
    ifTrue: [ DependencyList set: nil for: self ].
  
  newGsMethodDict _becomeDictionary: self.
  
  depList ~~ nil
    ifTrue: [ DependencyList set: depList for: self ].
] ensure:[
  prot _leaveProtectedMode
]
%

expectvalue %String
run
  "Per ANSI 5.6.8  , additional names for Float, to fix bug 40876"
| report |
report := String new .
{ #FloatE . #FloatD . #FloatQ } do:[:k |
  (Globals includesKey: k) ifFalse:[
     Globals at: k put: Float .
     (Globals associationAt: k) immediateInvariant .
     report add: 'added ' ; add: k ; lf .
  ].
].
report
%

!--------------------------------------

category: 'Repository Conversion'
method: Class
_unsafeDisallowSubclasses

""

self _unsafeAt: 2 "offset of format"
           put: (format bitOr: 32 "subclasses disallowed bit") .
self _refreshClassCache: false
%

category: 'Repository Conversion'
method: Class
_unsafeMakeClassNp

""

self _unsafeAt: 2 "offset of format"
           put: (format bitOr: 16r800"instancesNP bit" ) .
self _refreshClassCache: false
%

!--------------------------------------
run
| list |
  list := { Behavior . Class . ObsoleteMetaclass . Repository . UserProfileSet . 
     AbstractUserProfileSet . 
     CompiledMethod . Float . DecimalFloat . SmallFloat . SmallDouble . 
     Symbol . DoubleByteSymbol . 
     UnimplementedFloat1 . UnimplementedFloat2 . 
     MethodContext . Block . SelectionBlock . 
     GsObjectSecurityPolicy . SymbolList . System . UndefinedObject . UserProfile . Boolean . 
     Character . SmallInteger . Integer . LargeInteger . 
     JISCharacter . StackSegment . 
     StackBuffer . Activation . Process . VariableContext . BlockClosure . 

     ExecBlock . ExecutableBlock . SimpleBlock . ComplexBlock . ComplexVCBlock . 
     GsMethod . GsNMethod . GsNativeCode . 
     GsProcess . GsMethodDictionary . GsStackBuffer . 

     SelectBlock . BtreeBasicInteriorNode . BtreeBasicLeafNode . BasicSortNode . 
     RcIndexBucket . RcIndexBucketWithCache . RcIndexDictionary . NscBuilder . 

     PathTerm . ConstrainedPathTerm . SetValuedPathTerm  "fix bug 11484"
     }.

  list do: [ :aClass | aClass _unsafeDisallowSubclasses ] .

  list do: [ :aClass |
     aClass subclassesDisallowed ifFalse:[ ^ 'Error on ' , aClass name ].
     ] .
  ^ true
%
run
Semaphore _unsafeMakeClassNp .  "set instancesNonPersistent"
ProcessorScheduler _unsafeMakeClassNp .  "set instancesNonPersistent"
^ true
%

run
Class removeSelector: #_unsafeDisallowSubclasses .
Class removeSelector: #_unsafeMakeClassNp .
^ true
%

run
| list blk |

" allow adding instance variables to the following classes.  fixes bug 6569"
list := { UserProfile } .

blk := [ :aClass |
          aClass _unsafeAt: 4 "instVarNames"
                 put: (Array withAll: (aClass instVarAt: 4)) .
          aClass _unsafeAt: 5 "constraints"
                     put: (Array withAll: (aClass instVarAt: 5)) .
          aClass _refreshClassCache: false
       ].
list do: blk .
^ true
%

! deleted "Change the constraints on VariableContext..." , deletes fix 15424


!--------------------------------------

!--------------------------------------
! clear "conversion in progress flag"   
expectvalue false
run
ConversionStatus at: 1 put: false
%

!--------------------------------------
! Create class documentation for kernel classes that are only placeholders.

category: 'For Documentation Installation only'
classmethod: System
installPlaceholderDocumentation

| str |

str :=
' is a placeholder for another related GemStone product that has not currently
  been installed.' .

" list of placeholder classes for other products "
{ GsfClassDefinitionInfo . GsfModificationLog }
  do: [ :aClass |
    | doc txt |
    doc := GsClassDocumentation _newForPrivateGsClass: aClass.
    txt := (GsDocText new) details:
           ((String withAll: (aClass name)) add: str; yourself).
    doc documentClassWith: txt.
    aClass description: doc.
  ].

" list of placeholder classes within the server "
{ UnimplementedFloat1 . UnimplementedFloat2 }
  do: [ :aClass |
    | doc |
    doc := GsClassDocumentation _newForPrivateGsClass: aClass.
    doc documentClassWith: (GsDocText new).
    aClass description: doc.
  ].
%


!--------------------------------------
! Install class documentation for kernel classes only.

run

| classes |

System installPlaceholderDocumentation.

classes := ClassSet new.
(Globals values) do: [ :each |
  ( (each isKindOf: Class)
      and:[(each class includesSelector: #installDocumentation)] )
  ifTrue: [ classes add: each.  ].
  ].

classes do: [ :aClass | 
  aClass installDocumentation .
  GsFile gciLogServer:'installed doc for ', aClass name  . 
] .
classes do: [ :aClass |
  (aClass class) removeCategory: 'For Documentation Installation only'.
  ].
^true
%
commit

!------------------------------------------------------------
! Make various methods in the indexing subsystem 'protected'
!  now done via compilation directives during method compilation
!------------------------------------------------------------

!------------------------------------------------------------
! Make various classes in the indexing subsystem 'protected'.
! This prevents accessing an instance through structural access
! 31162: also protect classes for code security.
!------------------------------------------------------------
!
run
" list of classes which will be protected "
{ GsObjectSecurityPolicySet . 
   SetValuedPathTerm . 
   SetValuedPathEvaluator . UserSecurityData . 
"31162"
   GsMethod . Class . GsMethodDictionary . SymbolDictionary . SymbolList . 
   UserProfile
 }
    do: [ :aClass | aClass _makeProtected ].

^ true
%
lev 1
obj AbstractCharacter
lev 0
commit

!------------------------------------------------------------
! Build the dictionary of configuration file parameters
!------------------------------------------------------------
run
Globals at:#ConfigurationParameterDict put:
(System _configFileParameterDict) .
^ true
%
!------------------------------------------------------------
! Build the dictionary of software version information parameters
!------------------------------------------------------------
run
"dictionary to translate names to internal identifiers"
Globals at:#VersionParameterDict put:
(System _versionParameterDict) .
^ true
%
run
"dictionary of image version information"
Globals at:#ImageVersion put: SymbolDictionary new .
^ true
%
input $upgradeDir/version.topaz

! build the set of classes for which instances are disallowed
run
Globals at:#InstancesDisallowed put:
 ( IdentitySet withAll:
  { Activation . Character . Behavior . Block . BlockClosure . Boolean . 
      Class . GsMethod . ComplexBlock . ComplexVCBlock . DateTime . 
      ExecBlock . ExecutableBlock . Float . DecimalFloat . SmallFloat . BinaryFloat . 
      Integer . JISCharacter . 
      LargeInteger . ObsoleteMetaclass . MethodContext . Number . Process . 
      Repository . GsObjectSecurityPolicy . SelectBlock . SelectionBlock . SimpleBlock . 
      SmallInteger . StackBuffer . StackSegment . System . UndefinedObject . 
      UserProfile . VariableContext }) .
^ true
%
commit

!------------------------------------------------------------
! Create collating tables for raw ASCII collating order for use with
! String compare methods  in category 'Other comparisons'
!------------------------------------------------------------
run
| a |
a := String new .
0 to: 255 do:[:j| a add: (Character withValue: j )].
Globals at:#AsciiCollatingTable put: a .
a immediateInvariant .
^ a size == 256
%
run
| a |
a := DoubleByteString new .
0 to: 255 do:[:j| a add: (Character withValue: j )].
Globals at:#DoubleByteAsciiCollatingTable put: a .
a immediateInvariant .
^ a size == 256
%
run
| a |
a := QuadByteString new .
0 to: 255 do:[:j| a add: (Character withValue: j )].
Globals at:#QuadByteAsciiCollatingTable put: a .
a immediateInvariant .
^ a size == 256
%

!-----------
!  ensure bom has created the old canonicalized float zeros

! old oop 1873
expectvalue %Float
obj @239617

! old oop 1875
expectvalue %SmallFloat
obj @239873


! reset weak refs array 
expectvalue true
run
Globals at: #GcWeakReferences put: Array new.
^ true
%

! indexing  not supported, so clear SharedDependencyLists
expectvalue true
run
Globals at: #SharedDependencyLists put: DepListTable new .
^true
%

!-------------
! removal of _unsafesetoop method done later in filein


!------------------------------------------------------------
! fixes 31329 , 31540 - Remove references to Geode
!------------------------------------------------------------

expectvalue %String
run
" remove references to geode without creating Symbols unnecessarily"
| result geodeKeys globalsKeys removeList report |
result := String new .
geodeKeys := { 'Geode' . 'GeodeGlobals' . 'GeodeLauncher' . 
		'GeodeMaint' . 'GeODEDbfHistory' . 'ObsoleteGeode' }.
globalsKeys := Globals keys . "collection of Symbols"
removeList := Array new .
geodeKeys do:[ :aGeod |
  globalsKeys do:[ :aGlb | 
     aGlb asString = aGeod ifTrue:[ removeList add: aGeod ]
  ].
].
report := String new .
removeList do:[ :aKey |
  Globals removeKey: aKey asSymbol .
  report add: ' removed ' , aKey , ' ; ' .
  ].
^ report .
%


!------------------------------------------------------------
! Fix 31942, disallow gci store to instances of SymbolList .
! If customers have created subclasses of SymbolList , those are not
! changed .
expectvalue true
run
SymbolList _disallowGciCreateStore .
^ SymbolList _structuralUpdatesDisallowed
%

commit

expectvalue true
run
 "Check that certain kernel classes have only one version defined.
  The virtual machine contains optimizations which assume that
  the classes checked below have only one version, and thus the
  virtual machine can skip looking at the classHistory when doing
  certain isKindOf checks on arguments to primitives.  
  See fix 32339"

| clsList |
clsList := { AbstractCharacter . 
  Array . 
  ExecBlock . 
  ExecutableBlock . 
  GsNMethod . 
  GsProcess . 
  GsSocket . 
  IdentityBag . 
  Interval . 
  SequenceableCollection . 
  SmallFloat . 
  SmallDouble . 
  VariableContext  }.

clsList do:[ :aCls | | hist val sz |
  hist := aCls classHistory .
  sz := hist size .
  sz == 1 ifFalse:[ 
    nil error:'classHistory for ' , aCls name , ' too big' 
  ].
  val := hist at: 1 .
  val == aCls ifFalse:[
    nil error:'classHistory for ' , aCls name , ' is corrupt'
  ].
].   
"primitives for Float do not check classHistory, but explicitly
 check arguments for both SmallDouble and Float ."
true ifTrue:[ | hist val sz |
  hist := Float classHistory .
  sz := hist size .
  sz == 1 ifFalse:[
    nil error: 'classHistory for Float has wrong size.'
  ].
  val := hist at: 1 .
  val == Float ifFalse:[
    nil error:'classHistory for Float, bad value at:1' . 
  ].
].
^ true
%

expectvalue %String
run
| dcSeg |
dcSeg := Globals at:#DataCuratorObjectSecurityPolicy .
AllUsers objectSecurityPolicy == dcSeg ifTrue:[
  ^ 'objectSecurityPolicy ok'
] ifFalse:[
  AllUsers objectSecurityPolicy: dcSeg .
  AllUsers do:[ :aUp | 
    aUp objectSecurityPolicy == dcSeg ifFalse:[ aUp objectSecurityPolicy: dcSeg].
  ].
  ^ 'changed objectSecurityPolicy'
]
%
lev 1
obj AbstractCharacter
lev 0

expectvalue %String
run
	"ensure Association for FdcResults is in DataCurator objectSecurityPolicy"
| res dcSeg |
dcSeg := Globals at:#DataCuratorObjectSecurityPolicy .
(Globals includesKey:#FdcResults) ifTrue:[
  (Globals associationAt:#FdcResults) objectSecurityPolicy: dcSeg .
  res := 'changed objectSecurityPolicy of FdcResults' .
] ifFalse:[
  dcSeg setCurrentWhile:[
    Globals at:#FdcResults put: nil .
    res := 'created FdcResults' . 
  ]
].
^ res 
%

! =====================================================================
! adjust existing UserProfiles to grant new inverse privileges
run
| priv setBlk clearBlk inversePrivs newPrivs LF |
System myUserProfile userId = 'SystemUser' ifFalse:[ nil error:'wrong user'].
LF := Character lf .
newPrivs := #( #SessionPriority ) .
inversePrivs := #( #NoPerformOnServer #NoUserAction
             #NoGsFileOnServer #NoGsFileOnClient ) .
setBlk := [ :aUp :anArray :name | 
  anArray do:[ :aPriv | aUp addPrivilege: aPriv ] .
  GsFile gciLogClient:'set ', name , ' priv bits for ' , aUp userId , LF .
].
clearBlk := [ :aUp :anArray :name | 
  anArray do:[ :aPriv | aUp deletePrivilege: aPriv] .
  GsFile gciLogClient:'cleared ', name, ' priv bits for ' , aUp userId , LF .
].
AllUsers do:[ :aUp | | uid |
  uid := aUp userId .
  (UserProfile isSpecialUserId: uid) ifTrue:[
    (uid = 'SystemUser' or:[ uid = 'DataCurator']) ifTrue:[
      clearBlk value: aUp value: inversePrivs value: 'inverse' .
      setBlk value: aUp value: newPrivs value: 'new' .
    ] ifFalse:[  "gcuser, symboluser, nameless"
      setBlk value: aUp value: inversePrivs value: 'inverse' . 
    ].
  ] ifFalse:[
    "preexisting customer user profiles not affected. 
     If customers have blindly set all priv bits those Up's need adjustment.
    "
  ].
]. 
^ true
%


commit

! 38848: Add read-only support if needed
category: 'Repository Conversion'
method: UserProfile
_enableCommits

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _securityDataInstVarAt: 8 put: false.
] ensure:[
  prot _leaveProtectedMode
].
^ self
%

expectvalue %String
run
| usd result |
usd := Globals at: #UserSecurityData.
usd allInstVarNames size == 7
  ifTrue:[ "first add the inst var to the class"
    usd _instVarNames _unsafeAt: 8 put: #readOnly .
    usd _constraints  _unsafeAt: 8 put: Boolean .
    usd _unsafeAt: 3 put: 8 .
    "now fix the instances"
    AllUsers do:[:eachUserProfile| eachUserProfile _enableCommits].
    result := 'All User Profiles were updated'.
  ]
  ifFalse:[ result := 'No updates to UserProfiles were required.'].
"delete of unsafe method"
UserProfile removeSelector: #_enableCommits.
^result
%

expectvalue %String
run
Globals at: #GemStoneRCLock ifAbsent:[
  Globals at: #GemStoneRCLock put: Object new .
  ^ 'created'
].
^ 'found'
%

!------------------------------------------------------------
! Remove behavior from obsolete classes if repository originated
!  on Gs64 v2.3 or prior.  bom.c code run in first conversion session
!  has already renamed them !  to Obsolete*  if they existed.
!------------------------------------------------------------
expectvalue %String
run
 | names report eol |
 names := { 
  'ObsoleteExceptionA' . 
  'ObsoleteError' . 
  'ObsoleteMessageNotUnderstood' . 
  'ObsoleteZeroDivide' . 
  'ObsoleteNotification' . 
  'ObsoleteWarning' . 
  'ObsoleteTestFailure' . 
  'ObsoleteResumableTestFailure' . 
  'ObsoleteExceptionHandler' . 
  'ObsoleteExceptionSet' . 
  'ObsoleteFailedMessage' }.
 "Obsolete subclasses of  Number handled in respective files previous to here"
 report := String new .
 eol := Character lf .
 names do:[ :aName | | aSym aCls |
   (aSym := Symbol _existingWithAll:aName) ~~ nil ifTrue:[
     aCls := Globals at: aSym otherwise: nil .
     aCls ~~ nil ifTrue:[
       aCls removeAllMethods .
       aCls class removeAllMethods .
       report addAll: aName , ' removed all methods' .
     ] ifFalse:[
       report addAll: aName , ' not found in Globals'.
     ].
   ] ifFalse:[
     report addAll: aName , ' symbol does not exist'.
   ].
   report add: eol .
 ].
^ report 
%

expectvalue %String
run
  "For classes with no  classmethod with selector #comment ,
   if they have a description, generate a comment method
   from it."
| classes epoch lf emptySymList none found created errs |
System currentObjectSecurityPolicy: SystemObjectSecurityPolicy .
classes := Array new .
lf := Character lf .
none := 0 . found := 0 . created := 0 . errs := 0 .
emptySymList := SymbolList new .
Globals valuesDo:[ :aValue|
  (aValue isKindOfClass: Class) ifTrue:[ classes add: aValue].
].
classes do:[ :aClass | | meta |
  meta := aClass class .
  (meta includesSelector: #comment ) ifTrue:[
     "GsFile gciLogServer: '    ', aClass name,  ' already has comment method'."
     found := found + 1 .
  ] ifFalse:[ | str src desc |
     str := aClass description asString .
     str = 'nil' ifTrue:[
       ((aClass isSubclassOf: AbstractException) or:[
       (aClass isSubclassOf: GsCompilerIRNode) or:[
       (aClass isSubclassOf: ExecBlock)]]) ifFalse:[
         GsFile gciLogServer: aClass name , '  no documentation found '.
         none := none + 1 .
       ].
     ] ifFalse:[ | cm err |
       (src := 'comment' copy ) add: lf ; add: '^ '; add: $' ; 
         add: ( str copyReplaceAll: '''' with: '''''' ). "double up single quotes"
       src last == lf ifFalse:[ src add: lf ].
       src add: $' .
       err := meta compileMethod: src dictionaries: emptySymList category: 'Documentation' .
       err ifNotNil:[ 
         GsFile gciLogServer:'error compiling ' , aClass name , '(C)>>comment' .
          errs := errs + 1 .
       ] ifNil:[
         created := created + 1 .
       ]
     ].
  ]
].
errs > 0 ifTrue:[ Exception signal:'compilation errors' ].
^ ' none ', none asString, ' found ', found asString ,
   ' created ', created asString
%

expectvalue %String
run
  "Move GsCompilerIRNode and subclasses to a GsCompilerClasses dictionary"
  | dict |
  (Globals includesKey: #GsCompilerClasses) ifTrue:[
     (Globals includesKey: #GsCompilerIRNode)
       ifTrue:[ Globals removeKey: #GsCompilerIRNode ].
     ^ 'GsCompilerClasses already exists'
  ].
  dict := SymbolDictionary new .
  Globals associationsDo:[ : assoc | | val |
    val := assoc _value .
    (val isBehavior and:[ val isSubclassOf: GsCompilerIRNode ]) ifTrue:[ dict addAssociation: assoc ].
  ]. 
  dict associationsDo:[ : assoc | Globals removeAssociation: assoc ].
  dict name: #GsCompilerClasses .
  Globals at:#GsCompilerClasses put: dict .
  ^ 'created GsCompilerClasses dictionary'
%

expectvalue %String
run
  "Move selected obsolete classes to ObsoleteClasses, to fix 40985"
  | rpt obsG |
  obsG := Globals at: #ObsoleteClasses otherwise: nil .
  obsG ifNotNil:[
      Globals removeAllKeys: obsG keys .
    ^ 'ObsoleteClasses already exists'
  ].

  rpt := String new .
  obsG := SymbolDictionary new .
  Globals at:#ObsoleteClasses put: obsG . 
  #( Block
     MethodContext
     Obsolete23ClampSpecification
     Obsolete23GsFile
     ObsoleteClampSpecification
     ObsoleteDateTime
     ObsoleteDictionary
     ObsoleteGsFile
     ObsoleteIdentityCollisionBucket
     ObsoleteIdentityDictionary
     ObsoleteLanguageDictionary
     ObsoleteRcCollisionBucket
     ObsoleteSymbol
     ObsoleteSymbolAssociation
     ObsoleteSymbolDictionary
     ObsoleteSymbolKeyValueDictionary
     ObsoleteSymbolListDictionary
     ObsoleteSymbolSet
     ObsoleteVariableContext
     ReenterBlock
     SelectionBlock
  ) do:[:sym | | assoc |
     assoc := Globals associationAt: sym otherwise: nil .
     assoc ifNotNil:[
       obsG addAssociation: assoc .
       Globals removeKey: sym .
       rpt addAll: 'moved ' , sym ; lf .
     ].
  ].
  ^ rpt
%

commit

! end of bomlastconv.gs
