!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: eucstring.gs 26882 2011-09-23 22:25:27Z lalmarod $
!
! Superclass Hierarchy:
!   EUCString, JapaneseString, CharacterCollection, SequenceableCollection,
!   Collection, Object.
!
!=========================================================================

removeallmethods EUCString
removeallclassmethods EUCString

category: 'For Documentation Installation only'
classmethod: EUCString
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'Deprecated in 3.0 and later, replaced by Extended Character Set support
 and DoubleByteStrings/QuadByteStings.

 This class represents Japanese strings in Extended Unix Code format.'.
doc documentClassWith: txt.

self description: doc.
%

category: 'Formatting'
classmethod: EUCString
charSize

"Returns the number of bytes that make up a character"
^1
%

category: 'Formatting'
method: EUCString
charSize

"Returns the number of bytes that make up a character"
^1
%

category: 'Accessing'
method: EUCString
at: anIndex

"Returns the JISCharacter at the specified index."

<primitive: 244>

(anIndex _isSmallInteger)
ifTrue:[ ^ self _errorIndexOutOfRange: anIndex]
ifFalse:[^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #at: args: { anIndex } .
self _uncontinuableError
%

category: 'Accessing'
method: EUCString
size

"Returns the number of JISCharacters in the receiver."

<primitive: 76>

self _primitiveFailed: #size .
self _uncontinuableError
%

category: 'Updating'
method: EUCString
at: anIndex put: aCharacter

"Stores aCharacter at the specified location."

<primitive: 243>
| charClass |
charClass := aCharacter class .
(charClass == JISCharacter or:[ charClass == Character]) ifFalse:[
  ^ self at: anIndex put: aCharacter asJISCharacter .
  ].
(anIndex _isSmallInteger)
ifTrue:[ ^ self _errorIndexOutOfRange: anIndex]
ifFalse:[^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #at:put: args: { anIndex . aCharacter } .
self _uncontinuableError
%

category: 'Updating'
method: EUCString
size: anInteger

"Changes the size of the receiver to anInteger.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to nil."

<primitive: 245>

(anInteger _isSmallInteger)
ifTrue:[^ self _errorIndexOutOfRange: anInteger]
ifFalse:[^ self _errorNonIntegerIndex: anInteger].

self _primitiveFailed: #size: args: { anInteger } .
self _uncontinuableError
%

category: 'Formatting'
method: EUCString
asEUCString

"Returns the receiver."

"This reimplementation of from JapaneseString is crucial !."

^self
%

! category: 'Formatting'
! method: EUCString
! describe
! 
! "If the receiver is less than half a megabyte, quote it.
!  Otherwise, return the receiver."
! 
! self _basicSize > 500000
!   ifTrue: [ ^self ]
!   ifFalse: [ ^self quoted ]
! %

category: 'Converting'
method: EUCString
asSymbolKind

"Returns a copy of the receiver as an instance of Symbol."

^ self asSymbol
%

category: 'Comparing'
method: EUCString
= aCharCollection

"Returns true if all of the corresponding Characters in the receiver and
 argument are equal.  Returns false otherwise."

<primitive: 77>

^ super = aCharCollection
%

category: 'Comparing'
method: EUCString
< aCharCollection

"Returns true if the receiver collates before the argument.  Returns false
 otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) == -1]
  ifFalse: [ ^super < aCharCollection ]
%

category: 'Comparing'
method: EUCString
<= aCharCollection

"Returns true if the receiver collates before the argument or if all of the
 corresponding Characters in the receiver and argument are equal.  Returns
 false otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) <= 0]
  ifFalse: [ ^super <= aCharCollection ]
%

category: 'Comparing'
method: EUCString
> aCharCollection

"Returns true if the receiver collates after the argument.  Returns false
 otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) == 1]
  ifFalse: [ ^super > aCharCollection ]
%

category: 'Comparing'
method: EUCString
>= aCharCollection

"Returns true if the receiver collates after the argument or if all of the
 corresponding Characters in the receiver and arguments are equal.  Returns
 false otherwise."

((aCharCollection isKindOf: EUCString) or: [ aCharCollection isKindOf: String])
  ifTrue: [ ^(self _EUCcompare: aCharCollection) >= 0]
  ifFalse: [ ^super >= aCharCollection ]
%

category: 'Comparing'
method: EUCString
_EUCcompare: aCharCollection

"Returns -1 if self < aCharCollection, returns 0 if self = aCharCollection,
 and returns 1 if self > aCharCollection."

<primitive: 78>

^ self _primitiveFailed: #_EUCcompare: args: { aCharCollection }
%

! _hashCaseSensitive: deleted

! deleted copyFrom:to:into:startingAt: prim 246

category: 'Searching'
method: EUCString
indexOf: aCharacter startingAt: startIndex

"Returns the index of the first occurrence of aCharacter in the receiver,
 not preceding startIndex.  If the receiver does not contain aCharacter,
 this returns zero."

<primitive: 79>

^super indexOf: aCharacter startingAt: startIndex
%

category: 'Adding'
method: EUCString
add: aCharOrCharCollection

"Appends aCharOrCharCollection to the receiver.  The argument
 aCharOrCharCollection must be a CharacterCollection or a kind of
 AbstractCharacter."

<primitive: 242>

^ super add: aCharOrCharCollection
%

category: 'Adding'
method: EUCString
addAll: aCharOrCharCollection

"Equivalent to add: aCharOrCharCollection."

<primitive: 242>

^ super addAll: aCharOrCharCollection
%

category: 'Adding'
method: EUCString
addLast: aCharOrCharCollection

"Equivalent to add: aCharOrCharCollection."

<primitive: 242>

^ super addLast: aCharOrCharCollection
%

category: 'Adding'
method: EUCString
insertAll: aCharOrCharCollection at: anIndex

"Inserts aCharOrCharCollection at the specified index."

<primitive: 241>

anIndex _validateClass: SmallInteger.
(anIndex > (self size + 1))
ifTrue:[ ^ self _errorIndexOutOfRange: anIndex]
ifFalse:[ ^ super insertAll: aCharOrCharCollection at: anIndex]
%

category: 'Converting'
method: EUCString
asSymbol

"Returns a Symbol that contains the same bytes as the receiver."

^ (DoubleByteString withAll: self) asSymbol
%

category: 'Formatting'
method: EUCString
printString

"Returns an EUCString whose contents are a displayable representation of the
 receiver."

| ws str |

str := EUCString new.
ws := PrintStream printingOn: str.
self printOn: ws.
^str
%

! added trapping of error 2281 so we can get consistent error reports
!  in methtst, etc .
category: 'Formatting'
method: EUCString
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

[
  | idx lastIdx sz strmClass |
  strmClass := aStream _collection class .
  (strmClass == String or:[strmClass == ISOLatin]) ifTrue:[ 
    aStream _convertToEUC
  ].
  self size > 1000 ifTrue:[
    ^ aStream nextPutAll: self .
  ]. 
  aStream nextPut: $' .
  idx := self indexOf: $' startingAt: 1 .
  idx == 0 ifTrue:[
    aStream nextPutAll: self .
  ] ifFalse:[
    lastIdx := 1.
    [ idx == 0 ] whileFalse: [
      aStream nextPutAll: ( self copyFrom: lastIdx to: idx ) .
      aStream nextPut: $' .
      lastIdx := idx + 1 .
      idx := self indexOf: $' startingAt: lastIdx .
    ].
    sz := self size .
    lastIdx <= sz ifTrue: [
      aStream nextPutAll: (self copyFrom: lastIdx to: sz ).
    ].
  ].
  aStream nextPut: $' 
] onException: Error do:[:ex |
  ex number == 2281"RT_ERR_BAD_EUC_FORMAT" ifTrue:[
    aStream nextPutAll:'(corrupt EUCString with oop ' ;
    nextPutAll: (self asOop asString) ; nextPut: $) .
    ^ self
  ].
  ex outer
].
%

category: 'Converting'
method: EUCString
asArrayOfPathTerms

"Returns an Array of path substrings held by the receiver.  The receiver
 is assumed to be a period-separated list of substrings.  These substrings
 are extracted and collected in an Array.  If the receiver contains no
 periods, the Array will hold a copy of the receiver. The backslash character
 is no longer recongnized as an escape character. 

 Raises an error if an element is not a valid path term."

| nextName result period |

result := { } .
nextName := self class new.
period := JISCharacter withValue: ($.  asciiValue).
self do: [:c |
  (c == period) ifTrue: [
    nextName _isValidPathTermName
      ifFalse: [
        ^ self _error: #rtErrInvalidIndexPathExpression
               args: { nextName asSymbol }
    ].
    result add: nextName asSymbol .
    nextName := self class new.
  ]
  ifFalse: [
    nextName add: c
  ].
].
nextName size > 0 ifTrue: [
  nextName _isValidPathTermName
    ifFalse: [
      ^ self _error: #rtErrInvalidIndexPathExpression
             args: { nextName asSymbol }
  ].
  result add: nextName asSymbol
]
ifFalse: [
  result size == 0 ifTrue: [result add: nextName asSymbol ]
].
^result
%

category: 'Indexing Support'
method: EUCString
_isValidPathTermName

"Returns true if the receiver is a valid term in a path expression."

^ (String withAll: self) _isValidPathTermName
%
category: 'Accessing'
method: EUCString
codePointAt: anIndex

^ (self at: anIndex) eucValue
%

category: 'Updating'
method: EUCString
codePointAt: anIndex put: aValue

^ self at: anIndex put:(JISCharacter withEUCValue: aValue)
%

