!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: gspackagelib.gs 25988 2011-06-14 17:55:54Z stever $
!
! Superclass Hierarchy:
!   GsPackageLibrary, Object
!
!=========================================================================

expectvalue %String
run
| oldCls |
oldCls := Globals at:#GsPackageLibrary otherwise: nil .
oldCls == nil ifTrue:[
 Object subclass: 'GsPackageLibrary'
  instVarNames: #()
  classVars: #()
  classInstVars: #()
  poolDictionaries: { }
  inDictionary: Globals .
  ^ 'created class'
] ifFalse:[
  ^ 'existing class definition:  ' + oldCls definition
].
%

! Remove existing behavior from GsPackageLibrary
set class GsPackageLibrary
removeallmethods GsPackageLibrary
removeallclassmethods GsPackageLibrary

! ------------------- Class methods for GsPackageLibrary
category: 'Operations'
classmethod: GsPackageLibrary
uninstallPackageAndPrereqs: aPackage

    "Recursively uninstall the package and it's prereqs"

    | symDict pkg |
    symDict := aPackage symbolDict.
    aPackage prereqs do: [:symbolDict |
        pkg := symbolDict at: GsPackage globalName otherwise: nil.
        pkg ~~ nil ifTrue: [ GsPackageLibrary uninstallPackageAndPrereqs: pkg ].
    ].
    self userSymbolList remove: symDict ifAbsent: [].
    self sessionSymbolList remove: symDict ifAbsent: [].
%
category: 'Operations'
classmethod: GsPackageLibrary
uninstallPackage: aPackage

    self userSymbolList remove: aPackage symbolDict ifAbsent: [].
    self sessionSymbolList remove: aPackage symbolDict ifAbsent: [].
    GsPackagePolicy current refreshSessionMethodDictionary.
%
category: 'Operations'
classmethod: GsPackageLibrary
installPackageNamed: aSymbol

    	^self installPackage: (self packageNamed: aSymbol)
%
category: 'Operations'
classmethod: GsPackageLibrary
installPackage: aPackage

    	self installPackage: aPackage enable: true
%
category: 'Operations'
classmethod: GsPackageLibrary
installPackage: aPackage enable: aBool

    	| symDict pkgDict |
    	symDict := aPackage symbolDict.
	pkgDict := Globals.
    	aPackage prereqs do: [:symbolDict |  | p |
     		p := symbolDict at: aPackage class globalName otherwise: nil.
        	p ~~ nil 
			ifTrue: [ self installPackage: p  enable: false ].
		(self userSymbolList indexOf: symbolDict) < (self userSymbolList indexOf: pkgDict) 
			ifTrue: [ pkgDict := p symbolDict ].
    	].
	(self userSymbolList objectNamed: symDict name) == nil
		ifTrue: [ self userSymbolList add: symDict before: pkgDict ].
	(self sessionSymbolList objectNamed: symDict name) == nil
		ifTrue: [ self  sessionSymbolList add: symDict before: pkgDict ].
	aBool
		ifTrue: [ | policy |
			policy := GsPackagePolicy current.
			policy enable.
			policy homeSymbolDict:  aPackage symbolDict.
			policy externalSymbolList: (GsSession currentSession symbolList asArray copyWithout: aPackage symbolDict).
		].
%
category: 'Operations'
classmethod: GsPackageLibrary
createPackageNamed: aSymbol

    | package packageLibrary |
    package := self packageNamed: aSymbol.
    package ~~ nil ifTrue: [ ^package ].
    packageLibrary := self packageLibrary.
    packageLibrary createDictionaryNamed: aSymbol at: 1.
    ^GsPackage installIn: (packageLibrary objectNamed: aSymbol).
%
category: 'Searching'
classmethod: GsPackageLibrary
packageNamed: aSymbol

    	| symDict |
	symDict := self packageLibrary objectNamed: aSymbol.
	symDict ifNil:[ ^ nil ].
	^symDict at: GsPackage globalName otherwise: nil
%
category: 'Accessing'
classmethod: GsPackageLibrary
packageLibraryUserName

    ^'DataCurator'
%
category: 'Accessing'
classmethod: GsPackageLibrary
packageLibraryName

    ^#PackageLibrary
%
category: 'Accessing'
classmethod: GsPackageLibrary
packageLibrary

    | packageLibrary |
    packageLibrary := (AllUsers userWithId: self packageLibraryUserName ifAbsent: []) symbolList objectNamed: self packageLibraryName.
    packageLibrary == nil
        ifTrue: [
            packageLibrary := SymbolList new.
            self userGlobals at: self packageLibraryName put: packageLibrary.
        ].
    ^packageLibrary
%
category: 'Accessing'
classmethod: GsPackageLibrary
userGlobals

    ^self userSymbolList objectNamed: #UserGlobals
%
category: 'Accessing'
classmethod: GsPackageLibrary
userSymbolList

    ^System myUserProfile symbolList
%
category: 'Accessing'
classmethod: GsPackageLibrary
sessionSymbolList

    ^GsSession currentSession symbolList
%

category: 'Private - Installation Support'
classmethod: GsPackageLibrary
getSeasideDirectory
| basePath |
basePath := System gemEnvironmentVariable: 'SEASIDE'.
basePath == nil
 ifTrue:[
   basePath := System gemEnvironmentVariable: 'GEMSTONE'.
   basePath == nil
     ifTrue:[^nil]
     ifFalse:[basePath addAll: '/seaside/'].
  ]
  ifFalse:[
    basePath last == $/
      ifFalse: [ basePath add: $/ ].
  ].
^basePath
%
category: 'Private - Installation Support'
classmethod: GsPackageLibrary
getMonticelloRepositoryDirectory
| basePath |
basePath := self getSeasideDirectory .
basePath ==  nil
  ifTrue:[ ^nil ] .
basePath addAll: 'monticello/repository/' .
^ basePath
%


category: 'Monticello'
classmethod: GsPackageLibrary
loadMCZFile: mczFilename fromRepositoryPath: repositoryPath
	"load the given mcz file <mczFilename> into the current user's UserGlobals symbol dictionary from
	the repository located in the directory <repositoryPath> on the server."

	self installMonticelloPackagesDuring: [:globalsPackage | |repository |
		repository := (globalsPackage symbolDict at: #MCFilesystemRepository) new
			directory: ((globalsPackage symbolDict at: #FSDiskFilesystem) current pathFromString: repositoryPath) asReference.
		self loadMCZFile: mczFilename fromRepository: repository ].
%

category: 'Monticello'
classmethod: GsPackageLibrary
loadLastVersionOf: packageName fromRepositoryPath: repositoryPath
	"load the latest version of the package <packageName> into the current user's UserGlobals symbol dictionary from
	the repository located in the directory <repositoryPath> on the server."
	
	self installMonticelloPackagesDuring: [:globalsPackage | 
    | directory repository mczFilename |
		directory := ((globalsPackage symbolDict at: #FSDiskFilesystem) current 
										pathFromString: repositoryPath) asReference.
		mczFilename := self lastFileNameOf: packageName inDirectory: directory.

		repository := (globalsPackage symbolDict at: #MCFilesystemRepository) new directory: directory.
		self loadMCZFile: mczFilename fromRepository: repository 
  ]
%

category: 'Monticello'
classmethod: GsPackageLibrary
saveNewVersionOf: packageName author: authorInitials message: commitComment
  toPath: repositoryPath
  "write a new version of the package <packageName> 
   to the repository located in the directory <repositoryPath> on the server."
  self installMonticelloPackagesDuring: [:globalsPackage |
    | directory repository pkg wc version verName |
    directory := ((globalsPackage symbolDict at: #FSDiskFilesystem)
                current pathFromString: repositoryPath) asReference.
    repository := (globalsPackage symbolDict at: #MCFilesystemRepository)
                        new directory: directory.
    GsPackagePolicy current authorInitials: authorInitials .
    pkg := (globalsPackage symbolDict at:#MCPackage) named: packageName .
    wc := pkg workingCopy .
    version := wc newVersionWithName: (verName := wc uniqueVersionName )
                message: commitComment  .
    repository storeVersion: version.
    System commitTransaction ifFalse:[ Error signal:'commit failed' ].
    ^ 'wrote ' , verName asString , ' , ' version description .
  ].
%

category: 'Monticello'
classmethod: GsPackageLibrary
lastFileNameOf: packageName inDirectory: directory
  "returns a String"
  | versionNumberBlock mczFilename |
    versionNumberBlock :=  [:name | | base |
      base := name copyUpToLast: $..
      (base copyFrom: (base lastIndexOf: $.) + 1 to: base size) asNumber
    ].
    mczFilename := ((
       (directory childFiles collect: [:reference | reference basename ])
          select: [:mczName | mczName beginsWith: packageName ])
            sortBy: [:a :b | (versionNumberBlock value: a) <= (versionNumberBlock value: b) ]) 
        last.
   ^ mczFilename
%

category: 'Monticello'
classmethod: GsPackageLibrary
lastFileNameOf: packageName inPath: aPath
  "returns a String"
  self installMonticelloPackagesDuring: [:globalsPackage |
   | directory |
    directory := ((globalsPackage symbolDict at: #FSDiskFilesystem) current 
                     pathFromString: aPath ) asReference.
   ^ self lastFileNameOf: packageName inDirectory: directory
  ].
%


category: 'private'
classmethod: GsPackageLibrary
loadMCZFile: mczFilename fromRepository: repository
	"load the given <mczFilename> from the given GsMonticello repository <repository>"

        GsFile gciLogServer: 'loading ', mczFilename printString.
	(repository loadVersionFromFileNamed: mczFilename) load.
%

category: 'private'
classmethod: GsPackageLibrary
installMonticelloPackagesDuring: aBlock
        "install the libraries #'SessionMethods' and #'Monticell_Methods'.
         Enable #'SessionMethods' and configure for new classes to be installed into UserGlobals.
         Evaluate <aBlock> with the SymbolDictionary named #'Monticello_Globals' as the argument.
         Uninstall the library #'Monticell_Methods', but leave #'SessionMethods' enabled"

	| policy methodsPackage sessionMethodsPackage |
	methodsPackage := self packageNamed: #'Monticello_Methods'.
        sessionMethodsPackage := GsPackageLibrary createPackageNamed: #SessionMethods.

        [ self installPackage: sessionMethodsPackage.
	self installPackage: methodsPackage.

        policy := GsPackagePolicy current.
        policy homeSymbolDict:  (GsCurrentSession currentSession objectNamed: #UserGlobals).
        policy externalSymbolList: Array new.

	aBlock value: (self packageNamed: #'Monticello_Globals') ] ensure: [ self uninstallPackage: methodsPackage ].
%

! ------------------- Instance methods for GsPackageLibrary

!
! force definition of restrictedClasses during filein
!
doit
GsPackagePolicy restrictedClasses.
^true
%
