!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: gssocket.gs 26114 2011-07-08 16:18:18Z stever $
!
! Superclass Hierarchy:
!   GsSocket, IO, Object.
!
!=========================================================================

set class GsSocket
removeallmethods 
removeallclassmethods 

category: 'For Documentation Installation only'
classmethod:
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'GsSocket provides the means for creating and binding TCP sockets through the
 operating system of the machine that is running the session''s Gem process, and
 for communicating across those sockets.  Methods that block GemStone Smalltalk
 until the socket operation completes are interruptable by a hard break.  (You
 can get a hard break in Topaz by pressing the control-C key twice.  You can get
 a hard break in GemBuilder for C by calling the GciHardBreak function, and in
 GemBuilder for Smalltalk by calling the corresponding hard break method.)

 Beginning with Gs64 v3.0,  instances of GsSocket automatically have
 their C state closed when the instance is garbage collected or 
 when a persistent instance drops out of memory.

                              Warning:
    Do not retain an instance of GsSocket from one session to another.
    Instances of GsSocket are intended to exist only within a given GemStone
    session.  GsSockets that are used across sessions always generate an error.

 All instVars of GsSocket are private, for use by the implementation
 of socket methods, and for use by the ProcessorScheduler only.' .
doc documentClassWith: txt.

txt := (GsDocText new) details:
'Methods in this category are obsolete and are provided only for compatibility
 with earlier releases of GemStone.  They will be removed in a future release.'.
doc documentCategory: #'Deprecated' with: txt.

self description: doc.
%

! ------------------- Class methods for GsSocket
category: 'Drastic Measures'
classmethod:
closeAll

"Close all instances of GsSocket that are open."

self _zeroArgClassPrim: 6 .
^ self
%

category: 'Deprecated'
classmethod:
finalizeAll

"Returns the number of instances of kinds of GsSocket that are alive
 and not closed."

self deprecated: 'Obsolete, finalization is automatic in this release.
 Use closeAll to close all instances.'.
^ self _zeroArgClassPrim: 16 
%

category: 'Deprecated'
classmethod:
getServByName: serviceName
	"Returns the port number for the given service.  Returns nil if the service
	is undefined or an error occurs."

self deprecated: 'Obsolete in GemStone/64.  Use the getServicePortByName: method instead.'.
^ self getServicePortByName: serviceName
%

category: 'Queries'
classmethod:
getServicePortByName: serviceName

"Returns the port number for the given service.  Returns nil if the service
 is undefined or an error occurs."

^ self _twoArgClassPrim: 1  with: serviceName with: nil
%

category: 'Queries'
classmethod:
getServicePortByName: serviceName withProtocol: protocol

"Returns the port number for the given service and protocol.
 Returns nil if the service is undefined or an error occurs."

^ self _twoArgClassPrim: 1  with: serviceName with: protocol
%

category: 'Queries'
classmethod:
getServiceNameByPort: servicePort withProtocol: protocol

"Returns the service name for the given port number and protocol.
 Returns nil if the service is undefined or an error occurs."

^ self _twoArgClassPrim: 8 with: servicePort with: protocol
%

category: 'Queries'
classmethod:
getServiceNameByPort: servicePort

"Returns the service name for the given port number.
 Returns nil if the service is undefined or an error occurs."

^ self getServiceNameByPort: servicePort withProtocol: nil
%

category: 'Queries'
classmethod:
getHostNameByAddress: address

"Returns the host name for the given ip address.
 Returns nil if the host is undefined or an error occurs.

 The address needs to be a String in a form acceptable to 
 the inet_addr() network function."

^ self _twoArgClassPrim: 7 with: address with: nil
%

category: 'Queries'
classmethod:
getHostAddressByName: hostname

"Returns the ip address for the given hostname.
 Returns nil if the host is undefined or an error occurs.

 The resuling ip address is a String in a form acceptable to 
 the inet_addr() network function."

^ self _twoArgClassPrim: 12 with: hostname with: nil
%

category: 'Queries'
classmethod:
getLocalHostName

"Returns the name of the local host or nil if an error occurs."

^ self _zeroArgClassPrim: 5
%

category: 'Queries'
classmethod:
isAvailable

"Returns whether the supporting socket actions are available in the
 user's session."

^ true
%

category: 'Queries'
method:
raiseExceptionOnError

"Returns true if a socket error on the receiver will raise an exception.
 Returns false if not. The initial value is false."

^ self _twoArgPrim: 10 with: nil with: nil
%

category: 'Configuration'
method:
raiseExceptionOnError: bool

"Sets the value of raiseExceptionOnError to the Boolean bool for the receiver.
 Returns the receiver."

^ self _twoArgPrim: 10 with: bool with: nil
%

category: 'Instance Creation'
classmethod:
basicNew

"disallowed"

self shouldNotImplement: #basicNew
%

category: 'Private'
classmethod:
_basicNew

"creates an instance registered with VM for finalization of cData"

<primitive: 674>
^ self _primitiveFailed: #_basicNew
%

category: 'Instance Creation'
classmethod:
new

"Returns a new non-blocking TCP socket or nil if unable to create a new socket."

^ self _basicNew _zeroArgPrim: 1
%

classmethod:
newUdp

"Returns a new non-blocking UDP socket or nil if unable to create a new socket."

^ self _basicNew _zeroArgPrim: 26
%

category: 'Copying'
method:
copy

"Returns an instance with no associated C state. 
 There are no instVars to copy"

^ self class _basicNew
%

! GsSocket>>newWithId: deleted

category: 'Low Level Access'
classmethod:
getStandardId: index inband: bool
"Returns the socket id for the given index. Index must be >= 0.
 If bool is true then gets the inband socket id. Otherwise gets
 the outband socket id.

 If index is 0 then the socket id to the stone is returned. Starting with Gs64 v2.2,
 the socket id to the stone will be -1 if the session is using shared memory 
 communication to the stone.

 If index is 1 then the socket id to the rpc application is returned.

 Otherwise index represents the GCI session id (see GciGetSessionId) and
 the socket id to that session is returned.
 If a socket id does not exist for the given index then -1 is returned."

^ self _twoArgClassPrim: 15 with: index with: bool
%

category: 'Accessing'
method:
changeable: aFlag
  "Has no effect in this release "
  ^ self
%

category: 'Accessing'
method:
changeable

"Returns true if methods that would change the state of the receiver
 are allowed. Returns false if the state of the receiver can not be
 changed."

^ true
%

category: 'Error Reporting'
classmethod:
lastErrorString

"Returns a String containing information about the last error for 
 GsSocket class methods, or nil if there is no error.
 Clears the error information for the receiver."

^ self _zeroArgClassPrim: 7
%

category: 'Error Reporting'
classmethod:
lastErrorCode

"Returns an integer representing that last operating system error
 for GsSocket class methods. Returns zero if there is no error.
 Does not clear the error information for the receiver."

^ self _zeroArgClassPrim: 8
%

category: 'Private'
classmethod:
_getErrorSymbol: code

"Returns a symbol from SocketErrorSymbols for the given code.
 Returns nil if no error."

"Note: socketprim.c:raiseError does a recur from om to this method."

(code == 0) ifTrue: [^nil].
(code > (SocketErrorSymbols size)) ifTrue: [^SocketErrorSymbols at: 1].

^SocketErrorSymbols at: code
%

category: 'Error Reporting'
classmethod:
lastErrorSymbol

"Returns a Symbol representing that last operating system error
 for GsSocket class methods. Returns nil if there is no error.
 Does not clear the error information for the receiver."

^ self _getErrorSymbol: (self _zeroArgClassPrim: 19)
%

category: 'Error Reporting'
method:
lastErrorString

"Returns the string of the last error on the receiver, or nil if no error has
 occurred.  Clears the error information for the receiver."

^ self _zeroArgPrim: 10
%

category: 'Error Reporting'
method:
lastErrorCode

"Returns an integer representing the last operating system error on the
 receiver. Returns zero if there is no error.
 Does not clear the error information for the receiver."

 ^ self _zeroArgPrim: 11
%

category: 'Error Reporting'
method:
lastErrorSymbol

"Returns a Symbol representing the last operating system error on the
 receiver. Returns nil if there is no error.
 Does not clear the error information for the receiver."

^ self class _getErrorSymbol: (self _zeroArgPrim: 20)
%

category: 'Private'
classmethod:
_zeroArgClassPrim: opcode

"See GsSocket>>_zeroArgPrim: for a description of the opcode."

<primitive: 323>
^ self _primitiveFailed: #_zeroArgClassPrim: args: { opcode }
%

category: 'Private'
classmethod:
_twoArgClassPrim: opcode with: arg1 with: arg2

"See GsSocket>>_twoArgPrim:with:with: for a description of legal opcodes"
<primitive: 322>

^ self _primitiveFailed: #_twoArgClassPrim:with:with:
       args: { opcode . arg1 . arg2 }
%

category: 'Private'
method:
_zeroArgPrim: opcode

"opcode  function
   1     instance method: SocketCreate
   2     instance method: SocketClose
   3     instance method: SocketPort
   4     class method: SocketLocalHostName
   5     class method: GetLocalHostName
   6     class method: SocketCloseAll
   7     class method: SocketErrorString
   8     class method: SocketErrno
   9     instance method: SocketPeerAddress AF_INET
  10     instance method: SocketErrorString
  11     instance method: SocketErrno
  12     instance method: SocketId
  13     instance method: SocketIsConnected
  14     instance method: SocketPeerPort
  15     instance method: SocketAddress
  16     class method: GsSocketFinalizeAll
  17     instance method: SocketReadWillNotBlock
  18     instance method: SocketWriteWillNotBlock
  19     class method: _lastErrorSymbolOffset
  20     instance method: _lastErrorSymbolOffset
  23     instance method: shutdownReading
  24     instance method: shutdownWriting
  25     instance method: shutdownReadingAndWriting
  26     instance method: SocketCreate UDP
  27     instance method: SocketCreate UNIX Stream
  28     instance method: SocketCreate UNIX Dgram
  29     classmethod:  SocketPair UNIX Stream (returns array of fds)
  30    instance method: SocketAddress AF_UNIX
  31     instance method: SocketPeerAddress AF_UNIX
  32     instance method:  _peerSockAddr (returning a Ruby sockaddr String)
  33     instance method:  _socketLocation (returning a Ruby sockaddr String)
"

<primitive: 323>

^ self _primitiveFailed: #_zeroArgPrim: args: { opcode }
%

category: 'Private'
method:
_twoArgPrim: opcode with: arg1 with: arg2

"opcode  function
   1     class method: getServicePortByName
   2     instance method: SocketConnect
   3     instance method: SocketLinger
   4     instance method: SocketAccept
   5     instance method: SocketKeepAlive
   6     instance method: SocketBind
   7     class method: GetHostNameByAddress
   8     class method: GetServiceNameByPort
   9     instance method: GsSocketSetError
  10     instance method: GsSocketRaiseException
  11     instance method: SocketMakeListener
  12     class method: GetHostAddressByName
  13     instance method  UNIXServer>>bind: aPath
  14     instance method  UNIXSocket>>_setFd: 
  15     class method: GetStandardId
  16     instance method  UNIXServer>>connect: aPath 
  17     classmethod  _existingSocketForFd:
  18     classmethod  _gethostbyaddr:
  19     classmethod  _getsockaddr:host:
  20     classmethod  _unpackSockAddr:flags:
  21     classmethod  _getsockaddrUnix:
  22     classmethod  _unpackSockAddrUnix:
  23     instance method  GsSocket>>rubyConnect: addrString
  24     instance method _bindAddr:
"

<primitive: 322>

^ self _primitiveFailed: #_twoArgPrim:with:with: args: { opcode . arg1 . arg2 }
%

method:
_readInto: aString startingAt: anOffset maxBytes: numBytes

"Returns nil if an error occurred,
   false if non-blocking receiver would block,
   true if EINTR occurred,
   a SmallInteger number of bytes read.

 numBytes specifies maximum number of bytes to read, they
 are stored starting at (aString at: anOffset).

 Clears bits 16r3 from self.readyEvents if no error generated.
" 

<primitive: 489>
self _primitiveFailed: #_readInto:startingAt:maxBytes:
     args: { aString . anOffset . numBytes }
%

method:
_write: aByteObject startingAt: anOffset ofSize: numBytes

"returns nil if an error occurred,
   false if non-blocking receiver would block,
   true if EINTR occurred,
   a SmallInteger number of bytes written.
 Generates an error if aByteObject is not a String or ByteArray.
 Clears bits 16r4 from self.readyEvents if no error generated."

<primitive: 882>
self _primitiveFailed: #_write:startingAt:ofSize:
     args: { aByteObject . anOffset . numBytes }
%

category: 'Private'
classmethod:
_initSocketErrorSymbols

"Initializes the array of socket error symbols."

SocketErrorSymbols := #(
  "SYSERR_UNKNOWN" #UnknownSystemError
  "SYSERR_HOST_NOT_FOUND" #HostNotFound
  "SYSERR_TRY_AGAIN" #TryAgain
  "SYSERR_NO_RECOVERY" #NoRecovery
  "SYSERR_NO_DATA" #NoData
  "SYSERR_NO_ADDRESS" #NoAddress
  "SYSERR_NETDOWN" #NetworkDown
  "SYSERR_NETUNREACH" #NetworkUnreachable
  "SYSERR_NETRESET" #NetworkReset
  "SYSERR_CONNABORTED" #ConnAborted
  "SYSERR_CONNRESET" #ConnReset
  "SYSERR_NOBUFS" #NoBuffers
  "SYSERR_ISCONN" #AlreadyConnected
  "SYSERR_NOTCONN" #NotConnected
  "SYSERR_SHUTDOWN" #Shutdown
  "SYSERR_TIMEDOUT" #TimedOut
  "SYSERR_CONNREFUSED" #ConnectionRefused
  "SYSERR_NAMETOOLONG" #NameTooLong
  "SYSERR_INTR" #Interrupted
  "SYSERR_INPROGRESS" #InProgress
  "SYSERR_INVAL" #InvalidInput
  "SYSERR_ALREADY" #AlreadyDone
  "SYSERR_NOTSOCK" #NotSocket
  "SYSERR_DESTADDRREQ" #DestinationAddressRequired
  "SYSERR_MSGSIZE" #MsgSize
  "SYSERR_PROTOTYPE" #WrongProtocol
  "SYSERR_NOPROTOOPT" #OptionNotSupported
  "SYSERR_PROTONOSUPPORT" #ProtocolNotSupported
  "SYSERR_SOCKTNOSUPPORT" #TypeNotSupported
  "SYSERR_OPNOTSUPP" #OperationNotSupported
  "SYSERR_NOTSUP" #OperationNotSupported
  "SYSERR_PFNOSUPPORT" #ProtocolFamilyNotSupported
  "SYSERR_AFNOSUPPORT" #FamilyNotSupported
  "SYSERR_ADDRINUSE" #AddressInUse
  "SYSERR_ADDRNOTAVAIL" #AddressNotAvailable
  "SYSERR_WOULDBLOCK" #WouldBlock
  "SYSERR_BADF" #NotSocket
  "SYSERR_AGAIN"  #WouldBlock
  "SYSERR_NOMEM" #NoMemory
  "SYSERR_ACCES" #AccessDenied
  "SYSERR_FAULT" #Fault
  "SYSERR_PIPE" #BrokenPipe
  "SYSERR_HOSTDOWN" #HostDown
  "SYSERR_HOSTUNREACH" #HostUnreachable
  "SYSERR_NONET" #NoNetwork
  "SYSERR_WSANOTINITIALISED" #NotInitialized
  "SYSERR_NOSR" #NoStreams
  "SYSERR_PROTO" #ProtocolError
  "SYSERR_NXIO" #ServerExited
  "SYSERR_WSAEINTR" #BlockingCallCanceled
  "SYSERR_MFILE"    #ProcessOutOfDescriptors
  "SYSERR_NFILE"    #SystemOutOfDescriptors
  "GSSOCKETERR_NOHOSTNAME" #GsSocketNoHostName
).
%

category: 'Private'
method:
_isReadable

"Returns true if the receiver is marked as being ready to read."

^ (readyEvents bitAnd: 3) ~~ 0
%

category: 'Private'
method:
_isReadableOrException

"Returns true if the receiver is marked as being ready to read
 or has an exception."

^ (readyEvents bitAnd:16r3B) ~~ 0
%

category: 'Private'
method:
_isWritable

"Returns true if the receiver is marked as being ready to write."

^ (readyEvents bitAnd: 4) ~~ 0
%

category: 'Private'
method:
_isWritableOrException

"Returns true if the receiver is marked as being ready to write
 or has an exception."

^ (readyEvents bitAnd: 16r3c) ~~ 0
%

category: 'Private'
method:
_hadException

"Returns true if the receiver is marked as having had an exception."

^ (readyEvents bitAnd: 16r38) ~~ 0
%

category: 'Private'
method:
_enableRead

"Enables read detection by ProcessorScheduler>>_doPoll: .
 Returns receiver if successful, nil if socket not open."

<primitive: 731>
self _primitiveFailed: #_enableRead  "socket maybe set to blocking mode"
%

category: 'Private'
method:
_enableWrite

"Enables write detection by ProcessorScheduler>>_doPoll: .
 Returns receiver if successful, nil if socket not open."

<primitive: 732>
self _primitiveFailed: #_enableWrite  "socket maybe set to blocking mode"
%

category: 'Private'
method:
_disableEvent: aKind

"Disables the read or write detection specified by aKind.
 Used to cancel a previous  _enableRead or _enableWrite.
  aKind == true   cancels previous _enableWrite
  aKind == false  cancels previous _enableRead
  aKind == nil    cancels any previous _enableRead or _enableWrite
"

<primitive: 729>
self _primitiveFailed: #_disableEvent: args: { aKind }
%

category: 'Private'
method:
_setError: str

^self _twoArgPrim: 9 with: str with: nil
%


! ------------------- Instance methods for GsSocket
category: 'Deprecated'
method:
bind

self deprecated: 'Obsolete in GemStone/64.  Use the bindTo: method instead.'.
^self bindTo: nil 
%

category: 'Client Operations'
method:
bindTo: portNumber

"Binds the receiver to the specified port number.  Use the makeServer
 methods to do the bind when creating a server socket.
 If portNumber is nil then a random port is selected.
 This method is provided to bind a client socket to a specific port
 before it is connected.  Returns the port number actually
 bound to (should be the same as the argument unless argument is nil), 
 or nil if not successful."

^ self _twoArgPrim: 6 with: portNumber with: nil
%

category: 'Client Operations'
method:
bindTo: portNumber toAddress: address

"Binds the receiver to the specified port number and address. 
 Use the makeServer methods to do the bind when creating a server socket.
 This method is provided to bind a client socket to a specific port
 and/or address before it is connected. 
 If portNumber is nil then a random port is selected.
 If address is nil then any appropriate network interface(INADDR_ANY) is used.
 If address is -1 , then INADDR_BROADCAST is used (receiver should be from newUdp ).
 Returns the port number actually
 bound to (should be the same as the argument unless argument is nil), 
 or nil if not successful."

^ self _twoArgPrim: 6 with: portNumber with: address
%

category: 'Socket Operations'
method:
close

"Release any temporary system resources used by the receiver.  This includes
 closing the low level socket.  Returns self if the socket is closes
 successfully or the socket is already closed.  Returns nil if socket cannot
 be closed."
  
^ self _zeroArgPrim: 2
%

category: 'Client Operations'
method:
connectTo: portNumber

"Connect the receiver to the server socket on the local machine
 identified by portNumber.
 Returns true if the connection succeeded and false if not.

 If the underlying connect() returns EISCONN(already connected)
 this method will return true.  (Gs64 v3.0 document preexisting behavior)
 "

^ self connectTo: portNumber on: (self class getLocalHostName)
%

! changes to fix 34574 , 36607
category: 'Client Operations'
method:
connectTo: portNumber on: host

"Connect the receiver to the server socket identified by portNumber and
 host. host can be the name of the host or its address.
 Returns true if the connection succeeded and false if not."

| status |
status := self _twoArgPrim: 2 with: host with: portNumber.
status == self ifTrue:[ ^ true ].
(status == false) ifTrue: [
  "connect call has blocked, wait for it to complete"
  self _waitForWriteReady .
  ^ self peerAddress ~~ nil .
].
^ false
%

category: 'Accessing'
method:
option: optionName

"Returns the current value of the specified option.
 Returns nil if an error occurred.
 optionName can be any of the following:
   'BROADCAST': value is Boolean; permission to transmit broadcast messages.
   'DEBUG': value is Boolean; records debugging information.
   'DONTROUTE': value is Boolean; routing bypass for outgoing messages.
   'ERROR': value is SmallInt; error code of the socket.
   'KEEPALIVE': value is Boolean; detect broken connections.
   'NODELAY': value is Boolean; disables nagle algorithm for send coalescing.
   'NONBLOCKING': value is Boolean , non-blocking state of socket.
   'OOBINLINE': value is Boolean; reception of out-of-band data inband.
   'REUSEADDR': value is Boolean; allows local address reuse.
   'RCVBUF': value is SmallInt; buffer size for input.
   'SNDBUF': value is SmallInt; buffer size for output.
   'TYPE': value is SmallInt; type of the socket (tcp or udp).
   'USELOOPBACK': value is Boolean; bypass network card if possible.

  If an option name is not supported on a particular platform the
  error string will be 'The requested socket option does not exist'."

^ self _twoArgPrim: 5 with: optionName with: nil
%

category: 'Socket Operations'
method:
option: optionName put: value

"Sets the value of the specified option.
 Returns the receiver or nil if an error occurred.
 optionName can be any of the following:
   'BROADCAST': value is Boolean; permission to transmit broadcast messages.
   'CLOSEONEXEC': value is Boolean; close when system exec() is done.
   'DEBUG': value is Boolean; records debugging information.
   'DONTROUTE': value is Boolean; routing bypass for outgoing messages.
   'KEEPALIVE': value is Boolean; detect broken connections.
   'NODELAY': value is Boolean; disables nagle algorithm for send coalescing.
   'NONBLOCKING': value is Boolean , set state of non-blocking for socket.
   'OOBINLINE': value is Boolean; reception of out-of-band data inband.
   'REUSEADDR': value is Boolean; allows local address reuse.
   'RCVBUF': value is SmallInt; buffer size for input.
   'SNDBUF': value is SmallInt; buffer size for output.
   'USELOOPBACK': value is Boolean; bypass network card if possible.

  If an option name is not supported on a particular platform the
  error string will be 'The requested socket option does not exist'."

^ self _twoArgPrim: 5 with: optionName with: value
%
   
category: 'Socket Operations'
method:
keepAlive: bool

"Sets the receiver to periodically broadcast messages to clients.  If
 a client does not respond to a broadcast, its connection is severed.
 If bool is false, keepAlive is turned off.
 Returns the receiver or nil if an error occurred."

^ self option: 'KEEPALIVE' put: bool
%

category: 'Socket Operations'
method:
linger: bool length: timeOut

"Sets up the receiver so that if unsent data is waiting to be transmitted
 at the time the receiver is closed, the current process will block until
 either the data is transmitted, or the given timeOut expires.  timeOut is
 in units of seconds.

 Returns the receiver or nil if an error occurred."

^ self _twoArgPrim: 3 with: bool with: timeOut
%

category: 'Server Operations'
method:
makeListener: queueLength

"Turns the receiver into a listening socket.
 The queueLength argument specifies the size of the listen backlog queue for
 incoming connections.
 Returns the receiver or nil if an error occurred."

^ self _twoArgPrim: 11 with: queueLength with: nil
%

category: 'Server Operations'
method:
makeServer

"Turns the receiver into a server socket. Binds the receiver to a port and
 makes it a listening socket.  Returns the receiver or nil if an error
 occurred."

^ self makeServer: 5 atPort: nil atAddress: nil
%

category: 'Server Operations'
method:
makeServer: queueLength

"Turns the receiver into a server socket.  The queueLength argument specifies
 the size of the listen backlog queue for incoming connections.  Binds the
 receiver to a random port.  Returns the receiver or nil if an error occurred."

^ self makeServer: queueLength atPort: nil atAddress: nil
%

category: 'Server Operations'
method:
makeServerAtPort: portNum

"Turns the receiver into a server socket.  Binds the receiver to portNum and
 makes it a listening socket.  Returns the receiver or nil if an error
 occurred."

^ self makeServer: 5 atPort: portNum atAddress: nil
%

category: 'Server Operations'
method:
makeServer: queueLength atPort: portNum

"Turns the receiver into a server socket.  The queueLength argument specifies
 the size of the listen backlog queue for incoming connections.  Binds the
 receiver to portNum.  If portNum is nil then a random port is selected.
 Returns the receiver, or nil if an error occurred."

^ self makeServer: queueLength atPort: portNum atAddress: nil
%

category: 'Server Operations'
method:
makeServer: queueLength atPort: portNum atAddress: address

"Turns the receiver into a server socket.  The queueLength argument specifies
 the size of the listen backlog queue for incoming connections.  Binds the
 receiver to portNum and address.
 If portNum is nil then a random port is selected.
 If address is nil then any appropriate network interface is used.
 Returns the receiver, or nil if an error occurred."

((self bindTo: portNum toAddress: address) == nil) ifTrue: [^nil].
^ self makeListener: queueLength
%

category: 'Server Operations'
method:
speciesForAccept
"Returns a class, an instance of which should be used as the result
 of the accept method."

^self class
%

category: 'Server Operations'
method:
accept

"Accept a client request for a connection on the receiver.  
 Returns a socket created for a new connection, 
 or nil if there was some problem.
 The result is an instance of speciesForAccept , and has non-blocking state
 equal to non-blocking state of the receiver.

 For example, the following code does not return until there is a connection:
   
 sock := GsSocket new.
 sock makeServer.
 newsock := sock accept.
 msg := newsock read: 512."

| res aSocket |

aSocket := self speciesForAccept _basicNew .
res := true.
[ res == true or:[ res == false ]] whileTrue: [
  "primitive will wait until data available if C socket is blocking."
  res := self _twoArgPrim: 4 with: aSocket with: nil .
  (res == false) ifTrue: [
    "socket is non-blocking and would have blocked, process scheduler will wait"
    res := self _waitForReadReady .
  ].
].
(res == nil) ifFalse: [
  res := aSocket.
].
^res
%

! listen:  , listen:acceptingWith:   deleted as of Gs64 v3.0

category: 'Accessing'
method:
peerName

"For a bound socket, returns the hostname of the machine on which the 
 process at the other end of the connection is running.

 If the socket is not bound, or an error occurs, returns nil."

| addr |  

addr := self peerAddress.
(addr == nil) ifTrue: [^nil].
^ self class getHostNameByAddress: addr
%

category: 'Accessing'
method:
peerAddress

"For a bound socket, returns the address of the machine on which the 
 process at the other end of the connection is running.

 If the socket is not bound, or an error occurs, returns nil."
  
^ self _zeroArgPrim: 9
%

category: 'Accessing'
method:
peerPort

"For a bound socket, returns the port being used by the
 the other end of the connection.

 If the socket is not bound, or an error occurs, returns nil."

^ self _zeroArgPrim: 14
%

category: 'Accessing'
method:
port

"Returns the local port number of a socket, or nil if an error occurs.
 On some platforms an error is raised if the receiver is not bound.
 On others the port 0 is returned and no error is raised."
  
^ self _zeroArgPrim: 3
%

category: 'Accessing'
method:
address

"Returns the local ip address of a socket, or nil if an error occurs.
 On some platforms an error is raised if the receiver is not bound.
 On others the address 0.0.0.0 is returned and no error is raised."
  
^ self _zeroArgPrim: 15
%

!"Provided for compatibility with Geode."
category: 'Reading'
method:
read: maxBytes

"This method is equivalent to readString:."

 | bytes amount |
 bytes := String new.
 amount := self read: maxBytes into: bytes startingAt: 1.
 amount ifNil: [ ^nil ].
 ^ bytes
%

category: 'Reading'
method:
read: maxBytes into: byteObj

"Reads up to the given number of bytes into the given byte object (for
 example, a String) starting at index 1.
 Returns the number of bytes read, or nil if an error occurs, 
 or 0 if EOF on the receiver.
 byteObj is grown as needed but is not shrunk.

 If no data is available for reading, this blocks until data arrives.
 The readWillNotBlock or readWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for reading before calling this method."

^ self read: maxBytes into: byteObj startingAt: 1  
%

category: 'Reading'
method:
read: maxBytes into: byteObj startingAt: index

"Reads up to the given number of bytes into the given byte object (for
 example, a String).  The first byte read will go into the position
 indicated by index. 
 Returns the number of bytes read, or nil if an error, 
 or 0 if EOF on the receiver.

 maxBytes must be a SmallInteger > 0 and <= 10000000 .

 An error will be raised if index is greater than the size of byteObj.
 Note that index can be equal to the size of byteObj plus 1 in which
 case any bytes read will be appended to byteObj.

 If no data is available for reading, this blocks until data arrives.
 The readWillNotBlock or readWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for reading before calling this method."

| status |
[
  "primitive will wait until data available if C socket is blocking."
  status := self _readInto: byteObj startingAt: index maxBytes: maxBytes.
  (status == false) ifTrue: [
    "socket is non-blocking and would have blocked, process scheduler will wait"
    status := self _waitForReadReady
  ] "ifFalse:[ if status==true here, we get EINTR an must retry]" .
  status == true 
] whileTrue .
^ status
%

category: 'Reading'
method:
readString: amount untilFalse: aBlock

"Reads bytes from the receiver into a String which it creates and returns.
 Will keep reading until amount bytes are read or 
 the one argument Block  aBlock  returns false.
 Each time aBlock is evaluated it is given the number of bytes read so far.
 aBlock is only evaluated after a partial read.
 Returns the String or nil if an error occurs."

| bytes |

bytes := String new.
((self read: amount into: bytes untilFalse: aBlock) == nil) ifTrue: [
  ^nil
].

^bytes
%

category: 'Reading'
method:
read: amount untilFalse: aBlock

"Same as readString:untilFalse:."

^ self readString: amount untilFalse: aBlock.
%

category: 'Reading'
method:
read: amount into: byteObj untilFalse: aBlock

"Reads at most amount bytes from the receiver into the given byte object
 starting at index 1.
 Returns when amount bytes have been read, an error occurs, or 
 the one argument Block  aBlock   evaluates to false.

 See read:into:startingAt:untilFalse: for details.

 If aBlock is evaluated its argument is the total number of bytes read
 so far. Returns the number of bytes read, or nil if an error occurs."

^ self read: amount into: byteObj startingAt: 1 untilFalse: aBlock.
%

category: 'Reading'
method:
read: amount into: byteObj startingAt: index untilFalse: aBlock

"Reads at most amount bytes from the receiver into byteObj.
 The first byte read is put in byteObj at position specified by index.
 Returns when amount bytes have been read, an error occurs, or 
 the one argument Block   aBlock   evaluates to false.

 An error will be raised if index is greater than the size of byteObj.
 Note that index can be equal to the size of byteObj plus 1 in which
 case any bytes read will be appended to byteObj.

 Repeatedly does a read followed by an evaluation of aBlock if the
 read returned without amount bytes having been read.

 If the receiver is not ready for reading, this blocks until it is ready.
 The readWillNotBlock or readWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for reading before calling this method.
 They can also be used in aBlock to determine when it should return true
 so that another read will be done without blocking.

 If aBlock is evaluated its argument is the total number of bytes read
 so far. Returns the number of bytes read, or nil if an error occurs."

| total result offset numToRead |

total := 0.
offset := index.
numToRead := amount.

[result := self read: numToRead into: byteObj startingAt: offset.
 (result == nil) ifTrue: [^nil]. "Quit due to error"
 total := total + result.
 (total == amount) ifTrue: [^total]. "All done"
 numToRead := numToRead - result.
 offset := offset + result.
 aBlock value: total 
] untilFalse.

^total
%

category: 'Testing'
method:
isActive
"Returns true if the receiver's socket is in a usable state.
 Returns false if the receiver's socket is not usable.
 Sockets become unusable when they are closed."

((self id) == -1) ifTrue: [^false].
^ true
%

category: 'Testing'
method:
isConnected
"Returns true if the socket is connected to a peer.
 Returns false if the socket never was connected or has lost its connection
 to the peer and had no more data to read.
 Returns nil if an error occurs."

^ self _zeroArgPrim: 13.
%

category: 'Testing'
method:
readWillNotBlock

"Returns true if the socket is currently ready to receive input without
 blocking.  Returns false if it is not currently ready.  Returns nil if an error
 occurs.

 The receiver must already be connected for this method to work properly.  If it
 is not connected, then the value that this method returns is indeterminate.
 Use the peerName method to determine if the receiver is connected.

 Call this method to prevent subsequent read or accept operations from hanging.
 If it returns true for a connected socket, then the input operation will not
 hang.  However, a return value of true is no guarantee that the operation
 itself will succeed."
  
^ self _zeroArgPrim: 17
%

category: 'Testing'
method:
readWillNotBlockWithin: msToWait

"Returns true if the socket is ready to receive input without blocking within
 msToWait milliseconds from the time that this method is called.  Returns false
 if it is not ready after msToWait milliseconds.  Returns nil if an error
 occurs.

 If msToWait is 0, then this method reports the current readiness of the
 receiver.  If msToWait is -1, then this method never returns false, but waits
 until the receiver is ready to receive input without blocking, and then returns
 true.

 The receiver must already be connected for this method to work properly.  If it
 is not connected, then the value that this method returns is indeterminate.
 Use the peerName method to determine if the receiver is connected.

 Call this method to prevent subsequent read or accept operations from hanging.
 If it returns true for a connected socket, then the input operation will not
 hang.  However, a return value of true is no guarantee that the operation
 itself will succeed."

 | sched proc |
 msToWait == 0 ifTrue: [^ self readWillNotBlock ].
 self _isReadableOrException ifTrue:[ ^ true ].
 self _enableRead .
 sched := self _scheduler .
 proc := sched activeProcess .
 readWaiters ifNil:[  
   readWaiters := proc 
 ] ifNotNil:[  
   self _addReadWaiter: proc 
 ].
 msToWait == -1 ifTrue:[
   sched _waitForSocket: self .
   ^ true
 ] ifFalse:[
   sched _waitForSocket: self timeout: msToWait .
   ^ self _isReadableOrException
 ]
%

category: 'Private'
method:
_unscheduleProcess: aProcess
   self _cancelReadEventFor: aProcess .
   self _cancelWriteEventFor: aProcess
%

category: 'Private'
method:
_addReadWaiter: aProcess 
   "assumes that readWaiters is not nil, and aProcess will be second or 
    subsequent waiter."

   | waiters arr |
   (waiters := readWaiters) _isArray ifFalse:[ 
     arr := { waiters } .
     readWaiters := arr .
     waiters := arr .
   ].
   self _add: aProcess toArray: waiters .
%

category: 'Private'
method:
_addWriteWaiter: aProcess 
   "assumes that writeWaiters is not nil, and aProcess will be second or 
    subsequent waiter."

   | waiters arr |
   (waiters := writeWaiters) _isArray ifFalse:[ 
     arr := { waiters } .
     writeWaiters := arr .
     waiters := arr .
   ].
   self _add: aProcess toArray: waiters .
%

category: 'Testing'
method:
_waitForReadReady

"Returns true when socket is ready to receive input without blocking.
 Caller should have already attempted a read that failed because it would block,
 or checked status from _isReadableOrException.
 "
 | sched proc |
 self _enableRead .
 sched := self _scheduler .
 proc := sched activeProcess .
 readWaiters ifNil:[
   readWaiters := proc
 ] ifNotNil:[  
   self _addReadWaiter: proc 
 ].
 sched _waitForSocket: self . 
 ^ true
%
 

category: 'Accessing
method:
readWaiters
  "Return an Array containing the objects waiting on the receiver to be ready
   for reading."

^ self _getWaiters: readWaiters
%

category: 'Accessing
method:
writeWaiters
  "Return an Array containing the objects waiting on the receiver to be ready
   for writing."

^ self _getWaiters: writeWaiters
%


category: 'Private'
method:
_getWaiters: waiters
  waiters ifNotNil:[
    waiters _isArray ifTrue:[
      ^ waiters copy
    ] ifFalse:[
      ^ { waiters } 
    ] 
  ] ifNil:[
    ^ { } 
  ].
%

category: 'Private Scheduling'
method:
_whenReadableNotify: objToNotify

 objToNotify _canWaitOnSocket .  "DNU if waiting not supported on objToNotify"
 self _enableRead .
 readWaiters ifNil:[
   readWaiters := objToNotify
 ] ifNotNil:[  
   self _addReadWaiter: objToNotify
 ]
%  

category: 'Accessing
method:
hasReadWaiter: anObject
  | wtrs |
  (wtrs := readWaiters) ifNotNil:[
    wtrs _isArray ifTrue:[
      ^ wtrs includesIdentical: anObject
    ] ifFalse:[
      ^ wtrs == anObject 
    ].
  ].
  ^ false  
%

category: 'Accessing
method:
hasWriteWaiter: anObject
  | wtrs |
  (wtrs := writeWaiters) ifNotNil:[
    wtrs _isArray ifTrue:[
      ^ wtrs includesIdentical: anObject
    ] ifFalse:[
      ^ wtrs == anObject 
    ].
  ].
  ^ false  
%

category: 'Private Scheduling'
method:
_reapEvents
  "returns true if receiver has no more waiters after reaping events"

  | rwtrs wwtrs res revents |
  rwtrs := readWaiters .
  ((revents := readyEvents) bitAnd:16r3B) ~~ 0 ifTrue:[ "inline _isReadableOrException"
    rwtrs ifNotNil:[
      rwtrs _isArray ifTrue:[
        1 to: rwtrs size do:[:k |
          (rwtrs at: k) _reapSignal: self  .
        ].  
      ] ifFalse:[
        rwtrs _reapSignal: self 
      ].
      "GsFile gciLogServer:' reaped readable' .  "
      readWaiters := nil .
      rwtrs := nil .
    ].
  ].
  wwtrs := writeWaiters .
  (revents bitAnd: 16r3c) ~~ 0 ifTrue:[ "inline _isWritableOrException"
    wwtrs ifNotNil:[
      wwtrs _isArray ifTrue:[
        1 to: wwtrs size do:[:k |
          (wwtrs at: k) _reapSignal: self 
        ].  
      ] ifFalse:[
        wwtrs _reapSignal: self 
      ].
      " GsFile gciLogServer:' reaped writable' . "
      writeWaiters := nil .
      wwtrs := nil .
    ].
    res := rwtrs == nil .
  ] ifFalse:[
    res := rwtrs == nil and:[ wwtrs == nil] .
  ].
  " GsFile gciLogServer:'  returning ' , res asString .   "
  ^ res 
%

category: 'Private Scheduling'
method:
_cancelReadEventFor: objToNotify

  "Returns true if event has already been cancelled or reaped.
   Returns false if event is cancelled by this invocation."

  | wtrs |
  wtrs := readWaiters .
  wtrs _isArray ifTrue:[
    (wtrs removeIdentical: objToNotify otherwise:nil) ifNotNil:[ ^ true ].
    wtrs size == 0 ifTrue:[ readWaiters := nil. wtrs := nil ].
  ] ifFalse:[
    wtrs == objToNotify ifTrue:[ 
      readWaiters := nil.  wtrs := nil .
    ] ifFalse:[
      ^ true 
    ].
  ].
  wtrs ifNil:[
    self _disableEvent: false . "disableRead"
  ].
  ^ false
%

category: 'Private Scheduling'
method:
_cancelWriteEventFor: objToNotify

  "Returns true if event has already been cancelled or reaped.
   Returns false if event is cancelled by this invocation."

  | wtrs |
  wtrs := writeWaiters .
  wtrs _isArray ifTrue:[
    (wtrs removeIdentical: objToNotify otherwise: nil) ifNotNil:[ ^ true ].
    wtrs size == 0 ifTrue:[ writeWaiters := nil. wtrs := nil ].
  ] ifFalse:[
    wtrs == objToNotify ifTrue:[ 
      writeWaiters := nil. wtrs := nil .
    ] ifFalse: [
      ^ true 
    ].
  ].
  wtrs ifNil:[
    self _disableEvent: true . "disableWrite"
  ].
  ^ false

%

category: 'Accessing'
method:
hasWaiters

  ^ readWaiters ~~ nil or:[ writeWaiters ~~ nil ]
%

category: 'Private'
method:
_scheduler

"Returns   ProcessorScheduler scheduler "
<primitive: 457>

self _primitiveFailed: #_scheduler
%

category: 'Private'
method: 
_add: anObject toArray: aProcessArray

"Implements IdentitySet add: semantics for aProcessArray
 Returns true if anObject was already in aProcessArray, false otherwise"

<primitive: 725>
aProcessArray size > 100 ifTrue:[
  self error:'too many processes waiting on socket'.
].
self _primitiveFailed: #_add:toSet: args: { anObject . aProcessArray }
%


category: 'Deprecated'
method:
readReady

self deprecated: 'Obsolete in GemStone/64.  Use the readWillNotBlock or readWillNotBlockWithin: method instead.'.
^ self _zeroArgPrim: 17
%

category: 'Reading'
method:
readString: maxBytes

"Reads up to the given number of bytes, returning them in a String whose size is
 between 0 and maxBytes inclusive.  If an error occurs, nil is returned instead.

 If maxBytes is greater than the size of the operating system's buffer for 
 the socket, the size of the result string may be a function of this
 buffer size, even if more data is available from the sender.  Repeated
 invocation of this method may be necessary to obtain all of the data.

 For optimum performance, maxBytes should be <= 8192, and should typically be
 4096.
   
 If no data is available for reading, this blocks until data arrives.   
 The readWillNotBlock or readWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for reading before calling this method."
 
  | bytes amount |
  bytes := String new.
  amount := self read: maxBytes into: bytes startingAt: 1.
  amount ifNil: [ ^nil ].
  ^ bytes
%

category: 'Writing'
method:
write: byteObj

"Write out the given byte object.  Returns the number of bytes written,
 or nil if an error occurs.
  
 If the receiver is not ready for writing, this blocks until it is ready.
 The writeWillNotBlock or writeWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for writing before calling this method."

^ self write: (byteObj size) from: byteObj startingAt: 1
%

category: 'Writing'
method:
write: byteObj startingAt: index

"Write bytes from byteObj to the receiver.
 The first byte written will be from the position indicated by index.
 All of the bytes after it in byteObj will be written.
 Returns the number of bytes written, or nil if an error occurs.
  
 If the receiver is not ready for writing, this blocks until it is ready.
 The writeWillNotBlock or writeWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for writing before calling this method."

^ self write: ((byteObj size) - (index - 1)) from: byteObj startingAt: index
%

category: 'Writing'
method:
write: byteObj untilFalse: aBlock

"Writes the entire contents of byteObj to the receiver unless
 an error occurs or aBlock returns false.
 See write:from:startingAt:untilFalse: for details.
 Returns the number of bytes actually written."

^ self write: (byteObj size) from: byteObj startingAt: 1 untilFalse: aBlock.
%

category: 'Writing'
method:
write: byteObj startingAt: index untilFalse: aBlock

"Writes the contents of byteObj to the receiver unless
 an error occurs or aBlock returns false.
 The first byte written from byteObj is at the position specified by index.
 All of the bytes after it in byteObj will be written.
 See write:from:startingAt:untilFalse: for details.
 Returns the number of bytes actually written."

^ self write: ((byteObj size) - (index -1)) from: byteObj startingAt: index
       untilFalse: aBlock.
%

category: 'Writing'
method:
write: amount from: byteObj

"Write the given number of bytes from the given byte object.  Returns the
 number of bytes written, or nil if an error occurs.
  
 If the receiver is not ready for writing, this blocks until it is ready.
 The writeWillNotBlock or writeWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for writing before calling this method."

^ self write: amount from: byteObj startingAt: 1
%

category: 'Writing'
method:
write: amount from: byteObj startingAt: index

"Write bytes from byteObj to the receiver.
 The first byte written will be from the position indicated by index.
 The number of bytes written is specified by amount.
 It is in an error if amount bytes are not available.
 Returns the number of bytes written, or nil if an error occurs.
  
 If the receiver is not ready for writing, this blocks until it is ready.
 The writeWillNotBlock or writeWillNotBlockWithin: methods may be used to
 determine whether or not data is ready for writing before calling this method."
  
 | offset numToWrite |
 offset := index.
 numToWrite := amount.
 [ true ] whileTrue:[ | result |
   [
     result := self _write: byteObj startingAt: offset ofSize: numToWrite .
     result == true 
   ] whileTrue . "loop to handle EINTR "
 
   result _isSmallInteger ifTrue:[
     numToWrite  := numToWrite - result .
     (numToWrite <= 0) ifTrue: [ ^ amount ]. "All done"
     offset := offset + result .
   ] ifFalse: [
     result == false ifTrue:[  "non-blocking socket would have blocked."
       self _waitForWriteReady . 
     ] ifFalse:[
       result ifNil: [ ^ nil]. "socket error"
     ].
   ].
 ].
%

category: 'Writing'
method:
nbwrite: amount from: byteObj startingAt: index

"If receiver is a non-blocking socket,
 write at most amount bytes from byteObj to the receiver without blocking.

 If receiver is a blocking socket,
 write at most amount bytes from byteObj to the receiver blocking until
 the socket accepts data, or until the gem process receives SIGTERM.

 The first byte written will be from the position indicated by index.
 The maximum number of bytes to write is specified by amount.
 It is an error if amount bytes are not available.
 Returns the number of bytes written, or nil if an error occurs.
 Returns 0 if non-blocking socket would have blocked."

 | res |
 [
   res := self _write: byteObj startingAt: index ofSize: amount.
   (res == false) ifTrue: [
     "would have blocked"
     ^ 0
   ].
   res == true  "if res==true, socket got EINTR and we retry"
 ] whileTrue .
 ^res
%

category: 'Writing'
method:
write: amount from: byteObj startingAt: index untilFalse: aBlock

"Write bytes from byteObj to the receiver until amount bytes have
 been written, an error occurs, or aBlock's value is false.
 The first byte written will be from the position indicated by index.
 The number of bytes written is specified by amount.
 It is in an error if amount bytes are not available.
 Returns the number of bytes written, or nil if an error occurs.
 This method repeatedly tries to write as many bytes as it can
 and then, if more bytes still need to be written, evaluates aBlock.

 If the receiver is not ready for writing the one argument Block
 aBlock is called.
 The writeWillNotBlock or writeWillNotBlockWithin: methods may be used
 in aBlock to determine when it should return true so that another write
 will be done without blocking.
 Every time aBlock is evaluated its argument is the total number of bytes
 written so far.

 Implementation uses nbwrite, so unless aBlock uses 
 writeWillNotBlockWithin:-1 or other logic based on writeWillNotBlock ,
 to wait for the socket to be ready before returning false from
 aBlock, this method will run hot.  "

| total offset numToWrite |

total := 0.
offset := index.
numToWrite := amount.

[ | result |
 result := self nbwrite: numToWrite from: byteObj startingAt: offset.
 (result == nil) ifTrue: [^nil]. "Quit due to error"
 total := total + result.
 (total == amount) ifTrue: [^total]. "All done"
 numToWrite := numToWrite - result.
 offset := offset + result.
 aBlock value: total
] untilFalse.

^total
%

category: 'Testing'
method:
writeWillNotBlock

"Returns true if the socket is currently ready to take output without blocking.
 Returns false if it is not currently ready.  Returns nil if an error occurs.

 The receiver must already be connected for this method to work properly.  If it
 is not connected, then the value that this method returns is indeterminate.
 Use the peerName method to determine if the receiver is connected.

 Call this method to prevent subsequent write operations from hanging.  If it
 returns true for a connected socket, then a subsequent write will not hang.
 However, a return value of true is no guarantee that the write operation itself
 will succeed."
  
^ self _zeroArgPrim: 18    
%

category: 'Testing'
method:
writeWillNotBlockWithin: msToWait

"Returns true if the socket is ready to take output without blocking within
 msToWait milliseconds from the time that this method is called.  Returns false
 if it is not ready after msToWait milliseconds.  Returns nil if an error
 occurs.

 If msToWait is 0, then this method reports the current readiness of the
 receiver.  If msToWait is -1, then this method never returns false, but waits
 until the receiver is ready to take output without blocking, and then returns
 true.

 The receiver must already be connected for this method to work properly.  If it
 is not connected, then the value that this method returns is indeterminate.
 Use the peerName method to determine if the receiver is connected.

 Call this method to prevent subsequent write operations from hanging.  If it
 returns true for a connected socket, then a subsequent write will not hang.
 However, a return value of true is no guarantee that the write operation itself
 will succeed."
  
 | sched proc |
 msToWait == 0 ifTrue:[ ^ self writeWillNotBlock].
 self _isWritableOrException ifTrue:[ ^ true ].
 self _enableWrite .
 sched := self _scheduler .
 proc := sched activeProcess .
 writeWaiters ifNil:[  
   writeWaiters := proc 
 ] ifNotNil:[  
   self _addWriteWaiter: proc 
 ].
 msToWait == -1 ifTrue:[
   sched _waitForSocket: self .
   ^ true
 ] ifFalse:[
   sched _waitForSocket: self timeout: msToWait .
   ^ self _isWritableOrException
 ]
%

category: 'Testing'
method:
_waitForWriteReady

"Waits until socket is ready to take output without blocking
 and then returns true. Caller should have already done a write or connect
 which failed because it would have blocked, or checked status with _isWritableOrException."

 | sched proc |
 self _enableWrite .
 sched := self _scheduler .
 proc := sched activeProcess .
 writeWaiters ifNil:[
   writeWaiters := proc
 ] ifNotNil:[  
   self _addWriteWaiter: proc 
 ].
 sched _waitForSocket: self .
 ^ true
%


category: 'Private Scheduling'
method:
_whenWritableNotify: objToNotify

 objToNotify _canWaitOnSocket .  "DNU if waiting not supported on objToNotify"
 self _enableWrite .
 writeWaiters ifNil:[
   writeWaiters := objToNotify
 ] ifNotNil:[  
   self _addWriteWaiter: objToNotify 
 ].
%

!   fix Maglev Trac 806
method:
_changePriority: aGsProcess from: oldPriority

  ^ self  "do nothing"
%


category: 'Deprecated'
method:
writeReady

self deprecated: 'Obsolete in GemStone/64.  Use the writeWillNotBlock or writeWillNotBlockWithin: method instead.'.
^ self _zeroArgPrim: 18
%

category: 'Comparing'
method:
hash

"Returns a SmallInteger related to the value of the receiver.  If two instances
 of GsSocket are equal (as compared by the = method), then they must have the
 same hash value."

^ self id hash
%

category: 'Comparing'
method:
= aSocket

"Returns true if the receiver and aSocket represent the same operating system
 socket.  Returns false otherwise."

| idSelf |
(self == aSocket) ifTrue: [^true].
aSocket ifNil: [^false].
(aSocket isKindOf: self class) ifFalse: [^false].
idSelf := self id.
(idSelf == -1) ifTrue: [^false].
^idSelf == (aSocket id)
%

category: 'Accessing'
method:
id

"Returns the value of the low level socket.  If no low level socket exists,
 returns -1."

^ fileDescriptor ifNil:[ -1 ] ifNotNil:[ fileDescriptor ]
%

category: 'Comparing'
method:
~= aSocket

"Returns false if the receiver and aSocket represent the same operating system
 socket.  Returns true otherwise."

^ (self = aSocket) not
%

category: 'Examples'
classmethod:
clientExample 

^ self clientExample: true usingPort: 57785
%

category: 'Examples'
classmethod:
clientExample: logToGciClient usingPort: portNum

"logToGciClient is a Boolean, true if GsFile-based logging is to be done to
  the RPC Gci client, false if to the server process's log file .
  Use false if executing via topaz 'nbrun'  .

 This client will connect to a server created with serverExample, and read
 the string object that the server sends.
   
 Creates a socket, connect it to port portNum, read a string, close the
 socket, and check the resulting object.  Returns true if successful.
   
 The server should already be listening for connections when this method is
 invoked."

| socket dataString dataObj chunk firstChunk |

socket := GsSocket new.
(socket connectTo: portNum) ifFalse:[ 
  socket close .
  self halt: 'Unable to connect to port ' , portNum asString , '. Wait 30 seconds or so, then ensure serverExample is started first.' 
  ].
dataString := String new .

firstChunk := true .
[socket readWillNotBlockWithin: 5000] whileFalse: [ 
  GsFile gciLog: 'Waiting for server to write...' onClient: logToGciClient .
  ].
GsFile gciLog: 'Got something from the server to read.' onClient: logToGciClient .

[
  chunk := socket readString: 4000 "max bytes".
  firstChunk ifTrue:[
    firstChunk := false .
    chunk size <= 0 ifTrue:[ self halt: 'readWillNotBlock disagrees with read'].
    ] .
  chunk ifNil:[ self halt: 'Error in reading from socket' ].
  dataString addAll: chunk .
  (chunk at: chunk size) == (Character withValue: 0)  "until null terminator"
  ] untilTrue .
  
socket close.
dataString size: (dataString size - 1) "strip null terminator" .
dataObj := (PassiveObject newWithContents: dataString) activate.
dataObj size == 5000 ifFalse:[ self halt: 'bad size of dataObj' ].
1 to: 5000 do:[:j | 
  (dataObj at:j) == j ifFalse:[ self halt: 'invalid contents in dataObj' ].
  ] .
^ true. 
%

category: 'Examples'
classmethod:
serverExample

^ self serverExample: true usingPort: 57785
%

category: 'Examples'
classmethod:
serverExample: logToGciClient usingPort: portNum

"logToGciClient is a Boolean, true if GsFile-based logging is to be done to
  the RPC Gci client, false if to the server process's log file . 
  Use false if executing via topaz 'nbrun'  .

 Creates a socket, binds it to port portNum, and waits for a connection.
 When a connection is established, sends some data to the client, and
 closes the connection.  Returns true if successful.

 You will need two GemStone sessions running from two independent
 interface processes to run both this and the clientExample.  The Gem
 processes for the two sessions must be on the same machine. (For
 example two Topaz sessions.)
   
 Warning: This method will cause your current session to hang until a 
 connection is established."

| server client dataObj dataString numWritten dataStream errStr |

server := GsSocket new.

(server makeServerAtPort: portNum) ifNil: [
  errStr := server lastErrorString.
  server close.
  self halt: errStr.
  ].

server port == portNum ifFalse:[ self halt: 'bad port number' ] .

GsFile gciLog: 'Waiting for GsSocket clientExample to connect...' 
	onClient: logToGciClient .
[server readWillNotBlockWithin: 5000] whileFalse: [
  GsFile gciLog: 'waiting...' 
	onClient: logToGciClient
  ].
GsFile gciLog: 'Client connected.' onClient: logToGciClient .

client := server accept .
client ifNil: [
  errStr := server lastErrorString.
  server close.
  self halt: errStr.
].
client linger: true length: 10.  "wait after closing until data is processed"

dataObj := Array new:5000 .
1 to: 5000 do:[:j | dataObj at:j put: j ].
dataString := String new .
dataStream := WriteStream on: dataString . 
PassiveObject passivate: dataObj toStream: dataStream .
dataString add: (Character withValue: 0). "null terminate so client knows
                                           when to stop reading"
client writeWillNotBlock ifFalse:[ self halt: 'socket write will block' ].
numWritten := client write: dataString .
numWritten == dataString size ifFalse:[ self halt: 'error writing to socket'].
client close.
server close .
				"deleted   'reduce garbage' code"
^ true
%

category: 'Socket Operations'
method: GsSocket
shutdownReading

"Partially shutdown the socket such that further reads are disallowed.
 The socket is not closed.

 Returns true if successful, false if the method failed, and nil if 
 the receiver is not a valid GsSocket."
  
^ self _zeroArgPrim: 23
%

category: 'Socket Operations'
method: GsSocket
shutdownWriting

"Partially shutdown the socket such that further writes are disallowed.
 The socket is not closed.

 Returns true if successful, false if the method failed, and nil if 
 the receiver is not a valid GsSocket."
  
^ self _zeroArgPrim: 24
%

category: 'Socket Operations'
method: GsSocket
shutdownReadingAndWriting

"Shutdown the socket such that further reads and writes are disallowed.
 The socket is not closed.

 Returns true if successful, false if the method failed, and nil if 
 the receiver is not a valid GsSocket."
  
^ self _zeroArgPrim: 25
%
category: 'Private'
method: GsSocket
_waitingProcessesInto: anIdentitySet
  | waiters |
  1 to: (waiters := selectWaiters) size by: 2 do:[:j |
    anIdentitySet add: (waiters at: j)
  ]
%
method: GsSocket
_waitingProcessesInto: anIdentitySet inGroup: groupId
  | waiters |
  1 to: (waiters := selectWaiters) size by: 2 do:[:j | | proc |
    proc := waiters at: j .
    proc _groupOrNil == groupId ifTrue:[ anIdentitySet add: proc ].
  ]
%


expectvalue true
run
GsSocket _initSocketErrorSymbols .
^true
%
