! ========================================================================
! Copyright (C) by VMware, Inc. 1991-2011.  All Rights Reserved
!
! $Id: multibytestring.gs,v 1.14.6.1 2008-04-16 18:43:00 dhenrich Exp $
!
! ========================================================================

! Gs64 v3.0 MultiByteString always created by bom.c

removeallmethods MultiByteString
removeallclassmethods MultiByteString
set class MultiByteString

category: 'For Documentation Installation only'
classmethod:
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'MultiByteString is an abstract class for representing strings
 for which each Character occupies more than one byte.

 For historical reasons dealing with the now-deprecated constraints, 
 MultiByteString is in the ClassHistory of String.' .
doc documentClassWith: txt.

txt := (GsDocText new) details:
'Unless noted otherwise, comparisons between MultiByteStrings are 
 case-sensitive.

 Some of these methods determine whether one strings collates before
 another.  In collation, the Characters of the receiver and aString are
 compared Character-by-Character, from left to right, in case-sensitive
 fashion.  If two strings are of different length, and all Characters in the
 shorter string are equal to their counterparts in the longer strings, the
 shorter string collates before the longer.

 These methods require that the argument be a kind of CharacterCollection, not
 necessarily a String.'.
doc documentCategory: #Comparing with: txt.

self description: doc.
%

! ------------------- Class methods for MultiByteString

category 'Formatting'
classmethod:
charSize

"Returns number of bytes that make up a character for this string class."

^ self subclassResponsibility: #charSize
%

category 'Instance Creation'
classmethod:
new: aSize

^ self subclassResponsibility: #new:
%

! fix 40867
category 'Instance Creation'
classmethod:
new

^ self subclassResponsibility: #new
%

! ------------------- Instance methods for MultiByteString
category 'Formatting'
method:
charSize

"Returns number of bytes that make up a character for this string class."

^ self subclassResponsibility: #charSize
%

category: 'Formatting'
method:
printOn: aStream

"Puts a displayable representation of the receiver on the given stream."

"For classes whose instances can be literals, the result should contain
 formatting information.  For example, the following expression should
 evaluate to true:

 #abc asString = String withAll: '#abc'"

| idx lastIdx sz |

aStream _nextPut: $' .

idx := self indexOf: $' startingAt: 1 .
(idx == 0 or:[ idx > aStream maxSize ]) ifTrue:[
  aStream nextPutAll: self .
  ]
ifFalse:[
  lastIdx := 1.
  [ (idx == 0) or: [aStream isFull]] whileFalse: [
    aStream nextPutAll: ( self copyFrom: lastIdx to: idx) .
    aStream nextPut: $' .
    lastIdx := idx + 1 .
    idx := self indexOf: $' startingAt: lastIdx .
    ].
  sz := self size .
  ((lastIdx <= sz) and: [aStream isFull not]) ifTrue: [
    aStream nextPutAll: (self copyFrom: lastIdx to: sz) .
    ].
  ].
aStream _nextPut: $' 
%

! bugfix 39431: check when to return original string


category: 'Converting'
method:
asByteArray

	^ByteArray fromString: self
%

category: 'Converting'
method:
asDoubleByteSymbol

"Returns a symbol representation of the receiver."

^ DoubleByteSymbol withAll: self.
%

category: 'Converting'
method:
_asString

"Returns a (single byte) String representation of the receiver if all of the
 Characters in the receiver can be represented as single-byte Characters.
 Returns nil if any of the Characters in the receiver cannot be represented as
 single-byte Characters."

<primitive: 229> 

^ self _primitiveFailed: #_asString .
%

method:
asString

"Returns a (single byte) String representation of the receiver if all of the
 Characters in the receiver can be represented as single-byte Characters.
 Otherwise, returns the receiver."

| result |
(result := self _asString) ifNil:[ ^ self ].
^ result
%

category: 'Converting'
method:
_reduce

"Returns a reduced version of the receiver according to the range of the
 characters, as follows:
 
 For a DoubleByteString:
 Returns a (single byte) String representation of the receiver if all of the
 characters in the receiver can be represented as single-byte characters.
 Otherwise returns self.

 For A QuadByteString:
 Returns a (single byte) String representation of the receiver if all of the
 characters in the receiver can be represented as single-byte characters.
 Returns a DoubleByteString representation of the receiver if all of the
 characters in the receiver can be represented as double-byte characters.
 Otherwise returns self."

<primitive: 907> 

^ self _primitiveFailed: #_reduce .
%

category: 'Converting'
method:
asSymbol

"Returns a canonical Symbol representation of the receiver."

^ Symbol withAll: self .
%

category: 'Converting'
method:
asSymbolKind

"Returns a (single byte) Symbol representation of the receiver."

^ Symbol withAll: self 
%

! Fix 36125
category: 'Accessing'
method:
wordAt: anIndex

"Returns the integer value of the Character at anIndex in the receiver."

^ (self at: anIndex) asciiValue
%

method:
codePointAt: anIndex

"Returns the integer value of the Character at anIndex in the receiver."

^ (self at: anIndex) asciiValue
%


category: 'Updating'
method:
wordAt: anIndex put: aValue

"Stores the integer value aValue in the character cell of the receiver
 specified by anIndex. Return aValue."

self at: anIndex put: (Character withValue: aValue) .
^ aValue
%

method:
codePointAt: anIndex put: anInteger

^ self at: anIndex put: (Character withValue: anInteger)
%

category: 'Accessing'
method:
at: anIndex

"Returns the Character at anIndex."

^ self subclassResponsibility: #at:
%

category: 'Updating'
method:
at: anIndex put: aChar

") Stores aChar at the specified location and returns aChar."

^ self subclassResponsibility: #at:put:
%

category: 'Case-Sensitive Comparisons'
method:
= aCharCollection

"Returns true if corresponding Characters in the receiver and argument are equal
 and aCharCollection is comparable with the receiver, and aCharCollection is
 not a kind of Symbol.  Returns false otherwise.

 The comparison for 'equal' is case-sensitive .

 Note that 'kind of Symbol' means either an instance of Symbol or instance of
 MultiByteSymbol."

<primitive: 449>

(aCharCollection isKindOf: CharacterCollection)
   ifTrue:[ ^ aCharCollection = self]
   ifFalse:[ ^ false]
%

category: 'Accessing'
method:
numArgs

"Returns the number of arguments the receiver would take, if the receiver were
 a message selector."

self isInfix ifTrue: [ ^1 ].
self isKeyword ifTrue: [ ^self identicalOccurrencesOf: $: ].
^0
%

category: 'Searching'
method:
speciesForCollect

"(R) Returns a class, an instance of which should be used as the result
 of collect: or other projections applied to the receiver."

^ self subclassResponsibility: #speciesForCollect
%

category: 'Accessing'
method:
size

"Returns the number of Characters in the receiver"

^ self subclassResponsibility: #size
%

category: 'Updating'
method:
size: anInteger


"Changes the size of the receiver to anInteger Characters."

^ self subclassResponsibility: #size:
%

category: 'Accessing'
method:
valueAt: anIndex

"Returns the value of the (multi-byte) Character at anIndex."

^ (self at: anIndex) asciiValue
%

category: 'Copying'
method:
withLFs

"Returns a copy of the receiver with all back-slashes replaced by line-feeds."

^self copyReplaceAll: '\' with: (self class with: Character lf)
%

category: 'Compatibility'
method:
_at: anIndex equals: aCharCollection ignoreCase: aBoolean

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-insensitive."

^ aBoolean ifTrue:[ self at: anIndex equalsNoCase: aCharCollection ]
	  ifFalse:[ self at: anIndex equals: aCharCollection ]
%

category: 'Case-Insensitive Comparisons'
method:
at: anIndex equalsNoCase: aCharCollection

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-insensitive."

<primitive: 438>
anIndex _validateClass: Integer.
((anIndex <= 0) or: [(anIndex > self size)])
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex ].
aCharCollection _validateClass: CharacterCollection.
^ self _primitiveFailed: #at:equalsNoCase: args: { anIndex . aCharCollection }
%

category: 'Case-Sensitive Comparisons'
method:
at: anIndex equals: aCharCollection

"Returns true if aCharCollection is contained in the receiver, starting at
 anIndex.  Returns false otherwise.  The comparison is case-sensitive."

<primitive: 668>
anIndex _validateClass: Integer.
((anIndex <= 0) or: [(anIndex > self size)])
ifTrue: [ ^ self _errorIndexOutOfRange: anIndex ].
^ super at: anIndex equals: aCharCollection .
%

category: 'Case-Insensitive Comparisons'
method:
< aCharCollection

"Returns true if the receiver collates before the argument.
 Returns false otherwise.

 The comparison is case-insensitive."

"The ASCII letters A and a collate the same.
   'A' < 'a' returns false.
   'A' < 'b' returns true. "

<primitive: 440>

aCharCollection _validateClass: CharacterCollection .
^ aCharCollection > self
%

category: 'Case-Insensitive Comparisons'
method:
<= aCharCollection

"Returns true if the receiver collates before the argument or if all of the
 corresponding Characters in the receiver and argument are equal.
 Returns false otherwise.

 The comparison is consistent with that defined for the < method."

^(self > aCharCollection) not
%

category: 'Other Comparisons'
method:
asciiLessThan: aString

^ self subclassResponsibility: #asciiLessThan:
%

category: 'Case-Insensitive Comparisons'
method:
> aCharCollection

"Returns true if the receiver collates after the argument.  Returns false
 otherwise.

 The comparison is case-insensitive."

<primitive: 442>

aCharCollection _validateClass: CharacterCollection.
^ aCharCollection < self
%

category: 'Case-Insensitive Comparisons'
method:
>= aCharCollection

"Returns true if the receiver collates after the argument or if all of the
 corresponding Characters in the receiver and argument are equal.
 Returns false otherwise.

 The comparison is consistent with that defined for the < method."

^ (self < aCharCollection) not
%

category: 'Other Comparisons'
method:
lessThan: aString collatingTable: aTable

"Returns true if the receiver collates before the argument.

 The collating sequence is defined by aTable which must be
 a string with the same character size as the receiver.
 aTable is accessed with wordAt: to retrieve the collating
 sequence for a particular Character.  Use wordAt:put: to populate it.
 Characters beyond the range of aTable are collated using their raw character
 value."

<primitive: 359 >
aTable _validateSameCharSizeAs: self .
aString _validateClasses:{ String . MultiByteString }.
self _primitiveFailed: #lessThan:collatingTable: args: { aString . aTable }
%

category: 'Other Comparisons'
method:
greaterThan: aString collatingTable: aTable

"Returns true if the receiver collates after the argument.
 See lessThan:collatingTable: for description of aTable."

<primitive: 433 >
aTable _validateSameCharSizeAs: self .
aString _validateClasses:{ String . MultiByteString }.
self _primitiveFailed: #greaterThan:collatingTable: args: { aString . aTable }
%

category: 'Other Comparisons'
method:
greaterThanOrEqual: aString collatingTable: aTable

"Returns true if the receiver collates after or the same as the argument.
 The collating sequence is defined by aTable."

^ (self lessThan: aString collatingTable: aTable) not
%

category: 'Other Comparisons'
method:
lessThanOrEqual: aString collatingTable: aTable

"Returns true if the receiver collates before or the same as the argument.
 The collating sequence is defined by aTable."

^ (self greaterThan: aString collatingTable: aTable) not
%

category: 'Other Comparisons'
method:
equals: aString collatingTable: aTable

"Returns true if the receiver collates the same as the argument.
 See lessThan:collatingTable: for description of aTable.
 The collating sequence is defined by aTable."

<primitive: 446 >
aTable _validateSameCharSizeAs: self .
aString _validateClasses:{ String . MultiByteString }.
self _primitiveFailed: #equals:collatingTable: args: { aString . aTable }
%

category: 'Private'
method:
_validateSameCharSizeAs: aString

"Returns nil if self has same character size as aString,
 otherwise generates an error."

| cSize cls |
self _stringCharSize ~~ (cSize := aString _stringCharSize) ifTrue:[
  cSize == 1 ifTrue:[ cls := String ].
  cSize == 2 ifTrue:[ cls := DoubleByteString ].
  cSize == 4 ifTrue:[ cls := QuadByteString ].
  self _errorExpectedClass: cls 
].
^ nil
%

! deleted _idxCompareLessThan: v2.0

!deleted _idxCompareGreaterThan: v2.0

category: 'Case-Insensitive Comparisons'
method:
equalsNoCase: aCharCollection

"Returns true if corresponding Characters in the receiver and argument are equal
 and aCharCollection is comparable with the receiver, and aCharCollection is
 not a kind of Symbol.  Returns false otherwise.

 The comparison for equal is case-insensitive.

 Note that 'kind of Symbol' means either an instance of Symbol or instance of
 DoubleByteSymbol."

<primitive: 450>
| csiz |
csiz := aCharCollection _stringCharSize .
csiz ~~ 0 ifTrue:[
  self _primitiveFailed: #equalsNoCase: args: { aCharCollection } .
  ^ self _uncontinuableError
  ].
^super isEquivalent: aCharCollection
%

category: 'Case-Insensitive Comparisons'
method:
isEquivalent: aCharCollection
    "Returns true if the receiver is equivalent to aCharCollection.
    The receiver is equivalent to aCharCollection if the receiver
    contains the same Characters as aCharCollection regardless of case
    or internal representation.  For example, if $a is in
    aCharCollection, it is equivalent to any representation of an 'a'
    in the receiver's character set."

<primitive: 450>
| csiz |
csiz := aCharCollection _stringCharSize .
csiz ~~ 0 ifTrue:[
  self _primitiveFailed: #isEquivalent: args: { aCharCollection } .
  ^ self _uncontinuableError
  ].
^super isEquivalent: aCharCollection
%

category: 'Private'
method:
_findSmallString: subString startingAt: startIndex ignoreCase: aBoolean

"If a receiver contains subString beginning at some point at or after
 startIndex, this returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0."

<primitive: 30>
startIndex _validateClass: SmallInteger.
(startIndex < 1) | (startIndex > self size)
  ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { startIndex } ].
subString _validateClasses:{ String . MultiByteString }.
subString basicSize > 16272  "virtual machine constant, OBJ_BYTES_SIZE" ifTrue:[
  subString _halt: 'String too large for string search primitive.'
  ].
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_findSmallString:startingAt:ignoreCase:
       args: { subString . startIndex . aBoolean }
%

category: 'Private'
method:
_findLastSmallString: subString startingAt: startIndex ignoreCase: aBoolean

"Searches backwards through receiver    starting at startIndex  .
 Returns the index at which subString begins.  If the
 receiver does not contain subString, this returns 0."

<primitive: 451>

startIndex _validateClass: SmallInteger.
(startIndex < 1) | (startIndex > self size)
  ifTrue: [ ^ self _error: #objErrBadOffsetIncomplete args: { startIndex } ].
subString _validateClasses:{ String . MultiByteString }. 
subString basicSize > 16272 "virtual machine constant, OBJ_BYTES_SIZE" ifTrue:[ 
  subString error: 'String too large for string search primitive.' 
  ].
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_findLastSmallString:startingAt:ignoreCase:
       args: { subString . startIndex . aBoolean }
%

category: 'Case-Sensitive Searching'
method: 
includes: aCharacter

"Returns true if the receiver contains aCharacter, false otherwise.
 The search is case-sensitive."

^(aCharacter isKindOf: AbstractCharacter) and:
	[self includesValue: aCharacter].
%

category: 'Case-Sensitive Searching'
method:
includesValue: aCharacter

"Returns true if the receiver contains aCharacter.  The search is
 case-sensitive."

<primitive: 452>

"the following code is executed if aCharacter is a JISCharacter "

aCharacter _validateClass: AbstractCharacter .
^ self includesValue: aCharacter asCharacter .
%

category: 'Case-Sensitive Searching'
method:
indexOf: aCharacter startingAt: startIndex

"Returns the index of the first occurrence of aCharacter in the receiver,
 not preceding startIndex.  If the receiver does not contain aCharacter,
 returns zero.

 The search is case sensitive."

<primitive: 453>
startIndex _validateClass: Integer .
startIndex < 1 ifTrue:[ ^ self _errorIndexOutOfRange: startIndex].
^ self indexOf: aCharacter asCharacter startingAt: startIndex
%

category: 'Converting'
method:
asUppercase

"Returns a new instance of the receiver's class, with all lower-case Characters
 in the receiver changed to upper-case.  Lower-case Characters are the
 ASCII characters $a to $z inclusive.  If the receiver is a MultiByteSymbol, 
 returns an instance of MultiByteString."

<primitive: 455>

self _primitiveFailed: #asUppercase .
self _uncontinuableError
%

!  _compileInContext:symbolList: not supported yet

category: 'Execution'
method:
evaluate

"Compiles the receiver as an unbound method and executes it using the current
 default symbol list."

^ (self _compileInContext: nil 
       symbolList: GsSession currentSession symbolList)
  _executeInContext: nil
%

category: 'Execution'
method:
evaluateInContext: anObject symbolList: aSymbolList

"Compiles the receiver as an instance method for the class of anObject, using
 aSymbolList as the symbol list.  Executes the resulting GsMethod using anObject
 as self and returns the result of the execution.  Generates an error if
 compilation errors occur."

^ (self _compileInContext: anObject symbolList: aSymbolList)
    _executeInContext: anObject
%

category: 'Hashing'
method:
hash

"Returns a positive Integer based on a case-sensitive hash of the contents 
 of the receiver."

"The algorithm implemented is described in:

 [Pearson 90]
 Pearson, Peter K., Fast Hashing of Variable-Length Text Strings,
 Communications of the ACM 33, 6, (June 1990), 677-680."

<primitive: 282>
self _primitiveFailed: #hash .
self _uncontinuableError
%

category: 'Storing and Loading'
classmethod:
loadFrom: passiveObj

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| inst |

inst := String new: passiveObj readSize . "yes, make a String !!"
inst loadFrom: passiveObj.
inst _changeClassToMultiByte: self .  "Now change to double/quad byte"
^inst
%

category: 'Storing and Loading'
method:
writeTo: passiveObj

"Converts the receiver to its passive form and writes that information on
 passiveObj."

passiveObj writeClass: self class;
  writeSize: self _basicSize;
    nextPutAllBytes: self; space
%
category: 'Removing'
method:
removeFrom: startIndex to: stopIndex

"(R) Deletes the elements of the receiver from startIndex to stopIndex.  The
 size of the receiver is decreased by stopIndex - startIndex + 1.

 Both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex."

<primitive: 271>
(stopIndex < startIndex)
ifTrue:
   [ ^ startIndex _error: #rtErrBadCopyFromTo args: { stopIndex }].
((stopIndex > self size) or: [(stopIndex < 1)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].
(startIndex < 1)
   ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].
^ self _primitiveFailed: #removeFrom:to: args: { startIndex . stopIndex } 
%

category: 'Indexing Support'
method:
_changingSizeTo: newSize
 
"Notifies any modification tracking objects that the receiver (an indexable
 object) is having its size changed."
 
"the object manager reports size changes in number of bytes; convert this
 to number of characters."

^ super _changingSizeTo: newSize // (self charSize)
%
category: 'Testing'
method:
containsSeparator

"Returns true if the receiver contains a separator Character."

1 to: self size do: [:i |
  (self at: i) isSeparator ifTrue: [^true].
].
^false
%

category: 'New Indexing Comparison - prims'
method:
_idxPrimCompareLessThan: aCharCollection

"This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects.

 This method collates letters AaBb..Zz."

"The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #< .
 Same primitive as String>>lessThan: "

<primitive: 441>

aCharCollection == nil ifTrue: [ ^ false ].
aCharCollection _validateClass: CharacterCollection.
^ aCharCollection > self
%

category: 'New Indexing Comparison - prims'
method:
_idxPrimCompareGreaterThan: aCharCollection

"This comparison operation is used for the indexing subsystem to
 determine an ordering for insertion into indexing objects such as B-trees.

 This method collates letters AaBb..Zz."

"The comparison should be compatible with the case-insensitive semantics
 of the String method with selector #> .
 Same primitive as String>>greaterThan: ."

<primitive: 443>
aCharCollection == nil ifTrue: [ ^ true ].
aCharCollection _validateClass: CharacterCollection.
^ aCharCollection < self
%

category: 'Encoding'
method:
encodeUsing: encodingArray

  "A new instance of the class of the receiver will be returned where 
   each character in the receiver is replaced by the character or string 
   from encodingArray that is located at the position in encodingArray for  
   the character.  The ascii value of the character is used as an index
   into the encodingArray,  value 16r0 accesses   encodingArray at:1 .
   If the ascii value is >= size of the encodingArray,
   nil is returned.
   If the value from the encodingArray is a Character too large to
   fit in the receiver, or is a CharacterCollection with bytes-per-character 
   bigger than bytes-per-character of the receiver, 
   or is not a kind of CharacterCollection, nil is returned."

  <primitive: 654>
  ^ nil
%

category: 'Copying'
method:
_reverseFrom: aString

"copies elements of aString into receiver in reverse order.
 Size of aString must be same as size of receiver.
 Returns receiver. 
 aString==self is allowed. "

<primitive: 793>
aString _stringCharSize == self _stringCharSize ifFalse:[
  aString _error: #rtErrBadArgKind args:{ self class }.
].
aString size == self size ifFalse:[
  aString _error: #rtErrBadSize args:{ self size . aString size }.
].
self _primitiveFailed: #_reverseFrom args: { aString }
%

category: 'Copying'
method:
reverse

"Returns a copy of the receiver with its elements in reverse order."
| copy |
copy := self species new: self size .
^ copy _reverseFrom: self
%

category: 'Copying'
method:  
copyFrom: startIndex to: stopIndex

"Returns a new Array containing the elements of the receiver
 between startIndex and stopIndex, inclusive.  The result is of the same class
 as the receiver.

 If startIndex > stopIndex then an empty collection is returned.
 Otherwise both startIndex and stopIndex must be positive integers not larger than the
 size of the receiver, with startIndex <= stopIndex.
"

<primitive: 818>
(startIndex < 1) ifTrue: [ ^ self _errorIndexOutOfRange: startIndex].

((stopIndex > self size) or: [(stopIndex < 0)])
   ifTrue: [ ^ self _errorIndexOutOfRange: stopIndex].

self _primitiveFailed: #copyFrom:to: args: { startIndex . stopIndex } 
%

!
! 39484: next two methods moved up from DoubleByteString
!
category: 'Execution'
method:
_compileInContext: anObject symbolList: aSymbolList 
oldLitVars: litVarArray  environmentId: anEnvironmentId

"Compiles the receiver as an instance method for the class of
 anObject, using aSymbolList as the symbol list.

 If litVarArray is not nil, it must be an Array of SymbolAssociations,
 and this Array will be searched prior to searching aSymbolList to
 resolve literal variables within the method.

 anEnvironmentId must be a SmallInteger >= 0 and <= 255
 specifying a compilation environment.

 Returns a GsMethod that is not in the method dictionary of any class, or
 else generates an error if there are compilation errors."

<primitive: 150>
aSymbolList _validateClass: SymbolList .
^ self _primitiveFailed: #_compileInContext:symbolList:oldLitVars:environmentId:
       args: { anObject . aSymbolList . litVarArray . anEnvironmentId }
%

category: 'Execution'
method:
_compileInContext: anObject symbolList: aSymbolList

"Compiles the receiver as an instance method for the class of
 anObject, using aSymbolList as the symbol list, and using the
 default compilation environment.

 Returns a GsMethod that is not in the method dictionary of any class, or
 else generates an error if there are compilation errors."

^ self _compileInContext: anObject symbolList: aSymbolList oldLitVars: nil
      environmentId: 0
%
