!=========================================================================
! Copyright (C) VMware, Inc. 2008-2011.  All Rights Reserved.
!
! $Id: TCPSocket.gs 25988 2011-06-14 17:55:54Z stever $
!=========================================================================

set class TCPSocket

removeallmethods 
removeallclassmethods 

category: 'Ruby support'

classmethod:
new: aHostName port: portName

  "blocking connection attempt to portName on aHostName .
   The scheduler will allow other threads  to run of the connect blocks.
   The result will be a non-blocking Smalltalk socket, with
   isRubyBlocking==true , or nil if connection failed.
   portName maybe the name of a service such as 'http', or can be  a port number."
 
  | portNum hostAddr sock status |
  portNum := self resolvePort: portName .
  hostAddr := self getHostAddressByName: aHostName .
  hostAddr == nil ifTrue:[
    ^ self signalSocketError:'getHostAddressByName failed'.
  ].
  portNum == nil ifTrue:[ ^ self signalSocketError:'port name not found' ].
  sock := self new .

  status := sock connectTo: portNum on: hostAddr . "blocking connect"
  status ifFalse:[ ^ nil ].
  sock rubyBlocking: true .
  ^ sock .
%  

classmethod:
open: aHostName 
  "connection attempt to random port on aHostName .
   The scheduler will allow other threads  to run of the connect blocks.
   The result will be a non-blocking Smalltalk socket, with
   isRubyBlocking==true . "

  | portNum hostAddr sock status |
  hostAddr := self getHostAddressByName: aHostName .
  hostAddr == nil ifTrue:[
    ^ self signalSocketError:'getHostAddressByName failed'.
  ].
  sock := self new .
  portNum := sock bindTo: nil toAddress: hostAddr . "bind to a random port"
  portNum ifNil:[ self signalSocketError:'bind failed' ].
  status := sock connectTo: portNum on: hostAddr . "blocking connect"
  status ifFalse:[ self signalSocketError:'connect failed' ].
  sock rubyBlocking: true .
  ^ sock .
%  

classmethod:
resolvePort: portName

  "portName can be nil to specify a random port (in which case result is nil),  
   or can the name of a service such as 'http', or can be  a port number.

   Returns nil or a port number."

  | portNum |
  portName ~~ nil ifTrue:[
    (portName _isOneByteString) ifTrue:[
      portNum := self getServicePortByName: portName .
      portNum == nil ifTrue:[
        "portName is string, but not a service name; try as int"
         [ portNum := portName asInteger ] onSynchronous: AbstractException do:[ :ex|
             ^ self signalSocketError:'port not found' 
         ]
      ]
    ] ifFalse:[
       portName _isSmallInteger ifTrue: [ portNum := portName ]
    ].
    "At this point, portNum is either a number, or nil"
    (portNum _isSmallInteger and:[ portNum <= 0 or:[ portNum >= 65535]]) ifTrue:[
      ^ self signalSocketError:'invalid portName argument' 
    ].
  ].
  ^ portNum
%
