!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: userpro.gs 26905 2011-09-27 18:48:18Z lalmarod $
!
! Superclass Hierarchy:
!   UserProfile, Object.
!
!=========================================================================

removeallmethods UserProfile
removeallclassmethods UserProfile

category: 'For Documentation Installation only'
classmethod: UserProfile
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'Each instance of UserProfile contains a number of characteristics associated
 with a given system user.  For more information, see the GemStone System
 Administration Guide and the GemStone Programming Guide.'.
doc documentClassWith: txt.

txt := (GsDocText new) details:
'An Array of Symbols that identify the privileges (that is, the levels of
 access to certain privileged system functions) that may be assigned to a
 UserProfile.  See UserProfile | privileges for a list of privilege names and
 their associated privileged methods.'.
doc documentClassVar: #PrivilegeNames with: txt.

txt := (GsDocText new) details:
'A String that identifies the user to the system at login; limited to
 1024 Characters.  Methods in this class enforce uniqueness by the value of
 all user IDs of UserProfiles in AllUsers.'.
doc documentInstVar: #userId with: txt.

txt := (GsDocText new) details:
'An Array of SymbolDictionaries that are used for resolving compile-time
 Symbols.'.
doc documentInstVar: #symbolList with: txt.

txt := (GsDocText new) details:
'The GsObjectSecurityPolicy in which new objects are stored after each login.'.
doc documentInstVar: #defaultObjectSecurityPolicy with: txt.

txt := (GsDocText new) details:
'A SmallInteger describing the level of access to certain privileged system
 functions that are ordinarily performed by the GemStone data curator.'.
doc documentInstVar: #privileges with: txt.

txt := (GsDocText new) details:
'An IdentitySet; the groups to which the user belongs.'.
doc documentInstVar: #groups with: txt.

txt := (GsDocText new) details:
'Reserved for future use.'.
doc documentInstVar: #spare1 with: txt.

txt := (GsDocText new) details:
'Reserved for future use.'.
doc documentInstVar: #spare2 with: txt.

txt := (GsDocText new) details:
'Reserved for future use.'.
doc documentInstVar: #spare3 with: txt.

txt := (GsDocText new) details:
'Methods in this category are obsolete and are provided only for compatibility
 with earlier releases of GemStone.  They will be removed in a future release.'.
doc documentCategory: #'Deprecated' with: txt.

txt := (GsDocText new) details:
'Methods in this category are obsolete and are provided only for compatibility
 with earlier releases of GemStone.  They will be removed in a future release.'.
doc documentClassCategory: #'Deprecated' with: txt.

self description: doc.
%

category: 'Instance Creation'
classmethod: UserProfile
new

"Disallowed.  To create a new UserProfile, use newWithUserId:... instead."

self shouldNotImplement: #new
%


category: 'Accessing Privileges'
method: UserProfile
_validateCodeModificationPrivilege

"Generates an error if the receiver does not have the CodeModification
 privilege.  This uses hardcoded bit number to handle filein and upgrade
 cases."

System _inProtectedMode ifFalse: [
  (privileges bitAt: 12) == 1 ifFalse:[ 
    self _error: #rtErrNoPriv .
    self _uncontinuableError.
    ^ false
  ] .
].
^ true
%

category: 'Private'
classmethod: UserProfile
_validatePrivilege

" You must have the #OtherPassword to modify UserProfiles "

  ^ System myUserProfile _validatePrivilegeName: #OtherPassword
%

category: 'Private'
method: UserProfile
_validateIsMyUserProfileOrHasPrivilege: aPrivSymbol

"Validate that the receiver is my own UserProfile or that I have the
 privilege aPrivSymbol."

| myUserProfile |

myUserProfile := System myUserProfile .
self == myUserProfile ifFalse:[ 
  ^ myUserProfile _validatePrivilegeName: aPrivSymbol 
].
^ true
%

category: 'Private'
method: UserProfile
_validateUserProfileAccess

"Validate that the receiver is my own UserProfile or that I have the
 #OtherPassword privilege."

^ self _validateIsMyUserProfileOrHasPrivilege: #OtherPassword
%

category: 'Private'
method: UserProfile
_initialize

"Private.  Initialize private instance variables."

self _validatePrivilege.
%

category: 'Deprecated'
classmethod: UserProfile
newWithUserId: userIdString
password: passwordString
defaultSegment: anObjectSecurityPolicy
privileges: anArrayOfStrings
inGroups: aCollectionOfGroupStrings

"Deprecated - use methods in the Instance Creation category instead" 

self deprecated: 'UserProfile class>>newWithUserId:password:defaultSegment:privileges:inGroups: deprecated v3.0.  
Use #newWithUserId:password:defaultObjectSecurityPolicy:privileges:inGroups: instead.'.

^self
	newWithUserId: userIdString
	password: passwordString
	defaultObjectSecurityPolicy: anObjectSecurityPolicy
	privileges: anArrayOfStrings
	inGroups: aCollectionOfGroupStrings
%
category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString
password: passwordString
defaultObjectSecurityPolicy: anObjectSecurityPolicy
privileges: anArrayOfStrings
inGroups: aCollectionOfGroupStrings

"Creates a new UserProfile with the associated characteristics.  In so doing,
 creates a symbol list with three dictionaries: UserGlobals, Globals, and
 Published.  The first Dictionary (UserGlobals) is created for the user's
 private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group 'Subscribers' and of the groups in aCollectionOfGroupStrings.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 anObjectSecurityPolicy must be nil or a committed GsObjectSecurityPolicy.
"

| result newDict newSymList myAssociations mySymbols 
  userGroups newPrivSet publ |

self _validatePrivilege ifFalse:[ ^ nil ].
(userIdString _validateClasses: { String . DoubleByteString } ) ifFalse:[ ^ nil ].
(passwordString _validateClass: String) ifFalse:[ ^ nil ].
(passwordString equalsNoCase: userIdString) ifTrue:[ 
  ^ passwordString _error: #rtDisallowedPassword 
  ].
anObjectSecurityPolicy == nil ifFalse:[ 
  (anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy ) ifFalse:[ ^ nil ].
  anObjectSecurityPolicy isCommitted ifFalse:[ 
    anObjectSecurityPolicy _error: #rtErrObjMustBeCommitted .
    ^ nil
  ].
].
newPrivSet := UserProfile _privilegesSetFromArray: anArrayOfStrings .

(aCollectionOfGroupStrings _validateClass: Collection) ifFalse:[ ^ nil ].
userGroups := { }  .
aCollectionOfGroupStrings do: [ :aStr | | groupStr |
  (aStr _validateClass: CharacterCollection ) ifFalse:[ ^ nil ].
  (groupStr := AllGroups _validateGroupString: aStr)  ifNil:[ ^ nil ].
  userGroups add: groupStr .
  ].
userGroups add: 'Subscribers' . "by default all users are a member."

       "Create the new UserProfile and populate it using the given
        arguments."

GsObjectSecurityPolicy setCurrent: (Globals at: #AllUsers) objectSecurityPolicy  while:[ 
  | theUserId |
  result := super new _initialize .
  theUserId := userIdString asString copy immediateInvariant .
  result _userId: theUserId .
  result privileges: anArrayOfStrings.
  userGroups do: [ :each | result addGroup: each ].
].
result defaultObjectSecurityPolicy: anObjectSecurityPolicy .

   "Create UserGlobals Dictionary "
newDict := SymbolDictionary new.
newDict objectSecurityPolicy: anObjectSecurityPolicy .

   "Create all SymbolAssociations and Symbols and UserGlobals."

myAssociations := { } .
mySymbols := { } .
"it is important that the following Symbols are created by send withAll:
 instead of just using the literal, so that a new object will be
 generated each time this is run"
myAssociations add: (SymbolAssociation newWithKey: #UserGlobals
                                       value: newDict).
myAssociations do:[:assoc| 
  assoc objectSecurityPolicy: anObjectSecurityPolicy .
  newDict add: assoc 
].

"Create the new user's symbol list."

newSymList := SymbolList new .
newSymList add: newDict; add: Globals .
"fix 40939"
publ := (AllUsers userWithId: 'DataCurator') symbolList objectNamed: #Published.
publ ifNotNil: [newSymList add: publ].
result symbolList: newSymList.
newSymList objectSecurityPolicy: anObjectSecurityPolicy.

GsObjectSecurityPolicy setCurrent: (Globals at: #AllUsers) objectSecurityPolicy  while:[
  AllUsers add: result .  "add to AllUsers after all of above succeeded"
    " password must be assigned after adding to AllUsers"
  result password: passwordString.
].
^ result
%

category: 'Browser Methods'
method: UserProfile
_removeSymbol: aSymbol

"Remove aSymbol from the Symbol List by search the symbol list for an
 Association whose key is equal to aSymbol.  Returns true if successful.
 Otherwise no such Association was found, and it returns false."

| temp dict |
self _validatePrivilege ifTrue:[
  1 to: symbolList size do:[:j |
    dict := symbolList at: j .
    temp := dict at: aSymbol otherwise: nil .
    temp ifNotNil:[
      dict removeKey: aSymbol.
      ^ true
    ].
  ].
].
^ false
%

category: 'Browser Methods'
method: UserProfile
_createDictionary: aSymbol at: index

"This method creates a new SymbolDictionary named 'aSymbol' and inserts it
 in the user's symbol list at the given 'index'."

| newDict |

self _validatePrivilege ifTrue:[
  (aSymbol _validateClass: Symbol) ifTrue:[
    index _validateClass: Integer .
    newDict := SymbolDictionary new.
    newDict at: aSymbol put: newDict.
    self insertDictionary: newDict at: index.
  ].
].
^ newDict
%

! fix bug 8697
category: 'Browser Methods'
method: UserProfile
_renameDictionary: aDictionary to: newName

"Renames a SymbolDictionary in the user's symbol list from its current name
 to newName."

| oldName |

self _validatePrivilege ifFalse:[ ^ false ].
aDictionary _validateClass: SymbolDictionary.
(newName _validateClass: Symbol) ifFalse:[ ^ false ].

(self resolveSymbol: newName) ~~ nil ifTrue: [
   "There's already an object named newName - don't allow the rename"
   ^false ].

"Remove the old name (if there was one)"
oldName := aDictionary name.
oldName == nil ifFalse: [
    aDictionary removeKey: oldName ].
"Create the new name"
aDictionary at: newName put: aDictionary.
^true
%

!  fix 7603

category: 'Browser Methods'
method: UserProfile
_symbolListNamesAndDictionaries

"Returns an Array of Associations which name the dictionaries in the
 receiver's SymbolList."

| result |

result := { }  .
symbolList do: [ :aDict |
  result add: 
    ( aDict associationsDetect: [ :anAssoc | anAssoc value == aDict ]
             ifNone: [ SymbolAssociation newWithKey: #unnamed value: aDict ] ).
  ].
^ result. 
%

category: 'Deprecated'
method: UserProfile
defaultSegment

self deprecated: 'UserProfile>>defaultSegment deprecated v3.0. 
Use #defaultObjectSecurityPolicy instead.'.
^self defaultObjectSecurityPolicy.
%

category: 'Accessing'
method: UserProfile
defaultObjectSecurityPolicy

"Returns the default login GsObjectSecurityPolicy associated with the receiver."

^ defaultObjectSecurityPolicy
%

! fix bug 9732
category: 'Accessing'
method: UserProfile
groups

"Returns an IdentitySet, the set of groups to which the user belongs."

groups == nil ifTrue:[ ^ IdentitySet new ].
^ groups copy

%


category: 'Accessing'
method: UserProfile
nativeLanguage

"Returns a Symbol specifying the user's native language.  This value
 is no longer used by the base system, but is left for use by applications."

^ (self resolveSymbol: #NativeLanguage) ifNotNil:[ :assoc | assoc _value ]
					 ifNil:[ #English ]
%

category: 'Accessing'
method: UserProfile
userId

"Answers a String which is the user name of the receiver."

^ userId
%

category: 'Updating'
method: UserProfile
addGroup: aGroupString

"Adds the user to the group represented by aGroupString, and returns the
 receiver.  If the user already belongs to the group aGroupString, no action
 occurs.

 If aGroupString does not already exist in the global object AllGroups,
 generates an error."

| theGroup userGroups |

self _validatePrivilege ifTrue:[
  userGroups := groups .
  userGroups == nil ifTrue:[
    userGroups := IdentitySet new . 
    userGroups objectSecurityPolicy: self objectSecurityPolicy .  "fix 11447"
    ].
  (userGroups includesIdentical: aGroupString) ifFalse:[
    (theGroup := AllGroups _validateGroupString: aGroupString) ifNil:[ ^ nil ].
    groups ifNil:[ groups := userGroups ].
    groups add: theGroup .
    ] .
].
%

category: 'Deprecated'
method: UserProfile
defaultSegment: anObjectSecurityPolicy

 self deprecated: 'UserProfile>>defaultSegment: deprecated v3.0. 
Use #defaultObjectSecurityPolicy: instead.'.
 ^ self defaultObjectSecurityPolicy: anObjectSecurityPolicy
%

category: 'Updating'
method: UserProfile
defaultObjectSecurityPolicy: anObjectSecurityPolicy

"Redefines the default login GsObjectSecurityPolicy associated with the receiver, and returns
 the receiver.  The argument may be nil (which implies World write for 
 all newly created objects).

 If the receiver is the UserProfile under which this session is logged in,
 this method requires the DefaultObjectSecurityPolicy privilege.

 If the receiver is not the UserProfile under which this session is logged
 in, you must have write authorization for the GsObjectSecurityPolicy where the receiver
 resides.  Exercise extreme caution when executing this method.  If, at the
 time you commit your transaction, the receiver no longer had write
 authorization for anObjectSecurityPolicy, that user's GemStone session will be terminated
 and the user will be unable to log back in to GemStone."

anObjectSecurityPolicy ifNotNil:[ 
  (anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy) ifFalse:[ ^ nil].
] .
(self == System myUserProfile) ifTrue: [
  ^ self _defaultObjectSecurityPolicy: anObjectSecurityPolicy
].
defaultObjectSecurityPolicy := anObjectSecurityPolicy .
%

! comments and arg check to fix 34642
category: 'Updating'
method: UserProfile
nativeLanguage: aLanguageSymbol

"Redefines the user's native language to be aLanguageSymbol.
 aLanguageSymbol must be a Symbol or DoubleByteSymbol .

 If you use this method, you must commit, and the change will
 take effect in sessions which login using the receiver after the commit."

self _validatePrivilege ifTrue:[
  (aLanguageSymbol _validateClass: Symbol) ifTrue:[
    ((self resolveSymbol: #UserGlobals) value)
       at: #NativeLanguage
       put: aLanguageSymbol
  ].
].
%

category: 'Updating'
method: UserProfile
oldPassword: firstString
newPassword: secondString

"Modifies the receiver's password to be secondString.  Generates an error if
 either argument is not a String, if firstString is not the receiver's
 password, or if the receiver is not the UserProfile of the current session.
 Generates an error if secondString is equivalent to the userId of the
 receiver.  The new password (secondString) may not be longer than 
 1024 Characters.

 Raises an #rtErrAuthIllegalPasswordChange error if the authentication mode of
 the receiver is not set to #GemStone.

 This method allows you to change your own password.  To change the password of
 any other user, use password: instead.

 This method requires the UserPassword privilege."

 
<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[
  self == System myUserProfile ifFalse:[
    ^ self _error: #rtErrCantChangePassword
  ].

  (self _validatePrivilegeName: #UserPassword ) ifTrue:[
    "check old password"
    (self validatePassword: firstString) ifFalse:[ 
        ^ userId _error: #rtErrBadPassword.
    ].
    (self _validateNewPassword:  secondString) ifTrue:[
      self _changePasswordFrom: firstString to: secondString .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
]
%
category: 'Private'
method: UserProfile
_oldPasswordsIncludes: aString

"Returns true if the receiver's old passwords include the argument."

<protected primitive: 459>

self _primitiveFailed: #_oldPasswordsIncludes: args: { aString } .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_validateNewPassword: aString

"Generates an error if aString is not a valid new password for the receiver.
 Otherwise, returns true .  Note that the userId of the receiver is
 always a disallowed password."

<protected>

(aString equalsNoCase: userId) ifTrue:[ 
  aString _error: #rtDisallowedPassword  .
  ^ false
].
(AllUsers _validateNewPassword: aString ) ifFalse:[ ^ false ].

AllUsers disallowUsedPasswords ifTrue:[
  (self _oldPasswordsIncludes: aString) ifTrue:[ 
      aString _error: #rtDisallowedPassword .
      ^ false
  ].
].
^ true
%

category: 'Private'
method: UserProfile
_pendingReasonForDisabledAccount

" Returns either 'StaleAccount' or 'PasswordAgeLimit' if this 
  UserProfile will be disabled for that reason the next time it logs in.
  Otherwise returns nil."

| staleLimit passwordLimit |
self passwordNeverExpires ifTrue: [ ^ nil ].

" Check and see if we're using staleAccountLimits
  and if this account has exceeded the limit "
staleLimit := AllUsers staleAccountAgeLimit.
(staleLimit ~~ nil and: [ staleLimit > 0 ]) ifTrue: [
   | lastLogin |
   (lastLogin := self lastLoginTime) ifNotNil:[ 
         ((DateTime now) > (lastLogin addHours: staleLimit))
         ifTrue: [ ^ 'StaleAccount' ]]].

" Check and see if we're using passwordAging
  and if this account has exceeded the limit"
passwordLimit := AllUsers passwordAgeLimit.
(passwordLimit ~~ nil and: [ passwordLimit > 0 ]) ifTrue: [
      | lastPasswordChange |
      (lastPasswordChange := self lastPasswordChange) ifNotNil:[
            ((DateTime now) > (lastPasswordChange addHours: passwordLimit))
            ifTrue: [ ^ 'PasswordAgeLimit' ]]].
^ nil
%

category: 'Updating'
method: UserProfile
password: aString

"Modifies the receiver's password to be aString, and returns the receiver.  If
 the argument is not a String, generates an error.  
 Generates an error if aString is equivalent to the userId of the
 receiver.  aString may not be longer than 1024 Characters.

 This method allows you to change the password of another GemStone user.
 To change your own password, use oldPassword:newPassword: instead.

 Raises an #rtErrAuthIllegalPasswordChange error if the authentication mode of
 the receiver is not set to #GemStone.

 This method requires the #OtherPassword privilege."

<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    "no checks for format of new password, omit _validateNewPassword: "

    "disallow the receivers' userId as a password"
    (aString equalsNoCase: userId) ifTrue:[ 
      ^ aString _error: #rtDisallowedPassword 
    ].
    self _password: aString . "change the password"
    AllUsers addMsgToSecurityLog: '      new password set for ' , self userId , ' .' . 
  ].
] ensure:[
  prot _leaveProtectedMode
].
%

category: 'Private'
method: UserProfile
_password: aString

"Modifies the receiver's password to be aString, and returns the receiver.  If
 the argument is not a String, generates an error.  The new password (aString)
 may not be longer than 1024 Characters.

 Raises an #rtErrAuthIllegalPasswordChange error if the authentication mode of
 the receiver is not set to #GemStone.

 This method allows you to change the password of another GemStone user.
 To change your own password, use oldPassword:newPassword: instead.

 This method requires the #OtherPassword privilege."

<protected primitive: 309>
aString _validateClasses: { String . DoubleByteString } .
aString class isBytes ifFalse:[ aString _error: #objErrNotByteKind ] .
self _primitiveFailed: #_password: args: { aString } .
self _uncontinuableError
%

! fix 12574 
category: 'Updating'
method: UserProfile
removeGroup: aGroupString

"Removes the user from the group represented by aGroupString.  If aGroupString
 is not a group defined in the global collection AllGroups, generates an error.
 If the user does not belong to the group, no action occurs."

| theGroup |

self _validatePrivilege ifTrue:[
  (theGroup := AllGroups _validateGroupString: aGroupString) ifNotNil:[ 
    groups size > 0 ifTrue:[
      groups remove: theGroup ifAbsent:[ ^ self ].
    ]
  ].
].
%

! fixed bug 14712
category: 'Updating'
method: UserProfile
userId: newUserIdString

"Modifies the userId associated with the receiver to be newUserIdString.
 newUserIdString must not be the user ID of some other UserProfile in 
 AllUsers, or an error will be raised.  Has no effect if newUserIdString
 is equal to the current userId of the receiver.

 newUserIdString must be a kind of String , with size <= 1024 characters .

 WARNING: Be careful when using this method with UserProfiles which use an
          external authentication scheme.  Executing this method will 
          change the userId used with external authentication if a user ID
          alias has not been set.  See the method #userIdAlias: to set a
          alias for a UserProfile.

 This method may only be executed by SystemUser."
 
<primitive: 2001>
| prot |
prot := System _protectedMode .
[ | oldUserPro newUserId oldUserId duplicateUser|

  self _validatePrivilege ifFalse:[ ^ nil ].
  (newUserIdString _validateClass: String ) ifFalse:[ ^ nil ].
  newUserIdString size > 1024 ifTrue:[
    ^ newUserIdString _error: #errArgTooLarge args:{ 1024 }.
  ].

  (self class isSpecialUserId: userId)
    ifTrue:[ ArgumentError signal: 'illegal attempt to modify a special UserProfile' ].

  (self class isSpecialUserId: newUserIdString)
    ifTrue:[ ArgumentError signal: 'illegal attempt to assume the name of a special UserProfile' ].

  oldUserId := userId .
  newUserId := newUserIdString asString .
  oldUserId = newUserId ifTrue:[ 
    ^ self 
    ].
  oldUserPro := AllUsers userWithId: oldUserId ifAbsent:[ nil ].
     
  duplicateUser := AllUsers userWithId: newUserId ifAbsent:[ nil ].
  duplicateUser ~~ nil ifTrue:[ 
    ^ self _error: #rtErrUserIdAlreadyExists args: { newUserIdString }
    ].

  oldUserPro ~~ nil ifTrue:[
    AllUsers _oldUserId: oldUserId newUserId: newUserId for: self
    ].

  userId := newUserId .
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Updating'
method: UserProfile
_userId: newUserId

"Modifies the userId associated with the receiver to be aString."

self _validatePrivilege ifTrue:[
  userId := newUserId .
].
%

category: 'Updating'
method: UserProfile
_defaultObjectSecurityPolicy: anObjectSecurityPolicy

"Change the default login GsObjectSecurityPolicy of the receiver, which is the UserProfile of
 the current session.

 This method requires the #DefaultObjectSecurityPolicy privilege, 
 and requires write authorization for the receiver.

 This method does not change the GsObjectSecurityPolicy into which newly created objects
 are placed, that will happen after the current session commits
 at the next login using this UserProfile .
 See also System (C) >> currentObjectSecurityPolicy:  .
  
 Gs64 v2.2 , the requirement for write authorization for the receiver is new;
 it did not exist in Gemstone/S 6.x "

<primitive: 307>
anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy.
self _primitiveFailed: #_defaultObjectSecurityPolicy: 
     args: { anObjectSecurityPolicy } .
^ nil
%

category: 'Private'
method: UserProfile
_changePasswordFrom: firstString to: secondString

"Private.  Change the given password of the receiver, which is the UserProfile
 of the current session (firstString), to the new given password
 (secondString)."

"Tests AllUsers.disallowUsedPasswords, and if ok, saves firstString
  in encrypted form in receivers' UserSecurityData.oldPasswords. "

<protected primitive: 308>
firstString _validateByteClass: CharacterCollection.
secondString _validateByteClass: CharacterCollection.
self _primitiveFailed: #_changePasswordFrom:to: 
     args: { firstString . secondString } .
self _uncontinuableError
%

category: 'Deprecated'
method: UserProfile
dictionaryNames

self deprecated: 'Obsolete in GemStone/64.'.

"Returns a formatted String describing the position and name of each Dictionary
 in the receiver's symbol list.

 This method assumes that each Dictionary in the symbol list contains an
 Association whose value is that Dictionary.  If any Dictionary does not
 contain such an Association, it is represented in the result String as
 '(unnamed Dictionary)'."

^ symbolList namesReport
%

category: 'Clustering'
method: UserProfile
clusterDepthFirst

"This method clusters the receiver's user ID, password, and symbol list.
 Because the password and user ID are assumed to be byte objects, and because
 the symbol list may contain shared elements, no further traversal of the
 objects is done.  Returns true if the receiver has already been clustered
 during the current transaction; returns false otherwise."

self cluster
  ifTrue: [ ^ true ]
  ifFalse: [
    encryptedPassword cluster.
    userId cluster.
    symbolList cluster.
    ^ false
  ]
%

category: 'Updating Privileges'
method: UserProfile
_privileges: aSmallInteger

"Modifies the user's privileges (level of access to privileged system
 functions) to the level represented by the argument aSmallInteger."

self _validatePrivilege ifTrue:[
  (aSmallInteger _validateInstanceOf: SmallInteger ) ifTrue:[
    privileges := aSmallInteger
  ].
].
%

category: 'Updating Privileges'
method: UserProfile
addPrivilege: aPrivilegeString

"Adds aPrivilegeString to the receiver's privileges.  Generates an error if
 aPrivilegeString is not a valid privilege name (as defined in the class
 variable PrivilegeNames).

 Privileges may not be addedd to special UserProfiles.  Attempts to do so 
 will silently fail.

 Example: 'System myUserProfile addPrivilege: #GarbageCollection' "

<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifTrue:[
    self _addPrivilege: aPrivilegeString.
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

! 40937 - disallow messing with the privileges of special UserProfiles.

category: 'Updating Privileges'
method: UserProfile
_addPrivilege: aPrivilegeString

< protected >
| index privSet privSymbol |
privSet := UserProfile _privilegesSetFromArray: { aPrivilegeString }.
privSymbol := privSet _at: 1 .
(index := PrivilegeNames indexOf: privSymbol ) == 0 ifTrue:[
  aPrivilegeString _error: #rtErrBadPriv .  
  ^ self 
].
(self class isSpecialUserId: userId) ifTrue:[  
  (InversePrivileges indexOf: privSymbol) ~~ 0 ifTrue:[
    ^ self "do not set inverse privs in a special user"
  ].
].
privileges := privileges bitOr:( 1 bitShift: index - 1) .
%

category: 'Updating Privileges'
method: UserProfile
deletePrivilege: aPrivilegeString

"Removes aPrivilegeString from the receiver's privileges.  Generates an error
 if aPrivilegeString is not a valid privilege name (as defined in the class
 variable PrivilegeNames).

 Privileges may not be deleted from special UserProfiles.  Attempts to do so 
 will silently fail."

"Example: 'System myUserProfile deletePrivilege: #GarbageCollection'"

<primitive: 2001> "enter protected mode"
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifTrue:[
    self _deletePrivilege: aPrivilegeString.
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

! 40937 - disallow messing with the privileges of special UserProfiles.
category: 'Updating Privileges'
method: UserProfile
_deletePrivilege: aPrivilegeString

< protected >
| index privSet privSymbol |

privSet := UserProfile _privilegesSetFromArray: { aPrivilegeString }.
privSymbol := privSet _at: 1 .
(index := PrivilegeNames indexOf: privSymbol ) == 0 ifTrue:[
  aPrivilegeString _error: #rtErrBadPriv .
  ^ self
].
(self class isSpecialUserId: userId) ifTrue:[ 
  (InversePrivileges indexOf: privSymbol) == 0 ifTrue:[
     "not an inverse priv, so do not clear from special user"
     ^ self
  ].
].
privileges := privileges bitAnd:( 1 bitShift: index - 1) bitInvert
%

category: 'Updating Privileges'
method: UserProfile
privileges: anArrayOfStrings

"Redefines the receiver's privileges to be those specified by anArrayOfStrings.
 Any privileges not contained in anArrayOfStrings will be turned off.
 Generates an error if any element of anArrayOfStrings is not a valid privilege
 name (as defined in the class variable PrivilegeNames)."

"Example: System myUserProfile privileges: #('SessionAccess' 'DefaultObjectSecurityPolicy')"


| privSet myUserProfile |
self _validatePrivilege ifFalse:[ ^ nil ].
myUserProfile := ( self == System myUserProfile ).
privSet := UserProfile _privilegesSetFromArray: anArrayOfStrings .

" clear all privileges -- unless this is me, 
  in which case leave #OtherPassword on "

myUserProfile 
    ifTrue: [ privileges :=  
              ( 1 bitShift: ( PrivilegeNames indexOf: #OtherPassword ) - 1 ) ]
    ifFalse: [ privileges := 0 ].

privSet do: [ :aPrivSymbol |
   self addPrivilege: aPrivSymbol .
   ].

" if me, clean up the #OtherPassword flag "
myUserProfile ifTrue: [
    ( privSet includes: #OtherPassword ) ifFalse: [
        self deletePrivilege: #OtherPassword ] ].
%

category: 'Private'
classmethod: UserProfile
_privilegesSetFromArray: anArrayOfStrings

"Convert an Array of Strings to an IdentitySet of legal Privilege names.
 Raises an error if anArrayOfStrings is not an Array, or if any String
 within anArrayOfStrings cannot be translated to a canonical Symbol
 of a legal privilege name, otherwise returns an IdentitySet of Symbols."

| result |
(anArrayOfStrings _validateClass: Array) ifFalse:[ ^ nil ].
result := IdentitySet new .
anArrayOfStrings do:[ :aString | | aSymbol |
  aSymbol :=  Symbol _existingWithAll: aString .
  aSymbol ifNil:[ aString _error: #rtErrBadPriv ].
  "translate Segment for compatibility."
  aSymbol == #DefaultSegment ifTrue:[ aSymbol := #DefaultObjectSecurityPolicy ] .
  aSymbol == #SegmentCreation ifTrue:[ aSymbol := #ObjectSecurityPolicyCreation ] .
  aSymbol == #SegmentProtection ifTrue:[ aSymbol := #ObjectSecurityPolicyProtection ] .
  (PrivilegeNames includesIdentical: aSymbol) ifFalse:[
    aString _error: #rtErrBadPriv .  
    ^ self 
  ].
  result add: aSymbol 
].
^ result
%

category: 'Updating'
method: UserProfile
createDictionary: aSymbol

"Creates a new SymbolDictionary. The new Dictionary is created with a single
 SymbolAssociation, whose key is aSymbol and whose value is the new 
 SymbolDictionary itself.

 Also creates a SymbolAssociation in the receiver's UserGlobals dictionary, with
 aSymbol as the key and the new dictionary as its value.

 Returns the new SymbolDictionary. Generates an error if aSymbol is already
 defined in the receiver's symbol list, or if aSymbol is not a Symbol."

| newDictionary anAssoc userGlobalsOfMe |

self _validatePrivilege ifFalse:[ ^ nil ].
(aSymbol _validateClass: Symbol) ifFalse:[ ^ nil ].

anAssoc := self resolveSymbol: aSymbol.
 (anAssoc == nil)
    ifFalse:[ ^ self _error: #rtErrSymAlreadyDefined args: { aSymbol }].

newDictionary := (SymbolDictionary new).  "Create new dictionary"
"Name it by making a reference to itself"
newDictionary at: aSymbol put: newDictionary.

userGlobalsOfMe := (self resolveSymbol: #UserGlobals) value.
"Get my UserGlobals"
userGlobalsOfMe at: aSymbol put: newDictionary.

^newDictionary
%

category: 'Updating the Symbol List'
method: UserProfile
symbolList: aSymbolList

"Modifies the list of SymbolDictionaries associated with the receiver.
 If the receiver is identical to 'GsSession currentSession userProfile',
 replaces the contents of the session transient symbol
 list with the contents of aSymbolList.  If an error occurs as a result
 of the modification to the persistent symbol list, the transient list
 is left unmodified and the error is handled immediately.  If an error
 occurs as a result of the modification of the transient symbol list,
 the persistent symbol list is left in its new state, the transient
 symbol list is left in its old state, and the error is silently ignored.

 Note that if the transient and persistent lists have different contents when
 an abort transaction occurs they will not be automatically synchronized
 after the abort.  The persistent list will be rolled back to the
 committed state, but the transient list will not be rolled back."

"relax privilege to not require #OtherPassword if doing this to ourself"
(self == System myUserProfile
  ifTrue: [ self _validatePrivilegeName: #CodeModification ]
  ifFalse: [ self _validatePrivilege ]) ifTrue:[

  (aSymbolList _validateClass: SymbolList ) ifTrue:[
      symbolList ifNil:[  
         symbolList := aSymbolList 
      ] ifNotNil:[ "must preserve old identity to avoid authorization errors"
         symbolList ~~ aSymbolList ifTrue:[		"fix 41658"
           symbolList size: 0.
           symbolList addAll: aSymbolList .
         ]
      ] .
      (self == GsSession currentSession userProfile) ifTrue: [
         [ 
           GsSession currentSession symbolList replaceElementsFrom: symbolList  
         ] onException: Error do:[:ex | ^ nil ].
      ].       
  ].
].
^ self
%

category: 'Updating the Symbol List'
method: UserProfile
insertDictionary: aSymbolDictionary at: anIndex

"Inserts aSymbolDictionary into the receiver's symbol list.  If the
 receiver is identical to 'GsSession currentSession userProfile', inserts
 aSymbolDictionary into the transient session symbol list as well.  The
 insertion into the receiver's symbol list occurs first.

 If anIndex is less than or equal to the size of the receiver's symbol
 list, inserts aSymbolDictionary into the symbol list at anIndex.

 If anIndex is 1 greater than the size of the receiver's symbol list,
 appends aSymbolDictionary to the receiver's symbol list.

 If anIndex is  more than 1 greater than the size of the receiver's
 symbol list, or if anIndex is less than 1, generates an error.

 If an error occurs as the result of the persistent insertion, no
 further action is taken and an error is generated.  If the insertion
 completes correctly, aSymbolDictionary is inserted into the transient
 session symbol list.

 If anIndex is 1, prepend aSymbolDictionary to the beginning of the
 transient symbol list.

 Otherwise, finds the dictionary at (anIndex - 1) in the persistent symbol
 list, and searches for it by identity in the transient symbol list.  If
 found, insert aSymbolDictionary just after it in the transient symbol
 list.  If not found, append aSymbolDictionary to the end of the transient
 symbol list.

 If an error occurs as a result of the insertion in the transient symbol
 list, the persistent symbol list is left in its new state, the
 transient symbol list is left in its old state and the error is
 silently ignored.

 Note if the transient and persistent lists have different contents when
 an abort transaction occurs they will not be automatically synchronized
 after the abort.  The persistent list will be rolled back to the
 committed state, but the transient list will not be rolled back."

"Example: System myUserProfile
                        insertDictionary: (SymbolDictionary new) at: 3'"
| transient dict index |

(self == System myUserProfile
  ifTrue: [ self _validatePrivilegeName: #CodeModification  ]
  ifFalse: [ self _validatePrivilege ]) ifTrue:[

  (aSymbolDictionary _validateClass: SymbolDictionary) ifTrue:[
    self symbolList insertObject:  aSymbolDictionary at: anIndex.
    (self == GsSession currentSession userProfile) ifTrue: [
      transient := GsSession currentSession symbolList.
      anIndex == 1 ifTrue: [
	transient insertObject: aSymbolDictionary at: 1.
	]
      ifFalse: [
	dict := self symbolList at: (anIndex - 1).
	index := transient indexOfIdentical: dict.
	index ~~ 0 ifTrue: [
	  transient insertObject: aSymbolDictionary at: index + 1.
	  ]
	ifFalse: [
	  transient add: aSymbolDictionary.
	  ].
	].
      ].
  ]
].
^self.
%

category: 'Updating the Symbol List'
method: UserProfile
removeDictionaryAt: anIndex

"Removes the SymbolDictionary at position anIndex from the receiver's symbol
 list, thus decreasing the size of the receiver's symbol list by one,
 and, if the receiver is identical to 'GsSession currentSession
 userProfile', removes the SymbolDictionary at position anIndex from the
 transient symbol list's symbol list (subject to the constraints below).
 Returns the receiver's dictionaryNames String.

 Generates an error if anIndex is not both a positive integer and less than the
 size of the receiver's symbol list.  If an error occurs in removing
 from the receiver's symbol list, the transient symbol list is left
 alone, and the error is handled immediately.

 If the removal from the receiver's symbol list succeeds, search in the
 transient symbol list for a SymbolDictionary identical to the one
 removed from the transient symbol list, and if found, remove it from
 the list.  Removes only the first such element.  If no such dictionary
 is found, silently return.

 Note if the transient and persistent lists have different contents when
 an abort transaction occurs they will not be automatically synchronized
 after the abort.  The persistent list will be rolled back to the
 committed state, but the transient list will not be rolled back."

"Example: System myUserProfile insertDictionary: (SymbolDictionary new) at: 3.
                  System myUserProfile dictionaryNames.
                  System myUserProfile removeDictionaryAt: 3.
                  System myUserProfile dictionaryNames."

| dict |
(self == System myUserProfile
  ifTrue: [ self _validatePrivilegeName: #CodeModification  ]
  ifFalse: [ self _validatePrivilege]) ifTrue:[

   (anIndex _validateClass: Integer) ifTrue:[
     dict := self symbolList removeAtIndex: anIndex.
     (self == GsSession currentSession userProfile) ifTrue: [
             GsSession currentSession symbolList removeIdentical: dict ifAbsent: [ ]
     ].
  ].
].
^self dictionaryNames.
%

category: 'User Authorization'
method: UserProfile
validatePassword: aString

"This method allows an application to validate an operation that requires
 authentication by multiple individuals.  Returns true if aString is the
 password of the receiver, returns false otherwise.  

 Generates an error if the argument is not a String.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 306>
aString _validateByteClass: CharacterCollection.
self _primitiveFailed: #validatePassword: args: { aString } .
self _uncontinuableError
%

category: 'Accessing the Symbol List'
method: UserProfile
dictionaryAndSymbolOf: anObject

"Returns the symbol list Dictionary that names anObject, and also returns the
 name which that Dictionary associates with anObject.  More precisely, this
 returns an Array containing two elements:

 * The Dictionary in the user's symbol list that contains an Association whose
   value is anObject.
 * The Symbol which is that Association's key.

 The symbol list is searched in the same order that the compiler searches it.
 (For more information about symbol resolution, see the GemStone Programming
 Guide.)  If anObject is not found in the symbol list, returns nil."

^ symbolList dictionaryAndSymbolOf: anObject
%

category: 'Accessing the Symbol List'
method: UserProfile
resolveSymbol: aString

"Searches the receiver's symbol list for an Association whose key is equal to
 aString, and returns that Association.  If no such Association is found in the
 symbol list, this returns nil."

^ symbolList resolveSymbol: aString
%

category: 'Accessing the Symbol List'
method: UserProfile
symbolList

"Whenever the compiler is invoked, tokens in the method's source are bound to
 either Symbols found in the source, or to SymbolAssociations found in one of a
 number of SymbolDictionary passed to the compiler.  This method returns
 the Array of such SymbolDictionary that are associated with the
 receiver."

^ symbolList
%

category: 'Accessing the Symbol List'
method: UserProfile
symbolResolutionOf: aString

"Searches the receiver's symbol list for aString.  If aString is found, returns
 a formatted String describing the position in the symbol list of the
 Dictionary defining aString, the name of that Dictionary, and aString.

 Generates an error if aString is not defined for the receiver."

"Example: System myUserProfile symbolResolutionOf: #Integer"

^ self symbolList symbolResolutionOf: aString
%

category: 'Accessing Privileges'
method: UserProfile
_privileges

"Returns a SmallInteger specifying the user's level of access to certain
 privileged system functions ordinarily performed by the GemStone data
 curator."

"SystemControl (bit 0); ObsoleteStatistics (1); SessionAccess (2);  UserPassword (3);
 DefaultObjectSecurityPolicy (4); ReservedForFutureUse (5);  OtherPassword (6); 
 ObjectSecurityPolicyCreation (7); ObjectSecurityPolicyProtection (8); FileControl (9);
 GarbageCollection (10); CodeModification (11) ;
 NoPerformOnServer (12); NoUserAction (13); NoGsFileOnServer (14);
 NoGsFileOnClient (15); SessionPriority (16) "

^ privileges
%

category: 'Accessing Privileges'
method: UserProfile
_validatePrivilege: anInt

"Generates an error if the receiver does not have the privilege specified
 by anInt.  anInt is one based"

(privileges bitAt: anInt ) == 1 ifFalse:[ 
  self _error: #rtErrNoPriv .
  self _uncontinuableError .
  ^ false
].
^ true
%

category: 'Accessing Privileges'
method: UserProfile
_validatePrivilegeName: aSymbol

"Generates an error if the receiver does not have the privilege specified
 by aSymbol, or if aSymbol is not the name of a privilege."

 | bitNumber pNames |
 (pNames := PrivilegeNames) ifNil: [ ^ true ].         "filein bootstrap case"
 bitNumber := (pNames indexOfIdentical: aSymbol) .
 bitNumber < 1 ifTrue:[
   self _error: #rtErrNoPriv .
   self _uncontinuableError.
   ^ false
 ].
 ^ self _validatePrivilege: bitNumber
%


category: 'Accessing Privileges'
method: UserProfile
privileges

"Returns an Array of Symbols describing the receiver's privileges.  Those
 privilege Symbols specify the user's level of access to certain privileged
 system functions ordinarily performed by the GemStone data curator.

 See individual methods for specific required privileges.  For more
 information about privileges, see the GemStone Programming Guide.

 The following privileges are granted to UserProfiles, to allow them to
 perform privileged methods: 

 DefaultObjectSecurityPolicy
   DefaultSegment - translated to DefaultObjectSecurityPolicy 
 FileControl
 GarbageCollection
 OtherPassword
 ObjectSecurityPolicyCreation
   SegmentCreation - translated to ObjectSecurityPolicyCreation
 ObjectSecurityPolicyProtection
   SegmentProtection - translated to ObjectSecurityPolicyProtection
 SessionAccess
 SystemControl
 UserPassword
 CodeModification
 SessionPriority

 The following four privileges are inverse privilege; the operations can be
 performed unless the privilege is given to the user.  UserProfiles with 
 these privileges cannot perform the function.

 NoPerformOnServer 
    prevents perform of  System class >> performOnServer:

 NoUserAction
    prevents loading of any user action library.

 NoGsFileOnServer
    prevents any GsFile operation which accesses a file on the server,
    i.e. access from the gem process to the file system.

 NoGsFileOnClient
    prevents any GsFile operation which accesses a file on the client,
    i.e. access from the remote GCI process to the file system.

 The state of NoUserAction, NoGsFileOnServer, and NoGsFileOnClient are 
 cached in the Virtual Machine at session login, and changes to a 
 UserProfile for these 3 privileges will only be seen in sessions that 
 login after those changes have been committed.

 "

| result |

result := { } .

1 to: (PrivilegeNames size) do: [:counter |
    (privileges bitAt: counter) = 1 ifTrue: [
       result add: (PrivilegeNames at: counter).
    ].
].
^result
%

category: 'Accessing the Symbol List'
method: UserProfile
objectNamed: aSymbol

"Returns the first object in the receiver's symbol list that has the given
 name.  If no object is found with the given name, this returns nil."

| assn |
assn := symbolList resolveSymbol: aSymbol.
assn == nil ifTrue: [ ^nil ].
^assn value
%

category: 'Storing and Loading'
method: UserProfile
writeTo: aPassiveObject

"Instances of UserProfile cannot be converted to passive form.  This method
 writes nil to aPassiveObject and stops GemStone Smalltalk execution with a
 notifier."

  aPassiveObject writeObject: nil.
  self _error: #rtErrAttemptToPassivateInvalidObject.
%

category: 'Repository Conversion'
method: UserProfile
_migrateGroups

"Converts the groups collection from a SymbolSet to an IdentitySet of Strings."

groups := AllGroups _migrateSymbolSet: groups .
%

category: 'Private'
method: UserProfile
_securityDataInstVarAt: offset

"Returns the value of the specified instance variable in the UserSecurityData 
 of the receiver.  If the value is not a special object, returns a copy of the
 value of that instance variable.  

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

 "offset 0 implements 'isDisabled' and returns true or false.
 The primitive fails if you attempt to access the password or the 
 oldPasswords instVars."

<protected primitive: 460>
self _primitiveFailed: #_securityDataInstVarAt: args: { offset } .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_securityDataInstVarAt: offset put: aValue

"Private.  Modifies the specified instance variable in the UserSecurityData of
 the receiver.  The sender is responsible for making a copy of aValue and
 making the copy invariant, if appropriate.  If aValue is not a special 
 object, the primitive will assign aValue to the SecurityDataObjectSecurityPolicy.  

 Generates an error if you do not have the #OtherPassword privilege.

 The primitive fails if you attempt to modify the password instance variable
 of the receiver's security data."

<protected primitive: 461>
self _primitiveFailed: #_securityDataInstVarAt:put: args: { offset . aValue } .
self _uncontinuableError
%

category: 'Private'
method: UserProfile
_newSecurityData

"Creates a new instance of UserSecurityData for the receiver."

" only called from UserProfileSet>>_add: "
<protected primitive: 462>

self _primitiveFailed: #_newSecurityData .
self _uncontinuableError
%

category: 'Accessing'
method: UserProfile
lastLoginTime

"Returns a DateTime of the last login time of the receiver, or nil if
 no login time has been recorded.  If no stale account limits are set
 on the receiver, then no login times are recorded.  (See updating methods
 in UserProfileSet.)

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 2 .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

! lastLoginTime: only updated by primitives

category: 'Accessing'
method: UserProfile
lastPasswordChange

"Returns a DateTime that specifies when the password was last changed.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: 3 .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

! lastPasswordChange: only updated by primitives

category: 'Accessing'
method: UserProfile
activeUserIdLimit

"Returns the maximum number of concurrent logins allowed for the receiver.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 4 .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
activeUserIdLimit: aPositiveInteger

"Sets the maximum number of concurrent logins for the receiver to be
 aPositiveInteger.  This change will take effect after this session commits.

 Generates an error if you do not have the #OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    aPositiveInteger ifNotNil:[
      aPositiveInteger _isSmallInteger ifFalse:[
        ^ aPositiveInteger _validateClass: SmallInteger .
      ].
      aPositiveInteger < 0 ifTrue:[
        ^ aPositiveInteger _error: #rtErrArgNotPositive 
      ].
    ].
  ].
  self _securityDataInstVarAt: 4 put: aPositiveInteger .
] ensure:[
  prot _leaveProtectedMode
]
%

category: 'Accessing'
method: UserProfile
loginsAllowedBeforeExpiration

"Returns the number of logins allowed before the receiver's password will
 expire.  Zero or nil means unlimited logins.  A positive number is the number
 remaining before the password will expire.  A negative number means the account
 has expired.  The internal value is reset to zero when the password is
 changed.

 This method will always return nil if the authentication scheme of the 
 receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."


<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    self authenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: 5 .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
loginsAllowedBeforeExpiration: aPositiveInteger

"Sets the number of logins allowed using the receiver before the receiver's
 password will expire.  Zero means unlimited logins.  A positive number is 
 the number of logins to be allowed before the current password will expire.  
 The internal value is reset to zero when the password is next changed.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if you do not have the #OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validateAuthenticationSchemeIsGemStone ifTrue:[
    self _validatePrivilege ifTrue:[
      aPositiveInteger ifNotNil:[
        aPositiveInteger _isSmallInteger ifFalse:[
          ^ aPositiveInteger _validateClass: SmallInteger .
        ].
        aPositiveInteger < 0 ifTrue:[
          ^ aPositiveInteger _error: #rtErrArgNotPositive 
        ].
      ].
      self _securityDataInstVarAt: 5 put: aPositiveInteger .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
]
%

! isSpecialUserId also implemented in accountIsNeverDisabled() in pom.c
category: 'Private'
classmethod: UserProfile
isSpecialUserId: aUserId

 aUserId = 'SystemUser'  ifTrue:[ ^ true ].
 aUserId = 'SymbolUser'  ifTrue:[ ^ true ].
 aUserId = 'GcUser'      ifTrue:[ ^ true ].
 aUserId = 'DataCurator' ifTrue:[ ^ true ].
 aUserId = 'Nameless'    ifTrue:[ ^ true ].
 ^ false 
%

category: 'Accessing'
method: UserProfile
passwordNeverExpires

"Returns true if the receiver is an account that is immune from password
 expiration, or if the receiver uses an authentication scheme other than
 #GemStone.

 The special UserProfiles (SystemUser, SymbolUser, DataCurator, GcUser,
 and Nameless) are immune to password expiration and always answer true.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

self authenticationSchemeIsGemStone
  ifFalse:[ ^ true] .

(self class isSpecialUserId: userId)
  ifTrue:[ ^ true ].

^self _fetchPasswordNeverExpires
%

category: 'Private'
method: UserProfile
_fetchPasswordNeverExpires

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 12 .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Private'
method: UserProfile
_setPasswordNeverExpires: aBoolean

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    result := self _securityDataInstVarAt: 12 put: aBoolean .
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
setPasswordNeverExpires: aBoolean

"Makes the receiver immune from all forms of password expiration.  Once
 set, the receiver is never required to change the password and will 
 never be denied access because too much time has passed since the last
 successful login.

 This method does not modify any password validity restrictions for
 the receiver such as maximum repeating characters, disallowed passwords,
 etc.

 Password expiration is always disabled for the special UserProfiles 
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless).  Attempting
 to enabled it for any of these UserProfiles will raise an error.  An 
 attempt to disable password expiration for a special user profile is
 ignored.

 Running this method raises an error if the receiver is not using the
 #GemStone authentication scheme.

 Requires the #OtherPassword privilege."

self authenticationSchemeIsGemStone
  ifFalse:[ ^ self _error: #rtErrAuthIllegalOperation ].

(self class isSpecialUserId: userId)
  ifTrue:[ aBoolean  ifTrue:[ ^ self ] "Do nothing" 
                    ifFalse:[ ^ self _error: #rtErrIllegalPasswordExpiration ].].

^ self _setPasswordNeverExpires: aBoolean
%


category: 'Accessing'
method: UserProfile
isDisabled

"Returns true if logins for the receiver are disabled, false otherwise. 

 Always returns false if the receiver uses an authentication scheme other than
 #GemStone.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self authenticationSchemeIsGemStone
    ifFalse:[ ^ false ].
  result := self _securityDataInstVarAt: 1 . "isDisabled"
  result ifFalse: [ result := self _pendingReasonForDisabledAccount ~~ nil ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Accessing'
method: UserProfile
reasonForDisabledAccount

"Returns the String that contains the reason why the receiver's password
 has been automatically disabled or expired. Some possible strings are:
  'LoginLimitReached'
     The account's password in no longer valid because its login limit
     was reached.  See UserProfile>>loginsAllowedBeforeExpiration.
  'LoginsWithInvalidPassword'
     The account is disabled due to an attempt to login to it too many
     times with an incorrect password.  See the stone configuration
     option STN_DISABLE_LOGIN_FAILURE_LIMIT.
  'PasswordAgeLimit'
     The account's password is no longer valid due to it being too
     old as determined by UserProfileSet>>passwordAgeLimit.
  'StaleAccount'
     The account is disabled due to it being too old
     as determined by UserProfileSet>>staleAccountAgeLimit.
  'DisabledByAdmin'
     The account was disabled by an Administrator using the 
     UserProfile>>disable method.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Raises an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateAuthenticationSchemeIsGemStone ifTrue:[
    result := self _securityDataInstVarAt: 6 .
    result ifNil:[ result := self _pendingReasonForDisabledAccount ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
reasonForDisabledAccount: aString

"Updates the String that contains the reason why the receiver's password
 has been automatically disabled or expired.  

 Raises an error if the authentication scheme of the receiver is not #GemStone
 or if you do not have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ 
  self _validateAuthenticationSchemeIsGemStone ifTrue:[
    self _validatePrivilege ifTrue:[
     (aString isKindOf: String) ifFalse:[
       ^ aString _validateClass: String 
      ].
      "Note: primitive now stores a copy of aString"
      result := self _securityDataInstVarAt: 6 put: aString .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating'
method: UserProfile
clearOldPasswords

"Clears the set of old passwords for the receiver, thus permitting reuse of
 some passwords that had been previously disallowed.  

 Generates an error if you do not have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    result := self _securityDataInstVarAt: 7 put: Array new .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Updating Privileges'
method: UserProfile
_validatePrivilege

" You must have the #OtherPassword to modify UserProfiles "

^  System myUserProfile _validatePrivilegeName: #OtherPassword
%

! 38848
category: 'Updating Privileges'
method: UserProfile
disableCommits

"Disables commits for the receiver if they were previously disabled.
 The current transaction must be committed to apply this change.

 Requires the #OtherPassword privilege.

 Has no effect on existing sesssions.

 Commits will be disabled for the receiver at the next login attempt
 which occurs AFTER the current transaction is committed.

 Disabling commits for the reserved system accounts
 (SystemUser, DataCurator, GcUser, SymbolUser and Nameless) is not allowed
 and will always fail.

 Returns true if the operation was successful or if commits were already
 disabled for the receiver.  Returns false if the operation failed."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ result := true.
  self _validatePrivilege ifTrue:[
    (self class isSpecialUserId: userId)
      ifTrue:[ result := false] "illegal"
      ifFalse:[
        self isReadOnly
          ifFalse:[ result := (self _securityDataInstVarAt: 8 put: true) == true]].
  ].
] ensure:[ 
    prot _leaveProtectedMode "exit protected mode"
].
^ result
%

category: 'Updating Privileges'
method: UserProfile
enableCommits

"Enables commits for the receiver if they were previously disabled.
 The current transaction must be committed to apply this change.

 Requires the #OtherPassword privilege.

 Has no effect on existing sesssions.

 Commits will be renabled for the receiver at the next login attempt
 which occurs AFTER the current transaction is committed.

 Disabling commits for the reserved system accounts
 (SystemUser, DataCurator, GcUser, SymbolUser and Nameless) is not allowed.
 Therefore enabling commits for these user profiles will always succeed.

 Returns true if the operation was successful or if commits were already
 enabled for the receiver."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ result := true.
  self _validatePrivilege ifTrue:[
     (self class isSpecialUserId: userId)
     ifFalse:[
      self isReadOnly
        ifTrue:[  result := (self _securityDataInstVarAt: 8 put: false) == false]].
  ]
] ensure:[
  prot _leaveProtectedMode "exit protected mode"
].
^ result
%

category: 'Accessing Privileges'
method: UserProfile
isReadOnly

"Returns true if commits have been disabled for this user profile by the
 #disableCommits method.  Otherwise returns false.
 
 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .

[ self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 8.
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ result == true
%

category: 'External Authentication - Private'
classmethod: UserProfile
_validateAuthenticationScheme: aSymbol

^ (self authenticationSchemeSymbolToId: aSymbol ) ~~ nil 
%

category: 'External Authentication - Private'
classmethod: UserProfile
authenticationSchemeIdToSymbol: aSmallInt

aSmallInt == 0 ifTrue:[ ^ #GemStone ] .
aSmallInt == 1 ifTrue:[ ^ #UNIX ] .
aSmallInt == 2 ifTrue:[ ^ #LDAP ] .
aSmallInt _validateClass: SmallInteger .
aSmallInt _error: #rtErrInvalidAuthSchemeId .
^ nil
%

category: 'External Authentication - Private'
classmethod: UserProfile
authenticationSchemeSymbolToId: aSymbol

aSymbol == #GemStone  ifTrue:[ ^ 0 ] .
aSymbol == #UNIX      ifTrue:[ ^ 1 ] .
aSymbol == #LDAP      ifTrue:[ ^ 2 ] .
aSymbol _validateClass: Symbol .
aSymbol _error: #rtErrInvalidAuthSchemeSymbol .
^ nil
%

category: 'External Authentication'
method: UserProfile
authenticationScheme

"Returns a Symbol representing the authentication scheme for the receiver,
 while will be either #GemStone, #UNIX, or #LDAP.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 9.
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ self class authenticationSchemeIdToSymbol: result
%

category: 'External Authentication'
method: UserProfile
userIdAlias

"Returns a String which is the alias used by the receiver 
 for external authentication, or nil if an alias is not used.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| prot result |
prot := System _protectedMode .
[ self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 10.
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].
^ result
%

category: 'External Authentication - Private'
method: UserProfile
authenticationScheme: aSymbol

"Private - customers should not call this method directly.

 Sets the athentication scheme for the receiver to match aString, which 
 must be one of  #GemStone, #UNIX, or #LDAP.  The system user profiles,
 SystemUser, DataCurator, GcUser, SymbolUser and nameless, must always use
 'GemStone' authentication.  Attempts to change the authentication scheme
 for these UserProfiles will raise an error.

 Requires the OtherPassword privilege."

self _securityDataInstVarAt: 9 
        put: (self class authenticationSchemeSymbolToId: aSymbol).
^ self
%

category: 'External Authentication - Private'
method: UserProfile
userIdAlias: aString

"Private - customers should not call this method directly.

 Sets the alias user ID for the receiver to be aString.  When external
 authentication is performed, the alias user ID will be used instead of
 the userId instance variable.  If no alias is specified, then the userId
 instance variable is used.

 If UNIX authentication is enabled for the receiver, then aString must
 be a valid UNIX user ID, otherwise an error will be raised.

 Requires the OtherPassword privilege."

self _securityDataInstVarAt: 10 put: aString .
^ self
%

category: 'External Authentication - Private'
method: UserProfile
_validateUnixUserIdOrNil: aString

"Confirm that aString is nil or is a valid UNIX user ID on this host"

aString ifNil:[ ^ true ].
(System unixUserIdExists: aString) ifFalse:[
  aString _error: #rtErrInvalidUnixUserId .
  ^ false
].
^ true
%

category: 'External Authentication'
method: UserProfile
enableUnixAuthenticationWithAlias: aUnixUserIdString

"Modify the receiver so that UNIX authentication is used to validate
 all logins to GemStone by the receiver.  After this method has
 been run and successfully committed, all future logins of the receiver
 must specify the password for the given UNIX user ID.  GemStone will then
 validate the password using UNIX password calls.

 If aUnixUserIdString is nil, then the UNIX user ID and the UserProfile
 ID of the receiver are assumed to be the same.  At the next login, 
 UNIX password authentication will be performed using the user ID of the
 receiver and the password specified during the login.

 Special UserProfiles must always use GemStone authentication.  Attempting
 to change the authentication scheme for a special UserProfile will raise
 an error.  See the method #isSpecialUserId: in this class to see which
 UserProfiles are considered special.

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[ self _validatePrivilege ifTrue:[
    (self _validateUnixUserIdOrNil: aUnixUserIdString ) ifTrue:[
      self authenticationScheme: #UNIX .
      aUnixUserIdString ifNotNil:[ self userIdAlias: aUnixUserIdString ].
    ].
  ]
] ensure:[
  prot _leaveProtectedMode  "exit protected mode"
].

^ self
%

category: 'External Authentication'
method: UserProfile
enableGemStoneAuthentication

"Modify the receiver so that GemStone authentication is used to validate
 logins to GemStone by the receiver.  After this method has
 been run and successfully committed, all future logins of the receiver
 must specify a valid GemStone password.  

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .

[ self _validatePrivilege ifTrue:[
    self authenticationScheme: #GemStone  .
    self userIdAlias: nil.
  ]
] ensure:[
prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'External Authentication'
method: UserProfile
authenticationSchemeIsGemStone

"Answer true if the receiver uses GemStone for password authentication.  
 Otherwise answer false.  

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #GemStone
%


category: 'External Authentication'
method: UserProfile
authenticationSchemeIsUNIX

"Answer true if the receiver uses UNIX for password authentication.  
 Otherwise answer false.  

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #UNIX
%

category: 'External Authentication'
method: UserProfile
authenticationSchemeIsLDAP

"Answer true if the receiver uses LDAP for password authentication.  
 Otherwise answer false.  

 Requires the #OtherPassword privilege for any UserProfile other than
 your own."

^ self authenticationScheme == #LDAP
%

category: 'External Authentication - Private'
method: UserProfile
_validateAuthenticationSchemeIsGemStone

"Raises an error if the authentication scheme for the recevier is not
 #GemStone, otherwise returns true. If execution continued from the
 error, returns false."

self authenticationSchemeIsGemStone ifFalse:[ 
   self _error: #rtErrAuthIllegalOperation 
                 args: { self authenticationScheme } .
   ^ false
].
^ true
%

category: 'External Authentication - Private'
method: UserProfile
_invalidDnError: aString

self _error: #rtErrIllegalDn args: { aString }  .
self _uncontinuableError
%

category: 'External Authentication - Private'
method: UserProfile
_validateBaseDn: baseDn searchDn: filterDn

"The baseDn must be a String.  If the filterDn is nil,
 then the baseDn must contain the '%s' character sequence.
 If the filterDn is not nil, then it must be a string that
 contains the '%s' sequence, and baseDn must not contain
 the sequence.  If all of these requirements are met
 returns true, otherwise signals an error.  If execution is
 continued from the error, returns false.
"
 
(baseDn _validateClass: String ) ifFalse:[ ^ false ].
filterDn ifNil:[ 
  baseDn hasLdapWildcardPattern ifFalse:[ 
     self _invalidDnError: baseDn . ^ false 
  ]. 
] ifNotNil:[ 
   (filterDn _validateClass: String ) ifFalse:[  ^ false ].
   filterDn hasLdapWildcardPattern
     ifFalse:[ self _invalidDnError: filterDn . ^ false ].
   baseDn hasLdapWildcardPattern
     ifTrue:[ self _invalidDnError: baseDn . ^ false ].
  ].
^ true
%

category: 'External Authentication - LDAP'
method: UserProfile
ldapBaseDn: aString searchFilterDn: stringOrNil

"Specify the base distinguished name (DN) and optionally the search filter
 DN for LDAP logins done by the receiver.  Has no meaningful effect unless
 LDAP authenticaion is enabled for the receiver.

 Requires the OtherPassword privilege."


<primitive: 2001>
| prot |
prot := System _protectedMode .

[ self _validatePrivilege ifTrue:[
    (self _validateBaseDn: aString searchDn: stringOrNil ) ifTrue:[
       self ldapBaseDn: aString ; ldapFilterDn: stringOrNil .
    ]
  ]
] ensure:[
prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'Accessing'
method: UserProfile
ldapBaseDn

"If the receiver is using LDAP authentication, returns a copy of the base
 distinguished name (DN) string.  Otherwise returns nil.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 13 .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Accessing'
method: UserProfile
ldapSearchFilterDn

"If the receiver is using LDAP authentication, returns a copy of the search
 filter string used to resolve the receiver complete distinguished name
 during LDAP authentication.  Otherwise returns nil.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |

prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 11 .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'External Authentication'
method: UserProfile
enableLDAPAuthenticationWithAlias: aString baseDn: baseDn filterDn: filterDn

"Modify the receiver so that LDAP authentication is used to validate
 all logins to GemStone by the receiver.  After this method has
 been run and successfully committed, all future logins of the receiver
 must specify the password for the given LDAP user ID.  GemStone will then
 validate the password using an LDAP bind.

 If aString is nil, then the UNIX user ID and the UserProfile
 ID of the receiver are identical.  At the next login, an LDAP bind
 will be performed using the user ID of the receiver and the password
 specified during the login.

 Most LDAP binds require a fully qualified DN (distinguished name) to be
 specified in order to get a successful bind.  There are 2 ways to get
 thee fully qualified DN: specify it as baseDn, or have GemStone perform
 a query to resovle the DN before attempting the bind.

 If filterDn is aString, this filter will be used to search for the fully
 qualified DN, and the string must include the character sequence '%s' in
 place of the user ID.  The baseDn argument must not contain the '%s'
 sequence in this case.

 If no DN search is to be performed, then nil should be passed in the filterDn
 argument and the baseDn string must contain the character sequencey '%s'
 where the userId is to be inserted in the string.

 Refer to the LDAP section of the System Administrators Guide for more
 information on configuring GemStone for LDAP authentication.

 Special UserProfiles must always use GemStone authentication.  Attempting
 to change the authentication scheme for a special UserProfile will raise
 an error.  See the method #isSpecialUserId: in this class to see which
 UserProfiles are considered special.

 Requires the OtherPassword privilege."

<primitive: 2001>
| prot |
prot := System _protectedMode .

[ self _validatePrivilege ifFalse:[ ^ nil ].
  (self _validateBaseDn: baseDn searchDn: filterDn ) ifTrue:[
    aString ifNotNil:[
      (aString _validateClass: String) ifFalse:[ ^ nil ].
               self userIdAlias: aString 
    ].
    self ldapBaseDn: baseDn .
    filterDn == nil
      ifFalse:[ self ldapFilterDn: filterDn ].
    self authenticationScheme: #LDAP .
  ]
] ensure:[ prot _leaveProtectedMode  "exit protected mode"].

^ self
%

category: 'External Authentication - Private'
method: UserProfile
ldapBaseDn: aString

"Private - customers should not call this method directly."

self _securityDataInstVarAt: 13 put: aString .
^ self
%

category: 'External Authentication - Private'
method: UserProfile
ldapFilterDn: aString

"Private - customers should not call this method directly."

self _securityDataInstVarAt: 11 put: aString .
^ self
%

category: 'Repository Migration'
method: UserProfile
migrateSecurityData

"Migrate the UserSecurityData object to the correct size for this release
 of GemStone/64.  Returns true if the migration was performed, false if
 no migration is required.

 This method may only be executed by SystemUser."

<primitive: 2001>
| prot |
prot := System _protectedMode .
^ [
  self _securityDataInstVarAt: -1 put: nil .
] ensure:[
  prot _leaveProtectedMode
].
%

category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString password: passwordString

"Creates a new UserProfile with the given password and no privileges.
 In so doing, creates a symbol list with three dictionaries: UserGlobals,
 Globals, and Published.  The first Dictionary (UserGlobals) is created
 for the user's private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group 'Subscribers' and of the groups in aCollectionOfGroupStrings.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 The new UserProfile will have the default object security policy, which is nil. 
"

^ self newWithUserId: userIdString password: passwordString
  defaultObjectSecurityPolicy: nil privileges: #() inGroups: #()
%


category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString password: passwordString privileges: anArrayOfStrings inGroups: aCollectionOfGroupStrings

"Creates a new UserProfile with the given password and no privileges.
 In so doing, creates a symbol list with three dictionaries: UserGlobals,
 Globals, and Published.  The first Dictionary (UserGlobals) is created
 for the user's private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group 'Subscribers' and of the groups in aCollectionOfGroupStrings.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 The new UserProfile will have the default object security policy, which is nil. 
"

^ self newWithUserId: userIdString password: passwordString
  defaultObjectSecurityPolicy: nil privileges: anArrayOfStrings inGroups: aCollectionOfGroupStrings
%


category: 'Instance Creation'
classmethod: UserProfile
newWithUserId: userIdString password: passwordString defaultObjectSecurityPolicy: aPolicy

"Creates a new UserProfile with the given password and no privileges.
 In so doing, creates a symbol list with three dictionaries: UserGlobals,
 Globals, and Published.  The first Dictionary (UserGlobals) is created
 for the user's private symbols.  Returns the new UserProfile.

 Adds the new UserProfile to AllUsers.  Creates the new UserProfile as a member
 of the group 'Subscribers' and of the groups in aCollectionOfGroupStrings.

 Generates an error if passwordString is equivalent to userIdString ignoring
 case (equalsNoCase:).

 aPolic must be nil or a committed instance of GsObjectSecurityPolicy.
"

^ self newWithUserId: userIdString password: passwordString
  defaultObjectSecurityPolicy: aPolicy privileges: #() inGroups: #()
%

! Add disable methods to fix 40820
!
category: 'Updating'
method: UserProfile
disableWithReason: aString

"Disables the receiver when the transaction is committed and specifies aString
 as the reason.   Generates an error if you do not have OtherPassword privilege.
 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    (aString _validateClasses: { String . DoubleByteString } ) ifFalse:[ ^ nil ].
    self _securityDataInstVarAt: 1 put: nil.
    self _securityDataInstVarAt: 6 put: aString copy immediateInvariant .
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ self
%

category: 'Updating'
method: UserProfile
disable

"Disables the receiver when the transaction is committed.
 Generates an error if you do not have OtherPassword privilege.
 Returns the receiver."

^ self disableWithReason: 'Disabled by administrator'
%

category: 'Accessing'
method: UserProfile
passwordAgeLimit

"Answers the maximum number of hours for which the receiver can retain a password.
 When a password is set, it expires this number of hours later.  The user must change
 the password before it expires, or else GemStone disables the account.  Once
 a password has expired, an administrator must reset the password from another
 account before the user can login again.

 Returns nil if the repository-wide passwordAgeLimit, if any, will be used to 
 enforce the password age limit.

 Returns zero if password aging is disabled for the receiver.

 Sending this message to any of the special UserProfiles (SystemUser, SymbolUser,
 DataCurator, GcUser, and Nameless) always returns zero or an error.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."


<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: 14 .
    ]
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Accessing'
method: UserProfile
passwordAgeWarning

"The maximum number of hours prior to a password expiration time for which
 a login as the receiver can succeed without a warning.  If the user logs in
 to GemStone within this number of hours before the password is due to expire,
 GemStone issues a warning about the impending expiration.  This feature grants
 a user the opportunity to change the password conveniently and to prevent the
 account from being disabled.  

 Returns nil if the repository-wide passwordAgeWarning, if any, will be used to 
 generate password age warnings.

 Returns zero if password age warning is disabled for the receiver.

 Sending this message to any of the special UserProfiles (SystemUser, SymbolUser,
 DataCurator, GcUser, and Nameless) always returns zero or an error.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[
  self _validateUserProfileAccess ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      result := self _securityDataInstVarAt: 15 .
    ]
  ]
] ensure:[
  prot _leaveProtectedMode
].
^ result
%

category: 'Accessing'
method: UserProfile
staleAccountAgeLimit

"The maximum number of hours for which the receiver will remain enabled without
 a login.  Once the user logs in for the first time, he or she has up to this
 number of hours to login again, or else GemStone disables the account.  Once
 the account has been disabled, an administrator must reset the password from 
 another account before the user can login again.

 Returns nil if the repository-wide staleAccountAgeLimit, if any, will be used to 
 suspend stale accounts.

 Returns zero if the stale account age limit is disbled for the receiver.

 Sending this message to any of the special UserProfiles (SystemUser, SymbolUser,
 DataCurator, GcUser, and Nameless) always returns zero or an error.

 The staleAccountAgeLimit is enforced no matter what type of password authenticaiton
 scheme is used by the receiver.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege."

<primitive: 2001>
| result prot |
prot := System _protectedMode .
[ 
  self _validateUserProfileAccess ifTrue:[
    result := self _securityDataInstVarAt: 16 .
  ]
] ensure:[ prot _leaveProtectedMode ].
^ result
%

category: 'Updating'
method: UserProfile
passwordAgeLimit: numberOfHours

"Sets the maximum number of hours for which the receiver can retain a password.
 When a password is set, it expires this number of hours later.  The user must change
 the password before it expires, or else GemStone disables the account.  Once
 a password has expired, an administrator must reset the password from another
 account before the user can login again.

 Setting this value to nil causes the receiver to use the repository-wide 
 passwordAgeLimit setting in AllUsers, if enabled.

 Setting this value to zero disables password aging for the receiver.

 Requires the OtherPassword privilege.

 Has no effect if the receiver is one of the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless), which are always
 immune to the password age limit.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
  self _validateAuthenticationSchemeIsGemStone ifTrue:[
    (self class isSpecialUserId: userId)
      ifFalse:[ 
			numberOfHours == nil ifFalse: [
				(numberOfHours _validateClasses: { SmallInteger }) ifFalse: [^ nil ].
				numberOfHours < 0 ifTrue: [^numberOfHours _error: #rtErrArgNotPositive].
			].
		self _securityDataInstVarAt: 14 put: numberOfHours ] .
    ]
  ]
] ensure:[ prot _leaveProtectedMode "exit protected mode"].
^ self
%

category: 'Updating'
method: UserProfile
passwordAgeWarning: numberOfHours

"Sets the maximum number of hours prior to a password expiration time for which
 a login as the receiver can succeed without a warning.  If the user logs in
 to GemStone within this number of hours before the password is due to expire,
 GemStone issues a warning about the impending expiration.  This feature grants
 a user the opportunity to change the password conveniently and to prevent the
 account from being disabled.  

 Has no effect if the a password age limit is not set for the receiver.

 Setting this value to nil causes the receiver to use the repository-wide
 passwordAgeWarning setting, if is enabled.

 Setting this value to zero disables the password age warning for the receiver.

 Requires the OtherPassword privilege.

 Has no effect if the receiver is one of the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless), which are always
 immune to the password age warning.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      (self class isSpecialUserId: userId)
        ifFalse:[
			numberOfHours == nil ifFalse: [
				(numberOfHours _validateClasses: { SmallInteger }) ifFalse: [^ nil ].
				numberOfHours < 0 ifTrue: [^numberOfHours _error: #rtErrArgNotPositive].
			].
			self _securityDataInstVarAt: 15 put: numberOfHours ] .
    ]
  ]
] ensure:[ prot _leaveProtectedMode "exit protected mode"].
^ self
%

category: 'Updating'
method: UserProfile
staleAccountAgeLimit: numberOfHours

"Sets maximum number of hours for which the receiver will remain enabled without
 a login.  Once the user logs in, he or she has up to this number of hours to
 login again, or else GemStone disables the account.  Once the account has been
 disabled, an administrator must reset the password from another account before
 the user can login again.

 Setting this value to nil causes the receiver to use repository-wide
 staleAccountAgeLimit setting in AllUsers, if enabled.

 Setting this value to zero disables the stale account age limit for the 
 receiver.

 Requires the OtherPassword privilege.

 Has no effect if the receiver is one of the special UserProfiles
 (SystemUser, SymbolUser, DataCurator, GcUser, and Nameless), which are always
 immune to the stale account limit.

 The staleAccountAgeLimit is enforced no matter what type of password authenticaiton
 scheme is used by the receiver.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  self _validatePrivilege ifTrue:[
    (self class isSpecialUserId: userId)
      ifFalse:[
			numberOfHours == nil ifFalse: [
				(numberOfHours _validateClasses: { SmallInteger }) ifFalse: [^ nil ].
				numberOfHours < 0 ifTrue: [^numberOfHours _error: #rtErrArgNotPositive].
			].
			self _securityDataInstVarAt: 16 put: numberOfHours ] .
  ]
] ensure:[ prot _leaveProtectedMode "exit protected mode"].
^ self
%


category: 'Updating'
method: UserProfile
lastLoginTime: aVal

"Updates the lastLoginTime for the receiver to be aVal, which must be an
 instance of DateTime or nil.

 Raises an error if the authentication scheme of the receiver is not #GemStone.

 Generates an error if the receiver is not your UserProfile and you do not
 have the #OtherPassword privilege.

 Returns the receiver."

<primitive: 2001>
| prot |
prot := System _protectedMode .
[
  aVal ifNotNil:[ aVal _validateClass: DateTime ].
  self _validateUserProfileAccess ifTrue:[
    self _validateAuthenticationSchemeIsGemStone ifTrue:[
      self _securityDataInstVarAt: 2 put: aVal .
    ].
  ].
] ensure:[
  prot _leaveProtectedMode
].
^ self
%

category: 'Accessing'
classmethod: UserProfile
privilegeNames
  ^ PrivilegeNames
%
