!=========================================================================
! Copyright (C) VMware, Inc. 1986-2011.  All Rights Reserved.
!
! $Id: system.gs 26743 2011-09-08 19:41:19Z otisa $
!
! Superclass Hierarchy:
!   System, Object.
!
!=========================================================================

removeallmethods System
removeallclassmethods System
set class System

category: 'For Documentation Installation only'
classmethod: System
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'System is an abstract class that has no instances.  It implements class
 methods for object locking and for operations that are usually found in
 traditional operating systems.  The data curator may restrict user access to
 these messages.  For an explanation of the role of the data curator, refer to
 your GemStone System Administration Guide.'.
doc documentClassWith: txt.

txt := (GsDocText new) details:
'Methods in this category are obsolete and are provided only for compatibility
 with earlier releases of GemStone.  They will be removed in a future release.'.
doc documentClassCategory: #'Deprecated' with: txt.

txt := (GsDocText new) details:
'These methods are used in managing cluster buckets, the streams of disk pages
 in which objects are congregated during clustering.  Clustering is explained
 in the GemStone Programming Guide.'.
doc documentClassCategory: #Clustering with: txt.

txt := (GsDocText new) details:
'In each of these file system access methods, it is best to specify the full
 path name of the server text file in the method''s argument.

 Under Unix, be sure that the case of the argument matches the case of the Unix
 file name; Unix is case-sensitive.

 Also note that under Unix, each of these methods inherits environment
 variables from the GemStone session process, rather than from your user
 session.  In addition, the method performOnServer: invokes the Bourne shell,
 even if you use a different login shell.  For these reasons, you might want to
 avoid using environment variables in the arguments to these methods.'.
doc documentClassCategory: #'Host System Access' with: txt.

txt := (GsDocText new) details:
'GemStone maintains two sets of objects that you can manipulate with methods in
 this category.  The commit release locks set contains locked objects whose
 locks will be released as part of the next successful commit operation.  The
 commit-or-abort release locks set contains locked objects whose locks will be
 released as part of the next successful commit operation or abort operation.
 To gain complete control over the automatic releasing of locks at the end of a
 transaction, use these methods during the transaction to govern the membership
 of objects in these sets.'.
doc documentClassCategory: #'Releasing Locks' with: txt.

txt := (GsDocText new) details:
'Transactions are discussed in detail in the GemStone Programming Guide.'.
doc documentClassCategory: #'Transaction Control' with: txt.

self description: doc.
%

! method comments re: 33533
category: 'Deprecated'
classmethod: System
deleteServerFile: aFileSpec

self deprecated: 'Obsolete in GemStone/64.  
Use the GsFile>>removeServerFile: method instead.'.
^self _deleteServerFile: aFileSpec.
%

category: 'Deprecated'
classmethod: System
_deleteServerFile: aFileSpec

<primitive: 346>
aFileSpec _validateClass: String.
aFileSpec _error: #hostErrFileDelete args:  #()
%

category: 'Host System Access'
classmethod: System
performOnServer: aString

"This method causes the operating system commands in aString to be executed as
 a spawned subprocess.  Generates an error if aString cannot be executed by the
 operating system.

 Under Unix, commands in aString can have exactly the same form as a shell
 script.  For example, newlines or semicolons can separate commands, and a
 backslash can be used as an escape Character."

| resultStr arr |
arr := self _performOnServer: aString .
resultStr := arr at: 3 .
resultStr ifNil:[  | rawStatus childStatus errno errMsg |
  rawStatus := arr at: 1 .
  childStatus := arr at: 2 .
  errno := arr at: 5 . 
  errMsg := arr at: 4 .
  aString _error: #hostErrPerform args:{ errMsg . errno . rawStatus . childStatus }.
  ^ nil
].
^ resultStr
%

category: 'Private'
classmethod: System
_performOnServer: aString

"Returns an Array of 5 elements: 
  raw status Integer, 
  child process status Integer (after WEXITSTATUS macro applied), 
  result String (or nil if operation failed) ,
  errno value, a SmallInteger , 
  error message , a String
"
<primitive: 347>
aString _validateClass: String .  "prim fails only if arg is not a String"
^ aString _error: #hostErrPerform args:  #()  
%

category: 'Error Handling'
classmethod: System
signal: anInteger args: anArray signalDictionary: anErrorDict

"Deprecated.
 This method generates the specified Exception, along with its
 associated arguments, and signals it. 

 Gs64 v3.0  anErrorDict is ignored. "

^ (AbstractException _new: anInteger args: anArray ) signal
%

category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString

"Deprecated.
 Raise a user-defined signal with no arguments.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil."

^ UserDefinedError new _number: 2318;
         reason: errIdentifier ; details: aString ; signal
%

category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString arg: anArg

"Deprecated.
 Raise a user-defined signal with one argument.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil. " 

^ UserDefinedError new _number: 2318;
         reason: errIdentifier ; details: aString; 
	args: { anArg } ; signal .
%

category: 'Error Handling'
classmethod: System
genericSignal: errIdentifier text: aString args: anArray

"Deprecated.
 Raise a user-defined signal.

 The argument errIdentifier is a user-defined object, to distinguish user
 errors, and may be nil.  The argument aString appears in GemStone's error
 message for this error, and may be nil.  "

^ UserDefinedError new _number: 2318;
     reason: errIdentifier ; details: aString ; 
     args: (anArray  _isArray ifTrue:[ anArray ] ifFalse:[ { anArray } ]) ; 
     signal .
%

! stackLimit removed

category: 'Debugging Support'
classmethod: System
stackLimit

"Returns the approximate limit on the depth of the GemStone Smalltalk stack.
 The stack size is determined by the configuration file parameter
 GEM_MAX_SMALLTALK_STACK_DEPTH."

^ self _zeroArgPrim: 1
%

category: 'System Control'
classmethod: System
_deadNotReclaimedCount

"Return the number of dead not reclaimed objects in the system."
^ self _zeroArgPrim: 69
%

category: 'System Control'
classmethod: System
_scavengablePagesCount

"Return the number of scavengable pages in the system."
^ self _zeroArgPrim: 70
%

category: 'System Control'
classmethod: System
_possibleDeadSize

"Return the number of possible dead objects in the system."
^ self _zeroArgPrim: 71
%

category: 'System Control'
classmethod: System
_commitRecordBacklog

"Return the number of commit records in the system."
^ self _zeroArgPrim: 72
%

category: 'System Control'
classmethod: System
_totalSessionsCount

"Return the number of sessions logged into the system."
^ self _zeroArgPrim: 73
%

category: 'System Control'
classmethod: System
_remoteSharedCacheCount
"Return the number of remote shared page caches on the system."
^ self _zeroArgPrim: 74
%

category: 'Debugging Support'
classmethod: System
_enableTraceNewPomObjs
"Causes all newly committed objects to be added to hidden set 4
 immediately after they are committed.  Also causes hidden set 4
 to be cleared if tracing was previously disabled when this method
 is called."
^ self _zeroArgPrim: 75
%

category: 'Debugging Support'
classmethod: System
_disableTraceNewPomObjs
"Causes the system to discontinue adding newly committed objects 
 to hidden set 4 immediately after they are committed.  Does not
 modify the state of hidden set 4."
^ self _zeroArgPrim: 76
%


category: 'Debugging Support'
classmethod: System
stackLimit: anInteger

"Has no effect in this release.  Provided for compatibility."

anInteger _validateClass: SmallInteger .
(anInteger < 0) ifTrue: [ anInteger _error: #rtErrArgNotPositive ].
^ self
%

category: 'Debugging Support'
classmethod: System
stackDepth

"Returns current depth of the GemStone Smalltalk stack."

^ self _zeroArgPrim: 26
%

category: 'Debugging Support'
classmethod: System
stackDepthHighwater

"Returns largest depth of the GemStone Smalltalk stack since session login."

^ self _zeroArgPrim: 27
%

! _addBreak:  deleted
! _breakLimit deleted
! _breakpointReport deleted
! _clearAllBreaks deleted
! _clearBreakAt: deleted
! _setStepping: deleted
! _updateDebuggerCache deleted
! _breakReportArray deleted
! _breakReportAt: deleted

category: 'Instance Creation'
classmethod: System
new

"Disallowed.  You may not create new instances of System."

self shouldNotImplement: #new
%

category: 'Clustering'
classmethod: System
clusterAllSymbols

"This method clusters the AllSymbols hash dictionary and all of the symbols to
 which it refers.
 The clustering is performed by the Symbol Creation Gem, and other
 sessions may see a long latency on symbol creation requests while
 the clustering is in progress."

| result |
GsFile gciLogClient:'-- sending clusterAllSymbols to Symbol Creation gem'.
result := self _zeroArgCateg2Prim: 6 .
GsFile gciLogClient:'-- waiting 10 seconds for Symbol Creation gem commit'.
self sleep: 10 .
^ result
%

category: 'Clustering'
classmethod: System
_clusterImage

"For all classes in Globals, clusters behavior and descriptions "

| classes nonClasses behaviorBucket descriptionBucket otherBucket |
behaviorBucket := AllClusterBuckets at: 4 .
descriptionBucket := AllClusterBuckets at: 5 .
otherBucket := AllClusterBuckets at: 6  .

classes := Globals select:[:i| i isKindOf: Behavior ] .
nonClasses := Globals reject:[:i| i isKindOf: Behavior ] .
nonClasses removeAssociation: (Globals associationAt: #Globals ).

self clusterBucket: behaviorBucket.
classes associationsDo: [ :each |
    each value _alias . "for debugging"
    each value clusterBehavior.
    ].

self clusterBucket: descriptionBucket.
classes associationsDo: [ :each | 
    each value clusterDescription.
    ] .
^ true
%

category: 'Clustering'
classmethod: System
_clusterUserClassesVisibleFromUserProfile: aUserProfile

"Cluster all classes contained in symbol list dictionaries for the
 given user profile.  Kernel classes in Globals are not clustered;
 the _clusterImage method should be used to cluster kernel classes."

|behaviorBucket descriptionBucket symbolList|

symbolList := aUserProfile symbolList copy.
symbolList remove: Globals.
symbolList removeDictionaryNamed: #Published ifAbsent: [ ].
 
"Identify cluster buckets for class and method objects"
behaviorBucket := AllClusterBuckets at: 4.
descriptionBucket := AllClusterBuckets at: 5.
self clusterBucket: behaviorBucket.
symbolList do:[:eachSymbolDictionary|
  eachSymbolDictionary rebuildIfNeeded.
  eachSymbolDictionary cluster.
  eachSymbolDictionary associationsDo:[ :eachAssociation |
    eachAssociation cluster.	
    (eachAssociation value isKindOf: Behavior)
      ifTrue:[eachAssociation value clusterBehavior.].
  ].
].

self clusterBucket: descriptionBucket.
symbolList do:[:eachSymbolDictionary|
  eachSymbolDictionary associationsDo:[ :eachAssociation | |val|
  val := eachAssociation value.
  (val isKindOf: Behavior)
    ifTrue:[val clusterDescription.].
  ].
].
%

category: 'Clustering'
classmethod: System
currentClusterBucket

"Returns the instance of ClusterBucket that is the current default."

^ AllClusterBuckets at: self currentClusterId
%

category: 'Clustering'
classmethod: System
currentClusterId

"This method returns a SmallInteger that is the ID of the ClusterBucket that
 is the current default bucket."

<primitive: 96>
self _primitiveFailed: #currentClusterId .
self _uncontinuableError
%

category: 'Clustering'
classmethod: System
clusterBucket: aClusterBucketOrId

"This method sets the current default ClusterBucket to the ClusterBucket with
 the specified clusterId.  The argument may be an instance of ClusterBucket, or
 a positive SmallInteger which specifies an instance of ClusterBucket."

<primitive: 348>
self _primitiveFailed: #clusterBucket: args: { aClusterBucketOrId } .
self _uncontinuableError
%

category: 'Clustering'
classmethod: System
maxClusterBucket

"Returns the maximum legal clusterId as a SmallInteger."

^ AllClusterBuckets size
%

category: 'Deprecated'
classmethod: System
contentsOfServerDirectory: aSpecString

"Obsolete: Use the GsFile>>contentsOfDirectory: onClient:  method instead."

| result |
self deprecated: 'System class>>contentsOfServerDirectory: is obsolete in GemStone/S 64 Bit.
Replace with GsFile class >> contentsOfDirectory:onClient:'.
result := GsFile contentsOfDirectory: aSpecString onClient: false.
result ifNil: [ ^ { }  ].
^ result
%

category: 'Private'
classmethod: System
_initMap: aMap

"Initializes the underlying data structures used to perform the mapping
 of passivated objects.  The argument aMap must be in the range 1 - 4."

<primitive: 319>
self _primitiveFailed: #_initMap: args: { aMap }
%

category: 'Private'
classmethod: System
_inMap: aMap at: anOop putIfAbsent: newValue

"If there is no current value at anOop in the passive object map aMap,
 (that is, it is absent), then this method stores newValue and returns newValue.
 If the current value at anOop is valid however, it returns the value and does
 not store the newValue in the map.  

 The argument aMap must be in the range 1 - 4
 and newValue must be a positive SmallInteger.

 anOop must not be a special object."

<primitive: 320>
aMap _validateClass: SmallInteger .
(aMap < 1 or:[ aMap > 4]) ifTrue:[ aMap _halt:'argument out of range' ].
anOop isSpecial ifTrue:[ anOop _halt:'special object not allowed' ].
newValue _validateClass: SmallInteger .
newValue < 1 ifTrue:[ newValue _error: #rtErrArgNotPositive ].
self _primitiveFailed: #_inMap:at:putIfAbsent: 
     args: { aMap . anOop . newValue }
%

category: 'Private'
classmethod: System
_updateNoRollback: anyObj add: aBoolean

"Add or remove anyObj from the no-rollback set.  If an object is in the
 no-rollback set, then it is treated as a temporary and does not get rolled
 back on aborts."

aBoolean ifTrue: [self _add: anyObj to: 2]
         ifFalse: [self _remove: anyObj from: 2].
^System
%


category: 'Disk Space Management'
classmethod: System
findObjectsLargerThan: aSize limit: aLimit
 
"Searches GemStone for objects larger than aSize, and returns an Array of
 any such objects.  The search continues until all such objects have been
 found, or until the size of the result reaches the specified maximum aLimit.
 Both aSize and aLimit must be positive SmallIntegers.
 If aLimit == 0, the result size is unlimited.

 This method aborts the current transaction; if an abort would cause unsaved
 changes to be lost, it signals an error, #rtErrAbortWouldLoseData.    

 The result contains only permanent objects."

| res |

self needsCommit
  ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

res := (SystemRepository _scanPomWithMaxThreads: 2 waitForLock: 3 pageBufSize: 8
       percentCpuActiveLimit: 90 identSet: (IdentitySet with: aSize) 
       limit: aLimit scanKind: 7 toDirectory: nil) at: 2.
aLimit > 0 ifTrue:[
  res size > aLimit ifTrue:[ res size: aLimit ].
].
^res
%

category: 'Disk Space Management'
classmethod: System
fastFindObjectsLargerThan: aSize limit: aLimit
 
"Same as findObjectsLargerThan except that the scan is performed agressively
 in order to complete in as little time as possible."

| res |

self needsCommit
  ifTrue: [ self _error: #rtErrAbortWouldLoseData ] .

res := (SystemRepository _scanPomWithMaxThreads: 
          SystemRepository _aggressiveMaxThreadCount
       waitForLock: 3 pageBufSize: 8
       percentCpuActiveLimit: 95 identSet: (IdentitySet with: aSize) 
       limit: aLimit scanKind: 7 toDirectory: nil) at: 2.
aLimit > 0 ifTrue:[
  res size > aLimit ifTrue:[ res size: aLimit ].
].
^res
%


category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol

"Invokes the user-defined action represented by aSymbol.  Generates an error if
 the user action is not installed in this session, or if it expects any
 arguments.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction: args: { aSymbol }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: anArg

"Invokes the user-defined action represented by aSymbol, passing it the argument
 anArg.  Generates an error if the user action is not installed in this session,
 or if the number of arguments expected by the user action is not 1.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with: args: { aSymbol . anArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg and secondArg.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the user
 action is not 2.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:
       args: { aSymbol . firstArg . secondArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, and thirdArg.  Generates an error if the user
 action is not installed in this session, or if the number of arguments expected
 by the user action is not 3.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol withArgs: anArray

"Invokes the user-defined action represented by aSymbol, passing it the elements
 of anArray as arguments.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the user
 action is not the same as the number of elements in anArray.

 A maximum of 47 user actions may be active at any one time on the current
 GemStone Smalltalk stack."

<primitive: 201>
^ self _primitiveFailed: #userAction:withArgs: args: { aSymbol . anArray }
%

category: 'User-Defined Actions'
classmethod: System
userActionReport

"Returns a SymbolDictionary that provides information about all user actions
 installed in this GemStone session.  In that SymbolDictionary, the keys are
 the symbolic names of the user actions, and the values are Booleans (true if
 the user action is linked with your application, false if the user action is
 linked with the current GemStone session)."

 | anArray result |
 anArray :=  self _zeroArgCateg2Prim: 7 . "get an Array of SymbolAssociations"
 result := SymbolDictionary new .
 1 to: anArray size do:[:j | result addAssociation: (anArray at: j ) ].
 ^ result

%

category: 'User-Defined Actions'
classmethod: System
systemUserActionReport

"Returns a SymbolDictionary that provides information about GemStone system user
 actions.  These are user actions that are automatically installed in every
 GemStone session to support classes such as GsFile and GsSocket.

 In the resulting SymbolDictionary, the keys are the symbolic names of the user
 actions, and the values are Booleans.  A values is true if the user action is
 linked with your application, and false if the user action is linked with the
 current GemStone session."

| anArray result |

anArray :=  self _zeroArgCateg2Prim: 9 . "get an Array of SymbolAssociations"
result := SymbolDictionary new .
1 to: anArray size do:[:j | result addAssociation: (anArray at: j ) ].
^ result
%

category: 'User-Defined Actions'
classmethod: System
hasUserAction: aSymbol

"Returns true if the user action named aSymbol is installed in this GemStone
 session.  Returns false otherwise."

<primitive: 105>
aSymbol _validateClass: String.
^ self _primitiveFailed: #hasUserAction: args: { aSymbol }
%

category: 'User-Defined Actions'
classmethod: System
loadUserActionLibrary: aString

"Loads the session user action library specified by aString.
 This method always returns the receiver (System)."

<primitive: 426>
^ self _primitiveFailed: #loadUserActionLibrary: args: { aString }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, and fourthArg.  Generates an error if
 the user action is not installed in this session, or if the number of
 arguments expected by the user action is not 4."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg . fourthArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, and fifthArg.  Generates
 an error if the user action is not installed in this session, or if the number
 of arguments expected by the user action is not 5."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . 
               thirdArg . fourthArg . fifthArg}
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, and sixthArg.
 Generates an error if the user action is not installed in this session, or if
 the number of arguments expected by the user action is not 6."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg . 
               fourthArg . fifthArg . sixthArg}
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg with: seventhArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, sixthArg, and
 seventhArg.  Generates an error if the user action is not installed in this
 session, or if the number of arguments expected by the user action is not 7."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg . 
               fourthArg . fifthArg . sixthArg . seventhArg }
%

category: 'User-Defined Actions'
classmethod: System
userAction: aSymbol with: firstArg with: secondArg with: thirdArg
  with: fourthArg with: fifthArg with: sixthArg with: seventhArg
  with: eighthArg

"Invokes the user-defined action represented by aSymbol, passing it the
 arguments firstArg, secondArg, thirdArg, fourthArg, fifthArg, sixthArg,
 seventhArg, and eighthArg.  Generates an error if the user action is not
 installed in this session, or if the number of arguments expected by the
 user action is not 8."

<primitive: 200>
^ self _primitiveFailed: #userAction:with:with:with:with:with:with:with:with:
       args: { aSymbol . firstArg . secondArg . thirdArg . 
               fourthArg . fifthArg . sixthArg . seventhArg . eighthArg }
%

category: 'Version Management'
classmethod: System
_version

"Returns a String describing the versions of the pieces of the running GemStone
 system."

| result |
result := String new .
result addAll: 'GEMSTONE: ' ; addAll: self _gemVersion ; 
	addAll: '; IMAGE: ' ; addAll: self _imageVersion .
^ result
%

category: 'Version Management'
classmethod: System
_gemVersion

"Private."

^ self _zeroArgCateg2Prim: 3 
%

category: 'Version Management'
classmethod: System
_imageVersion

"Returns a String, which contains the history of the image."

^Globals at: #DbfHistory.
%

category: 'System Control'
classmethod: System
activeRepositories

"Returns an Array containing references to the repositories that are
 attached at the time the message is sent."

^ self _zeroArgCateg2Prim: 0
%

category: 'System Control'
classmethod: System
resumeLogins

"Allows new sessions to be initiated.  (Enables users to login.)  Logins are
 enabled when the GemStone system is started.  This message reverses the effect
 of System | suspendLogins.  Requires the SystemControl privilege."

self configurationAt: #StnLoginsSuspended put: 0
%

category: 'System Control'
classmethod: System
shutDown

"Aborts all current sessions, then terminates them.  Finally, the GemStone
 system is shut down.  The session issuing this message terminates with a
 broken connection.  Requires the SystemControl privilege."

^ self _zeroArgPrim: 8
%

category: 'System Control'
classmethod: System
suspendLogins

"Prevents any new sessions from being initiated.  That is, no new user is
 allowed to login.  However, users already active will be allowed to continue
 processing.

 To reenable logins, send the message System | resumeLogins.  (If you fail to do
 so, GemStone automatically reenables logins when the last user logs out.)

 Requires the SystemControl privilege."

self configurationAt: #StnLoginsSuspended put: 1 
%

category: 'Deprecated'
classmethod: System
concurrencyMode

"This method is obsolete, concurrency mode is always #NO_RW_CHECKS"

self deprecated: 'System class>>concurrencyMode is obsolete in GemStone/S 64 Bit.'.

^ #NO_RW_CHECKS
%

!  _gciDirtyObjsInitialized deleted

! edited to fix 36577
category: 'Private'
classmethod: System
_primitiveCommit: commitMode

"Do not call this method directly; it is private to GemStone.  You must use
 commitTransaction or commitAndReleaseLocks.  Otherwise Gem Builder breaks.

 The argument commitMode is a SmallInteger: 
    0 for normal, 
    1 for release locks,
    2 for checkpoint and release locks.
    3  value not used , primitive will fail
    4 checkpoint and start dpNsUnion (used by full backup)

 The result of this primitive is a SmallInteger: 
      -2 promote to checkpoint requested but checkpoints are suspended,  
           commit succeeded but did not promote
      -1 read-only  (there were no modified objects to commit),
       0 success, 
       1 rcFailure  (replay of changes to instances of Rc classes failed),
       2 dependencyMap failure  (concurrency conflict on dependencyMap),
       3 validationFailure  (concurrency conflict),
       4 retryFailure,   (previous commit attempt failed with an rcFailure)
       5 commitDisallowed (disallowed due to other error),
       6 retry limit exceeded "

<primitive: 336>
commitMode _validateClass: SmallInteger .

(commitMode < 0 or:[ commitMode > 2 ])
  ifTrue:[ commitMode _error: #rtErrArgOutOfRange args:{ 0 . 2 } .  ^ 1 "failure" ] .

self _primitiveFailed: #_primitiveCommit: args: { commitMode } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_primitiveAbort

"Do not call this method directly; it is private to GemStone.  You must use
 abortTransaction.  Otherwise the transparent GemStone Smalltalk interface will
 break.

 This method rolls back all modifications made to committed GemStone objects
 (connected to the root) and provides the session with a new view of the most
 current committed state.  These operations are performed whether or not the
 session was previously in a transaction.  If the transactionMode is set to
 #autoBegin, then a new transaction is started.  If the transactionMode is set
 to #manualBegin or #transactionless, then a new transaction is not started, 
 but the session's view of the database is updated.
 This method always returns the receiver (System)."

<primitive: 335>
self _primitiveFailed: #_primitiveAbort .
self _uncontinuableError
%

category: 'Transaction Control'
classmethod: System
startCheckpointSync
"Starts a synchronous checkpoint.  If a checkpoint is already
 in progress, this method will not start another.  Instead, it
 will block until the current checkpoint completes.
 
 If successful, this method returns after the a checkpoint has
 completed.  To start a new checkpoint and return before it
 completes, use the startCheckpointAsync method instead
 of this one.

 This method does not commit (or otherwise modify) the current
 transaction.

 Requires the SystemControl privilege.

 A result of true means a checkpoint was successfully finished.
 A result of false means a checkpoint could not be started.  This
 usually happens because checkpoints are suspended or no tranlog
 space is available, or the repository is in restore-from-log mode.
 It can also happen if more than one session
 calls this method at the same time."

 ^self _zeroArgPrim: 62
%

category: 'Transaction Control'
classmethod: System
startCheckpointAsync
"Starts an asynchronous checkpoint.  If a checkpoint is already
 in progress, this method will not start another.  Instead, it
 will return 'true', indicating a checkpoint is in progress.   If a
 checkpoint is not in progress, a new checkpoint is started and
 this method returns immediately.   To block until the checkpoint
 completes, use the startCheckpointSync method instead.

 Unlike the commitTransactionWithCheckpoint method, this method
 does not commit (or otherwise modify) the current transaction.

 Requires the SystemControl privilege.

 A result of true means a checkpoint was successfully started.
 A result of false means a checkpoint could not be started.  This
 usually happens because checkpoints are suspended or no tranlog
 space is available, or the repository is in restore-from-log state.
 It can also happen if more than one session
 calls this method at the same time."

 ^self _zeroArgPrim: 63
%

! commitTransactionWithCheckpoint deleted

category: 'Transaction Control'
classmethod: System
commitAndReleaseLocks

"Attempt to commit the transaction for the current session.

 This method is the same as 'commitTransaction' except for the handling of
 locks.  If the commit succeeds, this method releases all locks for the session
 and returns true.  Otherwise, it returns false and does not release locks.

 This method also clears the commit release locks and commit-or-abort release
 locks sets.  See the 'Releasing Locks' method category for more
 information.

 Returns true if commit was read-only or succeeded ,
 false if there was a failure.  "

^ self _commit: 1 "release locks if successful"
%

! fixed 31184
category: 'Transaction Control'
classmethod: System
transactionConflicts

"Returns a SymbolDictionary that contains an Association whose key is
 #commitResult and whose value is one of the following Symbols: #success,
 #failure, #retryFailure, #commitDisallowed, or #rcFailure .

 The remaining Associations in the dictionary are used to report the conflicts
 found.  Each Association's key indicates the kind of conflict detected; its
 associated value is an Array of OOPs for the objects that are conflicting.
 If there are no conflicts for the transaction, the returned SymbolDictionary
 has no additional Associations.

 The conflict sets are cleared at the beginning of a commit or abort and
 therefore may be examined until the next commit, continue or abort.

 The keys for the conflicts are as follows:

     Key                Conflicts
 Read-Write          StrongReadSet and WriteSetUnion conflicts.
 Write-Write         WriteSet and WriteSetUnion conflicts.
 Write-Dependency    WriteSet and DependencyChangeSetUnion conflicts.
 Write-WriteLock     WriteSet and WriteLockSet conflicts.
 Write-ReadLock      WriteSet and ReadLockSet conflicts.
 Rc-Write-Write      Logical write-write conflict on reduced conflict object.
 WriteWrite_minusRcReadSet  (WriteSet and WriteSetUnion conflicts) - RcReadSet)

 The Read-Write conflict set has already had RcReadSet subtracted from it.
 The Write-Write conflict set does not have RcReadSet subtracted .

 Beginning with Gemstone64 v1.1 , the WriteSet no longer includes 
 objects newly committed by this transaction.  Thus a 
 conflict between a lock and a newly committed object in prior
 releases will no longer show up as a conflict.

 The Write-Dependency conflict set contains objects modified (including DependencyMap
 operations) in the current transaction that were either added to, removed from, 
 or changed in the DependencyMap by another transaction. Objects in the 
 Write-Dependency conflict set may be in the Write-Write conflict set.

 Note: You should be sure to disconnect conflict sets before committing to
 avoid making them persistent."

 | arr res |

 arr := self _conflictsReport: true . 

 res := SymbolDictionary new.
 res at: #commitResult put: (arr at: 1) .

 2 to: arr size by: 2 do:[:j |
   res at: (arr at: j) put:(arr at: j + 1)
 ].
 ^ res
%

category: 'Transaction Control'
classmethod: System
conflictsReport

"Returns an Array describing conflicts or success of the last attempted commit.
 First element of result is a symbol describing the status. 
 If status was not #readOnly or #success, 
 the following elements are pairs of aKey,anArray  with each aKey
 describing the kind of conflict.  Empty sets are omitted from the result.

     Key                Conflicts
 Read-Write          StrongReadSet and WriteSetUnion conflicts.
 Write-Write         WriteSet and WriteSetUnion conflicts.
 Write-Dependency    WriteSet and DependencyChangeSetUnion conflicts.
 Write-WriteLock     WriteSet and WriteLockSet conflicts.
 Write-ReadLock      WriteSet and ReadLockSet conflicts.
 Rc-Write-Write      Logical write-write conflict on reduced conflict object.
 WriteWrite_minusRcReadSet  ((WriteSet and WriteSetUnion conflicts) - RcReadSet)

 see also System>>transactionConflicts for more documentation.
"

^ self _conflictsReport: false

%
category: 'Private'
classmethod: System
_conflictsReport: aBoolean

"if aBoolean true, 
 include conflict sets even if commit result read-only or success."

| status res |

status := self _commitResult + 2  .
res :=  { }  .
res add: ( #( #readOnly  		"must agree with _commitResult method"
		 #success 
		 #rcFailure 
	         #dependencyFailure 
                 #failure 
		 #retryFailure
                 #commitDisallowed
                 #retryLimitExceeded ) at:  status )  .
		   
(status > 2 or:[ aBoolean]) ifTrue:[  | blk wwConflicts rcReadSet rcObject sess |

  blk := [ :key :setNum | | anArr |
     (self _hiddenSetSize: setNum) > 0 ifTrue:[
        res add: key ; add:( anArr:= self _hiddenSetAsArray: setNum)
     ].
     anArr .
  ].

  blk value:#'Read-Write' value: 13 . "StrongRead - Write conflicts"

  wwConflicts := blk value:#'Write-Write' value: 15 .

  rcReadSet := blk value:#'RcReadSet' value: 7 .
  (wwConflicts size ~~ 0 and:[rcReadSet size ~~ 0]) ifTrue:[
    res add:#'WriteWrite_minusRcReadSet' ; 
      add:(Array withAll:(IdentitySet withAll: wwConflicts) - (IdentitySet withAll: rcReadSet)).
  ].
  blk value: #'Write-Dependency' value: 16 .
  blk value: #'Write-ReadLock' value: 17 .
  blk value: #'Write-WriteLock' value: 18 .
  rcObject := self rcValueCacheAt: #'Rc-Write-Write' for: self otherwise: nil.
  rcObject ~~ nil ifTrue: [
      res add: #'Rc-Write-Write' ; add: { rcObject }
  ].
  sess := self rcValueCacheAt: #'Synchronized-Commit' for: self otherwise: nil.
  sess ~~ nil ifTrue: [
      res add: #synchronizedCommitFailure ; add: { sess }
  ].
].
^ res
%


category: 'Transaction Control'
classmethod: System
abortTransaction

"Rolls back all modifications made to committed GemStone objects and provides
 the session with a new view of the most recently committed GemStone state.

 These operations are performed whether or not the session was previously in a
 transaction.  If the transaction mode is set to #autoBegin, then a new
 transaction is started.  If the transaction mode is set to #manualBegin, then
 a new transaction is not started."

| coordinator |

^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localAbort ]
  ifFalse: [ coordinator abort ].
%

category: 'Transaction Control'
classmethod: System
_localAbort

"Rolls back all modifications made to committed GemStone objects and provides
 the session with a new view of the most recently committed GemStone state.

 These operations are performed whether or not the session was previously in a
 transaction.  If the transaction mode is set to #autoBegin, then a new
 transaction is started.  If the transaction mode is set to #manualBegin, then
 a new transaction is not started."

self _pendingCommitAbort .  "Signal error to application if appropriate"

^ self _primitiveAbort "Abort the current transaction"
%

category: 'Transaction Control'
classmethod: System
_localBeginTransaction

"Starts a new transaction for the session."

self _pendingCommitAbort .  "Signal error to application if appropriate"
self _primitiveBegin: false  "Begin a transaction without dpnsunion "
%

category: 'Private'
classmethod: System
_localBeginTransactionStartDpnsu

"Starts a new transaction for the session, and starts tracking dpnsunion"

"Used by full backup code."

self _pendingCommitAbort .  "Signal error to application if appropriate"
self _primitiveBegin: true  "Begin a transaction with dpnsunion "
%

category: 'Transaction Control'
classmethod: System
beginTransaction

"Starts a new transaction for the session.  An abort is done before the new
 transaction is started - giving the session a new snapshot of the repository.

 If any permanent objects had been written by the session, their
 state is aborted.  This method returns the receiver (System)."

| coordinator |
^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localBeginTransaction ]
  ifFalse: [ coordinator beginTransaction ].
%

! deleted System | _beginTransactionStartDpnsu in 2.2.4

category: 'Transaction Control'
classmethod: System
continueTransaction

"Updates the session's view to the most recently committed GemStone state.

 If the session is in a transaction, this method preserves modifications 
 made to committed GemStone objects.  The read and write sets of the session 
 are carried forward and continue to accumulate until the session either 
 commits or aborts.  

 If the session is not in a transaction, this method performs an abort,
 discarding any accumulated changes to committed objects.

 If in a transaction and a previous attempt to commit the transaction
 failed due to conflicts, then continueTransaction will generate 
 error 2409.  After a failed attempt to commit, you must abort before
 continueTransaction can be used again.

 Returns true if accumulated modifications to the committed objects would not
 cause concurrency conflicts; otherwise returns false.  If the method 
 returns false, you can call the transactionConflicts method to determine
 the nature of the conflicts.

 This method can be used whether or not the session is outside of a transaction.
 Of course, the session cannot commit accumulated changes unless it is
 inside a transaction.

 If transaction mode is #manualBegin, then continueTransaction does not alter
 the inside/outside of transaction state of the session.

 Modifications made by other committed transactions are accumulated for
 retrieval by GciDirtyObjs() and GciDirtySavedObjs() just as they are
 accumulated for commitTransaction or abortTransaction.

 This method has no effect on object locks.  Locks in the release locks sets 
 are not released."

| continueResult |

continueResult := self _zeroArgPrim: 9.

continueResult <= 0 ifTrue: [ 
  continueResult >= -1 ifTrue:[
    ^ true  "read only or success"
  ].
] ifFalse: [
  continueResult == 1 ifTrue: [ 
    self _resolveRcConflicts ifTrue:[ 
      ^true 
    ] ifFalse: [ 
      self _disallowSubsequentCommits.
      ^false "RC validation failed" 
    ]
  ] ifFalse: [
    ^ false "validation failure"
    "3 means validation failure,
    4 means maximum number of commit retries was reached,
    5 means commit disallowed (usually because of indexes),
    -2 means promote to checkpoint failed"
  ]
].
self _uncontinuableError "logic error if we get here"
%

category: 'Private'
classmethod: System
_primitiveBegin: aBoolean

"Starts a new transaction for the session.  If the session is already
 in a transaction, aborts the transaction and starts a new transaction.

 aBoolean specifies whether to start tracking dpNsUnion

 If any permanent objects had been written by the session, their
 state is aborted.  This method returns the receiver (System)."

<primitive: 362>
aBoolean _validateClass: Boolean .
self _primitiveFailed: #primitiveBegin: args: { aBoolean } .
self _uncontinuableError
%

category: 'Transaction Control'
classmethod: System
commitTransaction

"Attempts to update the persistent state of the Repository to include changes
 made by this transaction.

 If the commit operation succeeds, then this method returns true, and the
 current transaction's changes, if any, become a part of the persistent
 Repository.  After the repository update, the session exits the current
 transaction.  If the transaction mode is #autoBegin, then the session enters
 a new transaction.  If the transaction mode is #manualBegin, then the session
 remains outside of a transaction.

 If conflicts prevent the repository update, then this method returns false.
 Call the transactionConflicts method to determine the nature of the
 conflicts.  If the session is outside of a transaction, then this method
 raises the error #rtErrPrimOutsideTrans. 

 This method also updates the session's view of GemStone.  If the commit
 operation succeeds, then all objects in the session's view are consistent with
 the current state of GemStone.  If the commit fails, then this method retains
 all the changes that were made to objects within the current transaction.
 However, commits made by other sessions are visible to the extent that changes
 in this transaction do not conflict with them.

 Returns true if commit was read-only or succeeded ,
 false if there was a failure.  "

^ self _commit: 0 "don't release locks"
%

category: 'Transaction Control'
classmethod: System
disableSignaledAbortError

"Disables the generation of an error when Stone signals the Gem session that it
 should abort when running outside of a transaction."

self _updateSignalErrorStatus: 2 toState: false
%

category: 'Transaction Control'
classmethod: System
enableSignaledAbortError

"Enables the generation of an error when the Stone has signaled that the Gem
 process should abort to connect to a more current GemStone root. 

 This method must be invoked after each delivery of the signal-abort error, to 
 reenable generation of the error. 

 If invoked when in a tranaction, the new state will take effect
 after the next commit or abort which exits the transaction.
 "

self _updateSignalErrorStatus: 2 toState: true
%

category: 'Transaction Control'
classmethod: System
disableSignaledFinishTransactionError

"Disables the generation of the TransactionBacklog notification ."

self _updateSignalErrorStatus: 4 toState: false
%

!edited to fix 41545
category: 'Transaction Control'
classmethod: System
enableSignaledFinishTransactionError

"Enables the generation of an error when the Stone has signaled that a Gem
 process which is in-transaction should abort, commit, or continueTransaction 
 to move to a newer transactional view.

 This method must be invoked after each delivery of the TransactionBacklog 
 notification to reenable generation of the error."

self _updateSignalErrorStatus: 4 toState: true
%

category: 'Signals'
classmethod: System
signalAlmostOutOfMemoryThreshold: anInteger

"Controls the generation of an error when session's temporary object memory
 is almost full .
 anInteger = -1, disable the generation of the error, resets threshold to 85% ,
		and suppress smalltalk stack printing before OOM.
 anInteger = 0,  enable generation of the error with previous threshold,
		default threshold after session login is 85% .
 0 < anInteger < 125 , enable generation of the error with specified threshold.

 The error is generated at the end of an in-memory markSweep, 
 when the amount of memory used exceeds the specified threshold.  
 If the session is executing a user action, or in index maintenance, 
 the error is deferred and generated when execution returns to the 
 bottom-level Smalltalk or GCI portion of the stack.

 This method or enableAlmostOutOfMemoryError must be invoked after 
 each delivery of the AlmostOutOfMemory error to reenable generation 
 of the error."

self _updateSignalErrorStatus: 5 toState: anInteger
%

category: 'Signals'
classmethod: System
enableAlmostOutOfMemoryError

"Enables or reenables error when session's temporary object memory
 is almost full , with previous threshold .  

 See signalAlmostOutOfMemoryThreshold:  for more details.

 This method or  signalAlmostOutOfMemoryThreshold:  must be
 invoked after each delivery of the 
 AlmostOutOfMemory error to reenable generation of the error.
 "

self _updateSignalErrorStatus: 5 toState: 0
%

category: 'Transaction Control'
classmethod: System
signaledAbortErrorStatus

"Returns true to indicate that the system generates an error when it receives
 the abort signal from Stone.  (In other words, verify that
 enableSignaledAbortError has been called to activate detection of the
 RT_ERR_SIGNAL_ABORT signal.)  Returns false otherwise.

 If in transaction, the result reflects what will happen to abort signals
 after the next commit or abort which exits the transaction.
 "

^self _signalErrorStatus: 2
%

category: 'Transaction Control'
classmethod: System
signaledFinishTransactionErrorStatus

"Returns true to indicate that the system generates an error when it receives
 the finishTranaction signal from Stone.  (In other words, verify that
 enableSignaledFinishTransactionError has been called ).  Returns false otherwise."

^self _signalErrorStatus: 4
%
category: 'Signals'
classmethod: System
almostOutOfMemoryErrorThreshold

"Returns a positive percentage of memory full at which the AlmostOutOfMemory
 error will be raised, or -1  if the error is not enabled. "

^self _signalErrorStatus: 5
%
category: 'Notification'
classmethod: System
signalTranlogsFullStatus

"Returns true to indicate that the the session will get an error when
 stone detects a tranlogs full condition.  Returns false otherwise." 

^self _signalErrorStatus: 6
%
category: 'Notification'
classmethod: System
enableSignalTranlogsFull

"Enables generation of error 2339 to this session when stone detects
 a tranlogs full condition."

self _updateSignalErrorStatus: 6 toState: true
%
category: 'Notification'
classmethod: System
disableSignalTranlogsFull

"Disables generation of error 2339 to this session when stone detects
 a tranlogs full condition."

self _updateSignalErrorStatus: 6 toState: false
%

category: 'Transaction Control'
classmethod: System
inTransaction

"Returns true to indicate that the session is in a transaction, false
 otherwise."

^ self _zeroArgPrim: 5
%

category: 'Transaction Control'
classmethod: System
transactionMode

"Returns the current transaction mode for the current GemStone session, either
 #autoBegin, #manualBegin or #transactionless.  The default is #autoBegin."

^ self _zeroArgPrim: 4
%

category: 'Transaction Control'
classmethod: System
_localTransactionMode: newMode

"Sets a new transaction mode for the current GemStone session and exits the
 previous mode by aborting the current transaction.  Valid arguments are
 #autoBegin, #manualBegin and #transactionless."

self _pendingCommitAbort .
^ self _primTransactionMode: newMode
%

category: 'Transaction Control'
classmethod: System
transactionMode: newMode

"Sets a new transaction mode for the current GemStone session and exits the
 previous mode by aborting the current transaction.  Valid arguments are
 #autoBegin, #manualBegin and #transactionless.
 The mode transactionless is intended primarily for idle sessions. Users
 may scan database objects, but are at risk of obtaining inconsistent views.
"

| coordinator |
^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localTransactionMode: newMode ]
  ifFalse: [ coordinator transactionMode: newMode ].
%

category: 'Transaction Control'
classmethod: System
_primTransactionMode: newMode

""

<primitive: 368>
self _primitiveFailed: #_primTransactionMode: args: { newMode } .
self _uncontinuableError
%

category: 'Reduced Conflict Support'
classmethod: System
_addEntireObjectToRcReadSet: anObject

"Adds anObject to the reduced-conflict read set (RcReadSet).  
 If the object is a large object, all nodes of the object are added.

 See also documentation for _addToRcReadSet:includingAllNodes: "

self _addToRcReadSet: anObject includingAllNodes: true.
%

category: 'Reduced Conflict Support'
classmethod: System
_addRootObjectToRcReadSet: anObject

"Adds anObject to the reduced-conflict read set (RcReadSet).  
 Only the root object is added (even if the object is a large object).

 See also documentation for _addToRcReadSet:includingAllNodes: "

self _addToRcReadSet: anObject includingAllNodes: false.
%

! System (C) >> _addToRcReadSet:includingAllNodes:
category: 'Reduced Conflict Support'
classmethod: System
_addToRcReadSet: anObject includingAllNodes: aBoolean

"Adds anObject to the reduced-conflict read set (RcReadSet).  
 All objects that are part of composite used to represent a reduced 
 conflict object should be added to this set to avoid unnecessary 
 conflict handling.  
 If aBoolean is true, the entire object (including all interior nodes 
 of a large object) are added.  If aBoolean is false, only the root object 
 is added.
 
 The RcReadSet is used as follows during a commit attempt .
   readWriteConflicts =   (strongReadSet * writeSetUnion) - RcReadSet .
   writeWriteConflicts = (writeSet * writeSetUnion) .
 If there are readWriteConflicts, the commit always fails.
 If writeWriteConflicts is not empty, 
 but (writeWriteConflicts - RcReadSet ) is empty , 
 then RC replay is executed and the commit is reattempted once .
 "

<primitive: 116>
aBoolean _validateClass: Boolean.
self _primitiveFailed: #_addToRcReadSet:includingAllNodes: 
     args: { anObject . aBoolean } .
self _uncontinuableError
%

category: 'Transaction Control'
classmethod: System
_pendingCommitAbort

""

| commitAction |
commitAction := self _sessionStateAt: 7 .
commitAction ifNotNil:[
  commitAction _isSmallInteger ifTrue:[
      "Signal commit-abort pending error"
      TransactionError new reason:#rtErrCommitAbortPending; _number: 6007 ; signal
      "expect the application to continue with GciContinue"
    ]
    ifFalse: [ self userAction: commitAction ]
].
%

category: 'Private'
classmethod: System
_processDeferredGciUpdates

"Process deferred GemBuilder for C updates and return the number of objects that
 had deferred updates."

"The deferred updates are stored in _sessionStateAt: 12.  These
 are updates to objects that do not allow direct structural update such
 as AbstractDictionary, Bag, and Set ."

<primitive: 2001>  "enter protected mode to disable adding to dirtys set..."
| res prot |
prot := self _protectedMode .
[ | arr count |
  arr := self _sessionStateAt: 13 "virtual machine constant".
  (count := arr size) ~~ 0 ifTrue:[
    1 to: count by: 2 do:[ :j |
      (arr at: j) _deferredGciUpdateWith: (arr at: j + 1).
      ].
    arr size: 0 .
    "reinitialize the identity dictionary"
    (self _sessionStateAt: 12 "virtual machine constant") 
	initialize: 751"a prime number for a large small object"
    ].
  res := count // 2
] ensure:[
  prot _leaveProtectedMode
].
^ res
%

category: 'Reduced Conflict Support'
classmethod: System
_commit: commitMode

"commitMode is a SmallInteger:

 0 for normal,
 1 for release locks,
 2 for checkpoint and release locks.

 Returns true if commit was read-only or succeeded ,
 false if there was a failure.  See also _localCommit: .  "

| coordinator |

^ (coordinator := self _commitCoordinator) == nil
  ifTrue: [ self _localCommit: commitMode ]
  ifFalse: [ coordinator commit: commitMode ].
%

category: 'Reduced Conflict Support'
classmethod: System
_localCommit: commitMode

"commitMode is a SmallInteger:

 0 for normal,
 1 for release locks,
 2 for checkpoint and release locks,
 3 for normal commit called from GciCommit .

 Returns true if commit was read-only or succeeded , 
 false if there was a failure.   

 For commitMode == 2 (request for checkpoint) will return true
 if the commit succeeded, regardless of whether promotion to
 checkpoint succeeded."

| commitResult actualMode |

self _processDeferredGciUpdates .

commitMode == 3 ifTrue:[
  "called from GciCommit, no pending commit action."
  actualMode := 0
  ]
ifFalse:[
  self _pendingCommitAbort .
  actualMode := commitMode
] .

commitResult := [ self __commit: actualMode 
                ] onException: Error do:[:ex |  
                  "ensure primitive commit is not on stack, and won't be retried
                   on continuation of execution."
                  ex pass .
                  5
                ].
commitResult <= 0 ifTrue: [ 
  ^ true 
] ifFalse: [
  commitResult == 1 ifTrue: [ 
    "Execute RC replay and then retry the commit."
    ^ self _resolveRcConflictsForCommit: actualMode 
  ].
].
^ false
%

category: 'Reduced Conflict Support'
classmethod: System
__commit: commitMode

"This method is provided to facilitate reimplementation of commit
 in Gem Builder.

 See  _primitiveCommit:   for the return values."

^ self _primitiveCommit: commitMode.
%

category: 'Reduced Conflict Support'
classmethod: System
clearRcValueCache

"Clears the cache of calculated values for reduced conflict classes by setting
 the temporary session state slot to nil."

self _sessionStateAt: 5 put: nil
%

category: 'Reduced Conflict Support'
classmethod: System
clearRedoLog

"Clear the redo log by setting the temporary session state slot to nil.

                            Warning:
 Clearing the redo log will probably prevent Reduced Conflict
 classes from resolving conflicts.  Sending this message negates this
 capability for the current transaction.

 This is a protected method."

< protected >

self _sessionStateAt: 4 put: nil
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCache

"Returns the cache dictionary that is stored in temporary session state
 used to hold calculated values for reduced conflict classes.  If it does
 not exist, create it."

| dict |
dict := self _sessionStateAt: 5.
dict == nil
    ifTrue: [
        dict := IdentityKeyValueDictionary new: 47.
        self _sessionStateAt: 5 put: dict
    ].
^ dict
%

category: 'Reduced Conflict Support'
classmethod: System
_rcValueCache

"Returns the cache dictionary that is stored in temporary session state
 used to hold calculated values for reduced conflict classes. "

^ self _sessionStateAt: 5
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey for: anObject ifAbsent: aBlock

"Returns the associated value at the given key for anObject.
 If the key is not present, execute the zero-argument block."

| valueArray cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aBlock value ].

valueArray := cache at: anObject ifAbsent: [ ^ aBlock value ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [ ^ valueArray at: (i + 1) ]
].

^ aBlock value
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey for: anObject otherwise: aValue

"Returns the associated value at the given key for anObject.
 If the key is not present, returns aValue."

| valueArray cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aValue ].

valueArray := cache at: anObject otherwise: nil.

valueArray == nil
    ifTrue: [ ^ aValue ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [ ^ valueArray at: (i + 1) ]
].

^ aValue
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey otherwise: aValue

"Returns the associated value at the given key for anObject.
 If the key is not present, returns aValue."

| cache |
cache := self _rcValueCache.
" if the cache does not exist, then the key is not present "
cache == nil
  ifTrue: [ ^ aValue ].

^ cache at: aKey otherwise: aValue.
%

category: 'Reduced Conflict Support'
classmethod: System
rcValueCacheAt: aKey put: aValue for: anObject

"Adds the given key/value pair for anObject.  Returns the receiver."

| valueArray |
valueArray := self rcValueCache at: anObject otherwise: nil.
valueArray == nil
  ifTrue: [
    valueArray := { } .
    self rcValueCache at: anObject put: valueArray.
  ].

1 to: valueArray size by: 2 do: [ :i |
    aKey == (valueArray at: i)
        ifTrue: [
            valueArray at: (i + 1) put: aValue.
            ^ self
        ]
].

"add a new entry"
valueArray addLast: aKey; addLast: aValue
%

category: 'Reduced Conflict Support'
classmethod: System
_redoLog

"Returns the redo log that is stored in the temporary session state."

^ self _sessionStateAt: 4
%

category: 'Reduced Conflict Support'
classmethod: System
redoLog

"Returns the redo log that is stored in the temporary session state.
 Create it if it does not exist."

| redoLog |
redoLog := self _sessionStateAt: 4.
redoLog == nil
    ifTrue: [
        redoLog := RedoLog new.
        self _sessionStateAt: 4 put: redoLog
    ].
^ redoLog
%

category: 'Reduced Conflict Support'
classmethod: System
_resolveRcConflictsForCommit: commitMode

""

| lockResult result |

self _processDeferredGciUpdates .

lockResult :=self _acquireRcLock.
(lockResult == 1 or: [ lockResult == 2074 ]) ifFalse: [
	self error: 'RC Lock not granted: ', lockResult printString.
	^ false
].

"GsFile gciLogServer:'---- RedoLog: ' , self _redoLog printString ."
self _resolveRcConflicts ifFalse: [ 
  self _disallowSubsequentCommits.
  self _releaseRcLock.
  ^ false 
].

result := self __commit: commitMode.

result == 4 ifTrue: [ 
  self _releaseRcLock.
  ^ false
] ifFalse: [ 
  result < 0 ifTrue:[ 
    self _releaseRcLock.
    "readonly commit, means that no pyhsical changes required by rc replay"
    ^true
  ] ifFalse:[ 
    result == 1 "rcFailure"
      ifTrue: [ 
        "Will recurse to WORK_COMMIT_RETRY_LIMIT (currently 15 total trys)
         Keep rcLock until we succeed or fail."
        ^self _resolveRcConflictsForCommit: commitMode 
    ].
    self _releaseRcLock.
    ^ (result == 0) 
  ]
]
%

category: 'Reduced Conflict Support'
classmethod: System class
_acquireRcLock

| rcLock |
rcLock := self _rcLockObject .
rcLock ~~ nil
  ifTrue: [ ^self waitForRcWriteLock: rcLock ].
^1 "granted"
%

category: 'Reduced Conflict Support'
classmethod: System
_releaseRcLock

| rcLock |
rcLock := self _rcLockObject.
rcLock ~~ nil
  ifTrue: [
    self removeLock:  rcLock.
  ].
%

category: 'Reduced Conflict Support'
classmethod: System
_disallowSubsequentCommits

"Disallows subsequent attempts to commit due to RC retry failure."

^ self _zeroArgPrim: 22
%

category: 'Modifying Classes'
classmethod: System
_disallowCommitClassModFailure

"Disallows subsequent attempts due to failure during class modification."
^ self _zeroArgPrim: 37
%

category: 'Removing Locks'
classmethod: System
removeLock: anObject

"Removes the lock held by the current session on anObject.
 Returns the receiver."

self removeLockAll: { anObject }.
^self
%

category: 'Private'
classmethod: System
_numPagesInSharedOt

"Returns the number of pages in the shared object table.
 This method aborts the current transaction."

^self _zeroArgPrim: 55
%

category: 'Private'
classmethod: System
flushAllExtents

"Deprecated, has no effect, extents are only flushed to file system
 by the stone process at checkpoints."

^ self
%

category: 'Private'
classmethod: System
_cpuClockTicksPerSecond

"Calls the UNIX function sysconf(_SC_CLK_TCK) to determine the number
 of clock ticks in a second.  Returns zero if the call is not supported."

^self _zeroArgPrim: 57
%


category: 'Removing Locks'
classmethod: System
removeLocksForSession

"Removes all locks held by this session.  Returns the receiver.
 This method succeeds even if the session no longer has
 read authorization for one or more of its locked objects."

^ self _zeroArgPrim: 0
%

category: 'Private'
classmethod: System
_zeroArgPrim: opcode

"Private."

"opcode 0 = removeLocksForSession
        1 = stackLimit
        2 = pageReads
        3 = pageWrites 
        4 = transactionMode   (also sent by GsCurrentSession)
        5 = inTransaction   (also sent by GsCurrentSession)
        6 = maxSessionId
        7 = myCacheProcessSlot
        8 = shutDown
        9 = continueTransaction
       10 = _readClock   (get CPU time used)
       11 = stoneName
       12 = _timeMsLegacy  
       13 = _timeGmtFloat
       14 = _generationScavenge
       15 = NOT USED,  was  _enableTraceObjs
       16 = NOT USED,  was _disableTraceObjs
       17 = voteState  (gets GC voteState from stone)
       18 = startAllGcSessions
       19 = voteStateString
       20 = clientIsRemote
       21 = sessionsReferencingOldestCr
       22 = _disallowSubsequentCommits  (Rc failure)
       23 = _approxOopHighWaterMark
       24 = (not used)
       25 = _hostCallDebugger
       26 = stackDepth
       27 = stackDepthHighwater
       28 = writtenObjects
       29 = _vmMarkSweep   was _flushConnectedToPom
       30 = _commitResult
       31 = _deadNotReclaimed
       32 = _validateTransaction
       33 = session
       34 = _writeLockWriteSubset
       35 = _removeLockWriteSubset
       36 = _approxOopNumberHighWaterMark
       37 = _disallowCommitClassModFailure
       38 = clearEpochGcState
       39 = _clearDeadObjects
       40 = stopAllGcSessions
       41 = forceEpochGc
       42 = stopAdminGcSession
       43 = stopAllReclaimGcSessions
       44 = stopSymbolCreationSession,   was _abortTransactionAndKeepAttached
       45 = startSymbolCreationSession, was _beginTransactionAndKeepAttached
       46 = symbolCreationSessionId,    was _doRecordDead
       47 = _lastGsErrorNumber
       48 = currentUserSessionCount
       49 = currentSessionCount
       50 = Repository pageSize
       51 = NOT USED, was weakDictionaryEnableDebugging
       52 = NOT USED, was weakDictionaryDumpContent
       53 = getHighAllocatedOopNum
       54 = _numPersistentObjsModified
       55 = _numPagesInSharedOt 
       56 = NOT USED, was  flushAllExtents
       57 = _cpuClockTicksPerSecond
       58 = _dumpFrameData
       59 = myPageServerProcessId
       60 = resumeCheckpoints
       61 = checkpointStatus
       62 = startCheckpointSync
       63 = startCheckpointAsync
       64 = adminGcGemSessionId
       65 = reclaimGcSessionCount
       66 = currentGcReclaimSessionsByExtent
       67 = startAdminGcSesssion
       68 = startAllReclaimGcSessions
       69 = _deadNotReclaimedCount
       70 = _scavengablePagesCount
       71 = _possibleDeadSize
       72 = _commitRecordBacklog
       73 = _totalSessionsCount
       74 = _remoteSharedCacheCount
       75 = _enableTraceNewPomObjs
       76 = _disableTraceNewPomObjs
       77 = _tempObjSpaceUsed
       78 = _tempObjSpaceMax
       79 = _tempObjSpacePercentUsed
       80 = prvLogPrint
       81 = shrLogPrint
       82 = hstLogStatus
       83 = shrLogLockHold (no method)
       84 = _inProtectedMode
       85 = currentSessions
       86 = _tempObjSpacePercentUsedLastMark
       87 = _comPrintOpcodesEmitted
       88 = _comPrintOpcodesNotEmitted
       89 = _printCHeap
       90 = _locale
       91 = _maxClusterId
       92 = _comClearOpcodesEmitted
       93 = _oopHighWaterMark
       94 = _oopNumberHighWaterMark
       95 = timeNs
       96 = _updateObjectsRead 
       97 = _disableTraceObjectsRead
       98 = _resumeTraceObjectsRead
       99 = _enableTraceObjectsRead
      100 = _numSharedCounters
      101 = unused
      102 = _setPrintStackAtError 
      103 = _timeUs
      104 = logout
      105 = inContinueTransaction
      106 = gemIsBigEndian
      107 = stoneIsBigEndian
      108 = latestCommitRecordPageId
      109 = sessionsReferencingOldestCrInTransaction
      110 = sessionsReferencingOldestCrNotInTransaction
      111 = myReadPageServerProcessId
      112 = commitsDisabledUntilLogout
      113 = commitsDisabledUntilAbort
      114 = continuationsEnabled  
      115 = enableContinuations  (in Ruby image)
      116 = disableContinuations  (in Ruby image)
      117 = fetchSystemStatNames
      118 = fetchSystemStats
      119 = _hostTimes
      120 = disableStoneGemTimeout
      121 = enableStoneGemTimeout
      122 = millisecondsSinceTransactionBoundary
      123 = millisecondsSinceLogin
      124 = secondsSinceStoneStarted
      125 = _enableTracePagesAccessed
      126 = _resumeTracePagesAccessed
      127 = _updatePagesAccessed
      128 = _disableTracePagesAccessed
      129 = refreshTransientSymbolList
      130 = hostCpuUsage
      131 = hostId
      132 = fast   System gemConfigurationAt:#GemConvertArrayBuilder 
      133 = stoneMessageKinds
      134 = commitsSinceCurrentView
      135 = hostCpuCount
      136 = sessionIdHoldingGcLock
      137 = sessionIsOnStoneHost
      138 = cacheStatisticsForAllShort
      139 = cacheStatisticsForAll
     1020 is  HostExplicitWaitForDebugger
     4001 = cacheStatisticsDescriptionForMonitor
     4002 = cacheStatisticsDescriptionForStone
     4004 = cacheStatisticsDescriptionForPageServer
     4008 = cacheStatisticsDescriptionForGem
"

<primitive: 98>

self _primitiveFailed: #_zeroArgPrim: args: { opcode } .
self _uncontinuableError
%

category: 'Version Management'
classmethod: System
gemVersionAt: aString

"Returns information about the Gem process of the current session. 
 aString must be equal to a key in VersionParameterDict, otherwise nil
 is returned.  The semantics of these keys are:

 aString              meaning
 -------              -------
 cpuKind             detailed CPU type obtained at runtime: 'sun4u', 'x86_64'.
 cpuArchitecture     target CPU for which GemStone was compiled:
                       'SPARC', 'x86-64'.
 gsBuildArchitecture operating system name and CPU for which GemStone 
                       was compiled: 'SPARC (Solaris)', 'x86-64 (Linux)'.
 gsBuildDate         time at which the Gem executable was compiled (a String).
 gsRelease           major and minor version of GemStone, such as '3.0.0'.
 gsVersion           major version of GemStone, such as '3.0'.
 imageKind           a Symbol: #server.
 nodeName            network node name: 'speedy'.
 osName              operating system name: 'SunOS', 'Linux'.
 osRelease           release number of the operating system: 
                       '5.10', '2.6.16.46-0.12-smp'.
 osVersion           vendor defined major version of the OS: 
                       'Generic_141444-09', '#1 SMP Thu May 17 14:00:09 UTC 2007'.
 processId           operating system process identifier (an Integer): 13529.
 processorCount      number of processors on the machine running the process."

| verId |
aString = 'imageKind' ifTrue:[ ^ self imageVersionAt: aString ] .
verId := VersionParameterDict at: aString ifAbsent:[ ^ nil ].

^ self _configurationAt: verId isStone: false kind: $V 
%

category: 'Private'
classmethod: System
_dumpFrameData

"Private.  Prints out shared page cache frame data to the session log file."

^ self _zeroArgPrim: 58
%

category: 'Version Management'
classmethod: System
stoneVersionAt: aString

"Returns version information about the Stone (repository monitor) process.

 See System (C) | gemVersionAt: for further details.
 aString = 'imageKind' returns nil for the Stone."

| verId |

aString = 'imageKind' ifTrue:[ ^ nil "image not applicable to Stone"].

verId := VersionParameterDict at: aString ifAbsent:[ ^ nil ].
  
^ self _configurationAt: verId isStone: true kind: $V 
%

category: 'Version Management'
classmethod: System
clientVersionAt: aString

"Returns version information about the client GemBuilder for C process.  If the
 client is a session using the linkable GemBuilder for C login, this method is
 equivalent to gemVersionAt:.

 See System(C) | gemVersionAt: for further details."

| verId |
verId := VersionParameterDict at: aString ifAbsent:[ ^ nil ].
^ GsFile _clientVersionAt: verId 
%

category: 'Version Management'
classmethod: System
imageVersionAt: aSymbol

"Returns information about the GemStone kernel class image where aSymbol is one
 of the following:

 #gsBuildDate  DateTime of last kernel class filein or upgrade.
 #gsRelease    Version String of last kernel class filein or upgrade: '3.0.0'.
 #gsVersion    Major version of image, such as '3.0' .
 #imageKind    A Symbol: #server.

 If aSymbol is not one of the above, returns nil."

^ ImageVersion at: aSymbol otherwise: nil
%

category: 'Version Management'
classmethod: System
_serverVersionReport: isStone

"Private, returns a StringKeyValueDictionary."

| result |
result := StringKeyValueDictionary new .
VersionParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := self _configurationAt: anId isStone: isStone kind: $V .
  aVal ~~ nil ifTrue:[ 
    result at: aName put: aVal
    ].
  ].
^ result 
%

category: 'Version Management'
classmethod: System
gemVersionReport

"Return a SymbolDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the Gem process."

^ self _serverVersionReport: false
%

category: 'Version Management'
classmethod: System
stoneVersionReport

"Return a StringKeyValueDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the Gem process."

^ self _serverVersionReport: true
%

category: 'Version Management'
classmethod: System
clientVersionReport

"Return a StringKeyValueDictionary whose keys are the names of operating system,
 hardware, or GemStone version attributes, and whose values are the
 current values of those attributes in the client GemBuilder for C process."

| result |
result := StringKeyValueDictionary new .
VersionParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := GsFile _clientVersionAt: anId . 
  aVal ~~ nil ifTrue:[ 
    result at: aName put: aVal
    ].
  ].
^ result 
%

category: 'Private'
classmethod: System
_versionParameterDict

"Returns a Dictionary of names for version information parameters.  The
 dictionary keys are Strings.  Its values are SmallInteger parameter IDs."

"gemstone64 changed  result keys from Symbols to Strings, to reduce creation
  of new Symbols at runtime. "

| result cfgId cfgName |
result := StringKeyValueDictionary new .
cfgId := 0 .
[
  cfgName := self _configParameterName: cfgId kind: $V .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId + 1 .
    ] .
  cfgName == nil 
  ] untilTrue .

^ result
%

category: 'Private GC'
classmethod: System
_generationScavenge_vmMarkSweep

"Execute an in-memory scavenge and an in-memory markSweep. 
 Should only be used when debugging internal problems. Frequent use
 can degrade performance."

self _generationScavenge ifTrue:[
  self _vmMarkSweep
  ].
%
 

category: 'Private GC'
classmethod: System
_generationScavenge

"Returns true if the scavenge succeeded, false if it got promoted
 to a markSweep 

 Explicitly triggers in-memory scavenge collection of temporary objects.
 Should only be used when debugging internal problems. Frequent use
 can degrade performance."

^ self _zeroArgPrim: 14 
%

category: 'Private GC'
classmethod: System
_vmMarkSweep

"Explicitly triggers in-memory markSweep collection of temporary objects.
 Should only be used when debugging internal problems. Frequent use
 can degrade performance."

^ self _zeroArgPrim: 29 
%

category: 'Private GC'
classmethod: System
_vmInstanceCounts: anInt

<primitive: 652> 
anInt _validateClass: SmallInteger.
(anInt < 0 or:[ anInt > 5 ])
  ifTrue:[ anInt _error: #rtErrArgOutOfRange args:{ 1 . 4 } ].

%
category: 'Private GC'
classmethod: System
_vmPrintInstanceCounts: anInt

"Print to VM's stdout or  topaz -l  output file 
 a report about how many instances of each in-memory class 
 are present in the specified portion of temporary object memory. 

   anInt = 0   all of object memory
           1   new generation (young temporary objects)
           2   pom generation (copies of committed objects)
           3   old generation (old temporary objects)
           4   perm generation (loaded classes)
           5   code generation (loaded methods)

   If printing all of object memory , also prints those classes
   which have zero instances in memory , otherwise classes with
   zero instances are not printed.  The printout contains one
   line per class, with className, number of instances, and total
   bytes occupied by the instances.
"
<primitive: 153> 
anInt _validateClass: SmallInteger.
(anInt < 0 or:[ anInt > 5 ])
  ifTrue:[ anInt _error: #rtErrArgOutOfRange args:{ 1 . 4 } ].

%

! deleted  _reclaimAll
! deleted _reclaimAllParallel
category: 'Private'
classmethod: System
_simpleCommitForReclaim: includeDeadBool

  "Calls stone to request a simple commit be performed.
   if includeDeadBool==true, returns true if both the
   number of dead objects and number of shadow pages needing reclaiming
   are zero .
   if includeDeadBool==false, returns true if 
   the number of shadow pages needing reclaiming is zero .

   Has no effect on this session's transactional view."

<primitive: 290> 

self _primitiveFailed: #_simpleCommitForReclaim: args: { includeDeadBool } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_clearDeadObjects

^ self _zeroArgPrim: 39 
%

category: 'Private'
classmethod: System
_zeroArgCateg2Prim: opcode

"Private."

"opcode 0 = activeRepositories
        1 = NOT used, was gemStatistics
        2 = NOT used, was stoneStatistics
        3 = gemVersion
        4 = (not used)
        5 = (not used)
        6 = clusterAllSymbols
        7 = userActionReport - returns an Array of SymbolAssociations
        8 = configConstantsArray - returns an Array of Assocations
        9 = systemUserActionReport
       10 = _committedDataPages
       11 = _uncommittedDataPages"

<primitive: 325> 

self _primitiveFailed: #_zeroArgCateg2Prim: args: { opcode } .
self _uncontinuableError
%

category: 'Removing Locks'
classmethod: System
removeLockAll: aCollection

"Removes all locks held by the current session on the objects in aCollection.
 If an object in aCollection is not locked by the current session, that
 object is ignored. Returns the receiver."

<primitive: 99>
| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self removeLockAll: anArray .
  ] .
aCollection _validateClass: Collection.
self _primitiveFailed: #removeLockAll: args: { aCollection } .
self _uncontinuableError
%

category: 'Session Control'
classmethod: System
clientIsRemote

"Returns true if the GemBuilder for C client for this session is in a different
 process than the Gem, otherwise returns false."

^ self _zeroArgPrim: 20
%

category: 'Deprecated'
classmethod: System
currentSegment

self deprecated: 'System class>>currentSegment deprecated v3.0. 
Use #currentObjectSecurityPolicy instead'.
^self currentObjectSecurityPolicy.
%

category: 'Deprecated'
classmethod: System
currentSegment: anObjectSecurityPolicy

self deprecated: 'System class>>currentSegment: deprecated v3.0. 
Use #currentObjectSecurityPolicy: instead'.
^self currentObjectSecurityPolicy: anObjectSecurityPolicy
%

category: 'Session Control'
classmethod: System
currentObjectSecurityPolicy

"Returns the GsObjectSecurityPolicy in which objects created in the current session 
 are stored.  The result can be nil, in which case objects are
 being created with World write  permission.

 At login, the current security policy is the default security policy of the
 UserProfile for the session of the sender."

<primitive: 341>
self _primitiveFailed: #currentObjectSecurityPolicy .
^ nil
%

category: 'Session Control'
classmethod: System
currentObjectSecurityPolicy: anObjectSecurityPolicy

"Redefines the GsObjectSecurityPolicy in which subsequent objects created in the
 current session will be stored.  Returns the receiver.

 If the argument is nil , subsequent objects are created
 with World write permission.

 The argument must be a committed GsObjectSecurityPolicy and you must have
 writeAuthorization to that objectSecurityPolicy, otherwise an error is generated.
"

<primitive: 339>
anObjectSecurityPolicy _validateClass: GsObjectSecurityPolicy.
self _primitiveFailed: #currentObjectSecurityPolicy: 
     args: { anObjectSecurityPolicy } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_sessionProfilesStartingAt: startSessionId 

"Returns an Array containing pairs of sessionId, UserProfile , 
 for some sessions currently logged in, beginning at the specified sessionId. 
 Some sessionIds may have a userProfile of nil
 if that UserProfile is not visible in the transactional view of 
 the current session.
 The size of the Array is limited by number of sessions logged in
 and/or gem to stone communication buffer sizes.  If there are
 no more valid sessionIds beginning with startSessionId, an
 Array of size 0 is returned. 

 This method requires SessionAccess privilege if there is more
 than one session logged in."

<primitive: 329>
startSessionId _validateClass: SmallInteger .
startSessionId _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #_sessionProfilesStartingAt: args: { startSessionId }
%

! fixed 34878
category: 'Private'
classmethod: System
_currentSessionProfiles

"Returns an Array containing pairs of sessionId, UserProfile
 for all sessions currently logged in.
 Some sessionIds may have a userProfile of nil

 This method requires SessionAccess privilege if there is more
 than one session logged in."

| sessId result batch |
sessId := 1 .
result := { }  .
[ sessId > 0 ] whileTrue:[
  batch := self _sessionProfilesStartingAt: sessId .
  batch size > 0 ifTrue:[
    result addAll: batch .
    sessId := (batch at: (batch size - 1)) + 1 .
  ] ifFalse:[
    sessId := -1 . "done"
  ].
].
^ result
%

category: 'Session Control'
classmethod: System
currentSessionNames

"Returns a formatted String containing, for each current GemStone session, the
 session number and userId.

 This method requires SessionAccess privilege if there is more
 than one session logged in."

| lf result profiles |

result := String new.  
profiles := self _currentSessionProfiles.  "sessId, userProfile pairs"
lf := Character lf.

1 to: profiles size by: 2 do:[:j| | aSessionNumber aUserProfile |
   aSessionNumber := profiles at: j .
   aUserProfile := profiles at: j + 1 .
   aUserProfile ~~ nil ifTrue: [
     result add: lf;
        addAll: 'session number: ';
        addAll: (aSessionNumber asString);
        addAll: '    UserId: ';
        addAll: (aUserProfile userId).
   ].
].
^ result
%
category: 'Session Control'
classmethod: System
otherSessionNames

"Returns a formatted String containing, for each current GemStone session
  other than the session executing this method, the
 session number and userId.

 This method requires SessionAccess privilege."

| lf result mySessNum profiles |

result := String new.  " string to return "
lf := Character lf.
mySessNum := self session .

profiles := self _currentSessionProfiles.  "sessId, userProfile pairs"
1 to: profiles size by: 2 do:[:j| | aSessionNumber aUserProfile |
   aSessionNumber := profiles at: j .
   aSessionNumber == mySessNum ifFalse:[
     aUserProfile := profiles at: j+ 1 .
     aUserProfile ~~ nil ifTrue: [
       result add: lf;
	  addAll: 'session number: ';
	  addAll: (aSessionNumber asString);
	  addAll: '    UserId: ';
	  addAll: (aUserProfile userId).
     ].
   ].
].
^result 
%

category: 'Session Control'
classmethod: System
currentSessions

"Returns an Array of SmallIntegers corresponding to all of the sessions
 currently running on the GemStone system."

^self _zeroArgPrim: 85
%

category: 'Session Control'
classmethod: System
currentUserSessionCount

"Return a SmallInteger which is the number of user sessions present in the system.
 Garbage collection sessions, the Symbol Gem, and the Page Manager Gem are not 
 included in the count."

^self _zeroArgPrim: 48
%

category: 'Session Control'
classmethod: System
currentSessionCount

"Return a SmallInteger which is the number of sessions present in the system,
 including the Symbol Gem, garbage collection sessions, but not the page manager
 session."

^self _zeroArgPrim: 49
%


category: 'Session Control'
classmethod: System
myUserProfile

"Returns the UserProfile of the current session."

"Implemented as a primitive to handle image bootstrap problems."

<primitive: 318>
^ self _primitiveFailed: #myUserProfile
%

category: 'Session Control'
classmethod: System
session

"Returns a SmallInteger representing the session of the sender."

^self _zeroArgPrim: 33
%

category: 'Session Control'
classmethod: System
stopUserSessions

"Prevents any new sessions from being initiated; then, for each
 active user session other than the session of the user executing this
 method, aborts the transaction and terminates that session.

 This method does not stop Gc Gem sessions nor does it stop 
 the Symbol Creation Gem .

 To reenable logins, send the message System resumeLogins.  Otherwise,
 logins are automatically reenabled when this session logs out.

 To execute this method, you must have explicit privilege for
 SessionAccess and SystemControl in your UserProfile."


"Performs a System | suspendLogins, then
 performs a System | stopSession."

| anArrayOfSessions mySession myPrivs |

myPrivs := self myUserProfile privileges.
((myPrivs includesValue: #SystemControl) and:
   [myPrivs includesValue: #SessionAccess])
   ifFalse: [ self _error: #rtErrNoPriv .  ^ self "do nothing" ].
self suspendLogins.  " Prevent anyone from logging in "

anArrayOfSessions := self currentSessions.  " All Session running "
mySession := self session.  " My Session number "

anArrayOfSessions do: " Loop all session and log then off if not me "
   [:aSessionNumber | (aSessionNumber == mySession )
                         ifFalse:[self stopSession: aSessionNumber].
   ].
%

category: 'Session Control'
classmethod: System
stopSession: aSessionId

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated
 session is not active, no operation is performed.

 Implemented by sending a stopSession out-of-band byte to the session's
 gem process.   If the session is not responsive to this request,
 it will eventually be killed per STN_GEM_TIMEOUT in stone's config file.
 
 Does not stop Garbage Collector or Symbol Creation Gem sessions.

 To execute this method, you must have explicit privilege from your
 system data curator."

| serialNum |

aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r0 timeout: -1
%

category: 'Session Control'
classmethod: System
stopGcSession: aSessionId

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated
 session is not active, no operation is performed.
 
 Will stop Garbage Collector sessions but will not stop the
 Symbol Creation Gem session .

 To execute this method, you must have explicit privilege from your
 system data curator."

| serialNum |

aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r1 timeout: -1
%

category: 'Session Control'
classmethod: System
stopSymbolCreationSession: aSessionId

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated
 session is not active, no operation is performed.
 
 Will stop any session including garbage Collector and 
 Symbol Creation Gem sessions. 

 Only SystemUser may execute this method, otherwise an error is generated."

| serialNum |

aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r3  timeout: -1
%

category: 'Session Control'
classmethod: System
terminateSymbolCreationSession: aSessionId timeout: aSeconds

"Will stop normal sessions, Garbage Collector Gem sessions,
 and  Symbol Creation Gem .

 Only SystemUser may use this method.
 Does not prevent auto-restart of Symbol Creation Gem sessions
 stopped by this method.

 Otherwise same as terminateSession:timeout:"

| serialNum |
aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r13 timeout: aSeconds.
%

! edits to fix 36706 , 41648
category: 'Session Control'
classmethod: System
terminateSession: aSessionId timeout: aSeconds

"Supports faster termination of the specified session than is
 possible with   stopSession:   .

 Sends a stopSession out-of-band byte to the specified session's gem
 process.
 The argument aSeconds is a SmallInteger. aSeconds >= 0 specifies number of
 seconds to wait after sending the stopSession out-of-band byte 
 before killing specified session's gem process by sending
 it a SIGTERM.  aSeconds < 0 means move the session to after_logout state
 immediately, but don't kill .  

 If the stone config file specifies non-default STN_GEM_TIMEOUT , 
 the actual timeout will be limited by    aSeconds min: STN_GEM_TIMEOUT*60  ,  
 otherwise timeout is limited in stone by     aSeconds min: 300 .

 If the current session is not logged in as SystemUser,
 this method requires SystemControl and SessionAccess privileges .

 Will stop normal sessions and Garbage Collector Gem sessions.
 Does not prevent auto-restart of Garbage Collector Gem sessions
 that are stopped by this method.

 Will not stop the Symbol Creation Gem."

| serialNum |
aSessionId _validateClass: SmallInteger.
serialNum := GsSession serialOfSession: aSessionId .
self _stopSession: serialNum kind: 16r11 timeout: aSeconds.
%

category: 'Session Control'
classmethod: System
_stopSession: aSerialNumber  kind: aKind timeout: aSeconds

"Aborts the transaction of the specified session (a SmallInteger), then
 terminates that session.  Returns the receiver.  If the indicated session is
 not active, no operation is performed.

 aKind has the following bits
   16r1  include GcGem sessions
   16r2  include SymbolCreation session

   16r10  timeout aSeconds per terminateSession:timeout: ,
	  and do not inhibit   restart of gc gems or symbol gem

 To execute this method, you must have explicit privilege from your system
 data curator."

<primitive: 332>  
aSerialNumber _validateClass: SmallInteger.
aSeconds _validateClass: SmallInteger.
aKind _validateClass: SmallInteger.
aKind _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #_stopSession:kind:timeout:
       args: { aSerialNumber . aKind . aSeconds }
%

category: 'Session Control'
classmethod: System
userProfileForSession: aSessionId

"Returns the UserProfile attached to the specified session (a SmallInteger).
 If the indicated session is not active, returns nil.

 Requires SessionAccess privilege if aSessionId is not the current session."

^ ( self descriptionOfSession: aSessionId) at: 1
%

category: 'Session Control'
classmethod: System
users

"Returns a Set of UserProfiles for all users known to the system."

^ AllUsers
%

category: 'Session Control'
classmethod: System
maxSessionId

"Returns a SmallInteger representing the maximum number of sessions
 allowed on the system based upon the Stone configuration parameter."

^ self _zeroArgPrim: 6
%

category: 'Session Control'
classmethod: System
__sleepMs: milliSecondsToSleep

"Sleep for the number of milliseconds specified by milliSecondsToSleep.
 The argument milliSecondsToSleep must be a positive SmallInteger.
 If milliSecondsToSleep is zero, this method has no effect.
 Returns time left to sleep , which will be > 0 if interrupted.
 The ProcessorScheduler is not used, and other waiting GsProcesses 
 will not execute during the sleep. Use a Delay or Semaphore to wait 
 while allowing other GsProcesses to run."

<primitive: 344>
milliSecondsToSleep _validateClass: SmallInteger.
milliSecondsToSleep < 0 
   ifTrue: [ milliSecondsToSleep _error: #errArgTooSmall args:{ 0 } ] .
^ self _primitiveFailed: #_sleepMs: args: { milliSecondsToSleep }
%

! Gs64 v2.1 ,  _sleepMs: is interruptable by soft-break and sigAbort 
category: 'Session Control'
classmethod: System
_sleepMs: milliSecondsToSleep

"Sleep for the number of milliseconds specified by milliSecondsToSleep.
 The argument milliSecondsToSleep must be a positive SmallInteger.
 If milliSecondsToSleep is zero, this method has no effect.
 Returns the receiver."

| timeLeft |
milliSecondsToSleep _isSmallInteger ifFalse:[ 
  milliSecondsToSleep _validateClass: SmallInteger 
].
milliSecondsToSleep < 0 ifTrue:[ 
  milliSecondsToSleep _error: #errArgTooSmall args:{ 0 } ].
timeLeft := milliSecondsToSleep .
[ true ] whileTrue:[
  timeLeft := self __sleepMs: timeLeft .
  timeLeft <= 0 ifTrue:[ ^ self ].
]
%

category: 'Session Control'
classmethod: System
_sleep: aTime

"Sleep for aTime seconds.  aTime must be a positive SmallInteger.  
 If aTime is zero, this method has no effect.

 Returns time left to sleep , which will be > 0 if interrupted.
 The ProcessorScheduler is not used, and other waiting GsProcesses 
 will not execute during the sleep. Use a Delay or Semaphore to wait 
 while allowing other GsProcesses to run."
 
<primitive: 370>
aTime _validateClass: SmallInteger .
aTime < 0 ifTrue: [ aTime _error: #errArgTooSmall args:{ 0 } ] .
^ self _primitiveFailed: #_sleep: args: { aTime }
%

! Gs64 v2.1 ,  sleep: is interruptable by soft-break
category: 'Session Control'
classmethod: System
sleep: aTime

"Sleep for aTime seconds.  aTime must be a positive SmallInteger.
 If aTime is zero, this method has no effect. 
 Returns the receiver.  
 The ProcessorScheduler is not used, and other waiting GsProcesses 
 will not execute during the sleep. Use a Delay or Semaphore to wait 
 while allowing other GsProcesses to run."

| timeLeft |
aTime _isSmallInteger ifFalse:[ aTime _validateClass: SmallInteger ].
aTime < 0 ifTrue: [ aTime _error: #errArgTooSmall args:{ 0 } ] .
timeLeft := aTime .
[ true ] whileTrue:[
  timeLeft := self _sleep: timeLeft . 
  timeLeft <= 0 ifTrue:[ ^ self ].
]
%

category: 'Session Control'
classMethod: System
sessionsReferencingOldestCr

"Returns an Array containing the sessionIds of the sessions that are
 currently referencing the oldest commit record.  Sessions both inside and outside
 of a transaction will be returned.  Because a session can update its commit
 record without committing a transaction, it is possible that no session
 actually references the oldest commit record.  Therefore, this method
 may return an empty Array."
       
^self _zeroArgPrim: 21
%

category: 'Session Control'
classmethod: System
descriptionOfSession: aSessionId

"Returns an Array describing the session as follows:

 1.  The UserProfile of the session, or nil if the UserProfile is recently
     created and not visible from this session's transactional view ,
     or the session is no longer active .
 2.  The process ID of the Gem process of the session (an Integer).
 3.  The hostname of the machine running the Gem process
     (a String, limited to 127 bytes).
 4.  Primitive number in which the Gem is executing, or 0 if it is not executing
     in a long primitive.
 5.  Time of the session's most recent beginTransaction, commitTransaction, or
     abortTransaction (from System timeGmt).
 6.  The session state (a SmallInteger).
 7.  A SmallInteger whose value is -1 if the session is in transactionless mode,
     0 if it is not in a transaction and 1 if it is in a transaction.
 8.  A Boolean whose value is true if the session is currently referencing the
     oldest commit record, and false if it is not.
 9.  The session's serial number (a SmallInteger).
 10. The session's sessionId (a SmallInteger).
 11. A String containing the ip address of host running the GCI process.
     If the GCI application is linked (using libgcilnk*.so or gcilnk*.dll) 
     this ip address is the address of the machine running the gem process .
 12. The priority of the session (a SmallInteger).
 13. Unique host ID of the host where the session is running (an Integer)
 14. Time of the session's most recent request to stone (from System timeGmt)
 15. Time the session logged in (from System timeGmt)
 16. Number of commits which have occurred since the session obtained its view.

 Because a session can update its commit record without committing a
 transaction, it is possible that no session actually references the oldest
 commit record.  Therefore, the eighth element may be false for all current
 sessions.

 To execute this method for any session other than your current session, you
 must have the SessionAccess privilege."

^ self _descriptionOfSessionSerialNum: 0 sessionId: aSessionId
%

category: 'Session Control'
classmethod: System
descriptionOfSessionSerialNum: aSerialNumber

"Returns an eleven-element Array describing the session identified by aSerialNumber.

 See System (C) | descriptionOfSession: for documentation on the contents of the
 result Array.

 Requires SessionAccess privilege if aSerialNumber is not the current session."

^ self _descriptionOfSessionSerialNum: aSerialNumber sessionId: -1
%

category: 'Session Control'
classmethod: System
_descriptionOfSessionSerialNum: serialNum sessionId: aSessionId

"Returns an eleven-element Array describing the session identified by the arguments.   
 The session is looked up by serial number if serialNum is > 0.  Otherwise
 it is looked up by sessionId.

 See System (C) | descriptionOfSession: for documentation on the contents of the
 result Array.
 Requires SessionAccess privilege if the session being looked up is not 
 the current session."

<primitive: 334>
aSessionId _validateClass: SmallInteger.
^ self _primitiveFailed: #_descriptionOfSessionSerialNum:sessionId:
       args: { serialNum . aSessionId }
%

category: 'Setting Locks'
classmethod: System
_lockError: errNum obj: anObj details: details

"Translate from SmallInteger to Symbol, for error reporting."
"Ugh."

| ex sym |
ex := LockError new _number: errNum ; object: anObj ; details: details .

(errNum == 2075) ifTrue: [  sym := #lockErrDenied ] ifFalse:[
(errNum == 2074) ifTrue: [  sym := #lockErrObjHasChanged] ifFalse:[
(errNum == 2071) ifTrue: [  sym := #lockErrUndefinedLock ] ifFalse:[
(errNum == 2073) ifTrue: [  sym := #lockErrIncomplete ] ifFalse:[
(errNum == 2418) ifTrue: [  sym := #lockErrDeadlock ] ifFalse:[
(errNum == 2419) ifTrue: [  sym := #lockErrTimeout ] ifFalse:[
]]]]]].

^ ex reason: sym ; signal
%

category: 'Setting Locks'
classmethod: System
readLock: anObject

"This method grants a read lock on anObject if another session already holds
 a read lock, but grants no lock if another session already holds a write
lock to anObject."

| result |
result := self _lock: anObject kind: 2 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
self _lockError: result obj: anObject details: 'read lock' 
%

category: 'Setting Locks'
classmethod: System
readLock: anObject ifDenied: denyBlock ifChanged: changeBlock

"Requests a read lock on anObject.  This method denies a read lock
 on anObject under any one of the following circumstances:

 * Another session has a write lock on the object.

 * The object is special.

 Returns the receiver if the requested lock was granted and was not dirty.

 If the requested lock is otherwise denied, it returns the value of
 the zero-argument block denyBlock.  If it grants a dirty lock, then it
 returns the value of the zero-argument block changeBlock.  In that case the
 lock remains, even after the transaction is aborted."

| result |

result := self _lock: anObject kind: 2 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
^ self _lockEvaluateErr: result obj: anObject denied: denyBlock changed: changeBlock
%

category: 'Setting Locks'
classmethod: System
readLockAll: aCollection

"Requests a read lock on each object in aCollection.  This method denies
 a read lock on an object under the following circumstances:

 * Another session already holds a write lock on the object.

 This method grants a read lock on an object whenever it finds no reason
 to deny it.  However, a lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 If a lock was acquired for every element of aCollection, and no locks are
 dirty, returns the receiver.

 This method generates an error if it is unable to acquire a lock for every
 element of aCollection, or if any lock that it acquires is dirty.  However,
 all the locks that it acquires remain in place, even after the current
 transaction is aborted."


| result |
result := self _lockAll: aCollection kind: 2 .
(result == self )
  ifTrue: [^self]
  ifFalse: " else we generate the error lock incomplete "
           [^ (result at: 1) _error: #lockErrIncomplete
                             args: { (result at:2) . (result at:3) }].
%

category: 'Setting Locks'
classmethod: System
readLockAll: aCollection ifIncomplete: incompleteBlock

"Requests a read lock on each object in aCollection.  This method denies
 a read lock on an object in the collection under any one of the following
 circumstances:

 * Another session already holds a write lock an object.

 * The object is special.

 If all requested locks were granted and none of the locks are 'dirty',
 returns the receiver.

 This method grants a read lock on an object whenever it finds no reason
 to deny it.  However, a lock that it grants may be dirty.  One session's
 lock is dirty if another session has committed a change to the locked object
 since the beginning of the first session's current transaction.  A session
 that holds a dirty lock cannot commit its transaction.  To clean its locks,
 it must abort the transaction and obtain updated values for each object whose
 lock is dirty.

 If this method is unable to acquire a lock for every element of aCollection,
 or if any lock that it acquires is dirty, then it returns the value of the
 three-argument block incompleteBlock.  The arguments to the block are:

 1.  An Array of objects that could not be locked.

 2.  An Array of objects that were locked but whose locks are dirty.

 3.  An empty Array, retained for backward compatibility with old GemStone versions.

 All the locks that it acquires remain in place, even after the current
 transaction is aborted."

| result |
result := self _lockAll: aCollection kind: 2 .
(result == System)
  ifTrue: [^self]
  ifFalse: " else we execute the incomplete block with 3 arguments "
          [^incompleteBlock value: (result at: 1)
                            value: (result at:2)
                            value: (result at:3)].
%


category: 'Setting Locks'
classmethod: System
writeLock: anObject

"Analogous to System | readLock:.  However, this method requests and
 grants write locks."

| result |

result := self _lock: anObject kind: 3 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
self _lockError: result obj: anObject details: 'write lock' 
%

category: 'Setting Locks'
classmethod: System
writeLock: anObject ifDenied: denyBlock ifChanged: changeBlock

"Analogous to System | readLock:ifDenied:ifChanged:.  However,
 this method requests and grants write locks."

| result |
result := self _lock: anObject kind: 3 autoRelease: false .
(result == 1) ifTrue:[ ^ self ].
^ self _lockEvaluateErr: result obj: anObject denied: denyBlock changed: changeBlock
%

category: 'Setting Locks'
classmethod: System
writeLockAll: aCollection

"Analogous to System | readLockAll:.  However, this method requests and
 grants write locks."

| result |
result := self _lockAll: aCollection kind: 3 .
(result == System)
  ifTrue: [^self]
  ifFalse: " else we generate the error lock incomplete "
           [^ (result at: 1) _error: #lockErrIncomplete
                             args: { (result at:2) . (result at:3) }].
%

category: 'Setting Locks'
classmethod: System
writeLockAll: aCollection ifIncomplete: incompleteBlock

"Analogous to System | readLockAll:ifIncomplete:.  However, this method
 requests and grants write locks."

| result |
result := self _lockAll: aCollection kind: 3 .
(result == System)
  ifTrue: [^self]
  ifFalse: " else we execute the incomplete block with 3 arguments "
          [^incompleteBlock value: (result at: 1)
                            value: (result at:2)
                            value: (result at:3)].
%

category: 'Setting Locks'
classmethod: System
waitForRcWriteLock: rcLockObject 

"Waits for the RcWriteLock using the specified object and
 returns a SmallInteger, one of 
   1  granted
   2074  dirty (object written by other session since start of this transaction)
   2075  denied (locked by another session)
   2418  deadlock
   2419  timeout  (per STN_OBJ_LOCK_TIMEOUT config parameter)

 All calls must use a persistent rcLockObject which is identical during the
 life of a stone process .
 If result is 1 or 2074, the rcLockObject is added to both 
 the CommitReleaseLocksSet and CommitOrAbortReleaseLocksSet 
 hidden sets of this session.

 Error 2418 deadlock  is returned if
   the requesting session already holds a readLock on rcLockObject
 or if 
  the session would have to wait for the lock on rcLockObject, but doing
  so would create a cycle in the graph of locks being waited for, i.e:

    the session already holds a read or write lock on some object X
      and some other session is waiting in an invocation of
        waitForRcWriteLock: X
      or
        waitForApplicationWriteLock: X ...
    and this session's attempt at
       waitForRcWriteLock: rcLockObject
    would have to wait to get the lock on rcLockObject
"

^ self _lock: rcLockObject kind: 4 autoRelease: true
%

category: 'Setting Locks'
classmethod: System
waitForApplicationWriteLock: lockObject queue: lockIdx autoRelease: aBoolean

"Waits for a write lock using the specified object and
 the Application lock queue specifed by lockIdx .

 lockIdx must be a SmallInteger,  >= 1  and  <= 10

 returns a SmallInteger, one of 
   1  granted
   2071  undefined lock (lockIdx out of range or lockObject is special object)
   2074  dirty (object written by other session since start of this transaction)
   2418  deadlock
   2419  timeout  (per STN_OBJ_LOCK_TIMEOUT config parameter)

 For a given value of lockIdx, all calls must use a persistent lockObject  
 which is identical during the life of a stone process .

 If result is 1 or 2074 and aBoolean == true,  the lockObject is added to both 
 the CommitReleaseLocksSet and CommitOrAbortReleaseLocksSet 
 hidden sets of this session.

 The result 2418 (deadlock)   is returned if
   the requesting session already holds a readLock on lockObject
 or if 
  the session would have to wait for the lock on lockObject, but doing
  so would create a cycle in the graph of locks being waited for, i.e:

    the session already holds a read or write lock on some object X
      and some other session is waiting in an invocation of
        waitForRcWriteLock: X
      or
        waitForApplicationWriteLock: X ...
    and this session's attempt at
       waitForApplicationWriteLock: lockObject
    would have to wait to get the lock on lockObject
"

lockIdx < 1 ifTrue:[ ^ 2071 "lockIdx out of range"].
^ self _lock: lockObject kind: lockIdx + 4 autoRelease: aBoolean
%

category: 'Setting Locks'
classmethod: System
_lockEvaluateErr: errNum obj: anObject denied: denyBlock changed: changeBlock

"Private."

(errNum == 2074) ifTrue:[ ^ changeBlock value ]. "object  written by other session"
(errNum == 2075) ifTrue:[ ^ denyBlock value ]. " lock denied "

^ self _lockError: errNum obj: anObject details: 'unexpected lock error'
%

category: 'Setting Locks'
classmethod: System
_lockAll: aCollection kind: lockKind

"Attempts to lock all the objects in aCollection.

 The lockKind argument specified the kind of lock and 
 must be a SmallInteger, one of 
    2  read
    3  write 

 An error is generated and no locks are obtained
 if aCollection is a byte format object.

 An error is generated and no locks are obtained if any element
 of aCollection is identical to AllSymbols .

 Returns the receiver if all locks were granted and not dirty .
 Otherwise returns an Array of size 3 with elements :
   1.  An Array of objects that could not be locked.
   2.  An Array of objects that were locked but whose locks are dirty.
   3.  An empty Array, retained for backward compatibility with old GemStone versions.
"

<primitive: 100>
"primitive fails if aCollection is not a kind of SequenceableCollection
 or IdentityBag "
| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self _lockAll: anArray kind: lockKind
  ] .
aCollection _validateClass: Collection.
lockKind _validateClass: SmallInteger .
self _primitiveFailed: #_lockAll:kind: args: { aCollection . lockKind } .
self _uncontinuableError
%

category: 'Setting Locks'
classmethod: System
_lock: anObject kind: lockKind  autoRelease: autoReleaseBoolean

"Attempt to lock a single object.  
 lockKind argument must be a SmallInteger , one of
    2  read
    3  write 
    4  wait for RcRetry write lock
    5  wait for Application write lock 1
    ... 
   14  wait for Application write lock 10

 lockKinds 4..14  may be used with only one unique persistent object 
 during the life of a stone process .  The first use of that lockKind 
 registers anObject with that transient queue in the stone process, and 
 subsequent uses of that lockKind with an object not identical to anObject 
 will generate an error.  An error will be generated if anObject is
 not committed.  The queues are reinitialized each time stone process restarts.

 Any attempt to lock AllSymbols generates an error.

 This method returns a SmallInteger.
 Attempt to lock a special object always returns  2075  denied .
 If lockKind is < 2 or > 14 ,   returns 2071 .

 read and write requests return one of  
   1  granted
   2074  dirty (object written by other session since start of this transaction)
   2075  denied (locked by another session)

 'wait for' requests return one of 
   2074  dirty (object written by other session since start of this transaction)
   2075  denied (locked by another session)
   2418  deadlock 
   2419  timeout  (per STN_OBJ_LOCK_TIMEOUT config parameter)

 If autoReleaseBoolean == true and result is either 1 or 2074 ,
 then the object is added to  both the CommitReleaseLocksSet
 and CommitOrAbortReleaseLocksSet  hidden sets .
"

<primitive: 97>
self _primitiveFailed: #_lock:kind:autoRelease:
     args: { anObject . lockKind . autoReleaseBoolean } .
self _uncontinuableError
%

category: 'Lock Status'
classmethod: System
lockKind: anObject

"Returns a Symbol (#none, #read, or #write) representing the kind
 of lock held on anObject by any session in the system."

| anArray |

anArray := self lockStatus: anObject.
^anArray at: 1.
%

category: 'Lock Status'
classmethod: System
lockOwners: anObject

"Returns an Array of session numbers (SmallIntegers) representing the sessions
 that hold a lock on anObject.  If the object is not locked by any session, the
 result Array is empty.  Note that a write lock can have only one owner."

| anArray |

anArray := self lockStatus: anObject.
^anArray at: 2.
%

category: 'Lock Status'
classmethod: System
lockStatus: anObject

"Returns a two-element Array, where the first element is a Symbol representing
 the kind of lock held on anObject (#none, #read or #write) and
 the second element is an Array of session numbers (SmallIntegers) representing
 the sessions that hold the lock.

 If there are no locks on anObject, the first element is the Symbol #none and
 the second element is an empty Array.

 Only locks on permanent objects are reported."

<primitive: 351>
self _primitiveFailed: #lockStatus: args: { anObject } .
self _uncontinuableError
%

category: 'Lock Status'
classmethod: System
sessionLocks

"Returns a three-element Array describing the locks held by the current
 session.  The first element is an Array of all read-locked objects, the second
 is an Array of all write-locked objects, and the third is empty array (which
 held exclusive-locked objects in older releases).  If the current session holds
 no locks of a particular kind (read, write), then the corresponding Array is
 empty.  If the current session holds no locks at all, then all three of these
 Arrays are empty."

<primitive: 352>
self _primitiveFailed: #sessionLocks .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_systemLocksPrim

"For use in implementation of System(C) | systemLocks.  Returns an Array of
 Associations, with key sessionId."

<primitive: 353>

self _primitiveFailed: #_systemLocksPrim .
self _uncontinuableError
%

category: 'Lock Status'
classmethod: System
systemLocks

"Returns a Dictionary describing all objects that are currently locked.  For
 each Association in the result Dictionary, the key is a SmallInteger (the
 session number of a GemStone session that holds locks) and the value is the
 three-element Array described in the sessionLocks method.  If no sessions hold
 any locks, the result Dictionary is empty.

 The Arrays in the result Dictionary contain only those objects that are
 visible to the current session.  This method does not return locks that the
 current session cannot see (objects that have been committed since the
 beginning of the current transaction, uncommitted objects from other sessions,
 or locks on objects for which this session has no read authorization).

 Locks on temporary objects in other sessions are not reported."

| anArray result |
anArray := self _systemLocksPrim  .
result := Dictionary new .
1 to: anArray size do:[ :j |
   result add: (anArray at:j)
].
^ result
%

category: 'Performance Monitoring'
classmethod: System
_approxOopHighWaterMark

"Returns an Integer that gives the approximate highest oop allocated in GemStone."

^self _zeroArgPrim: 23
%

category: 'Performance Monitoring'
classmethod: System
_oopHighWaterMark

"Returns an Integer that gives the highest oop allocated in GemStone."

^self _zeroArgPrim: 93
%

category: 'Performance Monitoring'
classmethod: System
_oopNumberHighWaterMark

"Returns an Integer that gives the highest oop number allocated in GemStone."

^self _zeroArgPrim: 94
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpaceUsed

"Returns the approximate number of bytes of temporary object memory being used
 to store objects."

^self _zeroArgPrim: 77
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpaceMax

"Returns the approximate maximum number of bytes of temporary object memory which is
 usable for storing objects."

^self _zeroArgPrim: 78
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpacePercentUsed

"Returns the approximate percentage of temporary object memory which is
 in use to store temporary objects.  This is equivalent to the 
 expression:

 (self _tempObjSpaceUsed * 100) // self _tempObjSpaceMax.

 Note that it is possible for the result to be slightly greater than 100%.
 This result indicates temporary memory is almost completely full."

^self _zeroArgPrim: 79
%

category: 'Performance Monitoring'
classmethod: System
_tempObjSpacePercentUsedLastMark

"Returns the approximate percentage of temporary object memory which is
 in use to store temporary objects, as of the last MarkSweep."

^self _zeroArgPrim: 86
%

category: 'Performance Monitoring'
classmethod: System
_approxOopNumberHighWaterMark

"Returns an Integer that gives the approximate highest oop number allocated in GemStone."

^self _zeroArgPrim: 36
%


category: 'Performance Monitoring'
classmethod: System
millisecondsToRun: aBlock

"Returns the number of CPU milliseconds used while evaluating aBlock.
 The argument aBlock must be a zero-argument block.

 Note that resolution of _readClock is on the order of 10ms so 
 this method should not be used to time operations that take 
 less than 50ms."

| startTime endTime |

startTime := self _readClock.
aBlock value.
endTime := self _readClock.
(endTime >= startTime) ifTrue: [
  ^ endTime - startTime
] ifFalse: [
  ^ endTime + (16rffffffff - startTime)
]
%

category: 'Performance Monitoring'
classmethod: System
pageReads

"Returns the number of Repository page read operations performed since the start
 of the Gem process.  If the Gem is remote, this corresponds to reads performed
 in the current session.  If the Gem is linked, it corresponds to reads
 performed since the application was invoked."

^ self _zeroArgPrim: 2
%

category: 'Performance Monitoring'
classmethod: System
pageWrites

"Returns the number of Repository page write operations performed since the
 start of the Gem process.  If the Gem is remote, this corresponds to writes
 performed in the current session.  If the Gem is linked, it corresponds to
 writes performed since the application was invoked."

^ self _zeroArgPrim: 3
%

category: 'Performance Monitoring'
classmethod: System
_readClock

"Returns an Integer indicating the amount of CPU time used by the current
 process, in units of milliseconds.  The resolution of the result is operating
 system dependent; typical resolution is 10 to 50 milliseconds."

^ self _zeroArgPrim: 10
%

category: 'Cache Statistics'
classmethod: System
myCacheProcessSlot

"Returns the process slot in the SharedPageCache that corresponds
 to my process.  If the SharedPageCache is not in use, returns -1."

^ self _zeroArgPrim: 7
%	

category: 'Cache Statistics'
classmethod: System
cacheSlotForSessionId: aSessionId

"Return the cache process slot number (a SmallInteger) for the given session
 ID.  The session must be connected to the same shared page cache as the 
 session invoking this method.  

 A return of nil indicates the session could not be located."

<primitive: 567>
aSessionId _validateClass: SmallInteger.
aSessionId _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #cacheSlotForSessionId: args: { aSessionId }
%

category: 'Cache Statistics'
classmethod: System
cacheStatsForGemWithName: aStringOrSymbol

"This method does a case-sensitive search for the gem with the given cache
 name and returns an array of cache statistics for the gem if found.

 Returns nil if the gem could not be located."

<primitive: 565>
aStringOrSymbol _validateClass: String.
^ self _primitiveFailed: #cacheStatsForGemWithName: args: { aStringOrSymbol }
%

category: 'Private'
classmethod: System
_primCacheStatsForSlotOrSession: arg opCode: code

"This primitive performs various functions relating to cache statistics per
 the following table:

arg  code    meaning
=======================================================================
 +     0     arg is slot.  Return all stats for all kinds* 
 +     1     arg is slot.  Return only stats for the process.
 -     0     arg is a gem -session ID.  Return all stats for all kinds* 
 -     1     arg is a gem -session ID.  Return only stats for the gem.
 +     2     arg is a PID.  Return only stats for the process.

 str   3     arg is a cache name string.  Return only stats for the
             process matching the name.
 +     4     arg is a slot.  Return  the Pid for slot.
 +     5     arg is a PID.  Return the slot for the PID.
 +     6     arg is a slot.  Return host cpu stats for slot.
 +     7     arg is a PID.  Host cpu stats for pid.
 +     8     arg is a pgsvr session ID.  Return only stats for the 
             pgsvr.
 +     9     arg is a slot.  Return the process kind for the slot
             (1 = monitor, 2 = stone, 3 = pgsvr, 4 = gem) or -1 if
             the slot is not in use.
=======================================================================

Legend:

+     a postive SmallInteger
-     a negative SmallInteger
str   aString
*     deprecated functionality which may be removed in a future release.
"

<primitive: 220>
code _validateClass: SmallInteger .
code == 3  
  ifTrue:[ arg _validateClass: String ]
  ifFalse:[ arg _validateClass: SmallInteger] .
(code < 0 or:[ code > 7])
  ifFalse:[ code _error: #rtErrArgOutOfRange args:{ 0 . 7 } ] .
arg _error: #rtErrArgOutOfRange .
^ self _primitiveFailed: #_primCacheStatsForSlotOrSession:opCode:
       args: { arg . code }
%

category: 'Authorization'
classmethod: System
canRead: anObject

"In Gemstone64 , an object must be faulted into memory before
 it can be passed as an argument to a method.  Thus by definition
 if you enter this method you can read the argument. "

^ true
%

category: 'Authorization'
classmethod: System
canWrite: anObject

"This method tests whether the user has authorization to write anObject without
 adding it to write set and returns a Boolean result."

<primitive: 103>
self _primitiveFailed: #canWrite: args: { anObject } .
self _uncontinuableError
%

category: 'Time'
classmethod: System
_timeMs

"Returns a SmallInteger representing the current relative time in milliseconds.
 The result is a SmallInteger equivalent to
    (System _timeGmtFloat * 1000) asInteger 

 The result is computed locally in the session process, using the offset 
 from Stone's time that was cached in the session at login.

 Gs64 v2.2, changed to no longer rollover to zero after 524287999 "
 
<primitive: 651>
^ self _primitiveFailed: #_timeMs
%
category: 'Time'
classmethod: System
_timeMsLegacy

"Returns a SmallInteger representing the current relative time in milliseconds.
 The result is a SmallInteger equivalent to
    (System _timeGmtFloat * 1000) asInteger \\ 524288000
 
 The result is computed locally in the session process, using the offset 
 from Stone's time that was cached in the session at login."

^ self _zeroArgPrim: 12
%

! edited to fix 37020
category: 'Time'
classmethod: System
_timeGmtFloat

"Returns a Float representing the time since January 1, 1970 in units
 of seconds, to microsecond resolution, or such resolution as provided
 by the operating system's gettimeofday() call.  This time is computed locally
 in the session process, using the offset from Stone's time that was
 cached in the session at login."

^ self _zeroArgPrim: 13 
%

category: 'Time'
classmethod: System
_timeGmt: aBoolean

"Returns a SmallInteger, the time since January 1, 1970, in seconds.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

<primitive: 371>
^ self _primitiveFailed: #_timeGmt: args: { aBoolean }
%

category: 'Time'
classmethod: System
_timeGmt95: aBoolean

"Returns a SmallInteger, the time since January 1, 1995, in seconds.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

self deprecated: 'Use timeGmt2005'.
^self _timeGmt95Prim: aBoolean.
%

category: 'Time'
classmethod: System
_timeGmt95Prim: aBoolean

<primitive: 381>
^ self _primitiveFailed: #_timeGmt95: args: { aBoolean }
%

category: 'Time'
classmethod: System
timeGmt

"Returns a SmallInteger, the time since January 1, 1970, in seconds.
 The time is computed from the clock of the machine on which the session is 
 running, using the offset from the clock on the Stone's (GemStone repository 
 monitor process) machine which is cached in the session at login."

^ self _timeGmt: false
%

category: 'Deprecated'
classmethod: System
timeGmt95

"Obsolete - use timeGmt2005 instead of this method.

 Returns a SmallInteger, the time since January 1, 1995, in seconds.
 The time is computed from the clock of the machine on which the session is 
 running, using the offset from the clock on the Stone's (GemStone repository 
 monitor process) machine which is cached in the session at login."

 self deprecated: 'System class>>timeGmt95 is obsolete in GemStone/S 64 Bit. 
Use #timeGmt2005 instead.'.

^ self _timeGmt95: false
%

category: 'Private'
classmethod: System
_signalErrorStatus: aSignal

"Returns status of the specified signal .
 Signals are specified by SmallIntegers, the following are defined:

 1.  SignaledObjects 
 2.  SignaledAbort (if in transaction, reflects status after transation finishes)
 3.  SignaledGemStoneSession 
 4.  SignaledFinishTransaction
 5.  SignalAlmostOutOfMemory 
 6   SignalTranlogsFull"

<primitive: 366>
self _primitiveFailed: #_signalErrorStatus: args: { aSignal } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_updateSignalErrorStatus: aSignal toState: newState

"This method changes the generation of the error for the specified signal
 generation is disabled.  Signals are specified by SmallIntegers, and the
 following are defined:

 1.  SignaledObjects
 2.  SignaledAbort       (delayed effect if currently in transaction)
 3.  SignaledGemStoneSession
 4.  SignaledFinishTransaction
 5.  SignalAlmostOutOfMemory .
 6   SignalTranlogsFull"

<primitive: 367>
self _primitiveFailed: #_updateSignalErrorStatus:toState: 
     args: { aSignal . newState } .
self _uncontinuableError
%

category: 'Notification'
classmethod: System
disableSignaledGemStoneSessionError

"Set the current GemStone session so that it cannot receive signals from
 other GemStone sessions."

self _updateSignalErrorStatus: 3 toState: false
%

category: 'Signals'
classmethod: System
sendSignal: aSignal to: aSessionId withMessage: aString

"Sends a signal (a SmallInteger) to the specified session (a SmallInteger) with
 aString as a message.  The aString argument is currently limited to 1023
 bytes."

aSessionId _validateClass: SmallInteger.
aSessionId < 1 ifTrue:[ aSessionId _error: #rtErrArgNotPositive  ].
self _sendSignal: aSignal toSess: aSessionId negated withMessage: aString
%

category: 'Signals'
classmethod: System
_sendSignal: aSignal toSess: sessionIdOrSerial withMessage: aString

"Sends a signal (a SmallInteger) to the specified session with aString as a
 message.  The aString argument is currently limited to 1023 bytes.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId."

<primitive: 330>
aSignal _validateClass: SmallInteger.
sessionIdOrSerial _validateClass: SmallInteger.
aString _validateClass: String.
self _primitiveFailed: #_sendSignal:toSerialNum:withMessage: 
     args: { aSignal . sessionIdOrSerial . aString } .
self _uncontinuableError
%

category: 'Deprecated'
classmethod: System
signalFromGemStoneSession

"Returns a three-element Array containing information about a signal from
 another GemStone session:

 1.  An instance of GsSession representing the session that sent the signal.
 2.  The signal value (a SmallInteger).
 3.  A signal message (a String).

 If there is no signal in the queue, returns an empty Array."

| result |
self deprecated: 'Obsolete in GemStone/64.'.
result := self _signalFromGemStoneSession .
result == nil ifTrue:[ ^ { } ].
result at: 1 put: (GsSession sessionWithSerialNumber: (result at: 1)).
^ result
%

category: 'Private'
classmethod: System
_signalFromGemStoneSession

"Returns a three-element Array containing information about a signal from
 another GemStone session:

 1.  The sessionSerialNumber of the session that sent the signal.
 2.  The signal value (a SmallInteger).
 3.  A signal message (a String).

 If there is no signal in the queue, returns nil."

<primitive: 369>
self _primitiveFailed: #signalFromGemStoneSession .
self _uncontinuableError
%

category: 'Notification'
classmethod: System
addToNotifySet: anObject

"Add anObject to the notify set.  The argument anObject cannot be a special
 object nor an uncommitted object, since neither can be modified by other
 sessions."

self _updateNotifySet: { anObject }  add: true
%

category: 'Notification'
classmethod: System
addAllToNotifySet: aCollection

"Add all the elements of aCollection to the notify set.  Special objects and
 uncommitted objects are not permitted, since neither of these can be modified
 by other sessions."

self _updateNotifySet: aCollection add: true
%

category: 'Notification'
classmethod: System
clearNotifySet

"Remove all of the objects that are currently in the notify set."

self removeAllFromNotifySet: (self notifySet)
%

category: 'Notification'
classmethod: System
disableSignaledObjectsError

"Disable the generation of an error when a member of the notify set
 is added to the signaled objects set."

self _updateSignalErrorStatus: 1 toState: false
%

! fixed 41577
category: 'Notification'
classmethod: System
enableSignaledObjectsError

"Enable the signalling of an ObjectsCommittedNotification exception when 
 a member of the notify set is added to the signaled objects set.  
 When a ObjectsCommittedNotification is signalled, further signalling
 is disabled to allow the exception handler to run without receiving 
 another signal.  Therefore, the handler for ObjectsCommittedNotification 
 should itself send enableSignaledObjectsError"

self _updateSignalErrorStatus: 1 toState: true
%

category: 'Notification'
classmethod: System
notifySet

"Returns an Array of the objects that the user has registered for notification
 when a new state is committed."

^self _hiddenSetAsArray: 25
%

category: 'Notification'
classmethod: System
removeFromNotifySet: anObject

"Removes anObject from the notify set.  Does not generate an error if anObject
 is not in the notify set."

self _updateNotifySet: { anObject } add: false
%

category: 'Notification'
classmethod: System
removeAllFromNotifySet: aCollection

"Removes all elements of aCollection from the notify set.  Does not generate an
 error if any of the elements are not in the notify set."

self _updateNotifySet: aCollection add: false
%

category: 'Notification'
classmethod: System
signaledObjects

"Returns an Array containing the objects that have been signaled since the last
 time this method was executed.  The elements in the Array are a subset of the
 notify set.  Clear the set of signaled objects."

<primitive: 365>
self _primitiveFailed: #signaledObjects .
self _uncontinuableError
%

category: 'Notification'
classmethod: System
signaledObjectsErrorStatus

"Returns true to indicate that the system generates errors when objects are
 added to the signaled objects set.  Returns false otherwise."

^self _signalErrorStatus: 1
%

! fixed bug 13621, 14253
category: 'Notification'
classmethod: System
_updateNotifySet: aCollection add: aBoolean

"Update the notify set by adding or removing the elements of the anArray to or
 from the set.  If aBoolean is true, the elements are added, otherwise they are
 removed."

<primitive: 363>
| anArray |
anArray := aCollection asArray.
anArray == aCollection ifFalse:[
  ^ self _updateNotifySet: anArray add: aBoolean
  ] .
aCollection _validateClasses: { Array . IdentityBag }.
self _primitiveFailed: #_updateNotifySet:add: 
     args: { aCollection . aBoolean } .
self _uncontinuableError
%

! fixed 41577
category: 'Notification'
classmethod: System
enableSignaledGemStoneSessionError

"Enable the current GemStone session to receive signals from other GemStone
 sessions.  One GemStone session receives a signal from another session when a
 InterSessionSignal exception is signalled.

 The receiving session processes the signal with an exception handler.  When
 GemStone raises one signal exception, it also disables further signal
 exceptions, to allow the exception handler to run without receiving another
 interrupt.  The exception handler should therefore re-enable signal exceptions
 when it is done with its other processing.

 A signal is not exactly an interrupt, and it does not automatically awaken an
 idle session.  Both the GemStone Smalltalk virtual machine and GemBuilder for C
 can raise the signal exception.  But the process of the session must activate
 the virtual machine or interface before the signal can be received."

self _updateSignalErrorStatus: 3 toState: true
%

category: 'Notification'
classmethod: System
signaledGemStoneSessionErrorStatus

"Returns true to indicate that the current GemStone session can receive signals
 from other GemStone sessions.  Returns false otherwise."

^self _signalErrorStatus: 3
%

!  removed EpochGcSignal methods

category: 'Transient Session State'
classmethod: System
_sessionStateAt: anIndex

"Returns an indexed instance variable of the Transient Session State object.
 This object is a temporary instance of Array created after each GciLogin.

 If the specified instance variable of the Session State has not been stored
 into during this session, the result is nil.  This is different from
 Array>>at: semantics, where accessing beyond the end of an Array
 generates an error.

 Users of this method must observe the following instance variable allocations
 for this Array.

    anIndex      assignedUse
    -------      -----------
    1            Reserved for SymbolDictionary of Temporaries
    2            Reserved for ephemerons finalization queue .
    3            Topaz session state
    4            Reduced Conflict classes logging info
    5            Cache for Reduced Conflict values
    6            The one instance of GsCurrentSession 
    7            Pending commit/abort errorNumber or userAction 
    8            Reserved for GemStone GBJ support
    9            Other known sessions (a dictionary keyed on sessionSerialNum)
    10           Reserved for GemStone relational support
    11           Reserved for GemStone IndexManager support
    12           GemBuilder for C (GCI) deferred update Dictionary.
    13           GemBuilder for C (GCI) deferred update Array.
    14           Commit coordinator.
    15           Reserved for GemStone ProcessorScheduler support and prim 457
    16           Reserved for GemStone Locale support. 
    17           Reserved for timezone support.
    18           Reserved for GemStone GsFile support, caching of std files.
    19           Reserved for Ruby support.
    20..         Available for customer use ."

<primitive: 111>

(anIndex _isSmallInteger)
  ifTrue: [self _errorIndexOutOfRange: anIndex .  ^ nil ]
  ifFalse: [self _errorNonIntegerIndex: anIndex .  ^ nil ].
self _primitiveFailed: #_sessionStateAt: args: { anIndex } .
self _uncontinuableError
%

category: 'Transient Session State'
classmethod: System
_sessionStateSize

"Return the current size of the Session State Array."

<primitive: 214>
self _primitiveFailed: #_sessionStateSize .
self _uncontinuableError
%

category: 'Transient Session State'
classmethod: System
_sessionStateAt: anIndex put: aValue

"Modifies an indexed instance variable of the Transient Session State Array.
 The Session State Array will be grown as required to accommodate a store
 into the specified instance variable.

 anIndex must be < approx 2034 (Session State Array may not be a large object).

 See additional important documentation in the method #_sessionStateAt: .
 Returns receiver."

<primitive: 372>
(anIndex _isSmallInteger)
  ifTrue: [self _errorIndexOutOfRange: anIndex]
  ifFalse: [self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #_sessionStateAt:put: args: { anIndex . aValue } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex incrementBy: aSmallInt

"Increments the value of the session statistic at the specified index 
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47.  -1 means increment the progressCount statistic.  -2 means
 increment the indexProgressCount statistic."

<primitive: 477>
anIndex _validateClass: SmallInteger.
aSmallInt _validateClass: SmallInteger.
((anIndex < -2) or: [ anIndex > 47]) ifTrue: [
   anIndex _error: #rtErrArgOutOfRange args: { -2 . 47 } ].
self _primitiveFailed: #_sessionCacheStatAt:incrementBy: 
     args: { anIndex . aSmallInt } .
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex incrementBy: aSmallInt

"Increments the value of the session statistic at the specified index
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47.  0 to 47 are for application use.  
 -1 means increment the progressCount statistic.  -2 means increment the 
 indexProgressCount statistic."

 ^self _sessionCacheStatAt: anIndex incrementBy: aSmallInt
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex

"Returns the value of the session statistic at the specified index, which  
 must be in the range -2 to 47.  -1 means return the value of the
 progressCount statistic.  -2 means return the value of the indexProgressCount
 statistic."

^ self _sessionCacheStatAt: anIndex incrementBy: 0
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex

"Returns the value of the session statistic at the specified index, which
 must be in the range -2 to 47.  0 to 47 are for application use.  -1 means 
 return the value of the progressCount statistic.  -2 means return the value 
 of the indexProgressCount statistic."

^ self _sessionCacheStatAt: anIndex incrementBy: 0 
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex decrementBy: aSmallInt

"Decrements the value of the session statistic at the specified index 
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47.  -1 means decrement the progressCount statistic.  -2 means
 decrement the indexProgressCount statistic."

^ self _sessionCacheStatAt: anIndex incrementBy: aSmallInt negated
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex decrementBy: aSmallInt

"Decrements the value of the session statistic at the specified index
 by aSmallInt and returns the new value.  anIndex must be in the range
 of -2 to 47. 0 to 47 are for application use.  -1 means decrement the 
 progressCount statistic.  -2 means decrement the indexProgressCount 
 statistic."

^ self _sessionCacheStatAt: anIndex incrementBy: aSmallInt negated
%

category: 'Private'
classmethod: System
_sessionCacheStatAt: anIndex put: i

"This method sets the session statistic at the specified index (which should be
 in the range -2 to 47) to the specified value i, which must be a SmallInteger.
 -1 means set the progessCount statistic to the value i.  -2 means set the 
 indexProgessCount statistic to the value i."

<primitive: 476>
anIndex _validateClass: SmallInteger.
i _validateClass: SmallInteger.
((anIndex < -2) or: [ anIndex > 47]) ifTrue: [
   anIndex _error: #rtErrArgOutOfRange args: { -2 . 47 } ].
self _primitiveFailed: #_sessionCacheStatAt:put: args: { anIndex . i } .
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatAt: anIndex put: i

"This method sets the session statistic at the specified index (which should be
 in the range -2 to 47) to the specified value i, which must be a SmallInteger.
 0 to 47 are for application use.  -1 means set the progessCount statistic to 
 the value i.  -2 means set the indexProgessCount statistic to the value i."

 ^ self _sessionCacheStatAt: anIndex put: i
%

category: 'Global Session Statistics'
classmethod: System
_updateGlobalSessionStat: index by: i overwrite: aBool

"Global session statistics are similar to session statistics except that global 
 statistics are accessible by every logged in session on every host.  Global
 sessions statistics are not transactional.  Therefore every session will see the
 same value in a given statistic regardless of its transactional view.  Reads and
 updates to these statistics are guaranteed to be atomic.

 This method updates the global session statistic at the specified index (which must be
 in the range 0 to 47) by the specified value i, which must be a SmallInteger.
 If overwrite is true, then the current value of the  statistic is replaced by i.
 If overwrite is false, then the current value of the  statistic is incremented by i.  

 Returns the new value of the statistic."

<primitive: 518>
index _validateClass: SmallInteger.
i _validateClass: SmallInteger.
aBool _validateClass: Boolean.
self _primitiveFailed: #_updateGlobalSessionStat:by:overwrite: 
     args: { index . i . aBool } .
%

category: 'Global Session Statistics'
classmethod: System
globalSessionStatAt: index
"Returns the current value of a global session statistic.
 See the method System>>_updateGlobalSessionStat: index by: i overwrite: aBool
 for more information on global session statistics."

^self _updateGlobalSessionStat: index by: 0 overwrite: false
%

category: 'Global Session Statistics'
classmethod: System
incrementGlobalSessionStatAt: index by: value
"Increments the value of the given global session statistic by newVal 
 and returns the new value.  See the method 
 System>>_updateGlobalSessionStat: index by: i overwrite: aBool
 for more information on global session statistics."

^self _updateGlobalSessionStat: index by: value overwrite: false
%


category: 'Global Session Statistics'
classmethod: System
globalSessionStatAt: index put: newVal
"Overwrites the value of the given global session statistic with newVal 
 and returns the new value.  See the method 
 System>>_updateGlobalSessionStat: index by: i overwrite: aBool
 for more information on global session statistics."

^self _updateGlobalSessionStat: index by: newVal overwrite: true
%

! _hasBreakpoints deleted

category: 'Private'
classmethod: System
_sessionCacheStatsForProcessSlot: aProcessSlot

"Return an array containing the 48 SmallIntegers representing the 
 session statistics for the given process slot.  

 Returns nil if the given process slot is not found or if the slot
 is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aProcessSlot
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatsForProcessSlot: aProcessSlot

"Return an array containing the 48 SmallIntegers representing the
 session statistics for the given process slot; the stats 
 corresponding to indexes 0 to 47, not including the stats for -2 
 and -1.

 Returns nil if the given process slot is not found or if the slot
 is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aProcessSlot
%

category: 'Private'
classmethod: System
_sessionCacheStatsForSessionId: aSessionId

"Return an array containing the 48 session statistics for the given
 session ID.  

 Returns nil if the given session is not found or if the slot is not
 in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aSessionId negated.
%

category: 'Transient Session State'
classmethod: System
sessionCacheStatsForSessionId: aSessionId

"Return an array containing the 48 session statistics for the given
 session ID; the stats corresponding to indexes 0 to 47, not including 
 the stats for -2 and -1.

 Returns nil if the given session is not found or if the slot is not
 in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

^self _sessionCacheStatsForSlotOrSessionId: aSessionId negated.
%

category: 'Private'
classmethod: System
_sessionCacheStatsForSlotOrSessionId: aProcessSlotOrSessionId

"Return an array containing the 48 session statistics for the given
 session ID or process slot number.  If the argument is positive
 it is assumed to be a process slot.  If it is negative it is assumed
 to be a negated session ID. 

 Returns nil if the given session or cache slot is not found or
 if the slot is not in use by a gem process.

 See additional information in the method #_sessionCacheStatAt: put:"

<primitive: 566>
aProcessSlotOrSessionId _validateClass: SmallInteger.
aProcessSlotOrSessionId _error: #rtErrArgOutOfRange .
^self _primitiveFailed: #_sessionCacheStatsForSlotOrSessionId:
      args: { aProcessSlotOrSessionId } .
%

category: 'Deprecated'
classmethod: System
myUserGlobals

"Obsolete in GemStone/64."

| assn |

self deprecated: 'System class>>myUserGlobals is deprecated in GemStone/S 64 Bit.  
Replace with (self myUserProfile resolveSymbol: #UserGlobals)'.

(assn := self myUserProfile resolveSymbol: #UserGlobals) == nil ifTrue: [
  ^self _error: #rtErrKeyNotFound args: #(#UserGlobals)
].
^assn value.
%

category: 'Debugging Support'
classmethod: System
_enableTraceObjectsRead

"Enables tracing of object read-faults, i.e. committed objects that are copied
 into the VM's memory.   Cancels any previous _enableTracePagesRead ,
 only one of _enableTraceObjectsRead or _enableTracePagesRead can be active
 at a time.

 Object read-faults are accumulated in internal buffers, which 
 are flushed to hidden set 5  each time the PomGenScavCount 
 statistic increments . 
 Use the method _updateObjectsRead to explicitly flush buffers into 
 hidden set 5 . 

 If tracing was previously enabled, this method clears hidden set 5 .  

 Note that hidden set 5 is an identity set, and if the same object
 is read multiple times during an interval defined by _enableTraceObjectsRead,
 _disableTraceObjectsRead, due to wraparound of the memory area holding 
 committed objects , that object will only show up once in the set .  
 Changes in the statistic ObjectsRead  can be compared to changes in the 
 set size to detect multiple reads of objects.
 Also note the statistic ObjectsRefreshed , which represents re-reads
 of objects to obtain new views after crossing transaction boundaries;
 re-reads.

 Hidden set 5 is in C heap memory ; this process could run out of C heap memory
 if this tracing is left enabled for a very long time on a large repository.
 Memory consumption varys from approximately 8 bytes per object for small sets 
 to 1 bit per object for sets spanning the entire shared object table.

 Returns self."

^ self _zeroArgPrim: 99
%

category: 'Debugging Support'
classmethod: System
_updateObjectsRead

"If tracing of object read-faults has been enabled by executing
 _enableTraceObjectsRead ,   flushes internal tracing buffers into
 hidden set 5 .  
 Returns the size of hidden set 5, after flushing the buffers.

 If tracing of object read-faults is not enabled, 
 or is currently disabled via _disableTraceObjectsRead , returns 0 ."

^ self _zeroArgPrim: 96
%

category: 'Debugging Support'
classmethod: System
_disableTraceObjectsRead

"Disables tracing of object read-faults , if it is enabled, and flushes
 any remaining trace buffers into hidden set 5 .
 Returns the size of hidden set 5 after flushing the buffers.

 If tracing of object read-faults was not enabled, returns 0 ."


^ self _zeroArgPrim: 97
%

category: 'Debugging Support'
classmethod: System
_resumeTraceObjectsRead

"Resumes tracing of object read-faults .
 Generates an error if hidden set 5 was most recently intialized by
 a _enableTracePagesAccessed .

 Same as _enableTraceObjectsRead, but does not clear hidden set 5 .  
 If tracing of object read-faults was already enabled, flushes the
 trace buffers into hidden set 5 .

 Returns the current size of hidden set 5."

^ self _zeroArgPrim: 98
%

category: 'Debugging Support'
classmethod: System
_enableTracePagesAccessed

"Enables tracing of page read accesses.
 A  'page read access'  is a page in the shared cache which was accessed
 the data source to satisfy an object read-fault .  Page accesses for 
 object table lookups are not included.

 Cancels any previous _enableTraceObjectsRead , only one of 
 _enableTraceObjectsRead or _enableTracePagesAccessed can be active at a time.

 PageIds of pages accessed are accumulated in internal buffers, which 
 are flushed to hidden set 5  approximately every 200 page read accesses.
 Use the method _updatePagesAccessed to explicitly flush buffers into 
 hidden set 5 . 

 If tracing was previously enabled, this method clears hidden set 5 .  

 Note that hidden set 5 is an identity set, and if the same page
 is read multiple times during an interval defined by _enableTracePagesAccessed,
 _disableTracePagesAccessed, that page will only show up once in the set .  

 Hidden set 5 is in C heap memory ; this process could run out of C heap memory
 if this tracing is left enabled for a very long time on a large repository.
 Memory consumption varys from approximately 8 bytes per page for small sets 
 to 1 bit per page for sets spanning the entire shared object table.

 Returns self."

^ self _zeroArgPrim: 125
%

category: 'Debugging Support'
classmethod: System
_updatePagesAccessed

"If tracing of page read accessses has been enabled by executing
 _enableTracePagesAccessed ,   flushes internal tracing buffers into
 hidden set 5 .  
 Returns the size of hidden set 5, after flushing the buffers.

 If tracing of page read accesses not enabled, 
 or is currently disabled via _disableTracePagesAccessed, returns 0 ."


^ self _zeroArgPrim: 127
%

category: 'Debugging Support'
classmethod: System
_disableTracePagesAccessed

"Disables tracing of page read accesses  , if it is enabled, and flushes
 any remaining trace buffers into hidden set 5 .
 Returns the size of hidden set 5 after flushing the buffers.

 If tracing of page read accesses was not enabled, returns 0 ."


^ self _zeroArgPrim: 128
%

category: 'Debugging Support'
classmethod: System
_resumeTracePagesAccessed

"Resumes tracing of page read accesses.
 Generates an error if hidden set 5 was most recently intialized by
 a _enableTraceReadFaults .

 Same as _enableTracePagesAccessed, but does not clear hidden set 5 .  
 If tracing of page read accesses was already enabled, flushes the
 trace buffers into hidden set 5 .

 Returns the current size of hidden set 5."

^ self _zeroArgPrim:126 
%


category: 'Hidden Set Support'
classmethod: System
HiddenSetSpecifiers

"Returns a list of the hiddenSet specifiers."

^#(
" 1"   'ListInstancesResult'
" 2"   'Reserved for GemStone'
" 3"   'RemovedDepMapEntries'
" 4"   'SaveNewPomObjs' "See method #_enableTraceNewPomObjs"
" 5"   'ObjectsRead'    "See method #_enableTraceObjectsRead"
" 6"   'Reserved for GemStone'
" 7"   'RcReadSet'
" 8"   'DepMapWriteSet'
" 9"   'PomWriteSet'  "Empty except after flush for commit, so only useful
		       after you get a transaction conflict."
"10"   'SaveDepMapChangedObjs'
"11"   'SaveWriteSetUnion'
"12"   'SaveWrittenObjs'
"13"   'ReadWriteConflicts'  "StrongRead-Write conflicts "
"14"   'SaveDepMapChangedUnion'
"15"   'WriteWriteconflicts'
"16"   'WriteDependencyConflicts'
"17"   'WriteReadLockConflicts'
"18"   'WriteWriteLockConflicts'
"19"   'Reserved for GemStone'
"20"   'AllocatedGciOops'
"21"   'Reserved for GemStone'
"22"   'ExportedDirtyObjs'
"23"   'TrackedDirtyObjs'
"24"   'enumeration of ReferencedSet'
"25"   'NotifySet'
"26"   'Reserved1'
"27"   'Reserved2'
"28"   'Reserved3'
"29"   'CommitReleaseLocksSet'
"30"   'CommitOrAbortReleaseLocksSet'
"31"   'Reserved for GemStone'
"32"   'Reserved for GemStone'
"33"   'Reserved for GemStone'
"34"   'GcCandidates'
"35"   'ObjsWithWeakRefs'
"36"   'WriteLockWriteSubset'
"37"   'NewDataPages'
"38"   'StrongReadSet'
"39"   'PureExportSet'
"40"   'GciTrackedObjs'
"41"   'For Customer Use'
"42"   'For Customer Use'
"43"   'For Customer Use'
"44"   'For Customer Use'
"45"   'For Customer Use'
)
%

category: 'Hidden Set Support'
classmethod: System
_add: anObject to: hiddenSetSpecifier

"Adds anObject to the hiddenSet."
<primitive: 168>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
self _primitiveFailed: #_add:to: args: { anObject . hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Private'
classmethod: System
_lockGc: aBool

"result is true for success, or a String with reason for failure"

<primitive: 525>
self _primitiveFailed: #_lockGc: args: { aBool } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_addAll: anArray to: hiddenSetSpecifier

"Adds contents of anArray to the hiddenSet (anArray not added to hiddenSet)."
<primitive: 520>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
self _primitiveFailed: #_addAll:to: args: { anArray . hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_removeAll: anArray from: hiddenSetSpecifier

"Removes the contents of anArray from the hiddenSet (anArray is not removed from 
 the hiddenSet).  Objects in the array which are not in the hidden set are ignored.
 Returns a SmallInteger or LargeInteger indicating the number of elements 
 successfully removed from the hidden set."

<primitive: 41>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
^self _primitiveFailed: #_removeAll:from: 
      args: { anArray . hiddenSetSpecifier }
%


category: 'Hidden Set Support'
classmethod: System
_remove: anObject from: hiddenSetSpecifier

"Removes anObject from the hiddenSet."

<primitive: 169>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
self _primitiveFailed: #_remove:from: 
     args: { anObject . hiddenSetSpecifier }  .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_testIf: anObject isIn: hiddenSetSpecifier

"Returns true if anObject is in the hiddenSet, false otherwise."

<primitive: 170>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
self _primitiveFailed: #_testif:isIn: args: { anObject . hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_hiddenSetAsArray: hiddenSetSpecifier

"Returns an Array containing the contents of the hiddenSet."

<primitive: 212>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
self _primitiveFailed: #_hiddenSetAsArray: args: { hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_primHiddenSetEnumerate: hiddenSetSpecifier limit: maxResultSize storeAsInts: aBool

"Returns an Array containing the first maxResultSize
elements of the hiddenSet.  If the hidden set contains fewer than
maxResultSize elements, returns an array containing
the contents of the hiddenSet.    If maxResultSize = 0,
the result will contain all elements of the hidden set.

All of the returned elements are removed from the hidden set.

If aBool is true, then the object ID's of the hidden set are stored in
the result array as SmallIntegers.  Otherwise the objects themselves 
are stored in the result array."

<primitive: 331>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
aBool _validateClass: Boolean.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
maxResultSize _validateClass: SmallInteger.
maxResultSize < 0 ifTrue:[ maxResultSize _error: #errArgTooSmall args:{ 0 } ].
self _primitiveFailed: #_primHiddenSetEnumerate:limit:storeAsInts:
     args: { hiddenSetSpecifier . maxResultSize . aBool } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_hiddenSetEnumerate: hiddenSetSpecifier limit: maxResultSize

"Returns an Array containing the first maxResultSize objects
in the hiddenSet.  If the hidden set contains fewer than
maxResultSize elements, returns an array containing
the contents of the hiddenSet.    If maxResultSize = 0,
the result will contain all elements of the hidden set.

All of the returned elements are removed from the hidden set."

^self _primHiddenSetEnumerate: hiddenSetSpecifier
      limit: maxResultSize 
      storeAsInts: false.
%

category: 'Hidden Set Support'
classmethod: System
_hiddenSetEnumerateAsInts: hiddenSetSpecifier limit: maxResultSize

"Returns an Array containing the numeric object identifiers of the 
first maxResultSize objects in the hiddenSet.  If the hidden set 
contains fewer than maxResultSize elements, returns an array containing
the contents of the hiddenSet.    If maxResultSize = 0,
the result will contain all elements of the hidden set.

All of the returned elements are removed from the hidden set."

^self _primHiddenSetEnumerate: hiddenSetSpecifier
      limit: maxResultSize 
      storeAsInts: true.
%

category: 'Hidden Set Support'
classmethod: System
_hiddenSetSize: hiddenSetSpecifier

"Returns the number of elments in the hiddenSet."

<primitive: 171>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
self _primitiveFailed: #_hiddenSetSize: args: { hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_hiddenSetReinit: hiddenSetSpecifier

"Reinitializes the hiddenSet to empty.  Must be system user for all
 hidden sets except the reserved ones."

<primitive: 172>
| sz |
hiddenSetSpecifier _validateClass: SmallInteger.
(hiddenSetSpecifier < 1 or:[
  hiddenSetSpecifier > (sz := self HiddenSetSpecifiers size)])
  ifTrue: [hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 1 . sz } ].
self _primitiveFailed: #_hiddenSetReinit: args: { hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_add: anObject toGciSet: hiddenSetSpecifier 

"Add the specified object to a GCI hidden set, where
   hiddenSetSpecifier = 39  specifies PureExportSet
   hiddenSetSpecifier = 40  specifies GciTrackedObjs .
 Other values of hiddenSetSpecifier are not allowed.
 Has no effect if anObject is a special object.

 If hiddenSetSpecifier = 39 and a userAction is 
 active an error is generated, since the PureExportSet
 cannot be modified from within a user action.

 If hiddenSetSpecifier = 40  and you have not
 executed  _gciDirtyInit:23 once since login,
 generates an error."

<primitive: 155>
hiddenSetSpecifier _validateClass: SmallInteger .
(hiddenSetSpecifier < 39 or:[ hiddenSetSpecifier > 40]) ifTrue:[
  hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 39 . 40 }
].
self _primitiveFailed: #_add:toGciSet: 
     args: { anObject . hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_addAll: anArray toGciSet: hiddenSetSpecifier 

"Add contents of anArray to a GCI hidden set, where
   hiddenSetSpecifier = 39  specifies PureExportSet
   hiddenSetSpecifier = 40  specifies GciTrackedObjs .
 Other values of hiddenSetSpecifier are not allowed.

 If hiddenSetSpecifier = 39 and a userAction is 
 active an error is generated, since the PureExportSet
 cannot be modified from within a user action.

 If hiddenSetSpecifier = 40  and you have not
 executed  _gciDirtyInit:23 once since login,
 generates an error."

<primitive: 156>
hiddenSetSpecifier _validateClass: SmallInteger .
(hiddenSetSpecifier < 39 or:[ hiddenSetSpecifier > 40]) ifTrue:[
  hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 39 . 40 }
].
anArray _validateClass: Array .
self _primitiveFailed: #_addAll:toGciSet: 
     args: { anArray . hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_remove: anObject fromGciSet: hiddenSetSpecifier 

"Remove the specified object to a GCI hidden set, where
   hiddenSetSpecifier = 39  specifies PureExportSet
   hiddenSetSpecifier = 40  specifies GciTrackedObjs .
 Other values of hiddenSetSpecifier are not allowed.
 Has no effect if anObject is a special object.

 If hiddenSetSpecifier = 39 and a userAction is 
 active an error is generated, since the PureExportSet
 cannot be modified from within a user action.

 If hiddenSetSpecifier = 40  and you have not
 executed  _gciDirtyInit:23 once since login,
 generates an error."

<primitive: 157>
hiddenSetSpecifier _validateClass: SmallInteger .
(hiddenSetSpecifier < 39 or:[ hiddenSetSpecifier > 40]) ifTrue:[
  hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 39 . 40 }
].
self _primitiveFailed: #_remove:fromGciSet: 
     args: { anObject . hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_removeAll: anArray fromGciSet: hiddenSetSpecifier 

"Remove contents of anArray to a GCI hidden set, where
   hiddenSetSpecifier = 39  specifies PureExportSet
   hiddenSetSpecifier = 40  specifies GciTrackedObjs .
 Other values of hiddenSetSpecifier are not allowed.

 If the argument anArray == true, all objects in
 the specified set are removed from that set .

 If hiddenSetSpecifier = 39 and a userAction is 
 active an error is generated, since the PureExportSet
 cannot be modified from within a user action.

 If hiddenSetSpecifier = 40  and you have not
 executed  _gciDirtyInit:23 once since login,
 generates an error."

<primitive: 158>
hiddenSetSpecifier _validateClass: SmallInteger .
(hiddenSetSpecifier < 39 or:[ hiddenSetSpecifier > 40]) ifTrue:[
  hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 39 . 40 }
].
anArray _validateClass: Array .
self _primitiveFailed: #_removeAll:fromGciSet:
     args: { anArray . hiddenSetSpecifier } .
self _uncontinuableError
%

category: 'Hidden Set Support'
classmethod: System
_gciDirtyInit: hiddenSetSpecifier

"Enable GCI tracking of dirty objects.
   hiddenSetSpecifier = 22, equivalent to GciDirtyObjsInit()
   hiddenSetSpecifier = 23, equivalent GciTrackedObjsInit()
 Other values of hiddenSetSpecifier are not allowed."

<primitive: 164>
hiddenSetSpecifier _validateClass: SmallInteger .
(hiddenSetSpecifier < 22 or:[ hiddenSetSpecifier > 23]) ifTrue:[
  hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 22 . 23 }
].
self _primitiveFailed: #_gciDirtyInit: args: { hiddenSetSpecifier } .
self _uncontinuableError
%


category: 'Hidden Set Support'
classmethod: System
_getAndClearGciDirtySet: hiddenSetSpecifier into: anArray

"Destructively enumerate a GCI hidden set, where
   anInt = 22  specifies ExportedDirtyObjs
   anInt = 23  specifies TrackedDirtyObjs .
 Other values of hiddenSetSpecifier are not allowed .

 Corresponding _gciDirtyInit: must have been sent once during the session
 before this method can be used.  It is intended that either this
 method or Gci calls be used to enumerate these hidden sets, not
 both within one session.  

 If the specified set is empty the result is nil.
 Otherwise the result is an Array containing up to the first 2034 elements 
 of the hidden set, and those element are cleared from the hidden 
 set prior to return from the primitive.   If the set contains
 more than 2034 elements, repeated invocation of this method
 is needed to enumerate the set completely.

 anArray may be nil in which case the result is a newly created Array.
 If anArray is non-nil it is used as the result and is grown as needed.

 For most efficient enumeration use this style:
   [ | arr |
     arr := self _getAndClearGciDirtySet:22 into: arr .
     1 to: arr size do:[:j | |aDirtyObj|
       aDirtyObj := arr at: j .
       aDirtyObj applicationHandleDirtyObj  .
     ].
     arr size == 0 .
   ] untilTrue .
 "

<primitive: 162>
hiddenSetSpecifier _validateClass: SmallInteger .
(hiddenSetSpecifier < 22 or:[ hiddenSetSpecifier > 23]) ifTrue:[
  hiddenSetSpecifier _error: #rtErrArgOutOfRange args:{ 22 . 23 }
].
anArray ~~ nil ifTrue:[ anArray _validateClass: Array ].
self _primitiveFailed: #_getAndClearGciDirtySet:into:
     args: { hiddenSetSpecifier . anArray } .
self _uncontinuableError
%


! fixed 13750
category: 'Lock Status'
classmethod: System
myLockKind: anObject

"Returns a Symbol that indicates what kind of lock the current session has on
 anObject: one of #none, #read or #write."

| status |
status := self lockStatus: anObject.
( (status at: 1) ~~ #none and:
[ (status at: 2) includesIdentical: self session] )
  ifTrue: [ ^status at: 1 ].

^#none
%

! v3.0, deleted _disableProtectedMode, replaced by special selector _leaveProtectedMode

category: 'Private'
classmethod: System
_protectedMode

"Returns a SmallInteger, the value of protected mode .

 The usage pattern for writing a protected method is

     <primitive: 2001> 
     | prot |
     prot := System _protectedMode .
     [ 
       ""method body""
     ] ensure:[
       prot _leaveProtectedMode
     ]

    primitive 2001 enters protected mode.  
    The _protectedMode method returns the current nesting level of 
    protected methods (a SmallInteger).
    _leaveProtectedMode is a special selector which compiles to a
     bytecode ."

<protected primitive: 691>
self _primitiveFailed: #_protectedMode .
self _uncontinuableError
%

category: 'Runtime Configuration Access'
classmethod: System
configurationAt: aName put: aValue

"Change the value of the specified configuration parameter.  

 The changeable parameters all require aValue to be a SmallInteger.

 Configuration parameters should not be changed unless there is a clear
 reason for doing so, since incorrect settings of parameters can
 have serious adverse effects on GemStone performance.

 Configuration parameters for Stone that are transferred to Gem processes
 are only read by the Gem at login, so changes using this method to
 Stone parameters may have no effect on existing sessions.

 Parameters in the Gem with the following names may be changed by any user at
 any time:

   * #GemIOLimit
   * #GemFreeFrameLimit
   * #GemPgsvrUpdateCacheOnRead
   * #GemKeepMinSoftRefs
   * #GemSoftRefCleanupPercentMem
   * #GemPomGenPruneOnVote
   * #GemTempObjPomgenScavengeInterval
   * #GemConvertArrayBuilder (takes effect at start of next method compile)

 Parameters in the Gem with the following names may be changed only by users
 who have the correct privilege and who follow any other restrictions:

   * #StnLoginsSuspended - Requires SystemControl privilege.
   * #StnLogLoginFailureLimit - Requires OtherPassword privilege.
   * #StnLogLoginFailureTimeLimit - Requires OtherPassword privilege.
   * #StnDisableLoginFailureLimit - Requires OtherPassword privilege.
   * #StnDisableLoginFailureTimeLimit - Requires OtherPassword privilege.
   * #StnNumGcReclaimSessions - Requires GarbageCollection privilege.
   * #StnAdminGcSessionEnabled - Requires GarbageCollection privilege.

   * #StnSignalAbortCrBacklog - Requires GarbageCollection privilege, 
	and minimum value is 2 .

 All other parameters in the Gem that are changeable at run time may be changed
 only by SystemUser, and should not be changed in the course of normal GemStone
 operation."

| cfgId |

cfgId := ConfigurationParameterDict at: aName .
self _atConfigId: cfgId put: aValue .
^ self configurationAt: aName
%

category: 'Runtime Configuration Access'
classmethod: System
_atConfigId: rtConfigId put: aValue

"Change the specified Stone runtime configuration parameter.  The current user
 must be SystemUser to set any configuration parameter except the parameters
 
 Changing a runtime parameter does not alter the configuration file.
 The new value will show up in the result of configuration report methods, 
 but will not be used when Stone next starts up."

<primitive: 355>
rtConfigId _validateClass: SmallInteger .
aValue _validateClass: SmallInteger .
(self _configParameterName: rtConfigId kind: $C ) == nil 
   ifTrue:[ rtConfigId _error: #rtErrArgOutOfRange ].
rtConfigId >= 0 ifTrue:[ 
  (self _configParameterName: rtConfigId kind: $C ) _error: #rtErrConfigReadOnly
  ].
   
self _primitiveFailed: #_atConfigId:put: args: { rtConfigId . aValue } .
self _uncontinuableError
%

category: 'Configuration File Access'
classmethod: System
_configParameterName: anInt kind: aChar

"Returns a String, which is the name of the configuration parameter whose
 internal identifier is anInt.  Returns nil if anInt is out of range.

 aChar must be one of
   $C - anInt is identifier of a configuration parameter
   $V - anInt is identifier of a version information item
"

<primitive: 342>
anInt _validateClass: SmallInteger .
^ self _primitiveFailed: #_configParameterName:kind: args: { anInt . aChar }
%

category: 'Configuration File Access'
classmethod: System
_configFileParameterDict

"Returns a Dictionary of names for configuration file parameters.  The
 dictionary keys are Symbols.  Its values are SmallInteger configuration IDs."

| result cfgId cfgName |
result := SymbolKeyValueDictionary new .
cfgId := 0 .
[
  cfgName := self _configParameterName: cfgId kind: $C .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId + 1 .
    ] .
  cfgName == nil 
  ] untilTrue .

cfgId := -1 .
[
  cfgName := self _configParameterName: cfgId kind: $C .
  cfgName == nil ifFalse:[
    result at: cfgName put: cfgId .
    cfgId := cfgId - 1 .
    ] .
  cfgName == nil 
  ] untilTrue .

^ result
%

! fixed 31813
category: 'Configuration File Access'
classmethod: System
_configConstantsDict

"Returns a Dictionary of internal configuration constants that
 are fixed at compilation of the gem and stone executables. The dictionary
 keys are Strings.  Its values are SmallInteger configuration constants."


| anArray result |
anArray := self _zeroArgCateg2Prim: 8 .
result := StringKeyValueDictionary new .
1 to: anArray size do:[ :j | | anAssoc |
  anAssoc := anArray at: j .
  result add: anAssoc .
  ].
^ result
%

category: 'Configuration File Access'
classmethod: System
gemConfigurationReport

"Returns a SymbolDictionary whose keys are the names of configuration file
 parameters, and whose values are the current settings of those parameters in
 the current session's Gem process.  Parameters that are not applicable to Gem
 and those that are undefined are not included in the result."

^ self _configurationReport: false 
%

category: 'Configuration File Access'
classmethod: System
stoneConfigurationReport

"Returns a SymbolDictionary whose keys are the names of configuration file
 parameters, and whose values are the current settings of those parameters in
 the repository monitor process (Stone).  Parameters that are not applicable to
 Stone and those that are undefined are not include in the result." 

^ self _configurationReport: true 
%

category: 'Configuration File Access'
classmethod: System
_configurationReport: isStone

"Private."

| result |
result := SymbolDictionary new .
ConfigurationParameterDict keysAndValuesDo:[ :aName :anId | | aVal |
  aVal := self _configurationAt: anId isStone: isStone kind: $C .
  aVal ~~ nil ifTrue:[ 
    result at: aName put: aVal
    ].
  ].
^ result 
%

category: 'Configuration File Access'
classmethod: System
configurationAt: aName 

"Returns the value of the specified configuration file parameter, giving
 preference to the Gem process if the parameter applies to the Gem."
 
| result |
result := self gemConfigurationAt: aName .
result == nil ifTrue:[
  result := self stoneConfigurationAt: aName
  ].
^ result
%

category: 'Configuration File Access'
classmethod: System
stoneConfigurationAt: aName

"Returns the value of the specified configuration file parameter from the
 repository monitor process (Stone).  Returns nil if that parameter is not
 applicable to the Stone."

| cfgId |
cfgId := ConfigurationParameterDict at: aName otherwise: nil .
cfgId == nil ifTrue:[ ^ nil ].
^ self _configurationAt: cfgId isStone: true kind: $C
%

category: 'Configuration File Access'
classmethod: System
stoneConfigurationAt: aName put: aValue

"Changes the value of the specified Stone configuration parameter.

 See comments in the method configurationAt:put: for complete documentation."

^ self configurationAt: aName put: aValue
%

category: 'Runtime Configuration Access'
classmethod: System
gemConfigurationAt: aName put: aValue

"Changes the value of the specified Gem configuration parameter.  

 See comments in the method configurationAt:put: for complete documentation."

^ self configurationAt: aName put: aValue
%

!  removeGemLogOnExit: added to fix 32245
category: 'Runtime Configuration Access'
classmethod: System
removeGemLogOnExit: aBoolean

"Set state in a gem process that overrides the state of the
 GEMSTONE_CHILD_LOG environment variable. See $GEMSTONE/sys/gemnetdebug
 for documentation on GEMSTONE_CHILD_LOG .

 If aBoolean is true, the gem log file will be deleted if the
 gem process exits normally. If aBoolean is false, the
 the gem log file will not be deleted .

 Has no effect in a linked session"

<primitive: 554>
aBoolean _validateClass: Boolean .
^self _primitiveFailed: #removeGemLogOnExit: args: { aBoolean }
%

category: 'Configuration File Access'
classmethod: System
gemConfigurationAt: aName

"Returns the value of the specified configuration file parameter from the
 current session.  Returns nil if that parameter is not applicable to a Gem."

| cfgId |
cfgId := ConfigurationParameterDict at: aName otherwise: nil .
cfgId == nil ifTrue:[ ^ nil ].
^ self _configurationAt: cfgId isStone: false kind: $C
%

category: 'Configuration File Access'
classmethod: System
_configurationAt: cfgId isStone: aBool kind: aKind

"Returns the configuration value specified. If aBool is true, get the
 configuration value from the Stone process, otherwise from the current
 session.

 cfgId must be a SmallInteger; if >= 0, returns the value of the specified
 configuration parameter;  if < 0, returns the value of the specified
 runtime control parameter.

 aKind must be either $C (for configuration info) or $V (for version info)"

<primitive: 338>
aBool _validateClass: Boolean .
cfgId _validateClass: SmallInteger .
aKind == $C ifTrue:[
  ConfigurationParameterDict keyAtValue: cfgId ifAbsent:[
     cfgId _error: #rtErrArgOutOfRange
     ].
  ]
  ifFalse:[
  VersionParameterDict keyAtValue: cfgId ifAbsent:[
     cfgId _error: #rtErrArgOutOfRange
     ].
  ].
self _primitiveFailed: #_configurationAt:isStone:kind:
     args: { cfgId . aBool . aKind } .
self _uncontinuableError
%

! _resolveSym: deleted

! fixed 13750
category: 'Releasing Locks'
classmethod: System
addToCommitReleaseLocksSet: anObject

"Add anObject to the commit release locks set.  If anObject is not locked 
 by the current session, then it is not added to the set."

((self myLockKind: anObject) == #none )
  ifFalse: [self _add: anObject to: 29.].
^self
%

category: 'Releasing Locks'
classmethod: System
addAllToCommitReleaseLocksSet: aCollection

"Add each element of aCollection to the commit release locks set.  If an
 element of aCollection is not locked by the current session, then that
 element is not added to the set."

aCollection accompaniedBy: self do: [:me :each | me addToCommitReleaseLocksSet: each ].
^self
%

! fixed 13750
category: 'Releasing Locks'
classmethod: System
addToCommitOrAbortReleaseLocksSet: anObject

"Add anObject to to the commit-or-abort release locks set.  If anObject
 is not locked by the current session, then it is not added to the set."

((self myLockKind: anObject) == #none )
  ifFalse: [self _add: anObject to: 30.].
^self
%

category: 'Releasing Locks'
classmethod: System
addAllToCommitOrAbortReleaseLocksSet: aCollection

"Add each element of aCollection to the commit-or-abort release locks set. 
 If an element of aCollection is not locked by the current session, then
 it is not added to the set."

aCollection accompaniedBy: self do: [:me :each | me addToCommitOrAbortReleaseLocksSet: each ].
^self
%

category: 'Releasing Locks'
classmethod: System
clearCommitReleaseLocksSet

"Remove all objects from the commit release locks set."

self _hiddenSetReinit: 29.
^self
%

category: 'Releasing Locks'
classmethod: System
clearCommitOrAbortReleaseLocksSet

"Remove all objects from the commit-or-abort release locks set."

self _hiddenSetReinit: 30.
^self
%

category: 'Releasing Locks'
classmethod: System
commitReleaseLocksSetIncludes: anObject

"Returns true if anObject is in the commit release locks set. 
 Returns false otherwise."

^self _testIf: anObject isIn: 29
%

category: 'Releasing Locks'
classmethod: System
commitOrAbortReleaseLocksSetIncludes: anObject

"Returns true if anObject is in the commit-or-abort release locks set.
 Returns false otherwise."

^self _testIf: anObject isIn: 30
%

category: 'Releasing Locks'
classmethod: System
removeFromCommitReleaseLocksSet: anObject

"Remove anObject from the commit release locks set.  If anObject is 
 not a member of the set, do nothing."

self _remove: anObject from: 29.
%

category: 'Releasing Locks'
classmethod: System
removeAllFromCommitReleaseLocksSet: aCollection

"Remove all elements of aCollection from the commit release locks set.
 If an element of aCollection is not a member of the set, it is ignored."

aCollection accompaniedBy: self do: [:me :each | me removeFromCommitReleaseLocksSet: each ].
^self
%

category: 'Releasing Locks'
classmethod: System
removeFromCommitOrAbortReleaseLocksSet: anObject

"Remove anObject from the commit-or-abort release locks set.  If anObject
 is not a member of the set, do nothing."

self _remove: anObject from: 30.
%

category: 'Releasing Locks'
classmethod: System
removeAllFromCommitOrAbortReleaseLocksSet: aCollection

"Remove all the elements of aCollection from the commit-or-abort release
 locks set.  If an element of aCollection is not a member of the set,
 it is ignored."

aCollection accompaniedBy: self do: [:me :each | me removeFromCommitOrAbortReleaseLocksSet: each ].
^self
%

category: 'Environment Access'
classmethod: System
stoneName

"Returns a Symbol whose value is the full network name of the
 Stone to which this session is logged in."

^ self _zeroArgPrim: 11
%

category: 'Environment Access'
classmethod: System
sessionPerformingBackup

"Returns the session ID of the session that is performing a backup.
 If there is no such session, returns -1."

^ self configurationAt: #SessionInBackup
%

! Deleted:  _obj: anObject isIdenticalTo: anotherObject

! _enableTraceObjs deleted
! _disableTraceObjs deleted
! deleted _markNotConnectedForCollection


category: 'Shared Counters'
classmethod: System
_sharedCounter: n setValue: i

"Shared counters provide a means for multiple sessions on the same host
 to share a common counter value (for creation of unique keys, etc.).
 The number of shared counters is determined by the configuration file
 parameter SHR_PAGE_CACHE_NUM_SHARED_COUNTERS, which has a default of 1900.
 Use the #_numSharedCounters method to determine how many shared counters
 are available on this shared page cache.
 
 Each shared counter is protected with a unique spinlock.

 This method initializes one of the counters (in the range 0 to
 [System _numSharedCounters - 1]) to the  specified value i and 
 returns the receiver.  Both n and i must be positive SmallInteger instances."

<primitive: 350>
| sz |
n _validateClass: SmallInteger.
i _validateClass: SmallInteger.
sz := self _numSharedCounters .
(n < 0 or:[ n >= sz ])
  ifTrue:[n _error: #rtErrArgOutOfRange args:{ 0 . sz - 1 } ] .
(i < 0)
  ifTrue:[i _error: #errArgTooSmall args:{ 0 } ] .
self _primitiveFailed: #_sharedCounter:value: args: { n . i }
%

category: 'Shared Counters'
classmethod: System
_sharedCounter: n incrementBy: i

"Shared counters provide a means for multiple sessions on the same host
 to share a common counter value (for creation of unique keys, etc.).
 The number of shared counters is determined by the configuration file
 parameter SHR_PAGE_CACHE_NUM_SHARED_COUNTERS, which has a default of 1900.
 Use the #_numSharedCounters method to determine how many shared counters
 are available on this shared page cache.
 
 Each shared counter is protected with a unique spinlock.

 This method increments one of the counters (in the range 0 to
 [System _numSharedCounters - 1]) by the  specified value i and 
 returns the result.  Both n and i must be positive SmallInteger instances.

 The maximum value of any shared counter is INT_MAX - 1 (2**31 - 1).
 Any attempt to increment a counter larger than this value will result
 in the counter being set to its maximum value."

<primitive: 354>
| sz |
n _validateClass: SmallInteger.
i _validateClass: SmallInteger.
sz := self _numSharedCounters .
(n < 0 or:[ n >= sz ])
  ifTrue:[n _error: #rtErrArgOutOfRange args:{ 0 . sz - 1 } ] .
(i < 0)
  ifTrue:[i _error: #errArgTooSmall args:{ 0 } ] .
self _primitiveFailed: #_sharedCounter:incrementBy: args: { n . i }
%

! Request 34022
category: 'Shared Counters'
classmethod: System
_sharedCounter: n decrementBy: i withFloor: f

"Shared counters provide a means for multiple sessions on the same host
 to share a common counter value (for creation of unique keys, etc.).
 The number of shared counters is determined by the configuration file
 parameter SHR_PAGE_CACHE_NUM_SHARED_COUNTERS, which has a default of 1900.
 Use the #_numSharedCounters method to determine how many shared counters
 are available on this shared page cache.

 This method decrements one of the counters (in the range 0 to
 [System _numSharedCounters - 1]) by the  specified value i and 
 returns the result.  Both n and i must be positive SmallInteger instances.
  
 f specifies the minimum final value of the shared counter.  Specifying a floor
 of nil means the minimum value is INT_MIN (-2**31), which is the lowest 
 possible value for any shared counter.  Attempting to decrement any counter
 to a final value less than f will set the counter to floor value f."

<primitive: 551>
| sz |
n _validateClass: SmallInteger.
i _validateClass: SmallInteger.
(f == nil)
  ifFalse:[f _validateClass: SmallInteger].
sz := self _numSharedCounters .
(n < 0 or:[ n >= sz ])
  ifTrue:[n _error: #rtErrArgOutOfRange args:{ 0 . sz - 1 } ] .
(i < 0)
  ifTrue:[i _error: #errArgTooSmall args:{ 0 } ] .
self _primitiveFailed: #_sharedCounter:decrementBy:withFloor:
     args: { n . i . f }
%

category: 'Shared Counters'
classmethod: System
_numSharedCounters
"Shared counters provide a means for multiple sessions on the same host
 to share a common counter value (for creation of unique keys, etc.).
 The number of shared counters is determined by the configuration file
 parameter SHR_PAGE_CACHE_NUM_SHARED_COUNTERS, which has a default of 1900.

 Answer the number of shared counters configured on this sessions
 shared page cache (SHR_PAGE_CACHE_NUM_SHARED_COUNTERS in the configuration
 file used to the shared page cache)."

  ^self _zeroArgPrim: 100
%

category: 'Debugging'
classmethod: System
_setPrintStackAtError
  "Set a flag in the VM so that a gem process will print error information
   and a Smalltalk stack to the gem log file when any error is generated."

  self _zeroArgPrim:102
%

category: 'Shared Counters'
classmethod: System
_sharedCounterFetchValuesFrom: firstCounter to: lastCounter
"Shared counters provide a means for multiple sessions on the same host
 to share a common counter value (for creation of unique keys, etc.).
 The number of shared counters is determined by the configuration file
 parameter SHR_PAGE_CACHE_NUM_SHARED_COUNTERS, which has a default of 1900.

 Returns an array containing the values from all shared counters starting
 with the counter at index firstCounter, up to and including the value
 from the counter lastCounter.

 It is an error if firstCounter is greater than lastCounter or if
 either value is less than 1.  It is also an error if lastCounter
 is greater than the total number of counters configured (see
 the #_numSharedCounters method to determine this value)."

<primitive: 589>
| sz |
firstCounter _validateClass: SmallInteger.
lastCounter _validateClass: SmallInteger.
(firstCounter < 0 or:[ firstCounter > lastCounter])
  ifTrue:[firstCounter _error: #rtErrArgOutOfRange args:{ 0 . lastCounter } ] .
sz := self _numSharedCounters .
(lastCounter < 0 or:[ lastCounter >= sz ])
  ifTrue:[lastCounter _error: #rtErrArgOutOfRange args:{ 0 . sz - 1 }].
self _primitiveFailed: #_sharedCounterFetchValuesFrom:to:
     args: { firstCounter . lastCounter }
%

category: 'Environment Access'
classmethod: System
gemEnvironmentVariable: varName

"Expands the environment variable named varName in the Gem
 process, returning a String. varName should be a kind of String.

 Returns nil if any of the following are true:

 * varName is not a byte format object.
 * There is no environment variable defined with name varName.
 * The value of the environment variable is more than approximately 8000 bytes.
 * The size of varName exceeds approximately 8000 bytes.
 * The NoGsFileOnServer privilege is set in the UserProfile.
"

(self myUserProfile _privileges bitAnd: 16r4000) = 0 ifTrue:[
 ^ GsFile _expandEnvVariable: varName isClient: false
] ifFalse:[
  ^ nil  "NoGsFileOnServer bit is set"
]
%

category: 'Environment Access'
classmethod: System
clientEnvironmentVariable: varName

"Expands the environment variable named varName in the GemBuilder for C client
 process, returning a String.  The varName argument should be a kind of String.

 Returns nil if any of the following are true:

 * varName is not a byte format object.
 * There is no environment variable defined with name varName.
 * The value of the environment variable is more than approximately 8000 bytes.
 * The size of varName exceeds approximately 8000 bytes.
 * The NoGsFileOnClient privilege is set in the UserProfile.
 "

(self myUserProfile _privileges bitAnd: 16r8000) = 0 ifTrue:[
  ^ GsFile _expandEnvVariable: varName isClient: true
] ifFalse:[
  ^ nil  "NoGsFileOnClient bit is set"
]
%

category: 'Private'
classmethod: System
_hostCallDebugger

"Private."

"For debugging.  Allows GemStone Smalltalk access to HostCallDebugger which
 causes a core dump in customer executables and sleeps in non-customer
 executables."

^ self _zeroArgPrim: 25
%

!  zeroArgPrim 1020 is just HostExplicitWaitForDebugger 

category: 'Private'
classmethod: System
_prepareToCommit

"This is a placeholder for the implementation of voting in a two-phase commit
 protocol.  The future implementation should use #hasConflicts to return
 whether the current transaction could commit at this exact point in time.

 The current design of #hasConflicts however will issue false positives,
 indicating that there are conflicts even when there are not (bug 40094).
 Because of this, certain types of operations would always cause 
 #_prepareToCommit to fail.  Until this is resolved, we'll default to
 always returning true -- later steps in the psuedo two-phase commit will
 still catch any problems.

 Note: See the comment for GsCurrentSession | hasConflicts to get a 
 description of the side-effects of hasConflicts."

 " ^ GsSession currentSession hasConflicts not "
 ^ true
%

category: 'Private'
classmethod: System
_committedDataPages

"Returns an Array of the committed data pages referenced from the session's
 view of GemStone.  The contents of this Array are invalid after the next
 commit or abort."

^ self _zeroArgCateg2Prim: 10
%

category: 'Private'
classmethod: System
_uncommittedDataPages

"Returns an Array of the data pages referenced from the session's shadowed
 view of GemStone.  After the next commit or abort, the contents of this
 Array are invalid and may be incomplete.  That is, very shortly after the
 commit or abort operation, they may not show all data pages, because more pages
 may be added by the currently running methods.  However, it might be useful to
 check on modified objects (in particular, reclustering) before the transaction
 commits."

^ self _zeroArgCateg2Prim: 11
%

category: 'System Control'
classmethod: System
addAllToStoneLog: aString

"Appends text to the Stone's informational log file.  First, this method writes
 a banner that identifies the session from which aString came.  It then appends
 aString itself.  The argument must be a kind of String or DoubleByteString."

| logicalSize chunkSize offset endOffset |

aString basicSize <= 16270 "virtual machine constant" ifTrue:[
  ^ self _addAllToStoneLog: aString 
  ].
aString _validateClasses:{ String . DoubleByteString }.
logicalSize := aString size .
chunkSize := 16270 "virtual machine constant" .
(aString isKindOf: DoubleByteString) ifTrue:[ chunkSize := 16270 / 2 ].
 
offset := 1.
[offset <= logicalSize] whileTrue:[
  endOffset := offset + chunkSize - 1 .
  endOffset > logicalSize ifTrue:[ endOffset := logicalSize ].
  self _addAllToStoneLog: (aString copyFrom: offset to: endOffset).
  offset := offset + chunkSize
  ].
% 

category: 'System Control'
classmethod: System
_addAllToStoneLog: aString

"Appends text to the Stone's informational log file.  First, this method writes
 a banner that identifies the session from which aString came.  It then appends
 aString itself.  The argument must be a kind of String or DoubleByteString.

 Generates an error if aString is larger than approximately 16000 bytes."

<primitive: 463>

aString _validateClasses:{ String . DoubleByteString }.
self _primitiveFailed: #addAllToStoneLog: args: { aString }
%

category: 'Private'
classmethod: System
changeCacheSlotFreeFrameLimit: aSlot to: aValue

"Changes the freeFrameLimit for the process associated with a cache slot to 
 aValue, which must be a positive SmallInteger.  The argument aSlot
 should be a SmallInteger between 0 and the number of process slots in the
 shared cache minus 1, inclusive. 

 To execute this method, you must have the SystemControl privilege."

<primitive: 349>

aSlot _validateClasses:{ SmallInteger }.
aValue _validateClasses:{ SmallInteger }.
self _primitiveFailed: #changeCacheSlotFreeFrameLimit:to:
     args: { aSlot . aValue }
%

category: 'Private'
classmethod: System
_commitCoordinator

"Returns the commit coordinator stored in temporary session state."

^ self _sessionStateAt: 14
%

category: 'Private'
classmethod: System
_commitCoordinator: aCoordinator

"Sets the commit coordinator stored in temporary session state."

^ self _sessionStateAt: 14 put: aCoordinator
%

category: 'Private'
classmethod: System
_clearAttachedPages

"Detaches the data pages currently attached."

^ self _zeroArgPrim: 28
%

! deleted _flushConnectedToPom

category: 'Private'
classmethod: System
_internalMessageForErrorNum: aNumber 

"Private.  Returns a String describing the error message generated from the
 errmsg.c module for GemStone error numbered aNumber.  Returns nil if no such
 error number or message exists in errmsg.c."

<primitive: 384>
aNumber _validateClass: SmallInteger .
self _primitiveFailed: #_internalMessageForErrorNum: args: { aNumber }
%

category: 'Private'
classmethod: System
_messageForErrorNum: aNumber 

"Private.  Returns a String describing the error message produced for GemStone 
 error numbered aNumber in native language English, or nil if no such error
 number or language exists."

^ self _messageForErrorNum: aNumber inLanguage: #English
%

category: 'Private'
classmethod: System
_messageForErrorNum: aNumber inLanguage: aSymbol

"Private.  Returns a String describing the error message produced for GemStone 
 error numbered aNumber in native language aSymbol, or nil if no such error
 number or language exists."

 | errArray msgParts result |

 errArray := GemStoneError at: aSymbol otherwise: nil . 
 errArray == nil ifTrue:[ ^ nil ].
 (aNumber < 1 or:[ aNumber > errArray size]) ifTrue:[ ^ nil ].
 msgParts := errArray at: aNumber .
 msgParts == nil ifTrue:[ ^ nil ].
 result := String new .
 1 to: msgParts size do:[:j | | element |
   element := msgParts at: j .
   element _isSmallInteger 
     ifTrue:[ result addAll: '<arg'; addAll: element asString; add: $> ] 
    ifFalse:[ result addAll: element ].
   ].
 ^ result 
%

category: 'Private'
classmethod: System
_allErrorMessagesReport

"Private."

| errorSymbolsArray symsWithNoMsg msgsWithNoSym report lf line errMsgsArray 
  firstNil last j |

errorSymbolsArray  := { }  .
ErrorSymbols keysAndValuesDo:[:aSym :aNum |
  aNum > errorSymbolsArray size ifTrue:[ errorSymbolsArray size: aNum ].
  (errorSymbolsArray at: aNum) ~~ nil ifTrue:[
    self _halt:'Duplicate symbol for error number ' , aNum asString .
    ].
  errorSymbolsArray at: aNum put: aSym .
  ].
errMsgsArray := GemStoneError at: #English .

msgsWithNoSym := String new .
symsWithNoMsg := String new .
report := String new .
lf := Character lf .

report addAll:'ERROR MESSAGES STORED IN THE REPOSITORY:'; add: lf; add: lf .

1 to: errorSymbolsArray size do:[:j | | aSym msgParts |
  aSym := errorSymbolsArray at: j .
  msgParts := errMsgsArray at: j .
  aSym == nil ifTrue:[
    msgParts ~~ nil ifTrue:[
       msgsWithNoSym addAll: '*** No error symbol for error number ';
		addAll: j asString; add: lf .
       ]
    ]
  ifFalse:[
    msgParts == nil ifTrue:[
       symsWithNoMsg addAll: '*** No message for error symbol #';
		addAll: aSym quoted; add: lf .
       ]
    ifFalse:[
       line := j asString. 
       line addAll: ', #' ; addAll: aSym; addAll: ' :  ' ;
            addAll: (self _messageForErrorNum: j)  .
       report addAll: (line _wrapTo: 80 indentingWith: '    '); add: lf .
       ]
    ].
  ].

report add: lf .
report addAll:
'ERROR MESSAGES FROM errmsg.c, TEXT FROM errmsg.ht '; add: lf ;
addAll: '  (Repository message followed by errmsg.ht message, if any)'; add: lf;
add: lf .

1 to: errorSymbolsArray size do:[:j | | aSym internalMsg msgFN |
  aSym := errorSymbolsArray at: j .
  aSym ~~ nil ifTrue:[
    internalMsg := self _internalMessageForErrorNum: j .
    internalMsg ~~ nil ifTrue:[
      msgFN := self _messageForErrorNum: j .
      msgFN == nil ifTrue:[ msgFN := 'NO MESSAGE FOR NUMBER ' , j asString ].
      line := j asString. 
      line addAll: ', #' ; addAll: aSym; addAll: ' :  ' ;
           addAll: msgFN ; add: lf ;
           addAll: internalMsg .
      report addAll: (line _wrapTo: 80 indentingWith: '    '); add: lf; add: lf.
      ].
    ].
  ].

report add: lf ; addAll:'UNUSED ERROR NUMBERS:'; add: lf .

firstNil := 1 .
j := 1.
last := errorSymbolsArray size .
[j < last] whileTrue:[
  [ (errorSymbolsArray at: j) == nil and:[j < last] ] whileTrue:[ j := j + 1 ]. 
  j > firstNil ifTrue:[
    report addAll: firstNil asString ; addAll: ' to: '; 
		addAll: (j - 1) asString; add: lf .
    ]
  ifFalse:[
    report addAll: firstNil asString; add: lf .
    ].
  firstNil := nil . 
  [ (errorSymbolsArray at: j) ~~ nil and:[j < last] ] whileTrue:[ j := j + 1 ]. 
  firstNil := j .
  ].

report add: lf .
msgsWithNoSym size ~~ 0 
  ifTrue:[ report addAll: msgsWithNoSym ]
  ifFalse:[ report addAll:'No messages found without an error symbol.'].
report add: lf .
symsWithNoMsg size ~~ 0 
  ifTrue:[ report addAll: symsWithNoMsg ]
  ifFalse:[ report addAll: 'No error symbols found without a message.'].
report add: lf .
^ report
%

category: 'Private'
classmethod: System
_commitResult

"Private.  Returns a SmallInteger which represents the result of the last 
 commitTransaction or continueTransaction operation.  If abortTransaction was 
 issued since the last commit or continue then it returns SmallInteger 0.  

 See   _primitiveCommit:  for documentation of the return values."

^ self _zeroArgPrim: 30
%

category: 'Private'
classmethod: System
_deadNotReclaimed

"Private.  Returns an Array that contains the current contents of the 
 global set of objects which have been identified as dead but have not 
 yet been reclaimed. Before accessing this set, the user should disable
 the reclaiming of dead (set reclaimDeadEnabled in GcUser's UserGlobals to 
 false).  The user should be sure to reset the size of the Array returned
 to zero when done analyzing the dead objects before re-enabling the
 reclaiming of dead objects."

^ self _zeroArgPrim: 31
%

category: 'Transaction Control'
classmethod: System
needsCommit

"Returns true if the session has made changes to the repository.  Returns false
 otherwise."

^ self _numPersistentObjsModified > 0
%

category: 'Transaction Control'
classmethod: System
_numPersistentObjsModified

"Returns number of persistent objects modified during current transaction. 
 Includes number of objects added/removed to/from depMap."

^ self _zeroArgPrim: 54
%

category: 'Reduced Conflict Support'
classmethod: System
_getRedoAndConflictObjects

"Builds a list of objects that need to be scanned because they need to
 have conflicts resolved.  If the result array is empty, then the transaction
 has conflicts that cannot be resolved because we should only get here if
 some Rc conflicts have been detected."
          
| wwConflicts scanArray redoLog |
     
	" Gs64 v2.0 optimization, access the hidden sets directly 
  	instead of computing complete transaction conflicts info."

" if there are read-write and write-read conflicts, we cannot resolve them "
"check for #'Read-Write' , i.e. strongRead-write conflicts "
(self _hiddenSetSize: 13) > 0 ifTrue:[
  "cannot resolve strongRead-write conflicts. The Read-Write conflict
   has already had RcReadSet subtracted from it by the commit primitive."
  ^ nil  
].
" check for #'Write-Dependency' conflicts"
(self _hiddenSetSize: 16) > 0 ifTrue:[
  "cannot resolve write-depencency conflicts. "
  ^ nil  
].

wwConflicts := self _hiddenSetAsArray: 15. "get #'Write-Write' conflicts"
wwConflicts size == 0 ifTrue: [ 
  "expected to find write-write conflicts to resolve, so fail"
  ^ nil 
].
              
scanArray := { } .  "expect this be relatively small"

redoLog := self _redoLog.

" make an initial scan to gather objects to be replayed "
redoLog == nil ifTrue: [
  " no redo log, treat objects themselves as the one on which conflicts
        must be resolved "
  1 to: wwConflicts size do: [ :i | | conflictObject |
    conflictObject := wwConflicts at: i.
    (scanArray includesIdentical: conflictObject) ifFalse: [ 
	scanArray add: conflictObject ; add: { conflictObject }.
    ].
  ].
] ifFalse: [
  1 to: wwConflicts size do: [ :i | | conflictObject redoObject |
    conflictObject := wwConflicts at: i.
  
    " get the object that has to be replayed due to this conflict "
    redoObject := redoLog getRedoObjectForConflictingObject: conflictObject.
    redoObject == nil ifTrue: [
      " no redo object in the redo log, treat the object itself as
	the one on which conflicts must be resolved "
      (scanArray includesIdentical: conflictObject)  ifFalse: [
	scanArray add: conflictObject ; add: { conflictObject }.
      ]
    ] ifFalse: [  | idx |
      idx := scanArray indexOfIdentical: redoObject.
      idx ~~ 0 ifTrue: [ | argArr |
	argArr := scanArray at: idx + 1.
	argArr add: conflictObject.
      ] ifFalse: [
	scanArray add: redoObject ; add: { conflictObject }.
      ].
    ].
  ].
].   
     
^ scanArray
%
 
category: 'Reduced Conflict Support'
classmethod: System
_resolveRcConflicts
        
"Checks for selected conflicts.  If some are found, attempt to resolve those
 conflicts.  If any conflicts could not be resolved, returns false."
  
|  scanArray redoObject commitFailed objsToRefresh conflicts |
              
scanArray := self _getRedoAndConflictObjects.
             
scanArray == nil ifTrue: [ 
  "no redo objects were found, cannot resolve conflicts"
  self _disallowSubsequentCommits.
  ^ false 
].
commitFailed := false.
objsToRefresh := { } .
1 to: scanArray size by: 2 do: [ :i |
  redoObject := scanArray at: i.
  conflicts := scanArray at: i + 1.
              
  " keep list of objects to selectively abort in case of commit failure "
  redoObject _refreshAfterCommitFailure ifTrue: [ 
    objsToRefresh add: redoObject 
  ].
  commitFailed ifFalse: [
    (redoObject _resolveRcConflictsWith: conflicts) ifFalse: [
      self rcValueCacheAt: #'Rc-Write-Write' put: redoObject for: self.
      commitFailed := true.
    ]
  ]
].      

commitFailed ifTrue: [
  " selectively abort objects that might be logically inconsistent "
  1 to: objsToRefresh size do: [ :i |
    (objsToRefresh at: i) _selectiveAbort  
  ].       
  " force subsequent attempts to commit to fail with retry limit error"
  self _disallowSubsequentCommits.
  ^ false
].      

^ true
%

category: 'Private'
classmethod: System
_validateTransaction

" Determine whether the current transaction could commit.

  See   _primitiveCommit:  for documentation of the return values.

  If the result is > 0 , the current transaction cannot commit and
  the transaction conflict hidden sets have
  been updated to reflect the conflicts preventing commit .  "


^ self _zeroArgPrim: 32
%

! added for 31693
category: 'Transaction Control'
classmethod: System
currentTransactionWWConflicts

"Returns an Array of objects which have write-write conflicts.
 The array is created by building the write set union of all
 commit records created since the session's current transaction
 and intersecting it with the session's write set."

 | status conflictDict wwConflicts |
 status := self _validateTransaction .
 status <= 0 ifTrue:[ 
   ^ #()  "read only or empty conflicts"
 ].
 conflictDict := self transactionConflicts .
 wwConflicts := conflictDict at: #'Write-Write' otherwise: #() .
 ^ wwConflicts .
% 

! added for 31693
category: 'Transaction Control'
classmethod: System
currentTransactionHasWWConflicts

"Returns a Boolean indicating if the current transaction has one or
 more write-write conflicts.  A result of true indicates a commit
 will most likely fail.  However some RC objects have conflict
 resolution mechanisms which could allow a commit to suceed.
 A result of false indicates no write-write conflicts exist.

 It is inefficient to invoke this method and then invoke 
 currentTransactionWWConflicts. If you want conflict details, invoke
 currentTransactionWWConflicts directly. 
 "

 ^ self currentTransactionWWConflicts size > 0
%

category: 'Transaction Control'
classmethod: System
currentTransactionWDConflicts

"Returns an Array of objects which have write-dependency conflicts.
 The array is created by building the union of all dependency 
 change sets from all commit records created since the session's 
 current transaction and intersecting it with the session's write 
 set."

 | status conflictDict wdConflicts |
 status := self _validateTransaction .
 status <= 0 ifTrue:[ 
   ^ #()  "read only or empty conflicts"
 ].
 conflictDict := self transactionConflicts .
 wdConflicts := conflictDict at: #'Write-Dependency' otherwise: #() .
 ^ wdConflicts .
% 

category: 'Transaction Control'
classmethod: System
currentTransactionHasWDConflicts

"Returns a Boolean indicating if the current transaction has one or
 more write-depencency conflicts.  A result of true indicates a commit
 will most likely fail.  A result of false indicates no write-dependency 
 conflicts exist.

 It is inefficient to invoke this method and then invoke 
 currentTransactionWDConflicts. If you want conflict details, invoke
 currentTransactionWDConflicts directly. 
 "

 ^ self currentTransactionWDConflicts size ~~ 0
%


Category 'Private'
classmethod: System
_reloginAsUser: aUserId password: aPassword encrypted: aBoolean
       
"Relogin this session with the specified user identification."

<primitive: 495>
aUserId _validateClass: String.
aPassword _validateClass: String.
aBoolean _validateClass: Boolean.
self _primitiveFailed: #_reloginAsUser:password:encrypted:
     args: { aUserId . aPassword . aBoolean }
%

category: 'Private'
classmethod: System
_publish: server port: port log: logname options: optstr
  "Description:
     Publishes information about a server so that the gslist utility
     can display it. The information will be published on the local machine.
   Input:
     server <String>: name of server to publish.
     port <SmallInteger>: port number of server listening socket.
     logname <String or nil>: name of log file or nil if none.
     optstr <String or nil>: startup options or nil if none.
   Result <Boolean>:
     true if publish was successful.
     false if server was already published.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 496>
self _primitiveFailed: #_publish:port:log:options:
     args: { server . port . logname . optstr }
%

category: 'Private'
classmethod: System
_unpublish: server on: machine
  "Description:
     Unpublishes information about a server on the given machine.
   Input:
     server <String>: name of server to unpublish.
     machine <String or nil>: name of machine to unplubish on or nil if local.
   Result <Boolean>:
     true if unpublish was successful.
     false if server was not published.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 497>
self _primitiveFailed: #_unpublish:on: args: { server . machine }
%

category: 'Private'
classmethod: System
_findPublished: server on: machine
  "Description:
     Find a published server on the given machine.
   Input:
     server <String>: name of server to findunpublish.
     machine <String or nil>: name of machine to find on or nil if local.
   Result <Array or nil>:
     nil if server was not found.
     instance of Array if server was found. The array contents are:
       1. <String>: the name of the server
       2. <SmallInteger>: the port number of ther servers listening socket.
       3. <Integer>: the proces id of the server's main process.
       4. <Boolean>: true if process exists, false if not.
       5. <String>: type of server: 'Stone', 'NetLDI', 'Cache', 'Broker',
                                    'Agent', 'RCPD', 'Service-Stone',
                                    'Service-NetLDI', or '?'.
       6. <String>: name of user that created the server.
       7. <String>: time server was created.
       8. <String>: name of machine server is running on.
       9. <String or nil>: version of GemStone used by server.
       10.<String or nil>: GemStone product directory.
       11.<String or nil>: log file name of server.
       12.<String or nil>: startup options of server.
       13.<String or nil>: GemStone system configuration file used by server.
       14.<String or nil>: GemStone executable configuration file used by server.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 498>
self _primitiveFailed: #_findPublished:on: args: { server . machine }
%

category: 'Private'
classmethod: System
_findAllPublishedOn: machine
  "Description:
     Find all published servers on the given machine.
   Input:
     machine <String or nil>: name of machine to find on or nil if local.
   Result <Array or nil>:
     nil if no servers were found.
     instance of Array that contains one element for each server found.
     Each element will itself be an Array with the contents described
     for the _findPublished:on: method.
   Exceptions:
     Illegal input raises various errors.
     Error 2367 raised if low level C code fails with system error.
  "

<primitive: 499>
self _primitiveFailed: #_findAllPublishedOn: args: { machine }
%

category: 'System Control'
classmethod: System
sendSigAbortToSession: sessionIdOrSerial

"Sends a sigAbort signal to the specified session.
 To execute this method, you must have the SystemControl privilege.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId.

 SigAborts sent to any session more frequently than 
 every 2 seconds are ignored."

self _sendTransactionSignal: sessionIdOrSerial kind: 0 
%

category: 'System Control'
classmethod: System
sendSignalFinishTransactionToSession: sessionIdOrSerial

"Sends a FinishTransaction signal to the specified session.
 To execute this method, you must have the SystemControl privilege.

 The sessionIdOrSerial argument is a SmallInteger specifying a session.  A
 positive value is interpreted as a serialNumber.  A negative value is
 interpreted as a negated sessionId."

self _sendTransactionSignal: sessionIdOrSerial kind: 1 
%

category: 'Private'
classMethod: System
_sendTransactionSignal: sessionIdOrSerial kind: aKind

"send a sigAbort or a finishTransaction signal to the specified
 session.  The signal will be ignored if the session has not enabled
 that specific signal. 

 You must have SystemControl privilege .

 aKind == 0 : sigAbort
 aKind == 1 : finishTransaction "

<primitive: 327>
sessionIdOrSerial _validateClasses:{ SmallInteger }.
aKind _validateClasses:{ SmallInteger }.
self _primitiveFailed: #sendSigAbortToSession: 
     args: { sessionIdOrSerial . aKind }
%

category: 'Private'
classMethod: System
_writeLockWriteSubset

"Start with the object manager dirty list ,
 and perform filtering of objects already locked, putting the results 
 in a new hidden set (writeLockWriteSubset).  Next, attempt to acquire a 
 write lock on the objects in the hidden set.  If all locks are acquired 
 cleanly, return true.  If all locks acquired but some are dirty, return 
 an array of dirty locked objects.  If not all locks could be acquired, 
 return false (locks that were acquired, including dirty locks are available
 in the writeLockWriteSubset hidden set.  Any locks that are acquired are 
 also put in the CommitOrAbortReleaseLocksSet."

^self _zeroArgPrim: 34
%

! deleted _exclusiveLockWriteSubset

category: 'Private'
classMethod: System
_removeLockWriteSubset

"Removes the locks acquired on any objects that were locked using
 _writeLockWriteSubset and empties the hidden set."

^self _zeroArgPrim: 35

%

category: 'Private'
classmethod: System
_cacheName: aString

"Sets the name the current session is known by in the shared cache
 to 'aString'. Does nothing if the current session is not attached
 to a shared cache. Raises an error if 'aString' is too long.
 Returns self."

<primitive: 509>
self _primitiveFailed: #_cacheName: args: { aString }
%

category: 'Private'
classmethod: System
_parseClientNrs: aString

"aString should be the result of reading a request sent to a netldi
 on a socket. A session reading nrs requests from clients can tell
 that a full request has come in by reading until it gets a zero byte.
 This method raises the error #genericKernelError if something is
 wrong with the input.
 Otherwise it returns a 8 element Array with the following contents:
 1  Symbol describing type of client. It will be one of the following:
      #RpcApplication, #Gem, #PageServer, #Stone, #Unknown
 2  Boolean: true if client is trusted.
 3  String: name of user.
 4  Boolean: true if using kerberos.
 5  Symbol describing type of request. It will be one of the following:
      #invalid, #file, #monitor, #server, #task, #spawn, #cmd, #dbf
 6  String: working directory
 7  String: logfile name
 8  String: body of request
"

<primitive: 515>
| syms |
" make sure we don't have to create new symbols at runtime"
syms := { #RpcApplication . #Gem . #PageServer . #Stone . #Unknown . 
	#invalid . #file . #monitor . #server . #task . #spawn . #cmd . #dbf } .
self _primitiveFailed: #_parseClientNrs: args: { aString }
%

category: 'Private'
classmethod: System
_reinitializeSharedCache

"This method shuts down the GcGem process and verifies that this session
 is the only one logged in.  If not the only user, then the primitive fails.
 If it is the only user, all attached pages for this session are detached 
 and the cache is swept.  All dirty pages are written.  All remaining 
 unattached pages are removed from the cache and their frames are put back
 in the free list.
 
 Returns self."

^self _zeroArgPrim: 36
%

category: 'Private'
classmethod: System
_ignoreModTracking

"Private. Disables Modification Tracking."

<primitive: 516>
self _primitiveFailed: #_ignoreModTracking
%

category: 'Private'
classmethod: System
_restoreModTracking

"Private. Enables Modification Tracking. 

 Has no effect in this release"

<primitive: 517>
self _primitiveFailed: #_RestoreModTracking
%
category: 'Private'
classmethod: System
_commitPrintingDiagnostics

    "Commits the current transaction. Answers true if successful,
    otherwise false. If false, it also prints to stderr the OOPs for
    each kind of conflict, ordered by kind. This is used in the
    upgrade scripts."

    self commitTransaction 
      ifTrue: [^true]
      ifFalse: [ | conflicts gsFil |
        gsFil := GsFile . "cache literalVariable, but don't use cached value
                           in the blocks to avoid making blocks complex."
	conflicts := self transactionConflicts.
	(conflicts at: #commitResult) == #success ifFalse: [
	gsFil gciLogClient: (conflicts at: #commitResult).
	gsFil gciLogClient: 'Read-Write Conflicts...'.
	(conflicts at: #'Read-Write' ifAbsent: [#()]) 
	    do: [ :each | GsFile gciLogClient: '    ', each asOop asString].

	gsFil gciLogClient: 'Write-Write Conflicts...'.
        (conflicts at: #'Write-Write' ifAbsent: [#()]) 
	    do: [ :each | GsFile gciLogClient: '    ', each asOop asString].
	gsFil gciLogClient: 'Write-Dependency Conflicts...'.
        (conflicts at: #'Write-Dependency' ifAbsent: [#()]) 
	    do: [ :each | GsFile gciLogClient: '    ', each asOop asString].
        gsFil gciLogClient: 'Write-ReadLock Conflicts...'.
        (conflicts at: #'Write-ReadLock' ifAbsent: [#()]) 
	    do: [ :each | GsFile gciLogClient: '    ', each asOop asString].
        gsFil gciLogClient: 'Write-WriteLock Conflicts...'.
        (conflicts at: #'Write-WriteLock' ifAbsent: [#()]) 
	    do: [ :each | GsFile gciLogClient: '    ', each asOop asString].
        gsFil gciLogClient: 'Rc-Write-Write Conflicts...'.
        (conflicts at: #'Rc-Write-Write' ifAbsent: [#()]) 
	    do: [ :each | GsFile gciLogClient: '    ', each asOop asString].
        gsFil gciLogClient: 'Synchronized-Commit Conflicts...'.
        (conflicts at: #'Synchronized-Commit' ifAbsent: [#()]) 
	    do: [ :each | GsFile gciLogClient: '    ', each asOop asString]].
        ^false
      ]
%

category: 'Garbage Collection Management'
classmethod: System
voteState

"Returns an integer, the voteState of the Stone garbage
 collection voting state machine.   
 The states are  0 IDLE, 1 VOTING,
	 2 DONE_VOTING, 3  IN_PDWSU_SWEEP, 4 PDWSU_DONE"

 ^ self _zeroArgPrim: 17
%

category: 'Garbage Collection Management'
classmethod: System
waitForVoteStateIdle

"Assumes the caller has voted by executing a commit or abort.
 returns when the vote state is idle."

[(self voteState) > 0] whileTrue: [self sleep: 1].
^ self
%

category: 'Garbage Collection Management'
classmethod: System
forceEpochGc
"Force an Epoch GC to run as soon as possible, regardless of the setting 
of the GcUser configuration parameters epochGc[Time|Trans]Limit.  
Note that the stone configuration parameter STN_EPOCH_GC_ENABLED must be 
TRUE for this method to have any effect.

This method will fail and return false under the following conditions:
  *Checkpoints are suspended
  *Another garbage collection operation is in progress
  *Unfinalized possible dead objects exist (i.e., System>>voteState 
    returns any value except 0).
  *The system is in restore mode.
  *The Admin GC session is not running.
  *Epoch GC is not enabled (STN_EPOCH_GC_ENABLED is set to FALSE)
  *The system is performing a reclaimAll.
  *A previous forceEpochGc operation was performed and the epoch has
   not yet started or completed.

If successful, this method sets the stone cache statistic EpochForceGc
to 1.  Once the Admin GcGem has started the epoch GC, EpochForceGc 
will return back to 0.

This method returns true if the epoch GC was started, false if not.

You must have GarbageCollection privileges to be able to run this method."

^self _zeroArgPrim: 41
%

category: 'Garbage Collection Management'
classmethod: System
clearEpochGcState

"Resets the epoch GC state which keeps track of objects eligible to be
 marked as possible dead during the next epoch GC operation.  Has no effect
 on objects already marked possible dead by previously completed epochs."
 
^self _zeroArgPrim: 38
%

category: 'Garbage Collection Management'
classmethod: System
disableEpochGc
"Disables epoch garbage collection from running and resets
 the epoch GC state.  Appends the new configuration state
 to the stones configuration file.

 No further epoch GC operations will be run after this method is
 successfully executed.  An epoch GC operation already in progress
 when this method is executed will not be interrupted.  Has no
 effect if epoch GC is already disabled.

 Requires the GarbageCollection privilege."

^self stoneConfigurationAt: #StnEpochGcEnabled put: 0.
%

category: 'Garbage Collection Management'
classmethod: System
enableEpochGc
"Enables epoch garbage collection to run and resets
 the epoch GC state.  Appends the new configuration state
 to the stones configuration file.

 Has no effect if epoch GC is already enabled.

 Requires the GarbageCollection privilege."

^self stoneConfigurationAt: #StnEpochGcEnabled put: 1.
%

category: 'Garbage Collection Management'
classmethod: System
voteStateString

"Returns a String which is the name of the current value of
 the voteState of the Stone garbage collection voting state
 machine. "

 ^ self _zeroArgPrim: 19
%

category: 'Garbage Collection Management'
classmethod: System
startAllGcSessions

"Start all enabled garbage collector gems that are not running.
 Does not start the Symbol Creation Gem.
 Requires the GarbageCollection privilege."

^ self _zeroArgPrim: 18
%

category: 'Garbage Collection Management'
classmethod: System
stopAllGcSessions

"Stop any and all Garbage Collection sessions. 
 Does not stop the Symbol Creation Gem .
 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 40
%

category: 'Garbage Collection Management'
classmethod: System
stopAdminGcSession

"Stop the Admin GC session if it is running.
 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 42
%

category: 'Garbage Collection Management'
classmethod: System
stopSymbolCreationSession

"Stop the Symbol Creation session if it is running.

 This is an extreme action that should never be done in normal
 operation of a system.  Stopping the Symbol Creation session
 can prevent other sessions from being able to commit until
 after the Symbol Creation session restarts and after
 the other sessions logout and login .

 Only SystemUser may execute this method, otherwise an error is generated."

^self _zeroArgPrim: 44
%

category: 'Garbage Collection Management'
classmethod: System
stopAllReclaimGcSessions

"Stop any and all reclaim GC sessions.
 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 43
%

category: 'Garbage Collection Management'
classmethod: System
startAdminGcSession

"Start the Admin GC session if it is not running.
 Returns true if the gem was started and false if
 it could not be started because it is already running
 or logins are suspended.
 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 67
%

category: 'Garbage Collection Management'
classmethod: System
startSymbolCreationSession

"Start the Symbol Creation session if it is not running.
 Returns true if the gem was started and false if
 it could not be started because it is already running
 or logins are suspended.

 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 45
%

category: 'Garbage Collection Management'
classmethod: System
startAllReclaimGcSessions

"Start all recalim GC sessions that are configured to run.
 Returns the number of reclaim sessions started.

 Requires the GarbageCollection privilege."

^self _zeroArgPrim: 68
%


category: 'Garbage Collection Management'
classmethod: System
adminGcGemSessionId

"Return the session ID of the Admin GC session.  Returns
 zero if the Admin GC session is not running."

^self _zeroArgPrim: 64
%

category: 'Garbage Collection Management'
classmethod: System
reclaimGcSessionCount

"Return the number of reclaim GC sessions currently running."

^self _zeroArgPrim: 65
%

category: 'Garbage Collection Management'
classmethod: System
symbolCreationSessionId

"Return the session ID of the SymbolCreation session.  Returns
 zero if the SymbolCreation session is not running."

^self _zeroArgPrim: 46
%


category: 'Garbage Collection Management'
classmethod: System
numberOfExtentsWithoutGC
"Return a SmallInteger indicating the number of extents not subject to
 garbage collection because they have no reclaim gem assigned."

|result reclaimGemsByExtent|

result := 0.
reclaimGemsByExtent := self currentGcReclaimSessionsByExtent.
1 to: reclaimGemsByExtent size do:[:j| | e |
  e := (reclaimGemsByExtent at: j) .
  (e == 0) ifTrue:[result := result + 1].
].

^result
%

category: 'Garbage Collection Management'
classmethod: System
numberOfExtentRangesWithoutGC
"Return a SmallInteger indicating the minimum number of additional
 reclaim gems that would need to be started to have all extents
 covered by a reclaim gem."

|result reclaimGemsByExtent startId stopId |

result := 0.
reclaimGemsByExtent := self currentGcReclaimSessionsByExtent.
startId := -1 .
stopId := -1 .
1 to: reclaimGemsByExtent size do:[:j| | e |
  e := (reclaimGemsByExtent at: j) .
  (e == 0) ifTrue:[
    startId < 0 ifTrue:[
      startId := j .
      result := result + 1 .
    ]
  ] ifFalse:[ 
    startId := -1 
  ]
].
^result
%

category: 'Garbage Collection Management'
classmethod: System
hasMissingGcGems
"Return a Boolean indicating if any garbage collection systems are
 missing from the system.  Answers false if the Admin GC session is
 running and each extent has been assigned a reclaim gem.  Answers
 true otherwise."

|reclaimGemsByExtent|

(self adminGcGemSessionId == 0)
  ifTrue:[^true]. "missing the Admin gem"

reclaimGemsByExtent := self currentGcReclaimSessionsByExtent.
reclaimGemsByExtent do:[:e| (e == 0) ifTrue:[^true].].

^false "all GC gems accounted for"
%


category: 'Garbage Collection Management'
classmethod: System
waitForAllGcGemsToStartForUpToSeconds: anInt
"Wait for all GC gems on the system to start for up to anInt
 seconds.  Returns true if all GC gems started, false if they
 did not within the specified amount of time."

|count|

count := 0.
[self hasMissingGcGems] whileTrue:[
  self sleep: 1.
  count := count + 1.
  (count > anInt)
   ifTrue:[^false]. "timeout waiting for GC gems to start"
].
^true "success!"
%


category: 'Garbage Collection Management'
classmethod: System
currentGcReclaimSessionsByExtent

"Return an array where each element represents the session ID
of the reclaim gem for the corresponding extent.  For example,
if the array returned was { 2 . 3}, that means session 2 is the
reclaim gem for extent 1 and session 3 is the reclaim gem for
extent 2.

A zero returned in the result array indicates there is no reclaim
gem for the corresponding extent."

^self _zeroArgPrim: 66
%

category: 'Garbage Collection Management'
classmethod: System
startReclaimGemForExtentRange: startExtent to: endExtent

"Start a reclaim GC gem to reclaim all extents starting with
 startExtent, up to and including endExtent.  The gem is started
 on the same machine as the stone.

 See the method:

 startReclaimGemForExtentRange: startExtent
 to: endExtent
 onHost: aHostNameOrNil
 stoneHost: stoneHostOrNil

 for further details.

 Requires the GarbageCollection privilege."

^self startReclaimGemForExtentRange: startExtent to: endExtent onHost: nil stoneHost: nil
%

category: 'Garbage Collection Management'
classmethod: System
startReclaimGemForExtentRange: startExtent to: endExtent onHost: aHostNameOrNil

"Start a reclaim GC gem to reclaim all extents starting with
 startExtent, up to and including endExtent on a remote host specified by
 aHostNameOrNil. aHostNameOrNil is a string containing a valid host name or IP 
 address where the reclaim gem will be spawned.  A value of nil means to spawn
 the reclaim gem on the stones host.   

 See the method:

 startReclaimGemForExtentRange: startExtent
 to: endExtent
 onHost: aHostNameOrNil
 stoneHost: stoneHostOrNil

 for further details.
 Requires the GarbageCollection privilege."

^self startReclaimGemForExtentRange: startExtent to: endExtent onHost: aHostNameOrNil stoneHost: nil
%

category: 'Garbage Collection Management'
classmethod: System
startReclaimGemForExtentRange: startExtent to: endExtent onHost: aHostNameOrNil stoneHost: stoneHostOrNil

"Start a reclaim GC gem to reclaim all extents starting with
 startExtent, up to and including endExtent.  Both startExtent
 and endExtent must be non-negative SmaallIntegers and 
 endExtent must be greater than or equal to endExtent.

 startExtent and endExtent are 1-based extent IDs, that is
 the first extent is extent 1.

 aHostNameOrNil if the host where the reclaim gem will be spawned.
 It may be a String containing a valid host name or IP address or
 it may be nil, which means the reclaim gem will be created on 
 the same node as the stone.

 stoneHostOrNil is the host name to use for the spawned reclaim
 gem to use to connect with the stone.  Normally this can be nil,
 which means use the default stone host name.  On multi-homed
 systems with more than one network connection between the remote
 host and the stone host, the host name or IP address may need to
 be specified to indicate which connection to use.

 Returns true if the reclaim GC gem was successfully started,
 otherwise returns false.

 The number of configured reclaim gems for the system is not
 altered by this method.  If reclaim gems were previously 
 disabled by exectuing:

 self stoneConfigurationAt: #StnNumGcReclaimSessions put: 0

 then the number of permitted reclaim gems must be changed to a non-zero
 value before this method can succeed.

 There are a number of conditions that will prevent a reclaim gem
 from starting, including:

 -logins are suspended.
 -repository in restore mode.
 -a reclaim gem already exists for any of the extents in the given
  range.
 -a reclaim gem is in the process of starting for any of the extents
  in the given range.
 -starting this reclaim gem would cause the total number of reclaim
  gems on the system to exceed the configured maximum
  (STN_NUM_GC_RECLAIM_SESSIONS).
 -no netldi process is running on the remote machine."

<primitive: 287>
startExtent _validateClass: SmallInteger.
endExtent _validateClass: SmallInteger.
(aHostNameOrNil ~~ nil) ifTrue:[aHostNameOrNil _validateClass: String].
(stoneHostOrNil ~~ nil) ifTrue:[stoneHostOrNil _validateClass: String].
(startExtent < 0) ifTrue:[^startExtent _error: #rtErrArgNotPositive].
(endExtent < 0) ifTrue:[^startExtent _error: #rtErrArgNotPositive].
(endExtent < startExtent) ifTrue:[^endExtent _error: #errArgTooSmall args:{ startExtent } ].
self _primitiveFailed: #startReclaimGemForExtentRange:to:onHost:stoneHost:
     args: { startExtent . endExtent . aHostNameOrNil . stoneHostOrNil }
%

! deleted _abortTransactionAndKeepAttached
! deleted _beginTransactionAndKeepAttached
! deleted _intToOopFiltered: array serial: serialNum status: statusArray


category: 'Error Handling'
classmethod: System
_lastGsErrorNumber

"Return the value of the last error number.  Also clears the error number back to zero."
^self _zeroArgPrim: 47
%
! deleted weakDictionaryAt:
! deleted weakDictionaryAt: key put: value toSurvive: anInt
! deleted weakDictionaryEnableDebugging
! deleted weakDictionaryInit
! deleted weakDictionaryLockedAt: key
! deleted weakDictionaryLockedAt: key put: aBoolean
! deleted weakDictionaryRemainingGcsAt: key
! deleted
! deleted weakDictionaryRemoveKeyAt: key
! deleted weakDictionarySize
! deleted weakDictionaryStatistics
! deleted weakDictionarySurvivedGcsAt: key
! deleted weakDictionaryDumpContents
! deleted weakDictionaryAt: key put: value insertOrUpdateToSurvive: anInt

category: 'Private'
classmethod: System
myPageServerProcessId
"Answer an Integer which is the process ID of the page server
on stone's machine for this session.  
Returns 0 if the session does not have a page server."

^ self _zeroArgPrim: 59 
%

category: 'Private'
classmethod: System
myReadPageServerProcessId
"Answer an Integer which is the process ID of the page server
being used for disk reads.
Returns 0 if the session does not have a page server.
Result may be different from the result of myPageServerProcessId if
the session is using a mid-level cache."

^ self _zeroArgPrim: 108
%

category: 'Online Backup Support'
classmethod: System
suspendCheckpointsForMinutes: anInt
"Suspend all checkpoints for the given number of minutes or until the
 resumeCheckpoints method is executed, which ever occurs first.  anInt
 must be a positive SmallInteger.
 
 Requires the SystemControl privilege.

 It is safe to copy the repository extents for backup purposes while
 checkpoints are suspended.

 Checkpoint suspension is not supported in partial tranlog mode.  This
 method will always return false STN_TRAN_FULL_LOGGING is set to FALSE.  

 Certain operations which require checkpoints are not permitted while
 checkpoints are suspended, such as full backups.

 Calling this method while checkpoints are already suspended has
 the effect of changing the duration of the suspension.

 If a checkpoint is in progress when this method is called, the call
 will block until the current checkpoint completes, at which time
 checkpoints will be suspended.  If any session has made this
 call and is waiting for the current checkpoint to complete, calls
 to this method by other sessions will fail.

 Returns true if checkpoints were successfully suspended.  Returns
 false if checkpoints could not be suspended because the repository
 is in partial log mode or is in restore from backup or log mode."

<primitive: 285>
anInt  _validateClass: SmallInteger.
(anInt < 0)
  ifTrue:[ anInt _error: #errArgTooSmall args:{ 0 } ] .
^self _primitiveFailed: #suspendCheckpointsForMinutes: args: { anInt }
%

category: 'Online Backup Support'
classmethod: System
resumeCheckpoints
"Resumes regular checkpoints if they were previously suspended by
 the System suspendCheckpointsForMinutes: method.
 
 Requires the SystemControl privilege.

 Returns the previous checkpoint state.  Returns true if checkpoints
 were suspended or false if checkpoints were not suspended."

 ^self _zeroArgPrim: 60
%

category: 'Online Backup Support'
classmethod: System
checkpointStatus
"Returns an array of 2 elements.  The first element is a boolean indicating
 if checkpoints are currently suspended.  The second element is an
 Integer indicating the number of seconds before checkpoints will be
 resumed by the stone."
 

 ^self _zeroArgPrim: 61
%

category: 'Historical Event Logging'
classmethod: System

hstLogStatus
"Prints out status of History Logging 
 Categories on stdout"

 ^self _zeroArgPrim: 82
%

category: 'Historical Event Logging'
classmethod: System

hstLogCategoryEnable: categorySymbol
"Enables the recording/printing of historical events 
 belonging to this category."

 ^self hstLogCategory: categorySymbol setup: true
%

category: 'Historical Event Logging'
classmethod: System

hstLogCategoryDisable: categorySymbol
"Disables the recording/printing of historical events 
 belonging to this category."

^self hstLogCategory: categorySymbol setup: false
%

category: 'Historical Event Logging'
classmethod: System

hstLogOpcodeEnable: opcodeSymbol
"Enables the recording/printing of historical events 
 belonging to this opcode."

 ^self hstLogOpcode: opcodeSymbol setup: true
%

category: 'Historical Event Logging'
classmethod: System

hstLogOpcodeDisable: opcodeSymbol
"Disables the recording/printing of historical events 
 belonging to this opcode."

^self hstLogOpcode: opcodeSymbol setup: false
%

category: 'Historical Event Logging'
classmethod: System

hstLogCategory: categorySymbol setup: aBoolean
"Enable/Disable the recording/printing of historical events 
 belonging to this category."

<primitive: 575>
^self _primitiveFailed: #hstLogCategory:setup: 
      args: { categorySymbol . aBoolean }
%

category: 'Historical Event Logging'
classmethod: System

hstLogOpcode: opcodeSymbol setup: aBoolean
"Enable/Disable the recording/printing of historical events 
 belonging to this opcode."

<primitive: 576>
^self _primitiveFailed: #hstLogOpcode:setup: args: { opcodeSymbol . aBoolean }
%

category: 'Historical Event Logging'
classmethod: System

prvLogPrint
"Print out the private historical event log to stdout"

 ^self _zeroArgPrim: 80
%

category: 'Historical Event Logging'
classmethod: System

shrLogPrint
"Print out the private historical event log to stdout"

 ^self _zeroArgPrim: 81
%

category: 'Historical Event Logging'
classmethod: System

prvLogRecordX1: x1 X2: x2 X3: x3

<primitive: 577>
^self _primitiveFailed: #prvLogRecordX1:X2:X3 args: { x1 . x2 . x3 }
%

category: 'Historical Event Logging'
classmethod: System

shrLogRecordX1: x1 X2: x2 X3: x3

<primitive: 578>
^self _primitiveFailed: #shrLogRecordX1:X2:X3 args: { x1 . x2 . x3 }
%

category: 'Private'
classmethod: System
_inProtectedMode

 "Return true if in protected mode"

  ^self _zeroArgPrim: 84
%

category: 'Version Management'
classmethod: System
_generatePassiveSourceFileTo: fileName
"generate a passivated data structure containing all source code for classes
 in Globals. The file generated by this method can be used by 
 System(C)>>_compareGlobalsWith:writeReportTo:generateFileinTo: to generate
 a report and/or filein script to reconcile kernel class differences between
 two versions of the image. If fileName is nil, file will be given a useful
 name like '1.0.0-sourceStrings.obj'.
"
|ver classes outfile outfilename targetDict |
(fileName == nil) ifTrue: [
  ver := self gemVersionReport at: #gsRelease.
  ver := (ver copyReplaceAll: ' ' with: '').
  outfilename := ver, '-sourceStrings.obj'.
] ifFalse: [outfilename := fileName].
classes := Dictionary new.
Globals do: [ :ea | | theArr meths methSrcDict classMethSrcDict |
  (ea isKindOf: Behavior) ifTrue: [  | methsBlk |
    theArr := Array new: 2.
    methSrcDict := Dictionary new.
    classMethSrcDict := Dictionary new.
    theArr at: 1 put: methSrcDict.
    theArr at: 2 put: classMethSrcDict.
    classes at: ea nameForFileout put: theArr.
    methsBlk := [ :mDict |
      mDict keysAndValuesDo: [ :k :aMeth | | envId aName |
        envId := aMeth environmentId .
        aName := envId == 0 ifTrue:[ k ] ifFalse:[ k asString ,':env_', envId asString].
        targetDict at: aName put: aMeth sourceString.
      ].
    ].
    targetDict := methSrcDict .
    ea persistentMethodDictsDo: methsBlk .
    targetDict := classMethSrcDict .
    ea class persistentMethodDictsDo: methsBlk .
  ]
].
outfile := GsFile openWriteOnServer: outfilename.
(outfile == nil) ifTrue: [^'error opening outfile', outfilename].
(PassiveObject passivate: classes toStream: outfile) == nil
    ifTrue: ['error writing outfile', outfilename].
outfile close.
^true
%

category: 'Version Management'
classmethod: System
_compareGlobalsWith: passiveSourceFileName 
writeReportTo: reportFileName 
generateFileinTo: fileinFileName
"returns a string, a report on Smalltalk method differences between this
 version and a previous version. 

 If reportFileName is nil, no report file is written. If reportFile is not
 nil, the report string will be written to a file.

 If fileinFileName is nil, no filein file will be generated. If fileinFileName
 is not nil, a topaz filein script will be generated. 

 See also System(C)>>_generatePassiveSourceFileTo:
"
| prevClasses prevClassesFile report reportFile lf addedMeths changedMeths
  deletedMeths addedClasses deletedClasses fileinFile |

lf := Character lf.
report := String new.
addedMeths := { } .
changedMeths := { } .
deletedMeths := { } .
addedClasses := { } .
deletedClasses := { } .

(fileinFileName == nil) ifFalse: [
  fileinFile := GsFile openWriteOnServer: fileinFileName.
  (fileinFile == nil) ifTrue: [ 
    ^'Can''t open ', fileinFileName, ' for writing.'
  ].
].

prevClassesFile := GsFile openReadOnServer: passiveSourceFileName.
(prevClassesFile == nil) ifTrue: [
  ^'Can''t open ', passiveSourceFileName, ' for reading.'
].

"read in the prevClasses from passivated object file"
prevClasses := (PassiveObject newOnStream: prevClassesFile) activate.
(prevClasses == nil) ifTrue: [
  ^'Failed to activate source dictionaries from ', passiveSourceFileName, '.'
].

"iterate classes in Globals and compare method source strings"
Globals do: [ :ea | 
  (ea isBehavior) ifTrue: [ | meths classMeths className |
    className := ea nameForFileout.
    (prevClasses includesKey: className) ifFalse: [
      addedClasses add: className.
      (fileinFileName == nil) ifFalse: [
        ea fileOutClassOn: fileinFile.
      ].
    ] ifTrue: [ | methsBlk clsMethStr prevDict theCls |

      methsBlk := [ :mDict |
        mDict keysAndValuesDo: [ :selector :aMeth | | envId aName |
          envId := aMeth environmentId .
          aName := envId == 0 ifTrue:[ selector ] ifFalse:[ 
             ':env_', envId asString , ':', selector asString 
          ].
          " check to see if same method exists in prevClasses"
          (prevDict includesKey: aName) ifFalse: [
            addedMeths add: className, '>>', clsMethStr, aName .
            (fileinFileName == nil) ifFalse: [
              fileinFile nextPutAll: (theCls fileOutMethod: selector environmentId: envId ).
            ].
          ] ifTrue: [
            " method exists in both versions, do a diff."
            (aMeth sourceString = (prevDict at: aName)) ifFalse: [
              changedMeths add: className, '>>', clsMethStr, aName.
              (fileinFileName == nil) ifFalse: [
                fileinFile nextPutAll: (theCls fileOutMethod: selector environmentId: envId).
              ].
            ].
          ].
        ].
      ].

      clsMethStr := '' .
      prevDict := (prevClasses at: className) at: 1 .
      theCls := ea .
      theCls persistentMethodDictsDo: methsBlk .

      clsMethStr := '(C)' .
      prevDict := (prevClasses at: className) at: 2 .
      theCls := ea class .
      theCls persistentMethodDictsDo: methsBlk .
    ] 
  ] 
]. "Globals do:"

"Now check to see if there are any deleted methods or classes"
prevClasses keysAndValuesDo: [ :clsNam :v | 
  ((Globals includesKey: clsNam) and: [(Globals at: clsNam) isBehavior]) ifFalse: [
    deletedClasses add: clsNam.
  ] ifTrue: [ | meths clsMethStr theCls envPrefix prefixSize |
    meths := v at: 1 .
    clsMethStr := '' . 
    theCls := Globals at: clsNam .
    envPrefix := ':env_' .
    prefixSize := envPrefix size .
    2 timesRepeat:[
      meths keysDo:[ :aName | | envId namSiz selectr |
        envId := 0 .
        selectr := aName .
        (aName at: 1 equals: envPrefix) ifTrue:[ | endIdx |
          endIdx := aName indexOf: $: startingAt: 2 . 
          envId := Integer fromString:( aName copyFrom: prefixSize + 1 to: endIdx - 1 ).
          selectr := aName copyFrom: endIdx + 1 to: aName size .
        ]. 
        (theCls includesSelector: selectr environmentId: envId) ifFalse:[
          deletedMeths add: clsNam, '>>', clsMethStr, aName.
          (fileinFileName == nil) ifFalse: [
            fileinFile nextPutAll: 'doit', lf, 
	      clsNam , ' removeSelector: #', selectr, ' environmentId:', envId asString ,
		lf, $%, lf.
          ].
        ].
      ].  
      meths := v at: 2 .
      clsMethStr := '(C)' .
      theCls := theCls class .
    ].
  ].
].
fileinFile close.

"put together the report"
report := '====', lf, 'Image change report', lf, '====', lf.
(addedClasses size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Added Classes:', lf, '----', lf.
  (addedClasses sortAscending) do: [ :ea | report := report, ea, lf ].
].
(deletedClasses size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Deleted Classes:', lf, '----', lf.
  (deletedClasses sortAscending) do: [ :ea | report := report, ea, lf ].
].
(addedMeths size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Added Methods:', lf, '----', lf.
  (addedMeths sortAscending) do: [ :ea | report := report, ea, lf ].
].
(deletedMeths size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Deleted Methods', lf, '----', lf.
  (deletedMeths sortAscending) do: [ :ea | report := report, ea, lf ].
].
(changedMeths size ~~ 0) ifTrue: [
  report := report, '----', lf, 'Changed Methods', lf, '----', lf.
  (changedMeths sortAscending) do: [ :ea | report := report, ea, lf ].
].

(reportFileName == nil) ifFalse: [
  reportFile := GsFile openWriteOnServer: reportFileName.
  (reportFile == nil) ifFalse: [
    reportFile nextPutAll: report; close.
  ].
].
^report
%

category: 'Hidden Set Support'
classmethod: System
_loadOrStoreHiddenSet: hiddenSetSpecifier toOrFromFile: aString opCode: anOpCode

"Performs hidden set to bitmap file operations.  If anOpCode is zero, read the
 given bitmap file into the given hidden set.  The file must exist and be readable
 by the gem process.   Also hiddenSetSpecifier must be the index of a hidden set
 that is modifiable by customers.  This check is bypassed if the session is logged in
 as SystemUser.  Extreme caution should be used in this case.  Be sure to call
 System>>_hiddenSetReinit: method first to ensure the hidden is empty.

 If anOpCode is one, then the given hidden set is written to the specified 
 bitmap file.  The file must not already exist and the path to the file must
 be in a directory which is writable by the process.  Any valid hidden set
 specifier may be used in this case.

 If anOpCode is two, then the file is assumed to be a sorted bitmap file
 of objects produced by one of the Repository>>listInstancesInPageOrder:
 methods.  The objects are loaded into the given hidden set, but the ordering
 by page ID is lost because hidden sets are always ordered by object ID.
 No validation of the object IDs is done by this load.

 Returns true if successful, false if an error occurred.  Additional error 
 information may be written stderr for the process (either the terminal
 or the gem log file)."
 
<primitive: 356>
hiddenSetSpecifier _validateClass: SmallInteger.
aString _validateClass: String.
anOpCode _validateClass: SmallInteger.
self _primitiveFailed: #_loadOrStoreHiddenSet:toOrFromFile:opCode:
     args: { hiddenSetSpecifier . aString . anOpCode }
%

category: 'Hidden Set Support'
classmethod: System
writeHiddenSet: hiddenSetSpecifier toFile: aString
"Refer to comments in the method
 System>>_loadOrStoreHiddenSet: hiddenSetSpecifier
           toOrFromFile: aString
           opCode: anOpCode"

^self _loadOrStoreHiddenSet: hiddenSetSpecifier toOrFromFile: aString opCode: 1
%

category: 'Hidden Set Support'
classmethod: System
readHiddenSet: hiddenSetSpecifier fromFile: aString
"Refer to comments in the method
 System>>_loadOrStoreHiddenSet: hiddenSetSpecifier
           toOrFromFile: aString
           opCode: anOpCode"

^self _loadOrStoreHiddenSet: hiddenSetSpecifier toOrFromFile: aString opCode: 0
%

category: 'Hidden Set Support'
classmethod: System
readHiddenSet: hiddenSetSpecifier fromSortedFile: aString
"Load a file assumed to be a sorted bitmap file of objects produced by one of
 the Repository>>listInstancesInPageOrder:  methods.  The objects are loaded
 into the given hidden set, but the ordering by page ID is lost because hidden
 sets are always ordered by object ID. 

 No validation of the object IDs is done by this method.

 Returns true if successful, false if an error occurred.  More error information
 may also be written to stdout."

^self _loadOrStoreHiddenSet: hiddenSetSpecifier toOrFromFile: aString opCode: 2
%

category: 'Hidden Set Support'
classmethod: System
_performSetArithmeticOnHiddenSets: first and: second storeResultsInto: third opCode: anOpCode

"Perform a set arithmetic operation on the hidden sets with the specifiers
 given by first and second and possiblly store the results into third.
 The behavior of this primitive is as follows:

 opCode 	Operation	 Result stored in third
 =================================================================
 0		Add		 Not used, third expected to be 0.
 1		Subtract	 Not used, third expected to be 0.
 2		Union 		 result of the union
 3		Difference	 result of difference
 4		Clear from start Not used
 3		Clear from end   Not used
 =================================================================

 The set arithmetic operations are described as follows:

 Add:	For each element in first, add that element to second.  Returns the number
        of elements added to second which were not already present.

 Subtract: For each element in first, remove that element from second.  Returns
           the number of elements removed from second.

 Intersection: Add elements to third which appear in both first and second.
               Returns the size of the intersection contained in third.

 Difference:   Add elements to third which appear in first but not in second.
               Returns the number of elements stored in third.

 Clear from start - remove the first count elements from the hidden set.  Elements
                    are removed in OOP order, from lowest to highest.

 Clear from end - remove the last count elements from the hidden set.  Elements
                    are removed in OOP order, from higest to lowest.
"

<primitive: 357>
first _validateClass: SmallInteger.
second _validateClass: SmallInteger.
third _validateClass: SmallInteger.
anOpCode _validateClass: SmallInteger.
self _primitiveFailed: 
     #_performSetArithmeticOnHiddenSets:and:storeResultsInto:opCode:
     args: { first . second . third . anOpCode }
%

category: 'Hidden Set Support'
classmethod: System
addHiddenSet: first to: second
"Add all the objects in hidden set first to hidden set second.  Returns the 
 number of objects added to second hidden set that were not already present.
 User must have permission to modify the second hidden set or be SystemUser."
 
^self _performSetArithmeticOnHiddenSets: first and: second storeResultsInto: 0 opCode: 0.
%

category: 'Hidden Set Support'
classmethod: System
removeContentsOfHiddenSet: first from: second
"Remove all objects in the first hidden  from the second hidden set second.  Returns the 
 number of objects removed from second hidden set.  User must have permission to modify
 the second hidden set or be SystemUser."
 
^self _performSetArithmeticOnHiddenSets: first and: second storeResultsInto: 0 opCode: 1.
%

category: 'Hidden Set Support'
classmethod: System
computeUnionOfHiddenSet: first and: second into: third
"Add every object to the third hidden set that is in both the first and second hidden
 sets.  Returns the number of elements added to the third hidden set.  User must have
 permission to modify the third hidden set or be SystemUser."
 
^self _performSetArithmeticOnHiddenSets: first and: second storeResultsInto: third opCode: 2.
%

category: 'Hidden Set Support'
classmethod: System
computeDifferenceOfHiddenSet: first and: second into: third
"Add every object to the third hidden set that is in the first and but not the second hidden
 set.  Returns the number of elements added to the third hidden set.  User must have
 permission to modify the third hidden set or be SystemUser."
 
^self _performSetArithmeticOnHiddenSets: first and: second storeResultsInto: third opCode: 3.
%

category: 'Hidden Set Support'
classmethod: System
removeFirst: count objectsFromHiddenSet: hiddenSetSpecifier
"Remove the first count objects from the given hidden set.  Objects are removed from
 the beginning, going from lowest to highest object ID.   
 
 Returns the number of objects removed from the hidden set.  User must have permission
 to modify the hidden set or be SystemUser."
 
^self _performSetArithmeticOnHiddenSets: hiddenSetSpecifier and: count storeResultsInto: 0 opCode: 4.
%

category: 'Hidden Set Support'
classmethod: System
truncateHiddenSet: hiddenSetSpecifier toSize: newSize
"Truncate the given hidden set by removing objects from the end of the set until it reaches a
 size of newSize.  Objects are removed in order, going from highest to lowest object ID.   
 
 Returns the number of objects removed from the hidden set.  User must have permission
 to modify the hidden set or be SystemUser."
 
^self _performSetArithmeticOnHiddenSets: hiddenSetSpecifier and: newSize storeResultsInto: 0 opCode: 5.
%

category: 'Private'
classmethod: System
_comClearOpcodesEmitted

^ self _zeroArgPrim: 92
%
category: 'Private'
classmethod: System
_comPrintOpcodesEmitted

^ self _zeroArgPrim: 87
%
category: 'Private'
classmethod: System
_comPrintOpcodesNotEmitted

^ self _zeroArgPrim: 88
%

category: 'Private'
classmethod: System
_printCHeap

"in a slow or fastdebug VM , calls UtlHeapPrint()"
^ self _zeroArgPrim: 89
%

category: 'Private'
classmethod: System
_locale
"Returns a Locale object setup according to the session's Locale configuration."

^ self _zeroArgPrim: 90.
%

! fix 36394 / 36587
! fix 37843/37871: modify category argument to be categorySym
category: 'Private'
classmethod: System
_setCategory: categorySym locale: localeStr

" Set Locale information.  See man page for setlocale( ) for details.

Valid category types include:

   LC_CTYPE        /* locale's ctype handline */
   LC_NUMERIC      /* locale's decimal handling */
   LC_TIME         /* locale's time handling */
   LC_COLLATE      /* locale's collation data */
   LC_MONETARY     /* locale's monetary handling */
   LC_MESSAGES     /* locale's messages handling */
   LC_ALL          /* name of locale's category name */

Applications should not call this directly -- instead use:

  Locale setCategory: <sym> locale: <str>' 

"

< primitive: 661>
categorySym _validateClass: Symbol.
localeStr _validateClass: String.
self _primitiveFailed: #_setCategory:locale: args: { categorySym . localeStr }
%

category: 'Private'
classmethod: System
_maxClusterId 

" return the maximum clusterId supported vy the VM."
^ self _zeroArgPrim: 91
%

category: 'Time'
classmethod: System
timeNs

"Returns a SmallInteger representing the current high-resolution
 real time in nanoseconds since some arbitrary time in the past.  The 
 result is not correlated in any way with the time of day."
 
^ self _zeroArgPrim: 95
%
category: 'Private'
classmethod: System
_removePageFromCache: aPageId

"aPageId must be an Integer >= 0 and <= 549755813887  .
 If the specified page is in gem's shared cache, 
 prints the page header and attempts to remove the page
 from the shared cache.   If the gem is running on stone's machine
 will also print the page header from the state of the page on disk.

 Returns true if successfully removed, false otherwise.
 The result will be false if the page was dirty, locked or pinned."
  
<primitive: 590>
aPageId  _validateClass: Integer.
(aPageId < 0 or:[aPageId > 549755813887 ]) ifTrue:[
  aPageId _error: #rtErrArgOutOfRange args:{ 0 . 549755813887 }
] .
^self _primitiveFailed: #_removePageFromCache: args: { aPageId }
%

category: 'Reduced Conflict Support'
classmethod: System
_rcLockObject

^ self rcValueCacheAt: #rcLockObj otherwise: (Globals at: #GemStoneRCLock otherwise: nil )
%

category: 'System Control'
classmethod: System
_startGcCacheWarmer

"Obsolete in GemStone64 2.4.x & 3.x"

self deprecated: 'System class>>_startGcCacheWarmer is deprecated in GemStone64 2.4.x & 3.x.
See System Administration Guide for warming cache on startup.'

%

category: 'Private'
classmethod: System
_bypassReadAuth

"For use by indexing and query subsystem only.

 Subsequent object read-faults will bypass
 read-authorization checks to allow indexed queries and index maintenance
 to execute.  This allows indexing code to execute comparisons on keys
 in btree nodes without regard to read-authorization of the keys.
 Query results will never contain objects for which the current user
 is not read-authorized.

 Has no effect if current user is SystemUser.

 Indexing code should use this method in combination with an ensure: block
 which contains   _exitBypassReadAuth

 The bypass is shutoff automatically (equivalent to _exitBypassReadAuth)
 upon return from a userAction, from a recurFromOm, , upon return to the GCI,
 or upon generation of an error which would unwind the VM C stack.
"

<protected primitive: 595>
self _primitiveFailed: #_bypassReadAuth
%

classmethod: System
_exitBypassReadAuth

"For use by indexing and query subsystem only.

 Any in-memory objects which were faulted in while bypass was enabled by 
 a preceeding _bypassReadAuth, and for which a read-authorization error would
 have otherwise occurred, will be marked not-valid, so that subsequent
 access will get a read-authorization error.

 Has no effect if current user is SystemUser."

<primitive: 692>
self _primitiveFailed: #_exitBypassReadAuth
%


category: 'Time'
classmethod: System
timeGmt2005

"Returns a SmallInteger or a LargeInteger, the time since January 1,
 2005, in seconds.  It will return a SmallInteger until January 5, 2022.
 The time is computed from the clock of the machine on which the session is
 running, using the offset from the clock on the Stone's (GemStone repository
 monitor process) machine which is cached in the session at login."

^ self _timeGmt2005: false
%

category: 'Time'
classmethod: System
_timeGmt2005: aBoolean

"Returns a SmallInteger or a LargeInteger, the time since January 1,
 2005, in seconds.  It will return a SmallInteger until January 5, 2022.
 aBoolean is true, call Stone and update our offset from Stone's clock,
 otherwise compute the current time locally using the offset from Stone's time
 that was cached in the session at login."

<primitive: 598>
^ self _primitiveFailed: #_timeGmt2005: args: { aBoolean }
%

category: 'Time'
classmethod: System
_timeUs

"Returns a SmallInteger between 0 and 999,999 representing the current number
 of microseconds that have passed within the current second."

^ self _zeroArgPrim: 103 .
%

! Request 36492
category: 'Session Control'
classmethod: System
logout

"Immediately logs out the current session and generates a fatal error
 (GS_ERR_GEM_NORMAL_SHUTDOWN).  This method does not return."

self _zeroArgPrim: 104
%

! deleted executeAsyncExceptionBlock:withArgs:

category: 'Shared Cache Management'
classmethod: System
_remoteCachesListStartingAt: anOffset midLevelOnly: aBoolean

"Return a portion of the result for remoteCachesList,
 starting at anOffset in stone's list.    anOffset is zero-based.

 Result size is zero if anOffset is beyond end of stone's list, 
 otherwise last element of result is a SmallInteger, the nextEnumIdx.
 If nextEnumIdx is < 0 there are no more elements in stone's list,
 otherwise pass nextEnumIdx as anOffset argument in next call to this 
 primitive to continue the enumeration. 

 Result size is limited by size of gem to stone communication buffers."

<primitive: 665>
anOffset _validateClass: SmallInteger .
anOffset < 0 ifTrue:[ anOffset _error: #errArgTooSmall args:{ 0 } ].
aBoolean _validateClass: Boolean .
^ self _primitiveFailed: #_remoteCachesListStartingAt:midLevelOnly:
       args: { anOffset . aBoolean }
%

category: 'Shared Cache Management'
classmethod: System
_remoteCachesList: forMidLevel

"Return an Array describing shared caches, per remoteCachesList."

| idx result batch |
idx := 0 .
result := { }  .
[ idx >= 0 ] whileTrue:[ | nextIdx batSize |
  batch := self _remoteCachesListStartingAt: idx midLevelOnly: forMidLevel .
  batSize := batch size .
  batSize > 0 ifTrue:[
    nextIdx := batch at: batSize .
    batch size: batSize - 1.
    result addAll: batch .
    idx := nextIdx . 
  ] ifFalse:[
    idx := -1 . "done"
  ].
].
^ result
%

category: 'Shared Cache Management'
classmethod: System
_formatCacheReport: anArray

"given an Array produced by remoteCachesList or midLevelCachesList
 return a String containing one line per element of the list."

| rpt buf LF formatBlk header |
rpt := String new .
buf := String new: 50 .
LF := Character lf .

formatBlk := [:aCache :aRpt :aBuf | |kind kindStr |
  aBuf size: 0 ; addAll: ((aCache at: 1) width: 30) . "hostname"
  aRpt addAll: aBuf .
  kind := aCache at: 2 .
  kind == true ifTrue:[ 
    kindStr := 'mid'
  ] ifFalse:[ 
    kind == false ifTrue:[ kindStr := 'remote' ]
                 ifFalse:[ kindStr := 'kind' . "header line"]
  ].
  kindStr := String withAll: kindStr .
  aBuf size: 0 ; addAll: (kindStr width: 10)  . "kind"
  aRpt addAll: aBuf .
  aBuf size: 0 ; addAll: ((aCache at: 3) asString width: -11 ). "sessions"
  aRpt addAll: aBuf .
  aBuf size: 0 ; addAll: ((aCache at: 6) asString width: -13 ). "sessions using as midCache"
  aRpt addAll: aBuf .
  aBuf size: 0 ; addAll: ((aCache at: 4) asString width: -10 ). "maxsessions"
  aRpt addAll: aBuf .
  aBuf size: 0 ; addAll: ((aCache at: 5) asString width: -10 ). "sizeKb"
  aRpt addAll: aBuf ; add: LF .
].  
  
header := { 'hostname' . 'kind' . 'sessions' . 'mid-sess''s' . 'maxSess' . 'sizeKB' }.
1 to: header size do:[:k | header at:k put:(String withAll:(header at:k)) ].

formatBlk value: header value: rpt value: buf  . "header line"

1 to: anArray size do:[:k || aCache |
  aCache := anArray at: k .
  formatBlk value: aCache value: rpt value: buf .
].
^ rpt
%

category: 'Shared Cache Management'
classmethod: System
remoteCachesList

"Return an Array describing all shared caches that the
 stone process is managing, not including the cache on
 the stone machine.

 Result contains one element per cache. Each
 element of result is a 6 element Array containing  
   hostName, 
   a Boolean - true if cache was created as a mid-level cache
   a SmallInteger - total number of sessions connected to the cache  
   a SmallInteger - max number of sessions for the cache  
   a SmallInteger - size of cache in KB 
   a SmallInteger - number of sessions using cache as a mid-level cache "


^ self _remoteCachesList: false
%

category: 'Shared Cache Management'
classmethod: System
midLevelCachesList

"Return an Array as described for remoteCachesList, 
 filtered to include only the mid-level caches. "

^ self _remoteCachesList: true
%

category: 'Shared Cache Management'
classmethod: System
remoteCachesReport

"Return the result of remoteCachesList formatted as a string,
 one line per cache."

^ self _formatCacheReport: self remoteCachesList

%

category: 'Shared Cache Management'
classmethod: System
midLevelCachesReport

"Return the result of midLevelCachesList formatted as a string,
 one line per cache."

^ self _formatCacheReport: self midLevelCachesList
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheConnect: hostName

"Attempt to connect to mid-level cache on the
 specified host, if cache already exists.

 Returns true if connection succeeded, or a String
 describing the failure.

 The session's gem process must be on a machine different from
 the machine running the stone process in order for this to succeed."

^ self midLevelCacheConnect: hostName cacheSizeKB:-1  maxSessions:-1
%

category: 'Shared Cache Management'
classmethod: System
midLevelCacheConnect: hostName cacheSizeKB: aSize maxSessions: nSess 

"If a mid-level cache does not exist on a specified host,
 and aSize > 0, attempt to start one.
 Then attempt to connect to the mid-level cache.
 Connection may fail if cache could not be started or
 if the cache could not accept another session .

 The session's gem process must be on a machine different from
 the machine running the stone process in order for this to succeed.

 If session A's gem process is running on a machine where the 
 shared cache is in use as a mid-level cache by gem processes
 on other machines, then session A is cannot use this method;
 i.e. cross-connected mid-level caches are not allowed .

 hostName must be a String, else an error is generated.

 If aSize <= 0, nSess is ignored , 
 otherwise nSess must be >= 1 and aSize must be >= 512 .
 The nSess value may be reduced by stone to the
 number of sessions stone is configured and licensed for. 

 When creating a cache
   aSize is cache size in K bytes ,
   nSess is maximum number of sessions ,

 The spinLockCount, targetFreeFrameCount, numFreeFrameServers,
 and numSharedCounters for creating a cache come from gem's config file
 just as for a cache the gem would have created as a side effect
 of login .
  
 Returns true if connection succeeded, or a String
 describing the failure."

<primitive: 664 >
hostName  _validateClass: String .
aSize _validateClass: SmallInteger .
aSize > 0 ifTrue:[
  nSess _validateClass: SmallInteger .
  (nSess < 1 or:[ nSess > 10000]) ifTrue:[ 
     nSess _error: #rtErrArgOutOfRange args:{ 1 . 10000 } ].
].
hostName size > 2000 ifTrue:[ hostName _error: #errArgTooLarge args:{ 2000 } ].
^ self _primitiveFailed: #midLevelCacheConnect:cacheSizeKB:maxSessions:
       args: { hostName . aSize . nSess }
%

category: 'Transaction Control'
classmethod: System
inContinueTransaction

"Returns a boolean indicating if the current transaction has been 
 continued.  Answers true if System>>continueTransaction has been
 executed and no abort or successful commit has been performed.
 Also answer true if a commit was attempted and failed and no
 abort has been executed.  Otherwise returns false."

^ self _zeroArgPrim: 105
%

category: 'Transaction Control'
classmethod: System
dirtyListId

"Return a SmallInteger identifying the currently active dirtyList"

^ self _zeroArgPrim:28 
%

category: 'Transaction Control'
classmethod: System
setDirtyList: listId

"Change the currently active dirtyList to be the one specified by
 listId .   A subsequent modification to a committed object, that
 is the first modification the object in this transaction,
 will add the object to the dirtyList specified by listId . 

 listId may be 0 or 1 .
 Returns the listId of the previously active dirtyList.
"

^ self _dirtyListOp: 0 id: listId 
%

category: 'Private'
classmethod: System
_dirtyListOp: opcode id: listId

"execute an operation on one of the object manager dirty lists
 opcode 0  setDirtyList  . listId may be 0 or 1 .
 opcode 1  rollbackDirtyList . listId may be -1 (current), 0, 1, or 2 .
                current means the list last specified by a setDirtyList .
 opcode 2  enumerateDirtyList.  listId may be -2 (all) -1(current) , 0, 1, or 2

  listId 2 is reserved for notTranlogged objects .
 "

<primitive: 276>
self _primitiveFailed: #_dirtyListOp:id: args: { opcode . listId } .
self _uncontinuableError
%


category: 'Transaction Control'
classmethod: System
rollbackDirtyList: listId

"For each object in the dirtyList specified by listId,
 rollback the object to its state as of the beginning of the
 current transaction.
 listId = -1  specifies the currently active dirtyList .
 listId = 2   specifies the list of dirty notTranlogged objects.
 May not be used after a failed commit (i.e. not for use during
 Rc replay operations).

 Returns the number of objects rolled back."

^ self _dirtyListOp: 1 id: listId 
%

category: 'Transaction Control'
classmethod: System
enumerateDirtyList: listId

"Returns an Array which is the result of enumerating
 the dirtyList(s) specified by listId.
 listId = -1  specifies the currently active dirtyList .
 listId = -2  specifies all dirtyLists.
 listId = 2   specifies the list of dirty notTranlogged objects.
"

^ self _dirtyListOp: 2 id: listId 
%


category: 'Private' 
classmethod: System
_writtenObjects

"Returns an Array which is the result of enumerating
 all of the object manager's lists of dirty committed objects ."

^ self _dirtyListOp: 2 id: -2
%
category: 'Runtime Configuration Access'
classmethod: System
gemIsBigEndian

" Returns true if the gem process is running
  on a machine using big endian byte ordering "

^ self _zeroArgPrim: 106

%
category: 'Runtime Configuration Access'
classmethod: System
stoneIsBigEndian

" Returns true if the stone process is running
  on a machine using big endian byte ordering "

^ self _zeroArgPrim: 107

%
category: 'Session Control'
classmethod: System
commitRecordPageForSessionId: aSessionId

"Return the page ID of the commit record referenced by the given session ID.
 The result will usually be a SmallInteger but could also be a 
 LargeInteger.
 
 Returns -1 if the session does not exist or if it does not currently reference a
 commit record (such as after a Lost OT root error).

 To execute this method for any session other than your current session, you
 must have the SessionAccess privilege."

<primitive: 663>
aSessionId _validateClass: SmallInteger .
^ self _primitiveFailed: #commitRecordPageForSessionId: args: { aSessionId } .
%
category: 'Session Control'
classmethod: System
latestCommitRecordPageId

"Return the page ID of the most recent (newest) commit record on the system.
 Returns -1 if an error occurs."

^ self _zeroArgPrim: 108
%

category: 'Session Control'
classMethod: System
sessionsReferencingOldestCrInTransaction

"Returns an Array containing the sessionIds of the sessions that are in
 transaction and currently reference the oldest commit record."       
^self _zeroArgPrim: 109
%

category: 'Session Control'
classMethod: System
sessionsReferencingOldestCrNotInTransaction

"Returns an Array containing the sessionIds of the sessions that are not in
 transaction and currently reference the oldest commit record."       
^self _zeroArgPrim: 110
%

! 38848
category: 'Session Control'
classMethod: System
disableCommits

"Disables further commits by the session.  See the method #disableCommitsWithReason:
 for more information."

^ self disableCommitsWithReason: nil
%

category: 'Session Control'
classMethod: System
disableCommitsWithReason: aString

"Prevents this session from performing any further commits.  Attempting to
 commit a transaction after this method is successfully executed will raise
 an #rtErrCommitDisallowed error and will include 'aString' as the reason.

 This method disables commits only for the current session, not permanently.
 Commits may be performed if the session logs out and logs in again.  Use the
 UserProfile>>disableCommits method to permanently disable commits for a
 UserProfile.

 aString must be an instance or subclass of String, or nil.  If aString is nil,
 a default reason of 'Commits disabled by user' will be used.

 Care should be taken when using this method.  If the session has any uncommitted
 changes, those changes will be lost after this method is executed.  Use the
 #needsCommit method to determine if uncommitted changes exist.

 This method returns true on success.  A return of false indicates commits were
 previously disabled by the user or by the system."

^ self _disableCommitsWithReason: aString untilAbort: false
%

category: 'Session Control'
classMethod: System
disableCommitsUntilAbortWithReason: aString

"Prevents this session from performing any further commits until the session
 performs an abort.  Attempting to commit a transaction after this method is
 successfully executed before aborting will raise an #rtErrCommitDisallowed
 error and will include 'aString' as the reason.

 aString must be an instance or subclass of String, or nil.  If aString is nil,
 a default reason of 'Commits disabled by user' will be used.

 Care should be taken when using this method.  If the session has any uncommitted
 changes, those changes will be lost after this method is executed.  Use the
 #needsCommit method to determine if uncommitted changes exist.

 This method returns true on success.  A return of false indicates commits were
 previously disabled by the user or by the system."

^ self _disableCommitsWithReason: aString untilAbort: true
%
category: 'Private'
classMethod: System
_disableCommitsWithReason: aString untilAbort: aBool

"Private - this method should not be called directly.  Call the #disableCommitsWithReason:
 method instead of this one."

<primitive: 701>
aBool _validateClass: Boolean .
aString == nil
  ifFalse:[ aString _validateClass: String].
self _primitiveFailed: #_disableCommitsWithReason:untilAbort:
     args: { aString . aBool }
%

category: 'Transaction Control'
classMethod: System
commitsDisabledUntilLogout

"Answers true if further commits have been disabled for the session.
 Otherwise answers false. 
 
 It is possible the session may still not be able to commit when this
 method returns true.  Possbile reasons are the session is not in transaction,
 commits are disabled at the UserProfile level, or commits have been disabled
 only until the next abort.

 Use the #sessionCanCommit method to definitively determine if the current session
 can commit."

^self _zeroArgPrim: 112.
%
 
category: 'Transaction Control'
classMethod: System
commitsDisabledUntilAbort 
 
"Answers true if further commits have been disabled until the session aborts.
 Otherwise answers false.
 
 It is possible the session may still not be able to commit when this
 method returns true.  Possbile reasons are the session is not in transaction,
 commits are disabled at the UserProfile level, or commits have been disabled
 until the session logs out. 
 
 Use the #sessionCanCommit method to definitively determine if the current session
 can commit."
 
^self _zeroArgPrim: 113.
% 
 
category: 'Transaction Control'
classMethod: System
sessionCanCommit

"Answers true if the session may commit the current transaction (if any).
 Otherwise answers false."
 
"Must be in a tranaction to commit."
self inTransaction
  ifFalse:[ ^ false].
self commitsDisabledUntilLogout 
  ifTrue:[ ^ false]. 
self commitsDisabledUntilAbort
  ifTrue:[^ false].
self myUserProfile isReadOnly 
  ifTrue:[^ false]. "This user can never commit" 
((SystemRepository _restoreStatusInfo at: 2) == nil)
  ifFalse:[^ false]. "restore from log or backup is active"
 
^true
% 
category: 'Session Control'
classmethod:
continuationsEnabled

"Returns true if Smalltalk (i.e. Seaside style) continuations are enabled
 for this session.
 Continuations are enabled by default at the start of a new session."

^ self _zeroArgPrim: 114
%

category: 'Persistent Counters'
classMethod: System
_updateSharedCounterAt: index by: amount withOpCode: code

"Persistent shared counters provide a means for multiple sessions
 to share common integer values.  There are 128 persistent shared
 counters, numbered from 1 to 128 (the index of the first counter is 1).

 Each update to a persistent shared counter causes a roundtrip 
 to the stone process.  However reading the value of a counter is
 handled by the gem (and its page server, if any) and does not
 cause a roundtrip to the stone.

 Persistent shared counters are globally visible to all sessions on all
 shared page caches.
 
 Persistent shared counters hold 64 bit values and may be set to any
 signed 64 bit integer value.  No limit checks are done when
 incrementing or decrementing a counter.  Attempts to increment/decrement
 the counter above/below the minimum/maximum value of a signed
 64-bit integer will cause the counter to 'roll over'.
 
 Persistent shared counters are independent of database transactions.
 Updates to counters are visible immediately and aborts have no
 effect on them.
 
 The values of all persistent shared counters are written to the 
 primary database extent at checkpoint time.  Updates between
 checkpoints are written to the transaction log by the stone.  
 Therefore the state of the persistent shared counters is
 recoverable after a crash, restore from backup, and restore from
 transaction logs.

 Persistent shared counter performance is affected by the stone
 configuration option STN_COMMITS_ASYNC.  Setting this option
 to TRUE will result in faster update performance because
 the stone will respond to update requests after the tranlog
 write has been queued but before it completes.  Operating
 in this mode leaves a small chance of losing data should the
 stone or host machine crash after the tranlog write was queued
 but before it completes.  If this value is set to FALSE, the
 stone will only respond to update requests after the tranlog
 write has completed."

<primitive: 669>
index _validateClass: SmallInteger .
amount _validateClass: Integer .
code _validateClass: SmallInteger .
(index < 1 or:[ index > 128 ])
  ifTrue:[ ^index _error: #rtErrArgOutOfRange args:{ 1 . 128 }] .
(code < 0 or:[ code > 1])
  ifFalse:[ ^code _error: #rtErrArgOutOfRange args:[0,1] ] .
(amount < -9223372036854775808 or:[amount > 9223372036854775807])
  ifTrue:[ ^amount _error: #rtErrArgOutOfRange args:{ -9223372036854775808 . 9223372036854775807 }] .
^self _primitiveFailed: #_updateSharedCounterAt:by:withOpCode:
      args: { index . amount . code }
%

category: 'Persistent Counters'
classMethod: System
persistentCounterAt: index incrementBy: amount

"Increments the persistent shared counter at index by the specified
 amount.  Amount must be a SmallInteger or LargeInteger.
 For a LargeInteger,  amount must be representable as a signed 
 64-bit integer (between -9223372036854775808 and 9223372036854775807).

 Returns the new value of the counter after the increment.

 See the method: 
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: amount withOpCode: 0
%

category: 'Persistent Counters'
classMethod: System
persistentCounterAt: index decrementBy: amount

"Decrements the persistent shared counter at index by the specified
 amount. Amount must be a SmallInteger or LargeInteger.
 For a LargeInteger,  amount must be representable as a signed 
 64-bit integer (between -9223372036854775808 and 9223372036854775807).

 Returns the new value of the counter after the decrement.

 See the method: 
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: amount negated withOpCode: 0
%
 
category: 'Persistent Counters'
classMethod: System
persistentCounterAt: index put: value

"Sets the persistent shared counter at index to the specified
 value.  value must be a SmallInteger or LargeInteger.
 For a LargeInteger,  amount must be representable as a signed 
 64-bit integer (between -9223372036854775808 and 9223372036854775807).

 Returns value, the new value of the counter.

 See the method: 
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: value withOpCode: 1
%
 
category: 'Persistent Counters'
classMethod: System
persistentCounterAt: index

"Returns the value of the persistent shared counter at index, which
 may be a SmallInteger or LargeInteger.

 See the method: 
   System>>_updateSharedCounterAt: index by: amount withOpCode: code
 for more information on persistent shared counters."

^self _updateSharedCounterAt: index by: 0 withOpCode: 0
%


category: 'Host System Access'
classmethod: System
fetchSystemStatNames

"On systems that support it, returns an array of Strings which describe
 the statistics returned by the #fetchSystemStats method.  The length
 of the result array is host technology dependent.

 Returns nil if the host system does not support system statistics."

^ self _zeroArgPrim: 117
%

category: 'Host System Access'
classmethod: System
fetchSystemStats

"On systems that support it, returns an array of Numbers which describe
 the statistics returned by the #fetchSystemStats method.  The length
 of the result array is host technology dependent.  A description of
 each element in the array can be obtained using the #fetchSystemStatNames
 method.

 While most elements in the result array will be SmallIntegers, the
 result may also contain other types of Numbers such as SmallDoubles, 
 Floats, LargeInteger, etc.

 Returns nil if the host system does not support system statistics."

^ self _zeroArgPrim: 118
%

category: 'Garbage Collection Management'
classmethod: System
gcConfigurationReport

"Returns a string describing the current garbage collection settings.  
 Sender must have permission to read GcUsers UserGlobals."

| gcug keys result lf |

result := String new.
lf := Character lf.
result addAll: 'Reclaim and Admin GC gem settings:' ; add: lf.
gcug := ((AllUsers userWithId: 'GcUser') resolveSymbol: #UserGlobals) value.
(keys := gcug keys) 
	remove: #NativeLanguage ifAbsent: [] ; 
	remove: #UserGlobals ; 
	remove: #GcUser ; 
	remove: #defaultValues.

keys do:[:eachSymbol| | val |
  val := gcug at: eachSymbol.
  result add: $#; addAll: eachSymbol asString; addAll: ' -> '; addAll: val asString; add: lf.
].
result add: lf; addAll: 'Stone GC settings:' ; add: lf.
result add: $# ; add: 'StnEpochGcEnabled'; addAll: ' -> ' .
(System stoneConfigurationAt: #StnEpochGcEnabled) == 1
  ifTrue:[result addAll: 'true']
  ifFalse:[result addAll: 'false'].
result add: lf.

^result
%

category: 'Host System Access'
classmethod: System
_hostTimes
" return a 4 element Array holding the elements of the struct tms result
  from   
      clock_t times(struct tms *buffer)

  The Array elements are tms_utime, tms_stime, tms_cutime, tms_cstime ,
  and each element ia s Float in units of seconds.

  tms_utime is the CPU time  used  while  executing
  instructions in the user space of the calling process.

  tms_stime is the CPU time used by the  system  on
  behalf of the calling process.

  tms_cutime is the sum of the  tms_utime  and  the
  tms_cutime of the child processes.

  tms_cstime is the sum of the  tms_stime  and  the
  tms_cstime of the child processes.
"
^ self _zeroArgPrim: 119
%

category: 'Private'
classMethod: System
_cleanupZombieSession: sessionId

"Requests stone forcibly remove a stuck session that is partially logged out.
 Only SystemUser may execute this method.
 Returns true if the operation was successful, false if not."

<primitive: 774>

sessionId _validateClass: SmallInteger .
(sessionId < 1)
  ifTrue:[ sessionId _error: #errArgTooSmall args:{ 1 } ].
self _primitiveFailed: #_cleanupZombieSession: args: { sessionId }
%

category: 'Host System Access'
classmethod: System
unixUserIdExists: aString

"Returns true or false indicating whether or not the UNIX user
 ID representing by aString exists on the host system."

<primitive: 820>
aString _validateClass: String .
self _primitiveFailed: #unixUserIdExists: args: { aString }
%

category: 'Session Control'
classmethod: System
disableStoneGemTimeout

"Prevents the current session from being killed due to inactivity 
 (lack of communication with the stone).

 Use the #enableStoneGemTimeout method to re-enable the stone gem timeout.

 Returns true if the action was successful or false if the stone gem timeout
 was previously disabled for this session.

 Refer to the STN_GEM_TIMEOUT configuration parameter for more information."

^ self _zeroArgPrim: 120
%

category: 'Session Control'
classmethod: System
enableStoneGemTimeout

"Re-enables the stone gem timeout for the current session after it was previously
 disabled by the #disableStoneGemTimeout method.

 Returns true if the action was successful.  Returns false if the stone gem timeout
 was already re-enabled or if it was not disabled for this session.

 Refer to the STN_GEM_TIMEOUT configuration parameter for more information."

^ self _zeroArgPrim: 121
%

category: 'Cache Statistics'
classmethod: System
myCacheStatistics

"Returns the cache statistics for the current session."

^ self cacheStatistics: self myCacheProcessSlot
%

! Request 40060
category: 'Session Control'
classmethod: System
lastTransactionTimeForSessionWithId: aSessionId

"Returns a SmallInteger representing the time of the session's most recent
 beginTransaction, commitTransaction, or abortTransaction (from System timeGmt).

 Equivalent to executing:
   (System descriptionOfSession: aSessionId) at: 5

 Returns -1 if the session with aSessionId could not be found."

<primitive: 834>
aSessionId _validateClass: SmallInteger .
^self _primitiveFailed: #lastTransactionTimeForSessionWithId: 
      args: { aSessionId }
%

category: 'Transaction Control'
classMethod: System
millisecondsSinceTransactionBoundary

"Answer a SmallInteger representing the number of milliseconds since this 
 session updated its view of the repository by executing #abortTransaction, 
 #beginTransaction, #continueTransaction, or #commitTransaction.  Note that
 an attempt to commit which fails (due to transaction conflicts) is considered
 a transaction boundary since the session is shifted to the most recent view
 by the failed commit attempt."

^ self _zeroArgPrim: 122
%

category: 'Transaction Control'
classMethod: System
millisecondsSinceLogin

"Answer a SmallInteger representing the number of milliseconds since this 
 session logged in to the repository."

^ self _zeroArgPrim: 123
%

category: 'Transaction Control'
classmethod: System
trapAddToClosureList: anObject

"If anObject is a non-committed object, signal an error 
 when a subsequent attempt to commit adds anObject to the closure list
 for the commit.  The error will be not-trappable within Gemstone Smalltalk.
 The error information will include the parent dirty or not-committed 
 object which caused anObject to be added to the closure list.  
 After the error is returned to topaz or GBS, you can then save the 
 stack and use Object>>findReferencesInMemory:
 to determine what paths might be causing anObject to become reachable 
 from persistent state.

 If anObject == nil, then cancel a previous trap object.
 Returns receiver."

<primitive: 850>
anObject isCommitted ifTrue:[ 
  ArgumentError signal: 
   'arg to trapAddToClosureList: must be nil or a non-committed object'
].
^self _primitiveFailed: #trapAddToClosureList: args: { anObject }
%

category: 'Signals'
classmethod: System
disableAlmostOutOfMemoryError

"Disables raising an error error when session's temporary object memory
 is almost full.

 See signalAlmostOutOfMemoryThreshold:  for more details. "

self _updateSignalErrorStatus: 5 toState: -1
%

category: 'Environment Access'
classmethod: System
secondsSinceStoneStarted

"Answer a SmallInteger which is the number of seconds since the stone started."

^ self _zeroArgPrim: 124
%

category: 'LDAP Support'
classmethod: System
validatePasswordUsingLdapServers: aUriArray baseDn: aBaseDn filterDn: aFilterDn userId: aUserId password: pw

<primitive: 859>
aUriArray == nil
  ifFalse:[ aUriArray _validateClass: Array ] .
aUriArray do:[:e| e _validateClass: String ].
aBaseDn _validateClass: String .
aFilterDn == nil
  ifFalse:[ aFilterDn _validateClass: String ] .
aUserId _validateClass: String .
pw _validateClass: String .

^ self _primitiveFailed: 
       #validatePasswordUsingLdapServers:baseDn:filterDn:userId:password
       args: { aUriArray . aBaseDn . aFilterDn . aUserId . pw }
%

category: 'Host System Access'
classmethod: System
validatePasswordForUser: uid password: pw isEncrypted: encrypted

"Validate the password pw for the user id uid using the getpwnam(), getspnam(),
 and crypt() C functions.  If encrypted is true, then pw is assumed to be
 encrypted and is compared with the encrypted password returned by getpwnam()
 or getspnam().  If encrypted is false, then pw is assumed to be the clear-text
 password.

 Both the uid and pw arguments must be instances of String.  encrypted must
 be true or false.

 Returns true if the password was successfully authenticated or false if 
 the password is incorrect.  Raises a #genericKernelError error with a brief
 error description if an error occurs.

 Requires the UserPassword privilege.  

 Some UNIX systems require the gem process to be running as root in order
 for this method to work correctly."

<primitive: 864>
uid _validateClass: String .
pw _validateClass: String .
encrypted _validateClass: Boolean .
^self _primitiveFailed: #validatePasswordForUser:password:isEncrypted:
      args: { uid . pw . encrypted }
%

category: 'Host System Access'
classmethod: System
encryptPassword: pw withSalt: salt

"Calls the C function crypt() to encrypt pw, which is assumed to be the
 clear text password to be encrypted.  Returns a new String which contains
 the encrypted password.

 Both the pw and salt arguments must be instances of String."

<primitive: 865>
pw _validateClass: String .
salt _validateClass: String .
^self _primitiveFailed: #encryptPassword:withSalt: args: { pw . salt }
%

category: 'Transaction Control'
classMethod: System
refreshTransientSymbolList

"Rebuilds the session's transient symbol list from the one in the UserProfile.
 Returns the receiver."

^ self _zeroArgPrim: 129
%

category: 'Private'
classMethod: System
_debugInfoForPage: anInt

"If anInt is the page ID of a data page, return an Array of 12 elements:

  1 - Time page was written in seconds since January 1, 1970 (SmallInteger)
  2 - Session ID that wrote the page. (SmallInteger)
  3 - Number of valid objects in the page. (SmallInteger)
  4 - Number of invalid objects in the page. (SmallInteger)
  5 - Array of valid object IDs (Array of SmallIntegers)
  6 - Array of classes of valid objects.
  7 - Array of invalid object IDs (Array of SmallIntegers)
  8 - Array of classes of invalid objects.
  9 - Array of correct page IDs for invalid objects.
 10 - Array of correct classes for invalid objects.
 11 - Array of session IDs that wrote the correct pages in element 9.
 12 - Array of timestamps that the correct pages in element 9 were written.

 This method is intended to be used only by GemStone staff to aid in the debugging of
 repository problems.

 This method may only be executed by SystemUser.

 Returns nil if the page is not a data page."

<primitive: 870>

anInt _validateClass: SmallInteger .
self _primitiveFailed: #_debugInfoForPage: args: { anInt }
%

category: 'Host System Access'
classmethod: System
hostCpuUsage

"Returns an Array of 5 SmallIntegers with values between 0 and 100 which
 have the following meanings:

 1 - Percent CPU active (user + system)
 2 - Percent CPU idle
 3 - Percent CPU user
 4 - Percent CPU system (kernel)
 5 - Percent CPU I/O wait

 On hosts with multiple CPUs, these figure represent the average across all
 processors.

 The results of the first call to this method are invalid and should be
 discarded.

 Returns nil if the host system does not support collecting CPU statistics."
 
^ self _zeroArgPrim: 130
%


category: 'Host System Access'
classmethod: System
hostId

"Return an Integer which represents a unique 64 bit identifier for the host
 where the session is running.  This identifier is created and managed by
 GemStone and is not related in anyway to the hostname or IP address of the
 host."

^ self _zeroArgPrim: 131
%

category: 'Session Control'
classmethod: System
setSessionPriority: anInt forSessionId: aSessionId

"Sets the priority of the given session to a new value.
 
 Session priority is used by the stone to order requests for 
 service by sessions.  anInt must be one of the following valid
 priorities:

 0 - lowest priority
 1 - low priority
 2 - normal priority (default)
 3 - high priority
 4 - highest priority

 By default, user sessions have normal (medium) priority and reclaim and
 Admin GC gems have lowest priority.

 A session which holds or is about the receive the commit token is always
 given highest priority by the stone regardless of what priority is assigned
 to the session.

 Session priority is only pertinent when more than one session requests
 service from the stone during the same stone wakeup interval.  A single
 session requesting service is always processed immediately.

 Sessions with the same priority are serviced in ascending session ID order.

 Requires the SessionAccess and SessionPriority privileges.

 If successful, this method answers a SmallInteger which indicates the previous
 priority of the session.  -1 is returned if aSessionId does not refer to
 a valid session.  An exception is raised if anInt is not a valid priority or
 the session executing this method does not have the required privileges."

^self _sessionPriorityPrim: anInt forSessionId: aSessionId
%

category: 'Session Control'
classmethod: System
_sessionPriorityPrim: anInt forSessionId: aSessionId

"Fetches or sets the priority for the given session ID.
 If anInt is -1, then the current priority for the given
 session ID is returned.  Otherwise anInt must be a valid
 priority between 0 and 4.  See the method:
   System(C)>>setSessionPriority:forSessionId:
 for more information on session priority."

<primitive: 898>
anInt _validateClass: SmallInteger .
aSessionId _validateClass: SmallInteger .
((anInt < -1) or:[ anInt > 4])
  ifTrue:[anInt _error: #rtErrArgOutOfRange args:{ -1 . 4 } .].
aSessionId < 1
  ifTrue:[aSessionId _error: #rtErrArgOutOfRange ].
^self _primitiveFailed: #_setSessionPriorityPrim:forSessionId:
      args: { anInt . aSessionId }
%

category: 'Session Control'
classmethod: System
priorityForSessionId: aSessionId

"Returns an integer between 0 and 4 indicating the current priority
 of the session with the given session ID.  Returns -1 if aSession ID
 does not refer to a valid session.

 Requires the SessionAccess privilege to query the priority of another
 session.  No privilege is required to query the priority of the current
 session."

^self _sessionPriorityPrim: -1 forSessionId: aSessionId
%

category: 'Cache Statistics'
classmethod: System
stoneMessageKinds

"Returns an array of Strings that lists the request kinds which can be sent
 from a session to the stone.  The numeric index of a message may be obtained
 from the 'MessagesToStone' (gem and page servers) or 'GsMsgKind' (stone)
 cache statistics.

 The result of this method may change with every new release of GemStone/64."

^ self _zeroArgPrim: 133
%

category: 'Transaction Control'
classmethod: System
commitsSinceCurrentView

"Returns an SmallInteger which is the number of commits which have occurred since
 the session obtained its current view.  If the session has no current view due to
 a lost OT root error, then -1 is returned.  A result of 0 indicates the session
 has the most current view of the repository."

^ self _zeroArgPrim: 134
%

category: 'Host System Access'
classmethod: System
hostCpuCount

"Returns an SmallInteger which is the number of CPUs on the host system.  In most cases,
 this number represents the total number of CPU cores on the host system."

^ self _zeroArgPrim: 135
%

category: 'Session Control'
classmethod: System
sessionIdHoldingGcLock

"Returns an SmallInteger which is the session ID of the session currently holding
 the garbage collection lock.  Returns 0 if the GC lock is free.

 Requires the SessionAccess privilege."

^ self _zeroArgPrim: 136
%

category: 'Private'
classmethod: System
_cacheStatWithName: aString opCode: anInt

"Return the value of the cache stat with the given name, which must match an
 element of the Array returned by the #cacheStatisticsDescription method.

 opCode 1 means the statistic is for the primary shared page cache monitor.
 opCode 2 means the statistic is for the stone.
 opCode 8 means the statistic is for the current session.

 This method may be used to access primary shared page cache monitor and stone
 statistics from hosts other than that on which the stone is running.

 Returns nil if a statistic matching aString was not found for the process kind
 specified by the opCode."

<primitive: 430>
aString _validateClass: String .
anInt _validateClass: SmallInteger .
^ self _primitiveFailed: #_cacheStatWithName:opCode:
       args: { aString . anInt }
%

category: 'Cache Statistics'
classmethod: System
stoneCacheStatisticWithName: aString

"Return the value of the cache stat with the given name, which must match an
 element of the Array returned by the #cacheStatisticsDescription method 
 applicable to the stone process.  The UserTime and SysTime statistics cannot
 be accessed using this method.

 Returns nil if a statistic matching aString was not found for the stone.

 This method may be used on hosts remote from the stone process."

^ self _cacheStatWithName: aString opCode: 2
%

category: 'Cache Statistics'
classmethod: System
primaryCacheMonitorCacheStatisticWithName: aString

"Return the value of the cache stat with the given name for the cache monitor
 process running on the stone's host.  The name must match an element of the
 Array returned by the #cacheStatisticsDescription method  applicable to the
 cache monitor process.  The UserTime and SysTime statistics cannot be accessed
 using this method.

 Returns nil if a statistic matching aString was not found for the cache monitor.

 This method may be used on hosts remote from the stone process."

^ self _cacheStatWithName: aString opCode: 1
%

category: 'Cache Statistics'
classmethod: System
myCacheStatisticWithName: aString

"Return the value of the cache stat with the given name for the current session.
 The name must match an element of the Array returned by the
 #cacheStatisticsDescription method applicable to a gem process.  The UserTime
 and SysTime statistics cannot be accessed using this method.

 Returns nil if a statistic matching aString was not found for a gem.

 This method may be used on hosts remote from the stone process."

^ self _cacheStatWithName: aString opCode: 8
%

category: 'Session Control'
classmethod: System
sessionIsOnStoneHost

"Answer true if this session is running on the same host computer as the stone
 process.  Otherwise answer false."

^ self _zeroArgPrim: 137
%

! Next 3 methods added for 41278
!
category: 'Transient Session State'
classmethod: System
incrementProgressCountBy: aSmallInt

"Increments the value of the progressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -1 incrementBy: aSmallInt
%

category: 'Transient Session State'
classmethod: System
decrementProgressCountBy: aSmallInt

"Decrements the value of the progressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -1 incrementBy: aSmallInt negated
%

category: 'Transient Session State'
classmethod: System
setProgressCountTo: aSmallInt

"Sets the value of the progressCount statistic to aSmallInt.
 Returns aSmallInt."

^ self _sessionCacheStatAt: -1 put: aSmallInt
%

category: 'Transient Session State'
classmethod: System
incrementIndexProgressCountBy: aSmallInt

"Increments the value of the indexProgressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -2 incrementBy: aSmallInt
%

category: 'Transient Session State'
classmethod: System
decrementIndexProgressCountBy: aSmallInt

"Decrements the value of the indexProgressCount statistic by aSmallInt and
 returns the new value."

^ self _sessionCacheStatAt: -2 incrementBy: aSmallInt negated
%

! fixed 41567
category: 'Transient Session State'
classmethod: System
setIndexProgressCountTo: aSmallInt

"Sets the value of the indexProgressCount statistic to aSmallInt.
 Returns aSmallInt."

^ self _sessionCacheStatAt: -2 put: aSmallInt
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForType: aProcessKind

"Returns an Array of Strings describing the result of the method #'cacheStatisticsAt:'. The
 argument <aProcessKind> is the StatType (as returned as element #4 by #'cacheStatisticsAt:')
 and must be one of the following:

   1 = Shared Page Cache Monitor
   2 = Stone
   4 = Page Server
   8 = Gem (including Topaz, GBS, and other GCI applications).

 The result includes only those appropriate to the specific executable (common and unique).
 Note that Statmonitor will capture additional statistics (e.g., on itself and the system)
 that are not actually stored in shared memory."

^ self _zeroArgPrim: aProcessKind + 4000
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForMonitor

"Returns an Array of Strings describing cache statistics applicable to the shared
 page cache monitor."

^ self _zeroArgPrim: 4001
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForStone

"Returns an Array of Strings describing cache statistics applicable to the 
 stone."

^ self _zeroArgPrim: 4002
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForPageServer

"Returns an Array of Strings describing cache statistics applicable to a 
 page server process."

^ self _zeroArgPrim: 4004
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionForGem

"Returns an Array of Strings describing cache statistics applicable to a 
 gem process."

^ self _zeroArgPrim: 4008
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsAt: aProcessSlot

"Returns an array of statistics appropriate to the specific executable (common + unique).
 The argument aProcessSlot is the same as described in #'cacheStatistics:' method.
 The contents are described by the result of sending element 4 of the result to the
 #cacheStatisticsDescriptionFor: method."

^ self _primCacheStatsForSlotOrSession: aProcessSlot opCode: 1
%

category: 'Cache Statistics'
classmethod: System
gemCacheStatisticsForSessionId: aSessionId

"Same as the cacheStatisticsAt: method except the argument is the session ID
 of a session currently connected to the shared memory cache.  In systems
 that use multiple shared memory caches, the session must exist on the same
 cache as the session invoking this method.

 Only cache statistics applicable to a Gem process are returned. 
 The description of these statistics can be determined by evaluating:
   'System cacheStatisticsDescriptionForGem'
 
 nil is returned if the session cannot be located."

^ self _primCacheStatsForSlotOrSession: aSessionId negated opCode: 1
%

category: 'Cache Statistics'
classmethod: System
stoneCacheStatistics

"Return the cache statistics for the stone if this session is located on 
 the same host as the stone process. 

 Only cache statistics applicable to the Stone process are returned. 
 The description of these statistics can be determined by evaluating:
   'System cacheStatisticsDescriptionForStone'
 
 nil is returned if the stone is not running the session's host."

^ self sessionIsOnStoneHost
    ifTrue:[ self cacheStatisticsAt: 1 ] "stone is always slot 1, if it's here at all"
    ifFalse:[ nil ]
%

category: 'Cache Statistics'
classmethod: System
sharedPageCacheMonitorCacheStatistics

"Return the cache statistics for the SPC monitor process.

 Only cache statistics applicable to the SPC monitor process are returned. 
 The description of these statistics can be determined by evaluating:
   'System cacheStatisticsDescriptionForMonitor'"

^ self cacheStatisticsAt: 0  "monitor is always slot 0"
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsProcessId: aPid

"Search the cache for a process id matching aPid.  If found, return the 
 statistics appropriate for that process.  Returns nil if the process 
 was not found."
 

^ self _primCacheStatsForSlotOrSession: aPid opCode: 2
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsForProcessWithCacheName: aString

"Search the cache for a process with cache name matcing aString.
 If found, return the statistics appropriate for that process.
 Returns nil if the process with the given cache name was not found."
 

^ self _primCacheStatsForSlotOrSession: aString opCode: 3
%

category: 'Cache Statistics'
classmethod: System
processIdForSlot: anInt

"Return the process ID of the process using the ncache slot at
 index anInt.  Return -1 if the slot is not in use or out of 
 range."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 4
%

category: 'Cache Statistics'
classmethod: System
cacheSlotForProcessId: anInt

"Return the cache slot at being used by the given process ID.
 Return -1 if the slot was not found."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 5
%

category: 'Cache Statistics'
classmethod: System
hostCpuStatsForProcessSlot: anInt

"For the process using the cache process slot anInt, return an Array of
 2 integers as follows:
   array[1] - user mode CPU milliseconds used
   array[2] - system mode CPU milliseconds used

 Both array elements are set to -1 if the process slot is out of range
 or not in use, or if this method is not supported for the host
 architecture."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 6
%

category: 'Cache Statistics'
classmethod: System
hostCpuStatsForProcessId: anInt

"Return an Array of 2 integers as follows:
  array[1] - user mode CPU milliseconds
  array[2] - system mode CPU milliseconds

 Both array elements will be -1 if the process slot is out of range
 or not in use or if this method is not supported for the host
 architecture.

 Is not required to be attached to the shared page cache or even be
 a GemStone process.  The method will success for any process for which
 the gem session has permission to view its CPU usage statistics."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 7
%

category: 'Cache Statistics'
classmethod: System
cacheStatsForPageServerWithSessionId: anInt

"Return the cache statistics for the page server with the given session 
 ID.  Page servers assume the same session ID as their client gem.

 Return -1 if the page server with the given session ID was not found."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 8
%

category: 'Cache Statistics'
classmethod: System
cacheName: aString

"Sets the name the current session is known by in the shared cache
 to 'aString'. Does nothing if the current session is not attached
 to a shared cache. Raises an error if 'aString' is too long.
 Returns self."

^ self _cacheName: aString
%

category: 'Cache Statistics'
classmethod: System
processKindForSlot: anInt

"Return the process kind occupying the given cache slot as follows:

   1 = Shared Page Cache Monitor
   2 = Stone
   4 = Page Server
   8 = Gem (including Topaz, GBS, and other GCI applications).

 A return value of -1 means the slot index was out of range or the
 slot is not in use."

^ self _primCacheStatsForSlotOrSession: anInt opCode: 9
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsDescriptionAt: aSlot

"Returns an array of strings which describe the cache statistics for the
 kind using the cache slot with index anInt.  Returns an empty array if
 the slot is not in use."

| kind |
kind := self processKindForSlot: aSlot .
^ kind == -1
    ifTrue:[ Array new ]
    ifFalse:[ self cacheStatisticsDescriptionForType: kind ]
%

category: 'Cache Statistics - Deprecated'
classmethod: System
cacheStatisticsDescription

"Returns an Array of Strings describing the result of the method
 cacheStatistics: . 

 A new Array of new Strings is created every  time this primitive 
 is called, so the application may wish to cache the result.

 This method is deprecated and may be removed in a future release.
 It is replaced by the following classmethods in class System:

  cacheStatisticsDescriptionForMonitor
  cacheStatisticsDescriptionForStone
  cacheStatisticsDescriptionForPageServer
  cacheStatisticsDescriptionForGem
  cacheStatisticsDescriptionAt:
  cacheStatisticsDescriptionForType:
"

self deprecated: 'System class>>cacheStatisticsDescription: deprecated v3.0. 
Use #cacheStatisticsDescriptionForType: instead'.

^ self _deprecatedCacheStatisticsDescription
%

category: 'Cache Statistics - Deprecated'
classmethod: System
_deprecatedCacheStatisticsDescription

<primitive: 580>
^ self _primitiveFailed: #_deprecatedCacheStatisticsDescription
%

category: 'Cache Statistics - Deprecated'
classmethod: System
cacheStatistics: aProcessSlot

"Returns an Array whose contents are described by the result of the
 cacheStatisticsDescription method.  The Array contains statistics for the
 specified slot in the GemStone shared memory cache to which this session is
 attached.

 The argument aProcessSlot should be a SmallInteger between 0 and the number of
 process slots in the shared cache minus 1, inclusive.  If aProcessSlot is 
 outside the range of valid process slots, or the session executing this
 method is not using a shared cache, generate an error.  If the slot specified
 by aProcessSlot is an inactive slot, returns nil.  The method
 cacheSlotForSessionId: may be used to determine the process slot of a session
 on the same shared cache.
 
 The process slots that are predefined are:

    slot 0: The shared page cache monitor.

    slot 1: The Stone if the cache is on the same machine as the Stone.
            Otherwise, a page server that is used to monitor the cache for
            the Stone.

 No other slots are guaranteed.  However, slot 2 is the often the page server
 and slot 3 is often the GcGem.  These depend to some extent on the relative
 speed of the processes during startup.  In addition, the GcGem can be 
 shut down, and when it is restarted, it is unlikely to end up at the same
 position.

 This method is deprecated and may be removed in a future release.
 It is replaced by the following classmethod in class System:

  cacheStatisticsAt:
"

self deprecated: 'System class>>cacheStatistics: deprecated v3.0. 
Use #cacheStatisticsAt: instead'.
 ^self _primCacheStatsForSlotOrSession: aProcessSlot opCode: 0
%

category: 'Cache Statistics - Deprecated'
classmethod: System
cacheStatisticsForSessionId: aSessionId

"Same as the cacheStatistics: method except the argument is the session ID
 of a session currently connected to the shared memory cache.  In systems
 that use multiple shared memory caches, the session must exist on the same
 cache as the session invoking this method.

 nil is returned if the session cannot be located.

 This method is deprecated and may be removed in a future release.
 It is replaced by the following classmethod in class System:

  gemCacheStatisticsForSessionId: 
"

self deprecated: 'System class>>cacheStatisticsForSessionId: deprecated v3.0. 
Use #gemCacheStatisticsForSessionId: instead'.
 ^self _primCacheStatsForSlotOrSession: aSessionId negated opCode: 0
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsForAllSlotsShort

"Return an Array with one entry for each process attached to the shared 
 cache.  Each element in the result is an array containing 5 elements.  
 The first 4 elements are the same first 4 elements returned when fetching
 cache statistics for a session, slot, or process.  The fifth element is the
 process slot for the process.  Each sub-array of 5 elements contain the 
 following elements:

  1 - ProcessName (String)
  2 - ProcessId   (SmallInteger)
  3 - SessionId   (SmallInteger)
  4 - StatType    (SmallInteger)
  5 - Cache Slot  (SmallInteger)
"

^ self _zeroArgPrim: 138
%

category: 'Cache Statistics'
classmethod: System
cacheStatisticsForAllSlots

"Return an Array with one entry for each process attached to the shared 
 cache.  Each element in the result contains the cache statistis for that 
 slot.  The last element in each array is the slot ID of the slot.
 The subarrays will be different sizes depending on what type of process
 is using the slot."

^ self _zeroArgPrim: 139
%
